--------------------------------------------------------------------------
--
--  File        : opb_pci9030.vhd
--
--  Title       : PLX PCI9030 local bus -> OPB Master
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : This code assemus that the PCI9030 local bus and the
--                OPB run from the same clock i.e. LCLK == OPB_Clk.
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity opb_pci9030 is
  generic (
    -- Bus protocol parameters, do not add to or delete
    C_OPB_AWIDTH    : integer               := 32;
    C_OPB_DWIDTH    : integer               := 32;
    C_FAMILY        : string                := "virtex2p"
    );
  port (
    -- PLX PCI9030 Local Bus Signals
    nREADY    : out std_logic;
    nLBE      : in std_logic_vector(3 downto 0);
    nBTERM    : out std_logic;
    LWnR      : in std_logic;
    nADS      : in std_logic;
    LAD       : inout std_logic_vector(31 downto 0);

    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    M_DBus          : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
    M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
    M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    M_busLock       : out   std_logic;
    M_request       : out   std_logic;
    M_RNW           : out   std_logic;
    M_select        : out   std_logic;
    M_seqAddr       : out   std_logic;
    OPB_errAck      : in    std_logic;
    OPB_MGrant      : in    std_logic;
    OPB_retry       : in    std_logic;
    OPB_timeout     : in    std_logic;
    OPB_xferAck     : in    std_logic;
    debug           : out   std_logic_vector(39 downto 0));

end entity opb_pci9030;

architecture implementation of opb_pci9030 is

  signal mst_request : std_logic;
  signal mst_select  : std_logic;
  
  -- mst_A holds the 32 bit OPB address
  signal mst_A  : std_logic_vector(0 to 31);
  -- slv_D holds the 32 bit OPB read data
  signal slv_D  : std_logic_vector(0 to 31);

  signal nREADY_int : std_logic;

begin

  M_request <= mst_request;
  M_select <= mst_select;
  nBTERM <= '1';
  nREADY <= nREADY_int;

  process (OPB_Clk, LWnR, nADS, LAD, OPB_Rst)
  begin
    if rising_edge(OPB_Clk) then
      -- nADS low starts new local bus cycle, we have to latch
      -- address here because the local bu is multiplexed
      if nADS = '0' then
        mst_A <= LAD;
      end if;
    end if;
  end process;
  
  local_data_bus: process (OPB_Clk, LWnR, nADS, nREADY_int)
    variable output_data : std_logic;
  begin
    LAD <= (others => 'Z');
    if output_data = '1' then
      LAD <= slv_D;
    end if;
    debug(39) <= output_data;
    if rising_edge(OPB_Clk) then
      if nADS = '0' and LWnR = '0' then
	output_data := '1';
      end if;
      if nREADY_int = '0' or OPB_Rst = '1' then
	output_data := '0';
      end if;	
    end if;
  end process;
  
  -- This interface will only perform one operation on one request,
  -- bus locking is not required.
  M_busLock <= '0';
  -- Address sequencing not supported.
  M_seqAddr <= '0';

  OPB_requesting : process (OPB_Clk, nADS, LWnR, OPB_Rst)
  begin
    if rising_edge(OPB_Clk) then
      -- nADS low starts new local bus cycle
      if nADS = '0' then
	mst_request <= '1';
      end if;
      if OPB_MGrant = '1' then
	mst_request <= '0';
      end if;
      if OPB_Rst = '1' then
	mst_request <= '0';
      end if;
    end if;
  end process;

  OPB_data_transfer : process (OPB_Clk, OPB_MGrant, OPB_xferAck, OPB_errAck,
			       nADS, LWnR, OPB_Rst)
    variable write_pend   : std_logic := '0';
    variable read_pend    : std_logic := '0';
    variable transfer_act : std_logic := '0';
  begin
    M_DBus <= (others => '0');
    if write_pend = '1' then
      M_DBus <= LAD(31 downto 0);
    end if;

    M_ABus <= (others => '0');
    if mst_select = '1' then
      M_ABus <= mst_A;
    end if;
    
    if rising_edge(OPB_Clk) then
      if nADS = '0' then
        if LWnR = '1' then
          write_pend := '1';
        else
          read_pend := '1';
        end if;
      end if;

      -- Data transfer starts with OPB_MGrant asserted for one cycle
      if OPB_MGrant = '1' and
         (write_pend = '1' or read_pend = '1') then
	transfer_act := '1';
      end if;

      nREADY_int <= '1';
      if transfer_act = '1' then
	if OPB_xferAck = '1' then
	  if read_pend = '1' then
            slv_D <= OPB_DBus;
	  end if;
          nREADY_int <= '0';
	  transfer_act := '0';
	  write_pend := '0';
	  read_pend := '0';
	end if;
        -- On timeout we do not activate ready, PCI9030 should
        -- be enabled to issue a timeout
	if OPB_timeout = '1' or OPB_errAck = '1' then
	  transfer_act := '0';
	  write_pend := '0';
	  read_pend := '0';
	end if;
      end if;
      
      M_RNW <= '0';
      mst_select <= '0';
      M_BE <= (others => '0');
      if transfer_act = '1' then
	mst_select <= '1';
	if read_pend = '1' or write_pend = '1' then
          M_BE <= not nLBE(3) & not nLBE(2) & not nLBE(1) & not nLBE(0);
	end if;
	if read_pend = '1' then
	  M_RNW <= '1';
	end if;
      end if;

      if OPB_Rst = '1' then
	write_pend := '0';
	read_pend := '0';
	transfer_act := '0';
      end if;
    end if;
  end process;

  debug(38) <= nREADY_int;
  debug(37) <= LWnR;
  debug(36) <= nADS;
  debug(35 downto 32) <= nLBE(3 downto 0);
  debug(31 downto 0) <= LAD(31 downto 0);
    
end implementation;
