--------------------------------------------------------------------------
--
--  File        : opb_ubicom.vhd
--                $RCSfile$
--                $Date$
--                $Revision$
--
--  Title       : Ubicom OPB Master
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  		  $Log$
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.opb_ubicom_pkg.all;

entity opb_ubicom is
  generic (
    
    -- Bus protocol parameters, do not add to or delete
    C_BASEADDR      : std_logic_vector      := X"00000000";
    C_HIGHADDR      : std_logic_vector      := X"0000FFFF";
    C_OPB_AWIDTH    : integer               := 32;
    C_OPB_DWIDTH    : integer               := 32;
    C_FAMILY        : string                := "virtex2p";
    C_AR0_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR0_HIGHADDR  : std_logic_vector      := X"00000000";
    C_AR1_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR1_HIGHADDR  : std_logic_vector      := X"00000000";
    C_AR2_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR2_HIGHADDR  : std_logic_vector      := X"00000000";
    C_AR3_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR3_HIGHADDR  : std_logic_vector      := X"00000000";
    C_AR4_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR4_HIGHADDR  : std_logic_vector      := X"00000000";
    C_AR5_BASEADDR  : std_logic_vector      := X"FFFFFFFF";
    C_AR5_HIGHADDR  : std_logic_vector      := X"00000000"
    );
  port (
    -- Ubicom interface signals
    ULE          : in std_logic;
    nUBERR       : out std_logic;
    nUACK        : out std_logic;
    nUIRQ        : out std_logic;
    nURD         : in std_logic;
    nUWR         : in std_logic;
    UA           : in std_logic_vector(16 downto 0);
                                  -- Ubicom address bits A16-A9 and A0,
                                  -- A8-A1 are multiplexed with D7-D0
    UD           : inout std_logic_vector(7 downto 0);

    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    M_DBus          : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
    M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
    M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    M_busLock       : out   std_logic;
    M_request       : out   std_logic;
    M_RNW           : out   std_logic;
    M_select        : out   std_logic;
    M_seqAddr       : out   std_logic;
    OPB_errAck      : in    std_logic;
    OPB_MGrant      : in    std_logic;
    OPB_retry       : in    std_logic;
    OPB_timeout     : in    std_logic;
    OPB_xferAck     : in    std_logic);

end entity opb_ubicom;

architecture implementation of opb_ubicom is

  signal ubiLE       : std_logic;
  signal nubiRD      : std_logic;
  signal nubiWR      : std_logic;
  signal nubiBERR    : std_logic;
  signal nubiACK     : std_logic;
  signal nubiIRQ     : std_logic;
  signal ubiA        : std_logic_vector(16 downto 0);
  signal ubiD        : std_logic_vector(7 downto 0);
  signal mst_request : std_logic;
  signal mst_select  : std_logic;
  
  -- mst_A holds the 32 bit OPB address for Ubicom OPB accesses
  signal mst_A  : std_logic_vector(0 to 31);
  -- mst_D holds the 16 bit OPB write data for Ubicom OPB accesses
  signal mst_D  : std_logic_vector(0 to 15);
  -- slv_D holds the 16 bit OPB read data for Ubicom OPB accesses
  signal slv_D  : std_logic_vector(0 to 15);

  signal write_trigger : std_logic;
  signal read_trigger  : std_logic;

begin

  nUBERR <= nubiBERR;
  nUACK <= nubiACK;
  nUIRQ <= nubiIRQ;
  M_request <= mst_request;
  M_select <= mst_select;
  
  nubiIRQ <= '1';
  
  sync_ubi_to_local_clock : process (OPB_Clk, ULE, nURD, nUWR, UA, UD)
  begin
    -- Synchronise all incoming signals from Ubicom to OPB clock (in IOBs)
    if rising_edge(OPB_Clk) then
      ubiLE <= ULE;
      nubiRD <= nURD;
      nubiWR <= nUWR;
      ubiA(16 downto 9) <= UA(16 downto 9);
      ubiA(0) <= UA(0);
      -- Latch multiplexed address bits
      if ubiLE = '1' then
	ubiA(8 downto 1) <= ubiD;
      end if;
      ubiD <= UD;
    end if;
  end process;

  master_address_generation : process (OPB_Clk, nubiWR, ubiA, ubiD,
				       mst_select, mst_A)
  begin
    M_ABus <= (others => '0');
    if mst_select = '1' then
      M_ABus(0 to 29) <= mst_A(0 to 29);
      M_ABus(30 to 31) <= "00";
    end if;
    mst_A(16 to 30) <= ubiA(15 downto 1);
    mst_A(31) <= '0';
    if rising_edge(OPB_Clk) then
      if ubiA(16) = '1' then
	if ubiA(UBICOM_REG_BITS downto 1) = UBICOM_REG_A32 then
	  if nubiWR = '0' then
	    if ubiA(0) = '0' then
	      mst_A(0 to 7) <= ubiD;
	    else
	      mst_A(8 to 15) <= ubiD;
	    end if;
	  end if;
	end if;
      end if;
    end if;
  end process;

  ubicom_data_bus : process (OPB_Clk, nubiRD, ubiA, slv_D)
  begin
    if rising_edge(OPB_Clk) then
      UD <= (others => 'Z');
      if nubiRD = '0' then
	if ubiA(0) = '0' then
	  UD <= slv_D(0 to 7);
	else
	  UD <= slv_D(8 to 15);
	end if;
      end if;
    end if;
  end process;
  
  -- This interface will only perform one operation on one request,
  -- bus locking is not required.
  M_busLock <= '0';
  -- Address sequencing not supported.
  M_seqAddr <= '0';

  OPB_access_triggering : process (OPB_Clk, nubiWR, nubiRD, ubiA, ubiD)
    variable nubiWR_1 : std_logic;
    variable nubiRD_1 : std_logic;
  begin
    if rising_edge(OPB_Clk) then
      write_trigger <= '0';
      read_trigger <= '0';
      if ubiA(16) = '0' then
	if nubiWR = '0' and nubiWR_1 = '1' then
	  if ubiA(0) = '0' then
	    mst_D(0 to 7) <= ubiD;
	  else
	    mst_D(8 to 15) <= ubiD;
	    -- Bits 7--0 are written last, so this triggers the actual write
	    write_trigger <= '1';
	  end if;
	end if;
	if nubiRD = '0' and nubiRD_1 = '1' and ubiA(0) = '0' then
	  read_trigger <= '1';
	end if;
      end if;
      nubiWR_1 := nubiWR;
      nubiRD_1 := nubiRD;
    end if;
  end process;

  OPB_requesting : process (OPB_Clk, write_trigger, read_trigger, OPB_Rst)
  begin
    if rising_edge(OPB_Clk) then
      if write_trigger = '1' or read_trigger = '1' then
	mst_request <= '1';
      end if;
      if OPB_MGrant = '1' then
	mst_request <= '0';
      end if;
      if OPB_Rst = '1' then
	mst_request <= '0';
      end if;
    end if;
  end process;

  OPB_data_transfer : process (OPB_Clk, OPB_MGrant, OPB_xferAck, OPB_errAck,
			       write_trigger, read_trigger, OPB_Rst)
    variable write_pend   : std_logic := '0';
    variable read_pend    : std_logic := '0';
    variable transfer_act : std_logic := '0';
  begin
    if rising_edge(OPB_Clk) then
      if write_trigger = '1' or read_trigger = '1' then
	nubiACK <= '1';
	nubiBERR <= '1';
      end if;
      if write_trigger = '1' then
	write_pend := '1';
      end if;
      if read_trigger = '1' then
	read_pend := '1';
      end if;
      -- Data transfer starts with OPB_MGrant asserted for one cycle
      if OPB_MGrant = '1' then
	transfer_act := '1';
      end if;

      if transfer_act = '1' then
	if OPB_xferAck = '1' then
	  if read_pend = '1' then
	    if mst_A(30) = '0' then
	      slv_D <= OPB_DBus(0 to 15);
	    else
	      slv_D <= OPB_DBus(16 to 31);
	    end if;
	  end if;
	  nubiACK <= '0';
	  transfer_act := '0';
	  write_pend := '0';
	  read_pend := '0';
	end if;
	if OPB_timeout = '1' or OPB_errAck = '1' then
	  nubiBERR <= '0';
	  transfer_act := '0';
	  write_pend := '0';
	  read_pend := '0';
	end if;
      end if;
      
      M_RNW <= '0';
      mst_select <= '0';
      M_DBus <= (others => '0');
      M_BE <= (others => '0');
      if transfer_act = '1' then
	mst_select <= '1';
	if read_pend = '1' or write_pend = '1' then
	  if mst_A(30) = '0' then
	    M_BE <= "1100";
	  else
	    M_BE <= "0011";
	  end if;
	end if;
	if read_pend = '1' then
	  M_RNW <= '1';
	end if;
	if write_pend = '1' then
	  M_DBus(16 to 31) <= mst_D;
	  M_DBus(0 to 15) <= mst_D;
	end if;
      end if;

      if OPB_Rst = '1' then
	nubiACK <= '1';
	nubiBERR <= '1';
	write_pend := '0';
	read_pend := '0';
	transfer_act := '0';
      end if;
    end if;
  end process;
    
end implementation;
