---------------------------------------------------------------------------
--
--  File        : opb_cr_csr.vhd
--
--  Title       : VME64x Configuration ROM (CR) and Control and Status
--                Register (CSR)
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.opb_vme_pkg.all;

entity opb_cr_csr is
  generic (
    -- Bus protocol parameters, do not add to or delete
    C_BASEADDR      : std_logic_vector      := X"00000000";
    C_BASEMASK      : std_logic_vector      := X"FF000000";
    C_OPB_AWIDTH    : integer               := 32;
    C_OPB_DWIDTH    : integer               := 32;
    C_FAMILY        : string                := "virtex2p"
    );
  port (
    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    Sl_DBus         : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
    Sl_errAck       : out   std_logic;
    Sl_retry        : out   std_logic;
    Sl_toutSup      : out   std_logic;
    Sl_xferAck      : out   std_logic;
    OPB_ABus        : in    std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE          : in    std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW         : in    std_logic;
    OPB_select      : in    std_logic;
    OPB_seqAddr     : in    std_logic;

    nGA           : in  std_logic_vector(5 downto 0);
                                  -- VME geographical addressing pins
                                  -- GA0* to GA4*, nGA5 is GAP*
    module_nBERR  : in  std_logic;
    module_fail   : in  std_logic;
    VME64x_BAR    : out std_logic_vector(7 downto 0);
    VME64x_ADER0  : out std_logic_vector(31 downto 0);
    VME64x_ADER1  : out std_logic_vector(31 downto 0);
    VME64x_ADER2  : out std_logic_vector(31 downto 0);
    SYSFAIL_ena   : out std_logic;
    module_ena    : out std_logic;
    module_reset  : out std_logic;
    VME_irq_level : out std_logic_vector(2 downto 0);
    VME_irq_vector : out std_logic_vector(7 downto 0);
    
    nSYSRESETO    : out std_logic := 'Z';
    PWRDWN        : out std_logic := '0';
    HSOPEN        : in std_logic;
    BLUEOFF       : out std_logic;
    PWRONIF       : out std_logic;
    nTBID         : in std_logic_vector(3 downto 0);
    debug_out     : out std_logic_vector(6 downto 0)
    );
end opb_cr_csr;

architecture rtl of opb_cr_csr is

  -- Base Address Register (BAR) offset 0x7ffff
  -- bits 7 downto 3 define the CR/CSR region and correspond to
  -- A[23..19] in CR/CSR space
  signal base_address_reg : std_logic_vector(7 downto 0);
  -- Bit Set Register offset 0x7fffb
  --        write 1                write 0  read
  -- bit 7: put board in reset       -      board state (1 = reset)
  -- bit 6: enable SYSFAIL driver    -      SYSFAIL driver state (1 = enabled)
  -- bit 5: set module fail          -      (1 = board has failed)
  -- bit 4: enable module            -      module state (1 = enabled)
  -- bit 3: set BERR* flag           -      (1 = module issued BERR*)
  -- bit 2: no effect                -      (1 = CRAM owned)
  -- Bit Clear Regitser offset 0x7fff7
  --        write 1                write 0  read
  -- bit 7: remove board from reset  -      board state (1 = reset)
  -- bit 6: disable SYSFAIL driver   -      SYSFAIL driver state (1 = enabled)
  -- bit 5: clear module fail        -      (1 = board has failed)
  -- bit 4: disable module           -      module state (1 = enabled)
  -- bit 3: clear BERR* flag         -      (1 = module issued BERR*)
  -- bit 2: clear CRAM_OWNER         -      (1 = CRAM owned)
  signal bit_status_reg      : std_logic_vector(7 downto 0);
  signal bit_set             : std_logic_vector(7 downto 0);
  signal bit_clear           : std_logic_vector(7 downto 0);
  -- CRAM Owner Register offset 0x7fff3
  signal CRAM_owner_reg      : std_logic_vector(7 downto 0);
  signal ADER_0_reg          : std_logic_vector(31 downto 0);
  signal ADER_1_reg          : std_logic_vector(31 downto 0);
  signal ADER_2_reg          : std_logic_vector(31 downto 0);
  signal write_bar           : std_logic;
  signal write_CRAM_owner    : std_logic;
  signal module_reset_reg    : std_logic;
  signal SYSFAIL_enable_reg  : std_logic := '0';
  signal BERR_reg            : std_logic := '0';
  signal module_fail_reg     : std_logic := '0';
  signal module_enable_reg   : std_logic := '0';
  -- User Bit Set Register
  --        write 1                write 0  read
  -- bit 7: drive nSYSRESET low      -      Handle Switch open (1 = open)
  --        (only when SYSCON)
  -- bit 6: Handle switch power    HS       Handle Switch power seq. enabled
  --        control enable         disable  (default = 1)
  -- bit 5: Blue led on            led off  Blue led status (1 = on)
  -- bit 4: Transition board       TB power Transition board power state
  --        power on               off      (1 = on)
  -- bit 3: -                      -        Transition board ID bit 3
  -- bit 2: -                      -        Transition board ID bit 2
  -- bit 1: -                      -        Transition board ID bit 1
  -- bit 0: -                      -        Transition board ID bit 0
  signal user_bit_status_reg : std_logic_vector(7 downto 0);
  signal user_bit_set        : std_logic_vector(7 downto 0);
  signal user_bit_clear      : std_logic_vector(7 downto 0);
  -- VME interrupt level register 0 - no irq
  signal irq_level           : std_logic_vector(2 downto 0) := "000";
  -- VME interrupt vector register
  signal irq_vector          : std_logic_vector(7 downto 0) := X"00";
  -- The serial number will be filled in by the Ubicom IP2022
  -- after configuration. The serial number is the same as the MAC
  -- address of the 10baseT interface
  signal SN_0                : std_logic_vector(7 downto 0) := X"00";
  signal SN_1                : std_logic_vector(7 downto 0) := X"0E";
  signal SN_2                : std_logic_vector(7 downto 0) := X"B2";
  signal SN_3                : std_logic_vector(7 downto 0) := X"00";
  signal SN_4                : std_logic_vector(7 downto 0) := X"00";
  signal SN_5                : std_logic_vector(7 downto 0) := X"00";

  signal CR_data             : std_logic_vector(7 downto 0);
  signal data_wr             : std_logic_vector(7 downto 0);
  signal ack                 : std_logic;
  signal opb_wr_ack          : std_logic;
  signal opb_rd_ack          : std_logic;

  signal Sl_select           : std_logic;

  component config_ROM
    port (
      address : in  std_logic_vector(10 downto 0);
      data    : out std_logic_vector(7 downto 0);
      clk     : in  std_logic
      );
  end component;

begin

  i_config_rom: config_ROM
    port map (
      address => OPB_ABus(19 to 29),
      data => CR_data,
      clk => OPB_Clk);

  VME64x_BAR <= base_address_reg;
  VME64x_ADER0 <= ADER_0_reg;
  VME64x_ADER1 <= ADER_1_reg;
  VME64x_ADER2 <= ADER_2_reg;
  VME_irq_level <= irq_level;
  VME_irq_vector <= irq_vector;

  debug_out(0) <= opb_wr_ack;
  debug_out(1) <= opb_rd_ack;
  debug_out(2) <= OPB_select;
  debug_out(3) <= OPB_RNW;
  debug_out(4) <= OPB_ABus(0);
  debug_out(5) <= OPB_ABus(1);
  debug_out(6) <= OPB_BE(3);

  OPB_address_decoding : process (OPB_ABus, OPB_select)
  begin
    Sl_select <= '0';
    if (OPB_ABus and C_BASEMASK) = C_BASEADDR and
      OPB_select = '1' then
      Sl_select <= '1';
    end if;
  end process;

  CRAM_writing: process (OPB_Clk, bit_set, bit_clear, write_CRAM_owner, data_wr)
    variable CRAM_owner : std_logic_vector(7 downto 0) := "00000000";
  begin
    if rising_edge(OPB_Clk) then
      if write_CRAM_owner = '1' then
	if CRAM_owner = "00000000" or data_wr = "00000000" then
	  CRAM_owner := data_wr;
	end if;
      end if;
      
      if bit_clear(2) = '1' then
	CRAM_owner := "00000000";
      end if;

      CRAM_owner_reg <= CRAM_owner;
    end if;
  end process;

  Module_reset_control: process (OPB_Clk, bit_set, bit_clear, OPB_Rst)
    variable mod_reset : std_logic := '1';
    variable OPB_Rst_1 : std_logic;
    variable OPB_Rst_2 : std_logic;
  begin
    if rising_edge(OPB_Clk) then
      module_reset <= mod_reset;
      module_reset_reg <= mod_reset;
      
      if OPB_Rst = '1' then
	mod_reset := '1';
      end if;

      -- detect rising edge on OPB_Rst
      if OPB_Rst_2 = '1' and OPB_Rst_1 = '0' then
	mod_reset := '0';
      end if;

      OPB_Rst_2 := OPB_Rst_1;
      OPB_Rst_1 := OPB_Rst;

      if bit_set(7) = '1' then
	mod_reset := '1';
      end if;

      if bit_clear(7) = '1' then
	mod_reset := '0';
      end if;
    end if;
  end process;

  SYSFAIL_control: process (OPB_Clk, bit_set, bit_clear, OPB_Rst,
			    SYSFAIL_enable_reg)
  begin
    SYSFAIL_ena <= SYSFAIL_enable_reg;
    if rising_edge(OPB_Clk) then
      if bit_set(6) = '1' or OPB_Rst = '1' then
	SYSFAIL_enable_reg <= '1';
      end if;
      if bit_clear(6) = '1' then
	SYSFAIL_enable_reg <= '0';
      end if;
    end if;
  end process;

  Module_fail_control: process (OPB_Clk, bit_set, bit_clear, module_fail,
				OPB_Rst)
    variable mod_fail_2 : std_logic;
    variable mod_fail_1 : std_logic;
  begin
    if rising_edge(OPB_Clk) then
      if bit_set(5) = '1' or (mod_fail_2 = '0' and mod_fail_1 = '1') then
	module_fail_reg <= '1';
      end if;

      mod_fail_2 := mod_fail_1;
      mod_fail_1 := module_fail;
      
      if bit_clear(5) = '1' or OPB_Rst = '1' then
	mod_fail_2 := '0';
	mod_fail_1 := '0';
	module_fail_reg <= '0';
      end if;
    end if;
  end process;

  Module_enable_control: process (OPB_Clk, bit_set, bit_clear, OPB_Rst,
				  module_enable_reg)
  begin
    module_ena <= module_enable_reg;
    if rising_edge(OPB_Clk) then
      if bit_set(4) = '1' or OPB_Rst = '1' then
	module_enable_reg <= '1';
      end if;

      if bit_clear(4) = '1' then
	module_enable_reg <= '0';
      end if;
    end if;
  end process;
  
  BERR_control: process (OPB_Clk, bit_set, bit_clear, module_nBERR, OPB_Rst)
  begin
    if rising_edge(OPB_Clk) then
      if bit_set(3) = '1' or module_nBERR = '0' then
	BERR_reg <= '1';
      end if;
      if bit_clear(3) = '1' or OPB_Rst = '1' then
	BERR_reg <= '0';
      end if;
    end if;
  end process;
  
  Status_register: process (OPB_Clk, CRAM_owner_reg)
  begin
    if rising_edge(OPB_Clk) then
      bit_status_reg(7) <= module_reset_reg;
      bit_status_reg(6) <= SYSFAIL_enable_reg;
      bit_status_reg(5) <= module_fail_reg;
      bit_status_reg(4) <= module_enable_reg;
      bit_status_reg(3) <= BERR_reg;
      bit_status_reg(2) <= '0';
      if CRAM_owner_reg /= "00000000" then
	bit_status_reg(2) <= '1';
      end if;
      bit_status_reg(1 downto 0) <= "00";
    end if;
  end process;
  
  CSR_writing: process (OPB_Clk, OPB_ABus, OPB_DBus, Sl_select, OPB_RNW,
			OPB_BE, OPB_Rst)
    variable BAR    : std_logic_vector(7 downto 3) := "11111";
    variable parity : std_logic;
  begin
    opb_wr_ack <= '0';
    if Sl_select = '1' and OPB_RNW = '0' and OPB_BE(3) = '1' then
      opb_wr_ack <= '1';
    end if;

    if rising_edge(OPB_Clk) then
      write_bar <= '0';
      write_CRAM_owner <= '0';
      bit_set <= (others => '0');
      bit_clear <= (others => '0');
      user_bit_set <= (others => '0');
      user_bit_clear <= (others => '0');
      if Sl_select = '1' and OPB_RNW = '0' and OPB_BE(3) = '1' then
	data_wr <= OPB_DBus(24 to 31);
	case OPB_ABus(13 to 29) is
	  when CSR_BAR => BAR(7 downto 3) := OPB_DBus(24 to 28);
	  when CSR_BIT_SET => bit_set <= OPB_DBus(24 to 31);
	  when CSR_BIT_CLEAR => bit_clear <= OPB_DBus(24 to 31);
	  when CSR_CRAM_OWNER => write_CRAM_owner <= '1';
	  when CSR_UD_BIT_SET => user_bit_set <= OPB_DBus(24 to 31);
	  when CSR_UD_BIT_CLEAR => user_bit_clear <= OPB_DBus(24 to 31);
	  when CSR_ADER0_3 => ADER_0_reg(31 downto 24) <= OPB_DBus(24 to 31);
	  when CSR_ADER0_2 => ADER_0_reg(23 downto 16) <= OPB_DBus(24 to 31);
	  when CSR_ADER0_1 => ADER_0_reg(15 downto 8) <= OPB_DBus(24 to 31);
	  when CSR_ADER0_0 => ADER_0_reg(7 downto 0) <= OPB_DBus(24 to 31);
	  when CSR_ADER1_3 => ADER_1_reg(31 downto 24) <= OPB_DBus(24 to 31);
	  when CSR_ADER1_2 => ADER_1_reg(23 downto 16) <= OPB_DBus(24 to 31);
	  when CSR_ADER1_1 => ADER_1_reg(15 downto 8) <= OPB_DBus(24 to 31);
	  when CSR_ADER1_0 => ADER_1_reg(7 downto 0) <= OPB_DBus(24 to 31);
	  when CSR_ADER2_3 => ADER_2_reg(31 downto 24) <= OPB_DBus(24 to 31);
	  when CSR_ADER2_2 => ADER_2_reg(23 downto 16) <= OPB_DBus(24 to 31);
	  when CSR_ADER2_1 => ADER_2_reg(15 downto 8) <= OPB_DBus(24 to 31);
	  when CSR_ADER2_0 => ADER_2_reg(7 downto 0) <= OPB_DBus(24 to 31);
	  when USER_CSR_SN0 => SN_0 <= OPB_DBus(24 to 31);
	  when USER_CSR_SN1 => SN_1 <= OPB_DBus(24 to 31);
	  when USER_CSR_SN2 => SN_2 <= OPB_DBus(24 to 31);
	  when USER_CSR_SN3 => SN_3 <= OPB_DBus(24 to 31);
	  when USER_CSR_SN4 => SN_4 <= OPB_DBus(24 to 31);
	  when USER_CSR_SN5 => SN_5 <= OPB_DBus(24 to 31);
	  when USER_CSR_IRQ => irq_level <= OPB_DBus(29 to 31);
	  when USER_CSR_IRQV => irq_vector <= OPB_DBus(24 to 31);
	  when others =>
	end case;
      end if;

      parity := not (nGA(0) xor nGA(1) xor nGA(2) xor nGA(3) xor nGA(4));
      if OPB_Rst = '1' then
	BAR(7 downto 3) := "11111";
	if parity = nGA(5) then
	  for i in 0 to 4 loop
	    BAR(3+i) := not nGA(i);
	  end loop;
	end if;
      end if;

      base_address_reg(7 downto 3) <= BAR;
      base_address_reg(2 downto 0) <= (others => '0');
    end if;
  end process;
			 
  CSR_reading: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			base_address_reg, bit_status_reg,
			CRAM_owner_reg, ADER_0_reg, ADER_1_reg, ADER_2_reg,
			user_bit_status_reg, CR_data, irq_vector, irq_level)
    variable opb_rd_ack_1 : std_logic;
    variable data_out : std_logic_vector(7 downto 0);
  begin
    Sl_DBus <= (others => '0');
    if Sl_select = '1' and OPB_RNW = '1' then
      Sl_DBus(24 to 31) <= data_out;
      if OPB_ABus(13 to 18) = "000000" then
	Sl_DBus(24 to 31) <= CR_data;
      end if;
    end if;

    if rising_edge(OPB_Clk) then
      case OPB_ABus(13 to 29) is
	when CSR_BAR => data_out := base_address_reg;
	when CSR_BIT_SET => data_out := bit_status_reg;
	when CSR_BIT_CLEAR => data_out := bit_status_reg;
	when CSR_CRAM_OWNER => data_out := CRAM_owner_reg;
	when CSR_UD_BIT_SET => data_out := user_bit_status_reg;
	when CSR_UD_BIT_CLEAR => data_out := user_bit_status_reg;
	when CSR_ADER0_3 => data_out := ADER_0_reg(31 downto 24);
	when CSR_ADER0_2 => data_out := ADER_0_reg(23 downto 16);
	when CSR_ADER0_1 => data_out := ADER_0_reg(15 downto 8);
	when CSR_ADER0_0 => data_out := ADER_0_reg(7 downto 0);
	when CSR_ADER1_3 => data_out := ADER_1_reg(31 downto 24);
	when CSR_ADER1_2 => data_out := ADER_1_reg(23 downto 16);
	when CSR_ADER1_1 => data_out := ADER_1_reg(15 downto 8);
	when CSR_ADER1_0 => data_out := ADER_1_reg(7 downto 0);
	when CSR_ADER2_3 => data_out := ADER_2_reg(31 downto 24);
	when CSR_ADER2_2 => data_out := ADER_2_reg(23 downto 16);
	when CSR_ADER2_1 => data_out := ADER_2_reg(15 downto 8);
	when CSR_ADER2_0 => data_out := ADER_2_reg(7 downto 0);
	when USER_CSR_SN0 => data_out := SN_0;
	when USER_CSR_SN1 => data_out := SN_1;
	when USER_CSR_SN2 => data_out := SN_2;
	when USER_CSR_SN3 => data_out := SN_3;
	when USER_CSR_SN4 => data_out := SN_4;
	when USER_CSR_SN5 => data_out := SN_5;
	when USER_CSR_IRQ => data_out(2 downto 0) := irq_level;
	when USER_CSR_IRQV => data_out(7 downto 0) := irq_vector;
	when others => 
      end case;

      opb_rd_ack <= '0';
      if Sl_select = '1' and OPB_RNW = '1' then
	if opb_rd_ack_1 = '0' then
	  opb_rd_ack <= '1';
	  opb_rd_ack_1 := '1';
	end if;
      else
	opb_rd_ack_1 := '0';
      end if;
    end if;
  end process;

  transfer_acknowledge : process (opb_wr_ack, opb_rd_ack)
  begin
    if opb_wr_ack = '1' or opb_rd_ack = '1' then
      Sl_xferAck <= '1';
    else
      Sl_xferAck <= '0';
    end if;
    Sl_errAck <= '0';
    Sl_retry <= '0';
    Sl_toutSup <= '0';
  end process;
  
  SYSRESET_control: process (OPB_Clk, user_bit_set)
  begin
    if rising_edge(OPB_Clk) then
      nSYSRESETO <= 'Z';
      if user_bit_set(7) = '1' then
	nSYSRESETO <= '0';
      end if;
    end if;
  end process;

  PWRDWN_control: process (OPB_Clk, user_bit_clear)
    variable power_down_hs_enable : std_logic := '1';
  begin
    if rising_edge(OPB_Clk) then
      if user_bit_set(6) = '1' then
	power_down_hs_enable := '1';
      end if;
      if user_bit_clear(6) = '1' then
	power_down_hs_enable := '0';
      end if;
      PWRDWN <= '0';
      if power_down_hs_enable = '1' and HSOPEN = '1' then
	PWRDWN <= '1';
      end if;
      user_bit_status_reg(6) <= power_down_hs_enable;
    end if;
  end process;

  BLUE_led_control: process (OPB_Clk, user_bit_set, user_bit_clear)
    variable blue_led : std_logic := '0';
  begin
    if rising_edge(OPB_Clk) then
      if user_bit_set(5) = '1' then
	blue_led := '1';
      end if;
      if user_bit_clear(5) = '1' then
	blue_led := '0';
      end if;
      user_bit_status_reg(5) <= blue_led;
      BLUEOFF <= not blue_led;
    end if;
  end process;

  IF_power_control: process (OPB_Clk, user_bit_set, user_bit_clear)
    variable if_power : std_logic := '0';
  begin
    if rising_edge(OPB_Clk) then
      if user_bit_set(4) = '1' then
	if_power := '1';
      end if;
      if user_bit_clear(4) = '1' then
	if_power := '0';
      end if;
      user_bit_status_reg(4) <= if_power;
      PWRONIF <= if_power;
    end if;
  end process;
  
  user_bit_status_reg(7) <= HSOPEN;
  user_bit_status_reg(3 downto 0) <= nTBID;

end rtl;
  
