--------------------------------------------------------------------------
--
--  File        : opb_vme.vhd
--
--  Title       : OPB to VME/MAC bridge
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.opb_vme_pkg.all;

entity opb_vme is
  generic (
    -- Bus protocol parameters, do not add to or delete
    C_ETHBASEADDR      : std_logic_vector      := X"00000000";
    C_ETHBASEMASK      : std_logic_vector      := X"FF000000";
    C_ADER0_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
    C_ADER0_CMP_LOW    : natural := 16; -- Size 16 = 64k
    C_ADER1_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
    C_ADER1_CMP_LOW    : natural := 16; -- Size 16 = 64k
    C_ADER2_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
    C_ADER2_CMP_LOW    : natural := 16; -- Size 16 = 64k
    C_CRCSR_CMP_HIGH   : natural := 23;
    C_CRCSR_CMP_LOW    : natural := 19;
    C_FUNC0_LOCAL_BASE : std_logic_vector      := X"00000000";
    C_FUNC1_LOCAL_BASE : std_logic_vector      := X"01000000";
    C_FUNC2_LOCAL_BASE : std_logic_vector      := X"02000000";
    C_CRCSR_LOCAL_BASE : std_logic_vector      := X"03000000";
    C_OPB_AWIDTH       : integer               := 32;
    C_OPB_DWIDTH       : integer               := 32;
    C_FAMILY           : string                := "virtex2p"
    );
  port (
    -- Shared VME/MAC signals
    LA           : inout std_logic_vector(31 downto 0);
    LD           : inout std_logic_vector(31 downto 0);
    
    -- VME signals
    vmeAM        : inout std_logic_vector(5 downto 0);
    nvmeIACK     : in std_logic;
    nvmeASI      : in std_logic;  -- VME AS* input
    nvmeASO      : out std_logic; -- VME AS* output, OE controlled by VMEMSTR
    nvmeDSI      : in std_logic_vector(1 downto 0);
                                  -- VME DS0*, DS1* input
    nvmeDSO      : out std_logic_vector(1 downto 0);
                                  -- VME DS0*, DS1* output,
                                  -- OE controlled by VMEMSTR
    nvmeRETRYI   : in std_logic;  -- VME RETRY* input
    nvmeRETRYO   : out std_logic; -- VME RETRY* output
    RETRYEN      : out std_logic; -- VME RETRY* OE
    nvmeIACKIN   : in std_logic;  -- VME IACKIN* input
    nvmeIACKOUT  : out std_logic; -- VME IACKOUT* output
    nvmeDTACKI   : in std_logic;  -- VME DTACK* input
    nvmeDTACKO   : out std_logic := '1'; -- VME DTACK* output
    DTACKEN      : out std_logic := '0'; -- VME DTACK* OE
    nvmeWRITEI   : in std_logic;  -- VME WRITE* input
    nvmeWRITEO   : out std_logic; -- VME WRITE* output,
                                  -- OE controlled by VMEMSTR
    nvmeBRI      : in std_logic_vector(3 downto 0);
                                  -- VME BR0* to BR3* inputs
    nvmeBRO      : out std_logic_vector(3 downto 0);
                                  -- VME BR0* to BR3* OC outputs
    nvmeBGIN     : in std_logic_vector(3 downto 0);
                                  -- VME BGIN0* to BGIN3* inputs
    nBGOUT       : out std_logic_vector(3 downto 0);
    				  -- VME BGOUT0* to BGOUT3* outputs,
    VMEMSTR      : out std_logic;
    nSYSCON      : in std_logic;  -- nSYSCON jumper input
    nvmeBERRI    : in std_logic;  -- VME BERR* input
    nvmeBERRO    : out std_logic; -- VME BERR* OC output
    nvmeBBSYI    : in std_logic;  -- VME BBSY* input
    vmeBBSYEN    : out std_logic; -- VME BBSY* OE
    nvmeBCLRI    : in std_logic;  -- VME BCLR* input
    nvmeBCLRO    : out std_logic; -- VME BCLR* output
    vmeBCLREN    : out std_logic; -- VME BCLR* OE
    nAOE         : out std_logic; -- VME address nOE
    ACLKAB       : out std_logic; -- VME address A to B clock (local -> VME)
    ACLKBA       : out std_logic; -- VME address B to A clock (VME -> local)
    ADIR         : out std_logic; -- VME address direction (1 = local -> VME)
    ALE          : out std_logic; -- VME address LE
    AMCLKAB      : out std_logic; -- VME AM A to B clock (local -> VME)
    AMCLKBA      : out std_logic; -- VME AM B to A clock (VME -> local)
    AMDIR        : out std_logic; -- VME AM DIR (1 = local -> VME) 
    AMLE         : out std_logic; -- VME AM LE
    nAMOE        : out std_logic; -- VME AM nOE
    nvmeLWORDI   : in std_logic;  -- VME LWORD* input
    nvmeLWORDO   : out std_logic; -- VME LWORD* output,
                                  -- OE controlled by VMEMSTR
    nDOE2        : out std_logic; -- VME D16 to D31 OE
    DCLKAB2      : out std_logic; -- VME D16 to D31 A to B clock (local -> VME)
    DCLKBA2      : out std_logic; -- VME D16 to D31 B to A clock (VME -> local)
    DDIR2        : out std_logic; -- VME D16 to D31 DIR (1 = local -> VME)
    DLE2         : out std_logic; -- VME D16 to D31 LE
    nDOE         : out std_logic; -- VME D0 to D15 OE
    DCLKAB       : out std_logic; -- VME D0 to D15 A to B clock (local -> VME)
    DCLKBA       : out std_logic; -- VME D0 to D15 B to A clock (VME -> local)
    DDIR         : out std_logic; -- VME D0 to D15 DIR (1 = local -> VME)
    DLE          : out std_logic; -- VME D0 to D15 LE
    nvmeIRQO     : out std_logic_vector(7 downto 1);
    -- MAC signals
    nEINTR       : in  std_logic;
    nESRDY       : in  std_logic;
    EBCLK        : out std_logic;
    nEDATACS     : out std_logic;
    nERDYRTN     : out std_logic;
    nEBE         : out std_logic_vector(3 downto 0);
    nECYCLE      : out std_logic;
    ESWR         : inout std_logic;
    nEVLBUS      : out std_logic;
    EAEN         : out std_logic;
   
    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    Sl_DBus         : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
    Sl_errAck       : out   std_logic;
    Sl_retry        : out   std_logic;
    Sl_toutSup      : out   std_logic;
    Sl_xferAck      : out   std_logic;
    OPB_ABus        : in    std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE          : in    std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW         : in    std_logic;
    OPB_select      : in    std_logic;
    OPB_seqAddr     : in    std_logic;
    M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
    M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    M_busLock       : out   std_logic;
    M_request       : out   std_logic;
    M_RNW           : out   std_logic;
    M_select        : out   std_logic;
    M_seqAddr       : out   std_logic;
    OPB_errAck      : in    std_logic;
    OPB_MGrant      : in    std_logic;
    OPB_retry       : in    std_logic;
    OPB_timeout     : in    std_logic;
    OPB_xferAck     : in    std_logic;
    IP2INTC_Irpt    : out   std_logic;

    opb_irq         : in    std_logic_vector(0 downto 0);
    VME_irq_level   : in    std_logic_vector(2 downto 0);
    VME_irq_vector  : in    std_logic_vector(7 downto 0);
    VME64X_ADER0    : in    std_logic_vector(31 downto 0);
    VME64X_ADER1    : in    std_logic_vector(31 downto 0);
    VME64X_ADER2    : in    std_logic_vector(31 downto 0);
    VME64x_BAR      : in    std_logic_vector(7 downto 0);
    usec_enable     : out   std_logic);

end entity opb_vme;

architecture implementation of opb_vme is

  signal usec_ena   : std_logic; -- active high for one cycle every 1 us
  
  signal MAC_select : std_logic;
  signal MAC_xferAck : std_logic;

  signal VME_select      : std_logic; -- This signal is active high during an
                                      -- OPB to VME cycle
  signal opb_vme_request : std_logic; -- This signal is active high for one
                                      -- cycle at the beginning of and OPB to
                                      -- VME cycle
  signal opb_vme_grant   : std_logic; -- This signal is active high when
                                      -- board is VME master
  signal VME_xferAck       : std_logic;
  signal VME_errAck        : std_logic;
  signal vme_opb_cycle     : std_logic; -- This signal is active high during a
                                        -- VME to OPB cycle
  signal vme_opb_cycle_ack : std_logic; -- Pulsed high for one OPB_clk cycle at
                                        -- end of OPB cycle
  
  signal nV_BR        : std_logic_vector(3 downto 0);
  signal nV_BGIN      : std_logic_vector(3 downto 0);
  signal nV_BBSY      : std_logic;
  signal nV_BCLR      : std_logic;
  signal nV_DTACK     : std_logic;
  signal nV_WRITE     : std_logic;
  signal nV_AS        : std_logic;
  signal nV_DS        : std_logic_vector(1 downto 0);
  signal nV_LWORD     : std_logic;
  signal nV_BERR      : std_logic;
  
  signal VME_LALD_grant   : std_logic;
  signal OPB_LALD_grant   : std_logic;
  signal VME_LALD_request : std_logic;
  signal OPB_LALD_request : std_logic;

  signal VME_OPB_BERR     : std_logic;
  signal VME_OPB_request  : std_logic;
  signal mst_request      : std_logic;
  signal mst_select       : std_logic;

  signal vme_blt          : std_logic;
  signal vme_mblt         : std_logic;
  signal vme_blt_addr_ack_cycle : std_logic;
  signal vme_mblt_next_cycle    : std_logic;
  
  -- mst_A holds the 32 bit OPB address for VME OPB accesses
  signal mst_A  : std_logic_vector(0 to 31);

  signal nIACKOUT_i       : std_logic;
  signal VME_irq_cycle    : std_logic;
  signal VME_addr_ack     : std_logic;
  
begin

  usec_enable <= usec_ena;
  
  M_busLock <= '0';
  M_request <= mst_request;
  M_select <= mst_select;
  M_seqAddr <= '0';
  IP2INTC_Irpt <= '0';

  -- Unused VME signals
  nvmeRETRYO <= '1';
  RETRYEN <= '0';

  nvmeBCLRO <= '1';
  vmeBCLREN <= '0';

  nEDATACS <= '1';
  nERDYRTN <= nESRDY;
  nEVLBUS <= '0';
  EAEN <= '0'; -- not MAC_select;

  usec_counter : process (OPB_Clk, OPB_Rst)
    variable cnt_usec : std_logic_vector(7 downto 0);
  begin
    if rising_edge(OPB_Clk) then
      usec_ena <= '0';
      if cnt_usec(cnt_usec'high) = '1' then
	usec_ena <= '1';
	cnt_usec := X"63";
      else
	cnt_usec := cnt_usec - 1;
      end if;
      if (OPB_Rst = '1')
	then cnt_usec := X"63";
      end if;
    end if;
  end process;
  
  local_data_bus : process (OPB_Clk, OPB_select, opb_vme_grant,
			    OPB_DBus, OPB_BE, OPB_RNW, OPB_Rst,
			    VME_LALD_grant, OPB_xferAck, nV_WRITE,
			    VME_OPB_request, VME_irq_cycle, OPB_LALD_grant)
    variable LD_out          : std_logic_vector(31 downto 0);
    variable OPB_select_1    : std_logic;
    variable LD_transparent  : std_logic;
  begin
    nDOE <= '1';
    DCLKAB <= '0';
    DCLKBA <= '0';
    DDIR <= '0';
    DLE <= '1';
    nDOE2 <= '1';
    DCLKAB2 <= '0';
    DCLKBA2 <= '0';
    DDIR2 <= '0';
    DLE2 <= '1';
    nvmeWRITEO <= '1';
    -- If OPB master writing to VME or
    -- VME master reading from OPB data direction OPB -> VME
    if (opb_vme_grant = '1' and OPB_RNW = '0') or
      (VME_OPB_request = '1' and nV_WRITE = '1') or
      VME_irq_cycle = '1' then
      DDIR <= '1';
      nDOE <= '0';
      DDIR2 <= '1';
      nDOE2 <= '0';
    end if;
    -- If OPB master reading from VME or
    -- VME master writing to OPB data direction VME -> OPB
    if (opb_vme_grant = '1' and OPB_RNW = '1') or
      (VME_OPB_request = '1' and nV_WRITE = '0') then
      DDIR <= '0';
      nDOE <= '0';
      DDIR2 <= '0';
      nDOE2 <= '0';
    end if;
    if opb_vme_grant = '1' and OPB_RNW = '0' then
      nvmeWRITEO <= '0';
    end if;
    LD <= (others => 'Z');
    if (OPB_LALD_grant = '1' and OPB_RNW = '0') or
      (VME_LALD_grant = '1' and nV_WRITE = '1') then
      LD <= LD_out;
    end if;
    if rising_edge(OPB_Clk) then
      if OPB_select = '1' and OPB_select_1 = '0' then
	LD_transparent := '1';
      end if;
      if LD_transparent = '1' then
	LD_out := OPB_DBus;
	if OPB_BE(2 to 3) = "00" then
	  LD_out(15 downto 0) := OPB_DBus(0 to 15);
	end if;
      end if;
      if VME_irq_cycle = '1' then
	LD_out(7 downto 0) := VME_irq_vector;
      end if;
      if OPB_Rst = '1' or OPB_xferAck = '1' or OPB_select = '0' then
	LD_transparent := '0';
      end if;
      OPB_select_1 := OPB_select;
    end if;
  end process;
  
  Slave_control : process (MAC_xferAck, VME_xferAck, VME_select,
			   OPB_LALD_grant, VME_errAck)
  begin
    Sl_errAck <= VME_errAck;
    Sl_retry <= '0';
    Sl_toutSup <= VME_select or MAC_select;
    Sl_xferAck <= MAC_xferAck or VME_xferAck;
  end process;
  
  Sl_DBus_Mux : process (OPB_ABus, OPB_select, LD, OPB_LALD_grant,
			 mst_select, OPB_BE, nV_WRITE, OPB_RNW,
                         vme_opb_cycle, nV_LWORD, LA)
  begin
    OPB_LALD_request <= '0';
    MAC_select <= '0';
    VME_select <= '0';
    Sl_DBus <= (others => '0');
    if OPB_select = '1' then
      if -- OPB_ABus(0 to 1) = "00" or -- 1G VME A32 outbound
	OPB_ABus(0 to 2) = "010" or -- 512M VME A32 outbound
	OPB_ABus(0 to 3) = "0110" or -- 256M VME A32 outbound
	OPB_ABus(0 to 4) = "01110" then -- A24 & A16 & CSR
	OPB_LALD_request <= '1';
	if OPB_LALD_grant = '1' then
	  VME_select <= '1';
	end if;
      end if;
      -- Check if 10/100 MAC/PHY access
      if (OPB_ABus and C_ETHBASEMASK) = C_ETHBASEADDR then
	OPB_LALD_request <= '1';
	if OPB_LALD_grant = '1' then
	  MAC_select <= '1';
	end if;
      end if;
    end if;
    if (VME_select = '1' and OPB_RNW = '1') or
      (MAC_select = '1' and OPB_RNW = '1') or
      (vme_opb_cycle = '1' and nV_WRITE = '0') then
      Sl_DBus <= LD;
      if vme_mblt = '1' and mst_A(29) = '0' then
        Sl_DBus <= LA;
        Sl_DBus(31) <= nvmeLWORDI;
      end if;
      if (OPB_BE(2 to 3) = "00" and VME_select = '1') or
	(nV_LWORD = '1' and LA(1) = '0' and vme_opb_cycle = '1') then
	Sl_DBus(0 to 15) <= LD(15 downto 0);
      end if;
    end if;
  end process;
  
  MAC_interface : process (OPB_Clk, MAC_select, nESRDY, OPB_RNW,
			   OPB_BE)
    variable bclk_counter : std_logic_vector(1 downto 0) := "00";
    variable bclk_ena     : std_logic;
    variable bclk_ena_1   : std_logic;
    variable cycle        : std_logic := '0';
    variable nECYCLE_int  : std_logic;
  begin
    ESWR <= 'Z';
    if MAC_select = '1' then
      ESWR <= not OPB_RNW;
    end if;
    if rising_edge(OPB_Clk) then
      MAC_xferAck <= '0';
      if MAC_select = '1' and nESRDY = '0' and bclk_ena = '1' then
	MAC_xferAck <= '1';
      end if;
      if MAC_select = '0' then
	cycle := '0';
      end if;
      nEBE(3) <= not OPB_BE(0);
      nEBE(2) <= not OPB_BE(1);
      nEBE(1) <= not OPB_BE(2);
      nEBE(0) <= not OPB_BE(3);
      if bclk_ena = '1' then
        nECYCLE <= nECYCLE_int;
	nECYCLE_int := '1';
	if MAC_select = '1' and cycle = '0' then
	  nECYCLE_int := '0';
	  cycle := '1';
	end if;
      end if;
      
      EBCLK <= '0';
      if bclk_counter = "00" then
        EBCLK <= '1';
      end if;
      -- Generate 33.333 MHz EBCLK from 100 MHz OPB clock
      if bclk_counter(1) = '1' then
	bclk_counter := "00";
      else
	bclk_counter := bclk_counter + 1;
      end if;
      bclk_ena := bclk_counter(1);
      bclk_ena_1 := bclk_counter(0);
    end if;
    
  end process;

  VME_sync : process (OPB_clk, nvmeBRI, nvmeBBSYI, nvmeBCLRI, nvmeLWORDI,
		      nvmeDTACKI, nvmeASI, nvmeDSI, nvmeWRITEI, nvmeBERRI)
  begin
    if rising_edge(OPB_Clk) then
      nV_BR <= nvmeBRI;
      nV_BBSY <= nvmeBBSYI;
      nV_BCLR <= nvmeBCLRI;
      nV_DTACK <= nvmeDTACKI;
      nV_WRITE <= nvmeWRITEI;
      nV_AS <= nvmeASI;
      nV_DS <= nvmeDSI;
      nV_BERR <= nvmeBERRI;
    end if;
  end process;
  
  VME_requester : process (OPB_Clk, VME_select, opb_vme_request,
			   nV_BGIN, OPB_Rst, nV_AS, opb_vme_grant)
    variable timer_BBSY   : std_logic_vector(4 downto 0);
    variable VME_select_1 : std_logic;
  begin
    if rising_edge(OPB_Clk) then
      if opb_vme_request = '1' then
	nvmeBRO <= "0111"; -- Request on level 3
      end if;
      nBGOUT <= nV_BGIN;
      if VME_select = '1' and nV_BGIN(3) = '0' then
	-- Wait for previous master to release DTB (ensure AS* is high)
	if nV_AS = '1' then
	  opb_vme_grant <= '1';
	end if;
	timer_BBSY := "01001"; -- Drive BBSY* low for a minimum of 90 ns
	nvmeBRO <= "1111";     -- Release bus request
	nBGOUT(3) <= '1';      -- Don't drive nBGOUT3
      end if;
      
      vmeBBSYEN <= '0';
      if opb_vme_grant = '1' or timer_BBSY(timer_BBSY'high) = '0' then
	vmeBBSYEN <= '1';
      end if;
      if timer_BBSY(timer_BBSY'high) = '0' then
	timer_BBSY := timer_BBSY - 1;
      end if;
      if VME_select = '0' then
	opb_vme_grant <= '0';
	if nV_BBSY = '0' then  -- Wait for BBSY* going low before releasing
	  nvmeBRO <= "1111";   -- bus request
	end if;
      end if;	
      if OPB_Rst = '1' then
	opb_vme_grant <= '0';
	timer_BBSY := "11111";
	nvmeBRO <= "1111";
      end if;

      opb_vme_request <= '0';
      if VME_select = '1' and VME_select_1 = '0' then
	opb_vme_request <= '1';
      end if;
      VME_select_1 := VME_select;
    end if;
  end process;
  
  VME_arbiter : process (OPB_Clk, nV_BR, nvmeBGIN, nV_BBSY, nV_BCLR,
			 nSYSCON, OPB_Rst, usec_ena)
    variable nBG        : std_logic_vector(3 downto 0);
    variable timer_BBSY : std_logic_vector(2 downto 0);
    variable timeout_BG : std_logic_vector(7 downto 0);
  begin
    if rising_edge(OPB_Clk) then
      if nBG = "1111" then
	timeout_BG := X"63";
	if timer_BBSY(timer_BBSY'high) = '1' then
	  -- Bus not granted and not busy
	  if nV_BR(3) = '0' then
	    nBG := "0111";
	  end if;
	  if nV_BR(2) = '0' then
	    nBG := "1011";
	  end if;
	  if nV_BR(1) = '0' then
	    nBG := "1101";
	  end if;
	  if nV_BR(0) = '0' then
	    nBG := "1110";
	  end if;
	end if;
      else
	-- Release bus grant when BBSY* low or timeout
	if nV_BBSY = '0' or timeout_BG(timeout_BG'high) = '1' then
	  nBG := "1111";
	end if;
	if usec_ena = '1' then
	  timeout_BG := timeout_BG - 1;
	end if;
      end if;
      if timer_BBSY(timer_BBSY'high) = '0' then
	timer_BBSY := timer_BBSY - 1;
      end if;
      if nV_BBSY = '0' then
	timer_BBSY := "010";
      end if;
      if OPB_Rst = '1' then
	nBG := "1111";
	timer_BBSY := "111";
      end if;
      nV_BGIN <= nBG;
      if nSYSCON = '1' then  -- If board is not system controller
	nV_BGIN <= nvmeBGIN; -- forward bus grants
      end if;
    end if;
  end process;

  VME_mastering : process (OPB_Clk, MAC_select, OPB_ABus,
			   opb_vme_grant, VME_select, nV_DTACK,
			   OPB_BE, OPB_Rst, nV_BERR)
    variable delay_DTACK : std_logic_vector(2 downto 0);
    variable timer_AS : std_logic_vector(2 downto 0);
  begin
    VMEMSTR <= opb_vme_grant;
    if rising_edge(OPB_Clk) then
      LA <= (others => 'Z');
      nAOE <= '0';
      if MAC_select = '1' then
	nAOE <= '1';
      end if;
      ADIR <= '0';
      if MAC_select = '1' or opb_vme_grant = '1' then
	LA <= OPB_ABus;
	if OPB_ABus(1) = '1' then
	  LA(30 downto 29) <= "00";
	end if;
      end if;
      ACLKAB <= '0';
      ACLKBA <= '0';
      ALE <= '1';
      AMCLKAB <= '0';
      AMCLKBA <= '0';
      AMDIR <= '0';
      AMLE <= '1';
      nAMOE <= '0';
      vmeAM <= "ZZZZZZ";
      VME_errAck <= '0';
      VME_xferack <= '0';
      nvmeASO <= '1';
      nvmeDSO <= "11";
      nvmeLWORDO <= '1';
      if opb_vme_grant = '1' then
	if nV_BERR = '0' then
	  VME_errAck <= '1';
	end if;
	if delay_DTACK = "000" then
	  VME_xferack <= '1';
	end if;
	if delay_DTACK(delay_DTACK'high) = '0' then
	  delay_DTACK := delay_DTACK - 1;
	end if;
	if nV_DTACK = '1' then
	  delay_DTACK := "011";
	end if;
	nAOE <= '0';
	ADIR <= '1';
	nAMOE <= '0';
	AMDIR <= '1';
	case OPB_ABus(0 to 7) is
	  when "00XXXXXX" => -- 1G VME A32 outbound
	    vmeAM <= AM_A32_SU_D;
	  when "010XXXXX" => -- 512M VME A32 outbound
	    vmeAM <= AM_A32_D;
	  when "0110XXXX" => -- 256M VME A32 outbound
	    vmeAM <= AM_A32_D;
	  when "01110000" => -- 16M VME A24 outbound
	    vmeAM <= AM_A24_D;
	  when "01110001" => -- 16M VME A24 outbound
	    vmeAM <= AM_A24_P;
	  when "01110010" => -- 16M VME A24 outbound
	    vmeAM <= AM_A24_SU_D;
	  when "01110011" => -- 16M VME A24 outbound
	    vmeAM <= AM_A24_SU_P;
	  when "01110100" => -- 64k VME A16 outbound
	    vmeAM <= AM_A16;
	  when "01110101" => -- 64k VME A16 outbound
	    vmeAM <= AM_A16_SU;
	  when others =>     -- 16M VME CR/CSR outbound
	    vmeAM <= AM_CR_CSR;
	end case;
	-- Assert address/data strobes 40 ns after address/AM
	LA(30) <= '0';
	if timer_AS(timer_AS'high) = '1' then
	  nvmeASO <= '0';
	  case OPB_BE(0 to 3) is
	    when "0001" => nvmeDSO <= "10";
			   LA(1) <= '1';
	    when "0010" => nvmeDSO <= "01";
			   LA(1) <= '1';
	    when "0100" => nvmeDSO <= "10";
			   LA(1) <= '0';
	    when "1000" => nvmeDSO <= "01";
			   LA(1) <= '0';
	    when "0011" => nvmeDSO <= "00";
			   LA(1) <= '1';
	    when "1100" => nvmeDSO <= "00";
			   LA(1) <= '0';
	    when others => nvmeDSO <= "00";
	                   nvmeLWORDO <= '0';
	  end case;
	end if;
	if timer_AS(timer_AS'high) = '0' then
	  timer_AS := timer_AS - 1;
	end if;
      else
	timer_AS := "011";
      end if;
      if OPB_Rst = '1' then
	delay_DTACK := "011";
      end if;
    end if;
  end process;

  VME_bus_timer : process (OPB_Clk, nV_DS, usec_ena, VME_OPB_BERR,
			   OPB_Rst)
    variable timeout : std_logic_vector(7 downto 0);
  begin
    if rising_edge(OPB_Clk) then
      if timeout(timeout'high) = '1' or VME_OPB_BERR = '1' then
	nvmeBERRO <= '0';
      end if;
      if usec_ena = '1' and timeout(timeout'high) = '0' then
	timeout := timeout - 1;
      end if;
      if nV_DS = "11" or OPB_Rst = '1' then
	timeout := X"63";
	nvmeBERRO <= '1';
      end if;
    end if;
  end process;

  LALD_arbitration : process (OPB_Clk, VME_LALD_request, OPB_LALD_request,
			      VME_LALD_grant, OPB_LALD_grant)
  begin
    if rising_edge(OPB_Clk) then
      if VME_LALD_grant = '0' and OPB_LALD_grant = '0' then
	if VME_LALD_request = '1' then
	  VME_LALD_grant <= '1';
	else
	  if OPB_LALD_request = '1' then
	    OPB_LALD_grant <= '1';
	  end if;
	end if;
      end if;

      if VME_LALD_request = '0' then
	VME_LALD_grant <= '0';
      end if;

      if OPB_LALD_request = '0' then
	OPB_LALD_grant <= '0';
      end if;
	
      if OPB_Rst = '1' then
	VME_LALD_grant <= '0';
	OPB_LALD_grant <= '0';
      end if;
    end if;
  end process;
  
  VME_OPB_access : process (OPB_Clk, vme_opb_cycle, opb_vme_grant,
			    nV_DS, LA, vmeAM, mst_A, VME_LALD_grant,
			    VME64X_ADER0, VME64X_ADER1, VME64X_ADER2,
			    VME64X_BAR)
    variable latched_addr  : std_logic_vector(31 downto 0);
    variable latched_AM    : std_logic_vector(5 downto 0);
    variable latched_nIACK : std_logic;
    variable masked_addr   : std_logic_vector(31 downto 0);
    variable sr_nV_AS      : std_logic_vector(1 downto 0) := "11";
    variable addr_inc      : std_logic_vector(10 downto 2);
  begin
    M_ABus <= (others => '0');
    if vme_opb_cycle = '1' then
      M_ABus(0 to 29) <= mst_A(0 to 29);
    end if;

    if nvmeASI = '1' then
      latched_addr := LA;
      latched_AM := vmeAM;
      latched_nIACK := nvmeIACK;
      nV_LWORD <= nvmeLWORDI;
    end if;
    
    if rising_edge(OPB_Clk) then
      VME_LALD_request <= '0';
      VME_OPB_request <= '0';
      -- Respond to accesses only from VME bus, not when board is master
      masked_addr := latched_addr;
      if latched_AM(5 downto 3) = "111" or
	latched_AM(5 downto 3) = AM_CR_CSR(0 to 2) then
	-- A24 addressing
	masked_addr(31 downto 24) := X"00";
      end if;
      if latched_AM = AM_A16 or latched_AM = AM_A16_SU then
	masked_addr(31 downto 16) := X"0000";
      end if;
      vme_blt <= '0';
      vme_mblt <= '0';
      if (latched_AM(5 downto 3) = "000" or latched_AM(5 downto 3) = "111") then
        if latched_AM(1 downto 0) = "11" then
          vme_blt <= '1';
        end if;
        if latched_AM(1 downto 0) = "00" then
          vme_mblt <= '1';
        end if;
      end if;
      -- Take address from VME on falling edge of AS*
      if nV_DS = "11" then
        vme_blt_addr_ack_cycle <= '0';
      end if;
      if sr_nV_AS = "10" then
        mst_A <= masked_addr;
        if vme_blt = '1' or vme_mblt = '1' then
          vme_blt_addr_ack_cycle <= '1';
        end if;
      end if;
      sr_nV_AS := sr_nV_AS(0) & nV_AS;
      if vme_opb_cycle_ack = '1' then
        addr_inc := mst_A(21 to 29) + 1;
        if vme_blt = '1' or vme_mblt = '1' then
          mst_A(21 to 29) <= addr_inc;
        end if;
      end if;
      nIACKOUT_i <= '1';
      VME_irq_cycle <= '0';
      VME_addr_ack <= '0';
      if opb_vme_grant = '0' then
	if nV_DS /= "11" then
	  VME_LALD_request <= '1';
	  if VME_LALD_grant = '1' then
	    if latched_nIACK = '1' then
	      -- Here we do not distinguish between supervisory and
	      -- non-priviledged accesses - we accept both and do not
	      -- compare AM2
	      if (latched_AM(5 downto 3) = VME64X_ADER0(7 downto 5) and
		  latched_AM(1 downto 0) = VME64X_ADER0(3 downto 2) and
		  VME64X_ADER0(1 downto 0) = "00" and
		  masked_addr(C_ADER0_CMP_HIGH downto C_ADER0_CMP_LOW) =
		  VME64X_ADER0(C_ADER0_CMP_HIGH downto C_ADER0_CMP_LOW)) then
		-- Access for function 0
		mst_A(0 to 7) <= C_FUNC0_LOCAL_BASE(0 to 7);
		mst_A(8 to 31-C_ADER0_CMP_LOW) <= (others => '0');
                if vme_blt_addr_ack_cycle = '0' then
                  VME_OPB_request <= '1';
                else
                  VME_addr_ack <= '1';
                end if;
	      end if;
	      if (latched_AM(5 downto 3) = VME64X_ADER1(7 downto 5) and
		  latched_AM(1 downto 0) = VME64X_ADER1(3 downto 2) and
		  VME64X_ADER1(1 downto 0) = "00" and
		  masked_addr(C_ADER1_CMP_HIGH downto C_ADER1_CMP_LOW) =
		  VME64X_ADER1(C_ADER1_CMP_HIGH downto C_ADER1_CMP_LOW)) then
		-- Access for function 1
		mst_A(0 to 7) <= C_FUNC1_LOCAL_BASE(0 to 7);
		mst_A(8 to 31-C_ADER1_CMP_LOW) <= (others => '0');
                if vme_blt_addr_ack_cycle = '0' then
                  VME_OPB_request <= '1';
                else
                  VME_addr_ack <= '1';
                end if;
	      end if;
	      if (latched_AM(5 downto 3) = VME64X_ADER2(7 downto 5) and
		  latched_AM(1 downto 0) = VME64X_ADER2(3 downto 2) and
		  VME64X_ADER2(1 downto 0) = "00" and
		  masked_addr(C_ADER2_CMP_HIGH downto C_ADER2_CMP_LOW) =
		  VME64X_ADER2(C_ADER2_CMP_HIGH downto C_ADER2_CMP_LOW)) then
		-- Access for function 2
		mst_A(0 to 7) <= C_FUNC2_LOCAL_BASE(0 to 7);
		mst_A(8 to 31-C_ADER2_CMP_LOW) <= (others => '0');
                if vme_blt_addr_ack_cycle = '0' then
                  VME_OPB_request <= '1';
                else
                  VME_addr_ack <= '1';
                end if;
	      end if;
	      if latched_AM = AM_CR_CSR and
		masked_addr(C_CRCSR_CMP_HIGH downto C_CRCSR_CMP_LOW) =
		VME64x_BAR(7 downto 3) then
		-- CR/CSR Access
		mst_A(0 to 7) <= C_CRCSR_LOCAL_BASE(0 to 7);
		VME_OPB_request <= '1';
	      end if;
	    else
	      -- IACK cycle
	      if nvmeIACKIN = '0' then
		if nIACKOUT_i = '1' and opb_irq(0) = '1' and
		  masked_addr(3 downto 1) = VME_irq_level and
		  VME_irq_level /= "000" then
		  VME_irq_cycle <= '1';
		else
		  nIACKOUT_i <= '0';
		end if;
	      end if;
	    end if;
	  end if;
	end if;
      end if;
    end if;
  end process;

  OPB_requesting : process (OPB_Clk, VME_OPB_request, OPB_MGrant, OPB_Rst)
    variable VME_OPB_req_1 : std_logic;
  begin
    if rising_edge(OPB_Clk) then
      -- Find rising edge of VME_OPB_request
      if (VME_OPB_request = '1' and VME_OPB_req_1 = '0') or vme_mblt_next_cycle = '1' then
	mst_request <= '1';
      end if;
      if OPB_MGrant = '1' then
	mst_request <= '0';
      end if;
      VME_OPB_req_1 := VME_OPB_request;
      if OPB_Rst = '1' then
	mst_request <= '0';
      end if;
    end if;
  end process;

  VME_OPB_data_transfer : process (OPB_Clk, OPB_MGrant, nV_DS, mst_A, nV_LWORD,
				   VME_OPB_request, OPB_xferAck, OPB_timeout,
				   OPB_Rst, nV_WRITE, VME_irq_cycle)
    variable DTACK_act      : std_logic;
    variable transfer_act   : std_logic;
    variable vme_strobes    : std_logic_vector(3 downto 0);
    variable irq_dtack_dly  : std_logic_vector(2 downto 0);
  begin
    vme_opb_cycle_ack <= '0';
    if transfer_act = '1' then
      if OPB_xferAck = '1' then
        vme_opb_cycle_ack <= '1';
      end if;
    end if;

    vme_mblt_next_cycle <= '0';
    if vme_mblt = '1' and OPB_xferAck = '1'and mst_A(29) = '0' then
      vme_mblt_next_cycle <= '1';
    end if;
    
    if rising_edge(OPB_Clk) then
      -- Data transfer starts with OPB_MGrant asserted for one cycle
      if OPB_MGrant = '1' then
	transfer_act := '1';
      end if;

      VME_OPB_BERR <= '0';
      if transfer_act = '1' then
	if OPB_xferAck = '1' then
          transfer_act := '0';
          if vme_mblt = '0' or mst_A(29) = '1' then
            DTACK_act := '1';
          end if;
	end if;
	if OPB_timeout = '1' then
	  transfer_act := '0';
	  VME_OPB_BERR <= '1';
	end if;
      end if;
      if VME_addr_ack = '1' then
        DTACK_act := '1';
      end if;
      
      vme_strobes := nV_DS & mst_A(30) & nV_LWORD;
      M_RNW <= '0';
      mst_select <= '0';
      M_BE <= (others => '0');
      if transfer_act = '1' then
	mst_select <= '1';
	M_RNW <= nV_WRITE;
	case vme_strobes is
	  when "0000" => M_BE <= "1111";
	  when "0001" => M_BE <= "1100";
	  when "0010" => M_BE <= "0110";
	  when "0011" => M_BE <= "0011";
	  when "1000" => M_BE <= "0111";
	  when "1001" => M_BE <= "0100";
	  when "1011" => M_BE <= "0001";
	  when "0100" => M_BE <= "1110";
	  when "0101" => M_BE <= "1000";
	  when "0111" => M_BE <= "0010";
	  when others =>
	end case;
      end if;
      
      -- VME DTACK control
      nvmeDTACKO <= '1';
      DTACKEN <= '0';
      if DTACK_act = '1' or irq_dtack_dly(irq_dtack_dly'high) = '1' then
	nvmeDTACKO <= '0';
	DTACKEN <= '1';
	-- Wait for data strobes to rise before releasing DTACK
	if nV_DS = "11" then
	  -- At end of cycle drive DTACK high for one cycle
	  nvmeDTACKO <= '1';
	  DTACKEN <= '1';
	  DTACK_act := '0';
	end if;
      end if;

      irq_dtack_dly := irq_dtack_dly(irq_dtack_dly'high - 1 downto 0) &
		       vme_irq_cycle;
      
      if OPB_Rst = '1' then
	transfer_act := '0';
	DTACK_act := '0';
	irq_dtack_dly := (others => '0');
      end if;

      vme_opb_cycle <= transfer_act;
    end if;
  end process;

  -- VME interrupt request
  process (OPB_Clk, VME_irq_level, opb_irq)
  begin
    if rising_edge(OPB_Clk) then
        nvmeIRQO <= (others => '1');
      if opb_irq(0) = '1' then
        case VME_irq_level is
          when "001" => nvmeIRQO(1) <= '0';
          when "010" => nvmeIRQO(2) <= '0';
          when "011" => nvmeIRQO(3) <= '0';
          when "100" => nvmeIRQO(4) <= '0';
          when "101" => nvmeIRQO(5) <= '0';
          when "110" => nvmeIRQO(6) <= '0';
          when "111" => nvmeIRQO(7) <= '0';
          when others =>
        end case;
      end if;
    end if;
  end process;

  -- VME IACKOUT* signal generation
  process (OPB_Clk, nvmeIACKIN, nIACKOUT_i, opb_irq)
  begin
    if rising_edge(OPB_Clk) then
      nvmeIACKOUT <= nvmeIACKIN;
      if opb_irq(0) = '1' then
	nvmeIACKOUT <= nIACKOUT_i;
      end if;
    end if;
  end process;
      
end implementation;
