---------------------------------------------------------------------------
--                                                                       --
--  File        : opb_vme_pkg.vhd                                        --
--                                                                       --
--  Title       : Local bus package
--                VME declarations
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

package opb_vme_pkg is
  constant AM_A64_MBLT    : std_logic_vector := "000000";
  constant AM_A64         : std_logic_vector := "000001";
  constant AM_A64_BLT     : std_logic_vector := "000011";
  constant AM_A32_MBLT    : std_logic_vector := "001000";
  constant AM_A32_D       : std_logic_vector := "001001";
  constant AM_A32_P       : std_logic_vector := "001010";
  constant AM_A32_BLT     : std_logic_vector := "001011";
  constant AM_A32_SU_MBLT : std_logic_vector := "001100";
  constant AM_A32_SU_D    : std_logic_vector := "001101";
  constant AM_A32_SU_P    : std_logic_vector := "001110";
  constant AM_A32_SU_BLT  : std_logic_vector := "001111";
  constant AM_A16         : std_logic_vector := "101001";
  constant AM_A16_SU      : std_logic_vector := "101101";
  constant AM_CR_CSR      : std_logic_vector := "101111";
  constant AM_A40_BLT     : std_logic_vector := "110111";
  constant AM_A24_MBLT    : std_logic_vector := "111000";
  constant AM_A24_D       : std_logic_vector := "111001";
  constant AM_A24_P       : std_logic_vector := "111010";
  constant AM_A24_BLT     : std_logic_vector := "111011";
  constant AM_A24_SU_MBLT : std_logic_vector := "111100";
  constant AM_A24_SU_D    : std_logic_vector := "111101";
  constant AM_A24_SU_P    : std_logic_vector := "111110";
  constant AM_A24_SU_BLT  : std_logic_vector := "111111";

  constant CSR_BAR          : std_logic_vector := "11111111111111111"; -- 7ffff
  constant CSR_BIT_SET      : std_logic_vector := "11111111111111110"; -- 7fffb
  constant CSR_BIT_CLEAR    : std_logic_vector := "11111111111111101"; -- 7fff7
  constant CSR_CRAM_OWNER   : std_logic_vector := "11111111111111100"; -- 7fff3
  constant CSR_UD_BIT_SET   : std_logic_vector := "11111111111111011"; -- 7ffef
  constant CSR_UD_BIT_CLEAR : std_logic_vector := "11111111111111010"; -- 7ffeb
  constant CSR_ADER7_0      : std_logic_vector := "11111111111110111"; -- 7ffdf
  constant CSR_ADER7_1      : std_logic_vector := "11111111111110110"; -- 7ffdb
  constant CSR_ADER7_2      : std_logic_vector := "11111111111110101"; -- 7ffd7
  constant CSR_ADER7_3      : std_logic_vector := "11111111111110100"; -- 7ffd3
  constant CSR_ADER6_0      : std_logic_vector := "11111111111110011"; -- 7ffcf
  constant CSR_ADER6_1      : std_logic_vector := "11111111111110010"; -- 7ffcb
  constant CSR_ADER6_2      : std_logic_vector := "11111111111110001"; -- 7ffc7
  constant CSR_ADER6_3      : std_logic_vector := "11111111111110000"; -- 7ffc3
  constant CSR_ADER5_0      : std_logic_vector := "11111111111101111"; -- 7ffbf
  constant CSR_ADER5_1      : std_logic_vector := "11111111111101110"; -- 7ffbb
  constant CSR_ADER5_2      : std_logic_vector := "11111111111101101"; -- 7ffb7
  constant CSR_ADER5_3      : std_logic_vector := "11111111111101100"; -- 7ffb3
  constant CSR_ADER4_0      : std_logic_vector := "11111111111101011"; -- 7ffaf
  constant CSR_ADER4_1      : std_logic_vector := "11111111111101010"; -- 7ffab
  constant CSR_ADER4_2      : std_logic_vector := "11111111111101001"; -- 7ffa7
  constant CSR_ADER4_3      : std_logic_vector := "11111111111101000"; -- 7ffa3
  constant CSR_ADER3_0      : std_logic_vector := "11111111111100111"; -- 7ff9f
  constant CSR_ADER3_1      : std_logic_vector := "11111111111100110"; -- 7ff9b
  constant CSR_ADER3_2      : std_logic_vector := "11111111111100101"; -- 7ff97
  constant CSR_ADER3_3      : std_logic_vector := "11111111111100100"; -- 7ff93
  constant CSR_ADER2_0      : std_logic_vector := "11111111111100011"; -- 7ff8f
  constant CSR_ADER2_1      : std_logic_vector := "11111111111100010"; -- 7ff8b
  constant CSR_ADER2_2      : std_logic_vector := "11111111111100001"; -- 7ff87
  constant CSR_ADER2_3      : std_logic_vector := "11111111111100000"; -- 7ff83
  constant CSR_ADER1_0      : std_logic_vector := "11111111111011111"; -- 7ff7f
  constant CSR_ADER1_1      : std_logic_vector := "11111111111011110"; -- 7ff7b
  constant CSR_ADER1_2      : std_logic_vector := "11111111111011101"; -- 7ff77
  constant CSR_ADER1_3      : std_logic_vector := "11111111111011100"; -- 7ff73
  constant CSR_ADER0_0      : std_logic_vector := "11111111111011011"; -- 7ff6f
  constant CSR_ADER0_1      : std_logic_vector := "11111111111011010"; -- 7ff6b
  constant CSR_ADER0_2      : std_logic_vector := "11111111111011001"; -- 7ff67
  constant CSR_ADER0_3      : std_logic_vector := "11111111111011000"; -- 7ff63

  constant USER_CSR_SN0     : std_logic_vector := "11111111011001001"; -- 7fb27
  constant USER_CSR_SN1     : std_logic_vector := "11111111011001000"; -- 7fb23
  constant USER_CSR_SN2     : std_logic_vector := "11111111011000111"; -- 7fb1f
  constant USER_CSR_SN3     : std_logic_vector := "11111111011000110"; -- 7fb1b
  constant USER_CSR_SN4     : std_logic_vector := "11111111011000101"; -- 7fb17
  constant USER_CSR_SN5     : std_logic_vector := "11111111011000100"; -- 7fb13
  constant USER_CSR_IRQV    : std_logic_vector := "11111111011000001"; -- 7fb07
  constant USER_CSR_IRQ     : std_logic_vector := "11111111011000000"; -- 7fb03
  
end opb_vme_pkg;
