---------------------------------------------------------------------------
--
--  File        : cpcievg220.vhd
--
--  Title       : CompactPCI Event Generator Top Level
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evg_bs_pkg.all;
use work.evg_pkg.all;
 
entity cpcievg220 is
  port (
    -- Transceiver transmitter disable signal
    TXDISB : out std_logic;

    -- Crosspoint switch select signals
    -- '0' selects transceiver input
    -- '1' selects passes signal from MGT output further
    TXSEL  : out std_logic;
    RXSEL  : out std_logic;
    
    -- BREFCLKx is sourced by RF input (via AD9515BCPZ, EVG only)
    BREFCLKP : in std_logic;
    BREFCLKN : in std_logic;

    -- BREFCLK2x is sourced by Micrel SY87739L
    BREFCLK2P : in std_logic;
    BREFCLK2N : in std_logic;

    -- Local Bus Signals
    LCLK      : in std_logic;
    nREADY    : out std_logic;
    nLBE      : in std_logic_vector(3 downto 0);
    nBTERM    : out std_logic;
    LINTi     : out std_logic_vector(2 downto 1);
    LWnR      : in std_logic;
    nADS      : in std_logic;
    LAD       : inout std_logic_vector(31 downto 0);
    
    -- I/O connector pins

    TTLIN           : in std_logic_vector(1 downto 1);
    TTLOUT          : out std_logic_vector(0 to C_EVG_TTL_OUTPUTS-1);
    UNIVIN          : in std_logic_vector(0 to C_EVG_UNIV_INPUTS-1);
    UNIVOUT         : out std_logic_vector(0 to C_EVG_UNIV_OUTPUTS-1);

    TBIN            : in std_logic_vector(0 to C_EVG_TB_INPUTS-1);
    TBOUT           : out std_logic_vector(0 to C_EVG_TB_OUTPUTS-1);

    -- PXI pins
    PXI_CLK10       : in std_logic;
    PXI_CLK10_IN_I  : in std_logic;
    PXI_CLK10_IN_O  : out std_logic;
    PXI_CLK10_DIR   : out std_logic; -- 0 - PXI CLK10_IN is output
                                     -- 1 - PXI CLK10_IN is input
    PXI_TRIG_DIR    : out std_logic_vector(7 downto 0);
                      -- PXI_TRIGx: 0 - output, 1 - input
    PXI_TRIG        : inout std_logic_vector(7 downto 0);
    PXI_STAR        : inout std_logic_vector(6 downto 0);
    nQSOE           : out std_logic; -- 0 - enable QuickSwitch
                                     -- 1 - disable QuickSwitch

    GA              : in std_logic_vector(4 downto 0);
    
    -- LEDS
    -- 1 - LNK (red)
    -- 2 - LNK (green)
    -- 3 - EVT (red)
    -- 4 - EVT (green)
    nLED       : out std_logic_vector(4 downto 1);
    
    -- Micrel SY87739L programming signals
    CGLOCK    : in std_logic;
    CGSK      : out std_logic;
    CGDI      : out std_logic;
    CGCS      : out std_logic;
    
    -- AD9515BCPZ RF divider select outputs
    RFSEL       : out std_logic_vector(5 downto 0);

    TBON      : out std_logic_vector(2 downto 1);

    RXN       : in std_logic_vector(4 downto 1);
    RXP       : in std_logic_vector(4 downto 1);
    TXN       : out std_logic_vector(4 downto 1);
    TXP       : out std_logic_vector(4 downto 1)
    );

end cpcievg220;

architecture structure of cpcievg220 is

  signal OPB_Clk         : std_logic;

  signal reset           : std_logic;
  
  signal gnd             : std_logic := '0';
  signal vcc             : std_logic := '1';

  signal pci_OPB_Clk : std_logic;
  signal pci_OPB_Rst : std_logic;
  signal pci_M_DBus : std_logic_vector(0 to 31);
  signal pci_OPB_DBus : std_logic_vector(0 to 31);
  signal pci_M_ABus : std_logic_vector(0 to 31);
  signal pci_M_BE : std_logic_vector(0 to 3);
  signal pci_M_busLock : std_logic;
  signal pci_M_request : std_logic;
  signal pci_M_RNW : std_logic;
  signal pci_M_select : std_logic;
  signal pci_M_seqAddr : std_logic;
  signal pci_OPB_errAck : std_logic;
  signal pci_OPB_MGrant : std_logic;
  signal pci_OPB_retry : std_logic;
  signal pci_OPB_timeout : std_logic;
  signal pci_OPB_xferAck : std_logic;

  signal evg_OPB_Clk : std_logic;
  signal evg_OPB_Rst : std_logic;
  signal evg_Sl_DBus : std_logic_vector(0 to 31);
  signal evg_Sl_errAck : std_logic;
  signal evg_Sl_retry : std_logic;
  signal evg_Sl_toutSup : std_logic;
  signal evg_Sl_xferAck : std_logic;
  signal evg_OPB_ABus : std_logic_vector(0 to 31);
  signal evg_OPB_BE : std_logic_vector(0 to 3);
  signal evg_OPB_DBus : std_logic_vector(0 to 31);
  signal evg_OPB_RNW : std_logic;
  signal evg_OPB_select : std_logic;
  signal evg_OPB_seqAddr : std_logic;
  signal evg_OPB_debug : std_logic_vector(39 downto 0);

  signal icon_control  : std_logic_vector(35 downto 0);
  
  signal LED_rx_link : std_logic;
  signal LED_ena : std_logic;
  
  component bufgdll is
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component system is
    port (
      sys_clk : in std_logic;
      sys_rst : in std_logic;
      opb_m_0_OPB_Clk : out std_logic;
      opb_m_0_OPB_Rst : out std_logic;
      opb_m_0_M_DBus : in std_logic_vector(0 to 31);
      opb_m_0_OPB_DBus : out std_logic_vector(0 to 31);
      opb_m_0_M_ABus : in std_logic_vector(0 to 31);
      opb_m_0_M_BE : in std_logic_vector(0 to 3);
      opb_m_0_M_busLock : in std_logic;
      opb_m_0_M_request : in std_logic;
      opb_m_0_M_RNW : in std_logic;
      opb_m_0_M_select : in std_logic;
      opb_m_0_M_seqAddr : in std_logic;
      opb_m_0_OPB_errAck : out std_logic;
      opb_m_0_OPB_MGrant : out std_logic;
      opb_m_0_OPB_retry : out std_logic;
      opb_m_0_OPB_timeout : out std_logic;
      opb_m_0_OPB_xferAck : out std_logic;
      opb_s_1_OPB_Clk : out std_logic;
      opb_s_1_OPB_Rst : out std_logic;
      opb_s_1_Sl_DBus : in std_logic_vector(0 to 31);
      opb_s_1_Sl_errAck : in std_logic;
      opb_s_1_Sl_retry : in std_logic;
      opb_s_1_Sl_toutSup : in std_logic;
      opb_s_1_Sl_xferAck : in std_logic;
      opb_s_1_OPB_ABus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_BE : out std_logic_vector(0 to 3);
      opb_s_1_OPB_DBus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_RNW : out std_logic;
      opb_s_1_OPB_select : out std_logic;
      opb_s_1_OPB_seqAddr : out std_logic;
      chipscope_icon_0_control1_pin : out std_logic_vector(35 downto 0);
      chipscope_opb_iba_0_iba_trig_in_pin : in std_logic_vector(39 downto 0));
  end component;

  component opb_pci9030 is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_OPB_AWIDTH    : integer               := 32;
      C_OPB_DWIDTH    : integer               := 32;
      C_FAMILY        : string                := "virtex2p"
      );
    port (
      -- PLX PCI9030 Local Bus Signals
      nREADY    : out std_logic;
      nLBE      : in std_logic_vector(3 downto 0);
      nBTERM    : out std_logic;
      LWnR      : in std_logic;
      nADS      : in std_logic;
      LAD       : inout std_logic_vector(31 downto 0);
      
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      M_DBus          : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
      M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
      M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      M_busLock       : out   std_logic;
      M_request       : out   std_logic;
      M_RNW           : out   std_logic;
      M_select        : out   std_logic;
      M_seqAddr       : out   std_logic;
      OPB_errAck      : in    std_logic;
      OPB_MGrant      : in    std_logic;
      OPB_retry       : in    std_logic;
      OPB_timeout     : in    std_logic;
      OPB_xferAck     : in    std_logic;
      debug           : out   std_logic_vector(39 downto 0));
  end component;
  
  component opb_evg is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      ACIN            : in std_logic;
      TTLIN           : in std_logic_vector(0 to C_EVG_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVG_TTL_OUTPUTS-1);
      UNIVIN          : in std_logic_vector(0 to C_EVG_UNIV_INPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVG_UNIV_OUTPUTS-1);

      TBIN            : in std_logic_vector(0 to C_EVG_TB_INPUTS-1);
      TBOUT           : out std_logic_vector(0 to C_EVG_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic;
      BREFCLKP        : in std_logic;
      BREFCLK2N       : in std_logic;  -- Reference clock input from 
      BREFCLK2P       : in std_logic;  -- fractional synthesizer

      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;
      
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXP
      EVRX_unusedTXN  : out std_logic; -- Unused TXP of receiving MGT
      EVRX_unusedTXP  : out std_logic; -- Unused TXP of receiving MGT
      EVTX_unusedRXN  : in std_logic;  -- Unused RXN of transmitting MGT
      EVTX_unusedRXP  : in std_logic;  -- Unused RXP of transmitting MGT
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXP

      -- AD9515BCPZ RF divider select outputs
      RFSEL           : out std_logic_vector(5 downto 0);
      EXTRFSEL        : out std_logic;

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic;

      icon_control : in std_logic_vector(35 downto 0)
      );
  end component;
  
begin
  
  i_bugdll : bufgdll
    port map (
      I => lclk,
      O => OPB_Clk);

  i_opb : system
    port map (
      sys_clk => OPB_Clk,
      sys_rst => reset,
      opb_m_0_OPB_Clk => pci_OPB_Clk,
      opb_m_0_OPB_Rst => pci_OPB_Rst,
      opb_m_0_M_DBus => pci_M_DBus,
      opb_m_0_OPB_DBus => pci_OPB_DBus,
      opb_m_0_M_ABus => pci_M_ABus,
      opb_m_0_M_BE => pci_M_BE,
      opb_m_0_M_busLock => pci_M_busLock,
      opb_m_0_M_request => pci_M_request,
      opb_m_0_M_RNW => pci_M_RNW,
      opb_m_0_M_select => pci_M_select,
      opb_m_0_M_seqAddr => pci_M_seqAddr,
      opb_m_0_OPB_errAck => pci_OPB_errAck,
      opb_m_0_OPB_MGrant => pci_OPB_MGrant,
      opb_m_0_OPB_retry => pci_OPB_retry,
      opb_m_0_OPB_timeout => pci_OPB_timeout,
      opb_m_0_OPB_xferAck => pci_OPB_xferAck,
      opb_s_1_OPB_Clk => evg_OPB_Clk,
      opb_s_1_OPB_Rst => evg_OPB_Rst,
      opb_s_1_Sl_DBus => evg_Sl_DBus,
      opb_s_1_Sl_errAck => evg_Sl_errAck,
      opb_s_1_Sl_retry => evg_Sl_retry,
      opb_s_1_Sl_toutSup => evg_Sl_toutSup,
      opb_s_1_Sl_xferAck => evg_Sl_xferAck,
      opb_s_1_OPB_ABus => evg_OPB_ABus,
      opb_s_1_OPB_BE => evg_OPB_BE,
      opb_s_1_OPB_DBus => evg_OPB_DBus,
      opb_s_1_OPB_RNW => evg_OPB_RNW,
      opb_s_1_OPB_select => evg_OPB_select,
      opb_s_1_OPB_seqAddr => evg_OPB_seqAddr,
      chipscope_icon_0_control1_pin => icon_control,
      chipscope_opb_iba_0_iba_trig_in_pin => evg_OPB_debug);

  i_opb_pci : opb_pci9030
    port map (
      nREADY => nREADY,
      nLBE => nLBE,
      nBTERM => nBTERM,
      LWnR => LWnR,
      nADS => nADS,
      LAD => LAD,
      OPB_Clk => OPB_Clk,
      OPB_Rst => pci_OPB_Rst,
      M_DBus => pci_M_DBus,
      OPB_DBus => pci_OPB_DBus,
      M_ABus => pci_M_ABus,
      M_BE => pci_M_BE,
      M_busLock => pci_M_busLock,
      M_request => pci_M_request,
      M_RNW => pci_M_RNW,
      M_select => pci_M_select,
      M_seqAddr => pci_M_seqAddr,
      OPB_errAck => pci_OPB_errAck,
      OPB_MGrant => pci_OPB_MGrant,
      OPB_retry => pci_OPB_retry,
      OPB_timeout => pci_OPB_timeout,
      OPB_xferAck => pci_OPB_xferAck,
      debug => evg_OPB_debug);
  
  i_opb_evg : opb_evg
    generic map (
      C_BASEADDR => EVG_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => OPB_Clk,
      OPB_Rst => evg_OPB_Rst,
      Sl_DBus => evg_Sl_DBus,
      Sl_errAck => evg_Sl_errAck,
      Sl_retry => evg_Sl_retry,
      Sl_toutSup => evg_Sl_toutSup,
      Sl_xferAck => evg_Sl_xferAck,
      OPB_ABus => evg_OPB_ABus,
      OPB_BE => evg_OPB_BE,
      OPB_DBus => evg_OPB_DBus,
      OPB_RNW => evg_OPB_RNW,
      OPB_select => evg_OPB_select,
      OPB_seqAddr => evg_OPB_seqAddr,

      ip_irq_out => LINTi(1),
    
      ACIN => TTLIN(1),
      TTLIN => TTLIN,
      TTLOUT => TTLOUT,
      UNIVIN => UNIVIN,
      UNIVOUT => UNIVOUT,

      TBIN => TBIN,
      TBOUT => TBOUT,
      
      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      BREFCLK2N => BREFCLK2N,
      BREFCLK2P => BREFCLK2P,

      LED_rx_link => LED_rx_link,
      LED_ena => LED_ena,
      LED_event_in => nLED(4),
      LED_event_out => nLED(3),
      LED_rx_fail => nLED(1),
    
      EVRXN => RXN(2),
      EVRXP => RXP(2),
      EVRX_unusedTXN => TXN(2),
      EVRX_unusedTXP => TXP(2),
      EVTX_unusedRXN => RXN(1),
      EVTX_unusedRXP => RXP(1),
      EVTXN => TXN(1),
      EVTXP => TXP(1),

      RFSEL => RFSEL,
      
      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS,
      CGLOCK => CGLOCK,
      icon_control => icon_control);

  LINTi(2) <= '0';
  
  -- Always enable transmitter
  TXDISB <= '0';

  -- Do not power on TB
  TBON <= "00";

  -- Output signal from MGT
  TXSEL <= '1';
  -- Get gigabit signal from transceiver input
  RXSEL <= '0';
  
  reset_generation : process (OPB_Clk)
    variable reset_cnt : std_logic_vector(1 downto 0) := "01";
  begin
    if rising_edge(OPB_Clk) then
      reset <= '1';
      if reset_cnt(1) = '1' then
	reset <= '0';
      end if;
      
      if reset_cnt(1) = '0' then
	reset_cnt := reset_cnt - 1;
      end if;
    end if;
  end process;

  enable_led: process (LED_rx_link, LED_ena)
  begin
    nLED(2) <= '1';
    if LED_rx_link = '0' or LED_ena = '0' then
      nLED(2) <= '0';
    end if;
  end process;

  PXI_CLK10_IN_O <= '0';
  PXI_CLK10_DIR <= '1';
  PXI_TRIG_DIR <= (others => '1');
  nQSOE <= '1';
  
end structure;
