---------------------------------------------------------------------------
--
--  File        : event_analyser.vhd
--
--  Title       : Event Analyser
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description :
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
LIBRARY UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity event_analyser is
  port (
    event_clk      : in std_logic;
    event_code     : in std_logic_vector(7 downto 0);
    dbus_data      : in std_logic_vector(7 downto 0);
    count_reset    : in std_logic;
    enable         : in std_logic;

    rd_clk         : in std_logic;
    event_time_out : out std_logic_vector(63 downto 0);
    event_out      : out std_logic_vector(7 downto 0);
    dbus_out       : out std_logic_vector(7 downto 0);
    rd_en          : in std_logic;
    overflow       : out std_logic;
    event_avail    : out std_logic;
    clear_overflow : in std_logic;
    fifo_reset     : in std_logic
    );
end event_analyser;

architecture structure of event_analyser is

  signal event_counter : std_logic_vector(63 downto 0);
  signal fifo_din      : std_logic_vector(79 downto 0);
  signal fifo_dout     : std_logic_vector(79 downto 0);
  signal fifo_wr       : std_logic;
  signal fifo_rd       : std_logic;
  signal fifo_full     : std_logic;
  signal fifo_empty    : std_logic;
  
  component fifo80x2048 is
    port (
      wr_clk: IN std_logic;
      rd_clk: IN std_logic;
      ainit: IN std_logic;
      din: IN std_logic_VECTOR(79 downto 0);
      wr_en: IN std_logic;
      rd_en: IN std_logic;
      dout: OUT std_logic_VECTOR(79 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;
 
begin

  i_fifo : fifo80x2048
    port map (
      wr_clk => event_clk,
      rd_clk => rd_clk,
      ainit => fifo_reset,
      din => fifo_din,
      wr_en => fifo_wr,
      rd_en => fifo_rd,
      dout => fifo_dout,
      full => fifo_full,
      empty => fifo_empty);

  fifo_din(79 downto 16) <= event_counter;
  fifo_din(15 downto 8) <= dbus_data;
  fifo_din(7 downto 0) <= event_code;
  event_out <= fifo_dout(7 downto 0);
  dbus_out <= fifo_dout(15 downto 8);
  event_time_out <= fifo_dout(79 downto 16);
  
  counting : process (event_clk, count_reset)
  begin
    if rising_edge(event_clk) then
      event_counter <= event_counter + 1;
      if count_reset = '1' then
	event_counter <= (others => '0');
      end if;
    end if;
  end process;
  
  fifo_writing : process (enable, event_code)
  begin
    fifo_wr <= '0';
    if event_code /= X"00" then
      fifo_wr <= enable;
    end if;
  end process;

  fifo_reading : process (rd_clk, fifo_empty, fifo_full, rd_en,
			  clear_overflow, fifo_reset)
    variable rd_en_1 : std_logic;
    variable reset_delay : std_logic_vector(2 downto 0);
  begin
    fifo_rd <= '0';
    if rd_en = '1' and rd_en_1 = '0' then
      fifo_rd <= '1';
    end if;

    if rising_edge(rd_clk) then
      event_avail <= not fifo_empty;
      if clear_overflow = '1' or reset_delay(2) = '1' then
	overflow <= '0';
      end if;
      if fifo_full = '1' then
	overflow <= '1';
      end if;
      
      rd_en_1 := rd_en;

      -- Reset delay is needed because fifo_reset controls async reset
      -- of fifo and overflow stays a while high after async reset
      reset_delay := reset_delay(1 downto 0) & fifo_reset;
    end if;
  end process;

end structure;
