---------------------------------------------------------------------------
--
--  File        : evg_pkg.vhd
--
--  Title       : Event Generator common definitions package
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

package evg_pkg is
  type integer_array is array (integer range <>) of integer;
  constant EVENT_RATE          : integer := 125000000;
  constant MGT_RX_PRESCALER    : integer := 1024;
  constant MGT_RX_TIMEOUT      : integer := 1024;
  constant MGT_RX_PWRDWN_TIME  : integer := 2;
  constant MGT_RX_LOCK_ACQ     : integer := 512;
  constant MGT_RX_K_COUNTER    : integer := 256;
  constant COUNT_10US          : integer := (EVENT_RATE / 100000);
  type cml_data_vector is array (integer range <>) of std_logic_vector(23 downto 0);
  type cml_we_vector is array (integer range<>) of std_logic_vector(2 downto 0);
  type count_vector is array (integer range<>) of std_logic_vector(31 downto 0);
  type we_vector is array (integer range<>) of std_logic_vector(3 downto 0);
  type data_vector is array (integer range<>) of std_logic_vector(31 downto 0);
  constant C_EVENT_CODE_BITS   : integer := 8;
  type event_code_vector is array (integer range<>) of std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  constant C_EVG_DBUS_BITS     : integer := 8;
  constant C_EVG_MXC_WIDTH     : integer := 32;
  type mx_count_vector is array (integer range<>) of std_logic_vector(C_EVG_MXC_WIDTH-1 downto 0);
  constant C_EVG_MAP_RAMS      : integer := 2;
  -- C_EVG_MAP_RAM_CODE_ABITS defines the number of address bits one event
  -- code in the mapping ram occupies
  -- currently only 4 i.e. 2**4 = 16 bytes i.e. 128 bit is supported
  constant C_EVG_MAP_CODE_ABITS : integer := 4;
  type event_map_vector is array (integer range<>) of std_logic_vector(0 to 8*(2**C_EVG_MAP_CODE_ABITS)-1);
  -- C_SIGNAL_MAP_BITS defines the number of mapping bits in the
  -- front panel mapping registers, universal output mapping registers
  -- and transition board mapping registers
  constant C_SIGNAL_MAP_BITS   : integer := 6;
  -- C_SIGNAL_MAP_DBUS defines the starting index of DBUS bit 0
  constant C_SIGNAL_MAP_DBUS   : integer := 32;
  -- C_SIGNAL_MAP_PRESC defines the starting index of the prescaler outputs
  constant C_SIGNAL_MAP_MXC    : integer := 40;
  -- C_SIGNAL_MAP_HIGH defines the index for state high output
  -- undefined indexes drive the output low
  constant C_SIGNAL_MAP_HIGH   : integer := 62;
  type signal_map_vector is array (integer range<>) of std_logic_vector(C_SIGNAL_MAP_BITS-1 downto 0);
  constant C_EVG_MAX_MXCS             : integer := 16;
  constant C_EVG_MXC_BITS             : integer := 32;
  -- C_EVG_REG_CMPMIN defines the first address bit to compare 
  constant C_EVG_REG_CMP_LOW          : integer := 16;
  constant C_EVG_REG_STATUS           : std_logic_vector(0 to 31) := X"00000000";
  constant C_EVG_REG_CONTROL          : std_logic_vector(0 to 31) := X"00000004";
  constant C_EVG_REG_IRQFLAG          : std_logic_vector(0 to 31) := X"00000008";
  constant C_EVG_REG_IRQENABLE        : std_logic_vector(0 to 31) := X"0000000C";
  constant C_EVG_REG_AC_CONTROL       : std_logic_vector(0 to 31) := X"00000010";
  constant C_EVG_REG_AC_MAP           : std_logic_vector(0 to 31) := X"00000014";
  constant C_EVG_REG_SW_EVENT         : std_logic_vector(0 to 31) := X"00000018";
  constant C_EVG_REG_DATABUF_CONTROL  : std_logic_vector(0 to 31) := X"00000020";
  constant C_EVG_REG_DBUS_MAP         : std_logic_vector(0 to 31) := X"00000024";
  constant C_EVG_REG_DBUS_TSEVENTS    : std_logic_vector(0 to 31) := X"00000028";
  constant C_EVG_REG_FW_VERSION       : std_logic_vector(0 to 31) := X"0000002C";
  constant C_EVG_REG_EVDCM_SAMPLE     : std_logic_vector(0 to 31) := X"00000030";
  constant C_EVG_REG_USEC_DIVIDER     : std_logic_vector(0 to 31) := X"0000004C";
  constant C_EVG_REG_CLOCK_CONTROL    : std_logic_vector(0 to 31) := X"00000050";
  constant C_EVG_REG_EVAN_CONTROL     : std_logic_vector(0 to 31) := X"00000060";
  constant C_EVG_REG_EVAN_EVENT       : std_logic_vector(0 to 31) := X"00000064";
  constant C_EVG_REG_EVAN_TIMEH       : std_logic_vector(0 to 31) := X"00000068";
  constant C_EVG_REG_EVAN_TIMEL       : std_logic_vector(0 to 31) := X"0000006C";
  constant C_EVG_REG_SQRC_BASE        : std_logic_vector(0 to 31) := X"00000070";
  constant C_EVG_REG_SQRC_CMP_HIGH    : integer := 27; -- last address bit to compare
  constant C_EVG_REG_SQRC_INDEX_LOW   : integer := 28;
  constant C_EVG_REG_SQRC_INDEX_HIGH  : integer := 29;

  constant C_EVG_REG_FRACDIV          : std_logic_vector(0 to 31) := X"00000080";
  constant C_EVG_REG_RX_INIT_PS       : std_logic_vector(0 to 31) := X"00000088";
  constant C_EVG_REG_EVENT_TRIG_BASE  : std_logic_vector(0 to 31) := X"00000100";
  constant C_EVG_REG_EVENT_CMP_HIGH   : integer := 25; -- last address bit to compare
  constant C_EVG_REG_EVENT_INDEX_LOW  : integer := 26;
  constant C_EVG_REG_EVENT_INDEX_HIGH : integer := 29;
  constant C_EVG_REG_MXC_BASE         : std_logic_vector(0 to 31) := X"00000180";
  constant C_EVG_REG_MXC_CMP_HIGH     : integer := 24; -- last address bit to compare
  constant C_EVG_REG_MXC_INDEX_LOW    : integer := 25;
  constant C_EVG_REG_MXC_INDEX_HIGH   : integer := 28;
  constant C_EVG_REG_MXC_CONTROL_SEL  : integer := 29;
  constant C_EVG_REG_FPOUT_BASE       : std_logic_vector(0 to 31) := X"00000400"; 
  constant C_EVG_REG_FPOUT_CMP_HIGH   : integer := 25; -- last address bit to compare
  constant C_EVG_REG_FPOUT_INDEX_LOW  : integer := 27;
  constant C_EVG_REG_FPOUT_INDEX_HIGH : integer := 29;
  constant C_EVG_REG_UNIVOUT_BASE      : std_logic_vector(0 to 31) := X"00000440"; 
  constant C_EVG_REG_UNIVOUT_CMP_HIGH  : integer := 25; -- last address bit to compare
  constant C_EVG_REG_UNIVOUT_INDEX_LOW : integer := 27;
  constant C_EVG_REG_UNIVOUT_INDEX_HIGH: integer := 29;
  constant C_EVG_REG_TBOUT_BASE       : std_logic_vector(0 to 31) := X"00000480"; 
  constant C_EVG_REG_TBOUT_CMP_HIGH   : integer := 24; -- last address bit to compare
  constant C_EVG_REG_TBOUT_INDEX_LOW  : integer := 25;
  constant C_EVG_REG_TBOUT_INDEX_HIGH : integer := 29;
  constant C_EVG_REG_FPIN_BASE        : std_logic_vector(0 to 31) := X"00000500"; 
  constant C_EVG_REG_FPIN_CMP_HIGH    : integer := 25; -- last address bit to compare
  constant C_EVG_REG_FPIN_INDEX_LOW   : integer := 29;
  constant C_EVG_REG_FPIN_INDEX_HIGH  : integer := 29;
  constant C_EVG_REG_UNIVIN_BASE      : std_logic_vector(0 to 31) := X"00000540"; 
  constant C_EVG_REG_UNIVIN_CMP_HIGH  : integer := 25; -- last address bit to compare
  constant C_EVG_REG_UNIVIN_INDEX_LOW : integer := 26;
  constant C_EVG_REG_UNIVIN_INDEX_HIGH: integer := 29;
  constant C_EVG_REG_TBIN_BASE        : std_logic_vector(0 to 31) := X"00000600"; 
  constant C_EVG_REG_TBIN_CMP_HIGH    : integer := 23; -- last address bit to compare
  constant C_EVG_REG_TBIN_INDEX_LOW   : integer := 24;
  constant C_EVG_REG_TBIN_INDEX_HIGH  : integer := 29;
  constant C_EVG_DATABUF_BASE         : std_logic_vector(0 to 31) := X"00000800";
  constant C_EVG_DATABUF_CMP_HIGH     : integer := 20; -- last address bit to compare
  constant C_EVG_REG_SEQRAM_BASE       : std_logic_vector(0 to 31) := X"00008000";
  constant C_EVG_REG_SEQRAM_CMP_HIGH   : integer := 16; -- last address bit to compare
  constant C_EVG_REG_SEQRAM_INDEX_LOW  : integer := 17;
  constant C_EVG_REG_SEQRAM_INDEX_HIGH : integer := 17;

  -- Status Register bit mappings
  constant C_EVG_STATUS_RXDBUS_HIGH    : integer := 31;
  constant C_EVG_STATUS_RXDBUS_LOW     : integer := 24;
  constant C_EVG_STATUS_TXDBUS_HIGH    : integer := 23;
  constant C_EVG_STATUS_TXDBUS_LOW     : integer := 16;
  -- Control Register bit mappings
  constant C_EVG_CTRL_MASTER_ENABLE    : integer := 31;
  constant C_EVG_CTRL_RX_DISABLE       : integer := 30;
  constant C_EVG_CTRL_RX_PWRDOWN       : integer := 29;
  constant C_EVG_CTRL_MXC_RESET        : integer := 24;
  constant C_EVG_CTRL_SEQRAM_ALT       : integer := 16;
  -- Interrupt Flag/Enable Register bit mappings
  constant C_EVG_IRQ_MASTER_ENABLE  : integer := 31;
  constant C_EVG_IRQFLAG_SEQSTOP    : integer := 12;
  constant C_EVG_IRQFLAG_SEQSTART   : integer := 8;
  constant C_EVG_IRQFLAG_DATABUF    : integer := 5;
  constant C_EVG_IRQFLAG_RXFIFOFULL : integer := 1;
  constant C_EVG_IRQFLAG_VIOLATION  : integer := 0;
  -- SW Event Register bit mappings
  constant C_EVG_SWEVENT_PENDING    : integer := 9;
  constant C_EVG_SWEVENT_ENABLE     : integer := 8;
  constant C_EVG_SWEVENT_CODE_HIGH  : integer := 7;
  constant C_EVG_SWEVENT_CODE_LOW   : integer := 0;
  -- AC Input Control Register bit mappings
  constant C_EVG_ACCTRL_BYPASS      : integer := 17;
  constant C_EVG_ACCTRL_ACSYNC      : integer := 16;
  constant C_EVG_ACCTRL_DIV_HIGH    : integer := 15;
  constant C_EVG_ACCTRL_DIV_LOW     : integer := 8;
  constant C_EVG_ACCTRL_DELAY_HIGH  : integer := 7;
  constant C_EVG_ACCTRL_DELAY_LOW   : integer := 0;
  -- AC Input Mapping Register bit mappings
  constant C_EVG_ACMAP_TRIG_BASE   : integer := 0;
  -- Databuffer Control Register bit mappings
  constant C_EVG_DATABUF_COMPLETE   : integer := 20;
  constant C_EVG_DATABUF_RUNNING    : integer := 19;
  constant C_EVG_DATABUF_TRIGGER    : integer := 18;
  constant C_EVG_DATABUF_ENA        : integer := 17;
  constant C_EVG_DATABUF_MODE       : integer := 16;
  constant C_EVG_DATABUF_SIZEHIGH   : integer := 11;
  constant C_EVG_DATABUF_SIZELOW    : integer := 2;
  -- Clock Control Register bit mapppings
  constant C_EVG_CLKCTRL_EXTRF        : integer := 24;
  constant C_EVG_CLKCTRL_DIV_HIGH     : integer := 21;
  constant C_EVG_CLKCTRL_DIV_LOW      : integer := 16;
  constant C_EVG_CLKCTRL_RECDCM_RUN    : integer := 15;
  constant C_EVG_CLKCTRL_RECDCM_INITD  : integer := 14;
  constant C_EVG_CLKCTRL_RECDCM_PSDONE : integer := 13;
  constant C_EVG_CLKCTRL_EVDCM_STOPPED : integer := 12;
  constant C_EVG_CLKCTRL_EVDCM_LOCKED : integer := 11;
  constant C_EVG_CLKCTRL_EVDCM_PSDONE : integer := 10;
  constant C_EVG_CLKCTRL_CGLOCK       : integer := 9;
  constant C_EVG_CLKCTRL_RECDCM_PSDEC : integer := 8;
  constant C_EVG_CLKCTRL_RECDCM_PSINC : integer := 7;
  constant C_EVG_CLKCTRL_RECDCM_RESET : integer := 6;
  constant C_EVG_CLKCTRL_EVDCM_PSDEC  : integer := 5;
  constant C_EVG_CLKCTRL_EVDCM_PSINC  : integer := 4;
  constant C_EVG_CLKCTRL_EVDCM_SRUN   : integer := 3;
  constant C_EVG_CLKCTRL_EVDCM_SRES   : integer := 2;
  constant C_EVG_CLKCTRL_EVDCM_RES    : integer := 1;
  constant C_EVG_CLKCTRL_USE_RXRECCLK : integer := 0;
  -- Event Analyser Control Register bit mappings
  constant C_EVG_EVANCTRL_RESET       : integer := 3;
  constant C_EVG_EVANCTRL_NOTEMPTY    : integer := 3;
  constant C_EVG_EVANCTRL_CLROVERFLOW : integer := 2;
  constant C_EVG_EVANCTRL_OVERFLOW    : integer := 2;
  constant C_EVG_EVANCTRL_ENABLE      : integer := 1;
  constant C_EVG_EVANCTRL_COUNTRES    : integer := 0;
  -- Sequence RAM Control Register bit mappings
  constant C_EVG_SQRC_RUNNING         : integer := 25;
  constant C_EVG_SQRC_ENABLED         : integer := 24;
  constant C_EVG_SQRC_ALTERNATE       : integer := 22;
  constant C_EVG_SQRC_SWTRIGGER       : integer := 21;
  constant C_EVG_SQRC_SINGLE          : integer := 20;
  constant C_EVG_SQRC_RECYCLE         : integer := 19;
  constant C_EVG_SQRC_RESET           : integer := 18;
  constant C_EVG_SQRC_DISABLE         : integer := 17;
  constant C_EVG_SQRC_ENABLE          : integer := 16;
  constant C_EVG_SQRC_TRIGSEL_LOW     : integer := 0;
  -- Sequence RAM Triggers
  constant C_EVG_SEQRAM_TRIGSEL_BITS  : integer := 5;
  constant C_EVG_SEQTRIG_MAX          : integer := 2**C_EVG_SEQRAM_TRIGSEL_BITS-1;
  type seqram_trigs_vector is array (integer range<>) of std_logic_vector(C_EVG_SEQTRIG_MAX downto 0);
  constant C_EVG_SEQTRIG_SWTRIGGER2   : integer := 18;
  constant C_EVG_SEQTRIG_SWTRIGGER1   : integer := 17;
  constant C_EVG_SEQTRIG_ACINPUT      : integer := 16;
  constant C_EVG_SEQTRIG_MXC_BASE     : integer := 0;
  -- Event Trigger bit mappings
  constant C_EVG_EVENTTRIG_ENABLE     : integer := 8;
  constant C_EVG_EVENTTRIG_CODE_HIGH  : integer := 7;
  constant C_EVG_EVENTTRIG_CODE_LOW   : integer := 0;
  -- Input mapping bits
  constant C_EVG_INMAP_TRIG_BASE      : integer := 0;
  constant C_EVG_INMAP_DBUS_BASE      : integer := 16;
  -- MXC Control Mapping bits
  constant C_EVG_MXC_READ             : integer := 31;
  constant C_EVG_MXC_POLARITY         : integer := 30;
  constant C_EVG_MXC_RESERVED_LIMIT   : integer := 29;
  constant C_EVG_MXCMAP_TRIG_BASE     : integer := 0;
  -- DBUS Mapping bits
  constant C_EVG_DBUS_SEL_BITS        : integer := 4;
  constant C_EVG_DBUS_SEL_MASK        : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0) := "0011";
  constant C_EVG_DBUS_SEL_OFF         : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0) := "0000";
  constant C_EVG_DBUS_SEL_EXT         : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0) := "0001";
  constant C_EVG_DBUS_SEL_MXC         : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0) := "0010";
  constant C_EVG_DBUS_SEL_FORWARD     : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0) := "0011";
  -- DBUS Special Timestamping event bit mappings
  constant C_EVG_DBUS_TS_1HZ          : integer := 5;
  constant C_EVG_DBUS_TS_0            : integer := 6;
  constant C_EVG_DBUS_TS_1            : integer := 7;
end evg_pkg;
