---------------------------------------------------------------------------
--
--  File        : opb_evg.vhd
--
--  Title       : OPB Event Generator IP
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : OPB Event Generator is the base for all event generators.
--                It connect to the OPB (IBM Core Connect On-Board
--                Peripheral Bus.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evg_pkg.all;
use work.evg_bs_pkg.all;
library unisim;
use unisim.vcomponents.all;

entity opb_evg is
  generic (
    -- Bus protocol parameters, do not add to or delete
    C_BASEADDR         : std_logic_vector      := X"7A000000";
    C_BASEMASK         : std_logic_vector      := X"FF000000";
    C_OPB_AWIDTH       : integer               := 32;
    C_OPB_DWIDTH       : integer               := 32;
    C_FAMILY           : string                := "virtex2p");
  port (
    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    Sl_DBus         : out   std_logic_vector(0 to 31);
    Sl_errAck       : out   std_logic;
    Sl_retry        : out   std_logic;
    Sl_toutSup      : out   std_logic;
    Sl_xferAck      : out   std_logic;
    OPB_ABus        : in    std_logic_vector(0 to 31);
    OPB_BE          : in    std_logic_vector(0 to 3);
    OPB_DBus        : in    std_logic_vector(0 to 31);
    OPB_RNW         : in    std_logic;
    OPB_select      : in    std_logic;
    OPB_seqAddr     : in    std_logic;
    
    ip_irq_out      : out std_logic;

    ACIN            : in std_logic;
    TTLIN           : in std_logic_vector(0 to C_EVG_TTL_INPUTS-1);
    TTLOUT          : out std_logic_vector(0 to C_EVG_TTL_OUTPUTS-1);
    UNIVIN          : in std_logic_vector(0 to C_EVG_UNIV_INPUTS-1);
    UNIVOUT         : out std_logic_vector(0 to C_EVG_UNIV_OUTPUTS-1);

    TBIN            : in std_logic_vector(0 to C_EVG_TB_INPUTS-1);
    TBOUT           : out std_logic_vector(0 to C_EVG_TB_OUTPUTS-1);

    -- Differential clocks from special brefclkx and brefclk2x pins
    -- Either pair may provide reference from fractional synthesizer
    -- or externally resynchronized clock
    BREFCLKN        : in std_logic;
    BREFCLKP        : in std_logic;
    BREFCLK2N       : in std_logic;
    BREFCLK2P       : in std_logic;

    RECCLKN         : out std_logic; -- Recovered clock output
    RECCLKP         : out std_logic;
    
    -- Front panel LEDs
    LED_fail        : out std_logic;
    LED_rx_link     : out std_logic;
    LED_ena         : out std_logic;
    LED_event_in    : out std_logic;
    LED_event_out   : out std_logic;
    LED_rx_fail     : out std_logic;
    LED_error       : out std_logic;
    
    -- MGT I/O pins
    EVRXN           : in std_logic;  -- Event Link RXN
    EVRXP           : in std_logic;  -- Event Link RXP
    EVRX_unusedTXN  : out std_logic; -- Unused TXP of receiving MGT
    EVRX_unusedTXP  : out std_logic; -- Unused TXP of receiving MGT
    EVTX_unusedRXN  : in std_logic;  -- Unused RXN of transmitting MGT
    EVTX_unusedRXP  : in std_logic;  -- Unused RXP of transmitting MGT
    EVTXN           : out std_logic; -- Event Link TXN
    EVTXP           : out std_logic; -- Event Link TXP

    -- AD9515BCPZ RF divider select outputs
    RFSEL           : out std_logic_vector(5 downto 0);
    EXTRFSEL        : out std_logic;
    
    -- Micrel SY87729L serial bus signals
    CGSK        : out std_logic;
    CGDI        : out std_logic;
    CGCS        : out std_logic;
    CGLOCK      : in std_logic;
    icon_control : in std_logic_vector(35 downto 0)
    );
end opb_evg;

architecture structure of opb_evg is

  constant C_EVG_FW_VERSION : std_logic_vector(31 downto 0) := X"2" &
                                                               C_FORM_FACTOR &
                                                               X"000002";

  signal gnd                : std_logic;
  signal vcc                : std_logic;
  signal gnd32              : std_logic_vector(31 downto 0);

  signal Sl_select          : std_logic;
  signal opb_rd_ack         : std_logic;
  signal opb_wr_ack         : std_logic;
  
  signal int_reset          : std_logic;
  signal mgt_reset          : std_logic;
  signal event_clk          : std_logic;
  signal use_rxrecclk       : std_logic; -- for EVGs with low jitter
                                         -- clock resynchronizing force
                                         -- use of internal recovered clock
  signal event_clk_rx       : std_logic;
  signal event_clk_rx_nobuf : std_logic;
  signal refclksel          : std_logic;
  signal reg_ref_extrf      : std_logic;
  signal reg_ext_rfsel      : std_logic_vector(5 downto 0);
  
  signal evdcm_reset        : std_logic;
  signal evdcm_clkout       : std_logic;
  signal evdcm_locked       : std_logic;
  signal evdcm_stopped      : std_logic;
  signal evdcm_sample_reset : std_logic;
  signal evdcm_sample_run   : std_logic;
  signal evdcm_sample_low   : std_logic_vector(15 downto 0);
  signal evdcm_sample_high  : std_logic_vector(15 downto 0);
  signal evdcm_psinc        : std_logic;
  signal evdcm_psdec        : std_logic;
  signal evdcm_psdone       : std_logic;

  signal recdcm_init_ps_done : std_logic;
  signal recdcm_init_ps     : std_logic_vector(7 downto 0);
  signal recdcm_running     : std_logic;
  signal recdcm_reset       : std_logic;
  signal recdcm_psinc       : std_logic;
  signal recdcm_psdec       : std_logic;
  signal recdcm_psdone      : std_logic;

  signal reg_fp_input_map   : data_vector(C_EVG_TTL_INPUTS-1 downto 0);
  signal reg_fp_output_map  : signal_map_vector(C_EVG_TTL_OUTPUTS-1 downto 0);
  signal reg_univ_input_map : data_vector(C_EVG_UNIV_INPUTS-1 downto 0);
  signal reg_univ_output_map : signal_map_vector(C_EVG_UNIV_OUTPUTS-1 downto 0);
  signal reg_tb_input_map   : data_vector(C_EVG_TB_INPUTS-1 downto 0);
  signal reg_tb_output_map  : signal_map_vector(C_EVG_TB_OUTPUTS-1 downto 0);
  signal reg_pulse_irq_map  : std_logic_vector(C_SIGNAL_MAP_BITS-1 downto 0);
  signal reg_usec_divider   : std_logic_vector(15 downto 0);
  signal reg_clock_control  : std_logic_vector(15 downto 0);
  signal usec_enable        : std_logic;
  signal reg_micrel_word    : std_logic_vector(31 downto 0);
  signal micrel_reload      : std_logic;

  signal seqram_control     : data_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_data        : data_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_we          : we_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_triggers    : std_logic_vector(C_EVG_SEQTRIG_MAX downto 0);
  signal event_seqram_code  : event_code_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal event_seqram_pend  : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal event_seqram_ack   : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_timeout     : data_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_enable      : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_disable     : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_recycle     : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_single      : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_alternate   : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_reset       : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_ena_read    : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_running     : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_altseq      : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_endseq      : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_sw_trig     : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_sw_reset    : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_debug       : std_logic_vector(63 downto 0);

  signal seqram_start_irq   : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_stop_irq    : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_start_irq_reset : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal seqram_stop_irq_reset  : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);

  signal reg_seqram_enable  : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal reg_seqram_disable : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal reg_seqram_trigger : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal reg_seqram_reset   : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal reg_mxc_seq_map    : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  
  signal event_trig_code    : event_code_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
  signal event_trig_trig    : std_logic_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
  signal event_trig_ena     : std_logic_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
  signal event_trig_pend    : std_logic_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
  signal event_trig_ack     : std_logic_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
  
  signal ac_trig            : std_logic;
  signal ac_trigger         : std_logic;
  signal reg_ac_divider     : std_logic_vector(7 downto 0);
  signal reg_ac_delay       : std_logic_vector(7 downto 0);
  signal reg_ac_sync        : std_logic;
  signal reg_ac_bypass      : std_logic;
  signal reg_ac_map         : std_logic_vector(31 downto 0);
  
  signal event_tx_code      : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  signal event_rx_code      : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  signal event_rx_pend      : std_logic;
  signal event_rx_ack       : std_logic;
  signal dbus_tx            : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  signal dbus_rx            : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  signal dbus_ext           : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  signal dbus_forward       : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  
  signal databuf_tx_data    : std_logic_vector(7 downto 0);
  signal databuf_tx_k       : std_logic;
  signal databuf_tx_ena     : std_logic;
  signal databuf_ena        : std_logic;
  signal databuf_mode       : std_logic;
  signal databuf_trigger    : std_logic;
  signal databuf_txsize     : std_logic_vector(31 downto 0);    
  signal databuf_running    : std_logic;
  signal databuf_complete   : std_logic;
  signal databuf_data       : std_logic_vector(31 downto 0);
  signal databuf_we         : std_logic_vector(3 downto 0);

  signal rx_fifo_full       : std_logic;
  signal rx_fifo_reset      : std_logic;
  signal rx_fifo_full_clr   : std_logic;
  signal rx_disable         : std_logic;
  signal rx_pwrdown         : std_logic;
  signal rx_link_ok         : std_logic;
  signal rx_violation       : std_logic;
  signal rx_violation2      : std_logic;

  signal reg_sw_trigger     : std_logic;
  signal event_sw_ena       : std_logic;
  signal event_sw_trig      : std_logic;
  signal event_sw_pend      : std_logic;
  signal event_sw_ack       : std_logic;
  signal event_sw_code      : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  
  signal reg_dbus_event     : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  signal event_dbus_ack     : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
  signal event_dbus_pend    : std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);

  signal reg_mxc_control    : data_vector(C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  signal reg_mxc_count      : mx_count_vector(C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  signal mxc_out            : std_logic_vector(C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  signal mxc_ena_out        : std_logic_vector(C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  signal mxc_reset          : std_logic;
  signal reg_mxc_reset      : std_logic;
  signal reg_mxc_polarity   : std_logic_vector(C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  signal reg_dbus_map       : std_logic_vector(C_EVG_DBUS_SEL_BITS*C_EVG_MULTIPLEXED_COUNTERS-1 downto 0);
  
  signal evan_count_reset   : std_logic;
  signal evan_enable        : std_logic;
  signal evan_time_out      : std_logic_vector(63 downto 0);
  signal evan_event_out     : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  signal evan_dbus_out      : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  signal evan_rd_en         : std_logic;
  signal evan_overflow      : std_logic;
  signal evan_not_empty     : std_logic;
  signal evan_clr_overflow  : std_logic;
  signal evan_reset         : std_logic;
  
  signal master_enable      : std_logic;
  
  signal reset_rxfifofull_irq : std_logic;
  signal reset_violation_irq  : std_logic;
  
  signal flag_lostlock      : std_logic;
  
  signal master_irq_enable  : std_logic;
  signal irq_rxfifofull_ena : std_logic;
  signal irq_rxviol_ena     : std_logic;  
  signal irq_databuf_ena    : std_logic;
  signal irq_seq_start_ena  : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  signal irq_seq_stop_ena   : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0);
  
  signal irq_out            : std_logic;
  
  signal event_sseq_pend    : std_logic;
  signal event_sseq_ack     : std_logic;
  signal event_sseq_code    : std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
  
  signal refdcm_reset       : std_logic;
  signal brefclk            : std_logic;
  signal brefclk2           : std_logic;
  signal brefrecclk         : std_logic;
  signal brefclk_dcm        : std_logic;
  signal brefclk2_dcm       : std_logic;

  component bufg is
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component bufgmux is
    port (
      O : out std_ulogic;
      I0 : in std_ulogic;
      I1 : in std_ulogic;
      S : in std_ulogic);
  end component;

  component ibufgds
  port (O  : out std_ulogic;
        I  : in std_ulogic;
        IB : in std_ulogic);
  end component;

    component ac_input
    port (
      clk      : in std_logic;
      usec_enable : in std_logic;
      reset    : in std_logic;
      ac_divider  : in std_logic_vector(7 downto 0);
      shift    : in std_logic_vector(7 downto 0);
      ac_in    : in std_logic;
      bypass   : in std_logic;
      trigger  : out std_logic);
  end component;
  
  component seq_ramb is
    generic (
      max_trigger_bits  : integer := C_EVG_SEQTRIG_MAX+1;
      max_trigsel_bits  : integer := C_EVG_SEQRAM_TRIGSEL_BITS);
    port (
      -- Sequence RAMB write interface
      data_in           : in std_logic_vector(31 downto 0);
      addr_in           : in std_logic_vector(31 downto 0);
      data_out          : out std_logic_vector(31 downto 0);
      wr_en             : in std_logic_vector(3 downto 0);
      clk               : in std_logic;
      
      -- Event interface
      event_code        : out std_logic_vector(7 downto 0);
      event_pending     : out std_logic;
      event_ack         : in std_logic;
      event_time_out    : out std_logic_vector(31 downto 0);
      
      -- Sequence control interface
      seq_clk           : in std_logic;
      
      seq_enable        : in std_logic;
      seq_disable       : in std_logic;
      seq_triggers      : in std_logic_vector(C_EVG_SEQTRIG_MAX downto 0);
      seq_trig_sel      : in std_logic_vector(C_EVG_SEQRAM_TRIGSEL_BITS-1 downto 0);
      seq_recycle       : in std_logic;
      seq_single        : in std_logic;
      seq_reset         : in std_logic;
      seq_alternate     : in std_logic;
      seq_alt_in        : in std_logic;
      
      seq_enabled_out   : out std_logic;
      seq_running_out   : out std_logic;
      seq_end_out       : out std_logic;
      irq_start_out     : out std_logic;
      irq_stop_out      : out std_logic;
      irq_start_reset   : in std_logic;
      irq_stop_reset    : in std_logic;

      reset             : in std_logic;
      debug_out         : out std_logic_vector(31 downto 0));
  end component;
  
  component trigger
    port (
      trigger_input  : in  std_logic;
      trigger_enable : in  std_logic;
      event_pending  : out std_logic;
      event_ack      : in  std_logic;
      event_clk      : in  std_logic);
  end component;

  component transceiver is
    port (
      brefclk         : in std_logic;
      brefclk2        : in std_logic;
      brefclk_dcm     : in std_logic;
      brefclk2_dcm    : in std_logic;
      refclksel       : in std_logic;
      event_clk       : out std_logic;
      rxreset         : in std_logic;
      rx_polarity     : in std_logic;
      tx_polarity     : in std_logic;
      
      rxclk_out       : out std_logic;
      
      -- Transmitter side connections
      tx_event        : in std_logic_vector(7 downto 0);
      tx_dbus         : in std_logic_vector(7 downto 0);
      tx_databuf      : in std_logic_vector(7 downto 0);
      k_databuf       : in std_logic;
      ena_databuf     : out std_logic;
      databuf_mode    : in std_logic;
      
      -- Receiver side connections
      event_code_out  : out std_logic_vector(7 downto 0);
      event_pending   : out std_logic;
      event_ack       : in std_logic;
      dbus_upstream   : out std_logic_vector(7 downto 0);
      
      fifo_full       : out   std_logic;  -- fifo full flag (latched)
      fifo_reset      : in    std_logic;  -- active high fifo reset
      fifo_clear_full : in    std_logic;  -- active high clear fifo full flag
      
      rx_disable      : in    std_logic;
      rx_pwrdown      : in    std_logic;
      rx_link_ok      : out   std_logic;
      rx_violation    : out   std_logic;
      rx_violation2   : out   std_logic;
      rx_clear_viol   : in    std_logic;
      
      reset           : in    std_logic;
      tx_reset        : in    std_logic;
      tx_powerdown    : in    std_logic;
      
      -- MGT I/O pins
      RXN1            : in std_logic;
      RXP1            : in std_logic;
      EVTXN           : out std_logic;
      EVTXP           : out std_logic;
      
      EVRXN           : in std_logic;
      EVRXP           : in std_logic;
      TXN2            : out std_logic;
      TXP2            : out std_logic);
  end component;

  component mx_counter is
    generic (
      counter_range  : integer := C_EVG_MXC_BITS);
    port (
      reg_width      : in    std_logic_vector(C_EVG_MXC_BITS-1 downto 0);
      pulse_out      : out   std_logic;
      ena_out        : out   std_logic;
      clk            : in    std_logic;
      sync           : in    std_logic;
      polarity       : in    std_logic;
      reset          : in    std_logic);
  end component;

  component event_analyser is
    port (
      event_clk      : in std_logic;
      event_code     : in std_logic_vector(7 downto 0);
      dbus_data      : in std_logic_vector(7 downto 0);
      count_reset    : in std_logic;
      enable         : in std_logic;
      
      rd_clk         : in std_logic;
      event_time_out : out std_logic_vector(63 downto 0);
      event_out      : out std_logic_vector(7 downto 0);
      dbus_out       : out std_logic_vector(7 downto 0);
      rd_en          : in std_logic;
      overflow       : out std_logic;
      event_avail    : out std_logic;
      clear_overflow : in std_logic;
      fifo_reset     : in std_logic
      );
  end component;

  component memory_buffer is
    port (
      -- Memory buffer RAMB read/write interface
      data_in           : in std_logic_vector(31 downto 0);
      data_out          : out std_logic_vector(31 downto 0);
      addr_in           : in std_logic_vector(31 downto 0);
      wr_en             : in std_logic_vector(3 downto 0);
      clk               : in std_logic;
      
      -- Data stream interface
      event_data_out    : out std_logic_vector(7 downto 0);
      event_k_out       : out std_logic;
      event_clk         : in std_logic;
      buf_clkena        : in std_logic;
      
      -- Control interface
      enable            : in std_logic;
      trigger           : in std_logic;
      bufsize_words     : in std_logic_vector(11 downto 2);
      tx_running        : out std_logic;
      tx_complete       : out std_logic;
      
      reset             : in std_logic
      );
  end component;

  component micrel_load is
    port (
      clk           : in std_logic;
      data          : in std_logic_vector(31 downto 0);
      reload        : in std_logic;
      reset         : in std_logic;
      
      -- Micrel SY87729L serial bus signals
      CGSK          : out std_logic;
      CGDI          : out std_logic;
      CGCS          : out std_logic);
  end component;

  component refclock is
    port (
      BREFCLKN     : in std_logic; -- Reference clock
      BREFCLKP     : in std_logic;
      reset        : in std_logic;
      brefclk      : out std_logic;
      brefclk_dcm  : out std_logic);
  end component;

  component event_dcm is
    port (
      recclkin      : in std_logic;
      event_clk     : in std_logic;
      reset         : in std_logic;
      sample_reset  : in std_logic;
      sample_run    : in std_logic;
      clkout        : out std_logic;
      sample_low    : out std_logic_vector(15 downto 0);
      sample_high   : out std_logic_vector(15 downto 0);
      dcm_locked    : out std_logic;
      clkin_stopped : out std_logic;
      psclk         : in std_logic;
      psinc         : in std_logic;
      psdec         : in std_logic;
      psdone        : out std_logic
      );
  end component;

  component recclk_out is
    port (
      clkin         : in std_logic;
      reset         : in std_logic;
      clkoutn       : out std_logic;
      clkoutp       : out std_logic;
      rx_link_ok    : in std_logic;
      dcm_run_out   : out std_logic;
      init_ps_done  : out std_logic;
      init_ps       : in std_logic_vector(7 downto 0);
      psclk         : in std_logic;
      psinc         : in std_logic;
      psdec         : in std_logic;
      psdone        : out std_logic
      );
  end component;

  component mgt_cmlout_bram is
    port (
      clk          : in std_logic;
      data_in      : in std_logic_vector(23 downto 0);
      data_out     : out std_logic_vector(23 downto 0);
      addr         : in std_logic_vector(1 downto 0);
      we           : in std_logic_vector(2 downto 0);
      event_clk    : in std_logic;
      brefclk      : in std_logic;
      brefclk2     : in std_logic;
      refclksel    : in std_logic;
      pulse        : in std_logic;
      enable       : in std_logic;
      powerdown    : in std_logic;
      RXN          : in std_logic;
      RXP          : in std_logic;
      txreset      : in std_logic;
      TXN          : out std_logic;
      TXP          : out std_logic);
  end component;

  component sync_domain is
    port (
      clk_in   : in std_logic;
      trig_in  : in std_logic;
      clk_out  : in std_logic;
      trig_out : out std_logic);
  end component;

  component ila32
    port
    (
      control     : in    std_logic_vector(35 downto 0);
      clk         : in    std_logic;
      trig0       : in    std_logic_vector(31 downto 0)
    );
  end component;

begin

  gnd <= '0';
  vcc <= '1';
  gnd32 <= X"00000000";

  ila: ila32
    port map
    (
      control   => icon_control,
      clk       => event_clk,
      trig0     => seqram_debug(31 downto 0)
    );
  
  event_clock_resynchronizing:
    if C_EVG_USE_RESYNC generate
      i_recclk : recclk_out
	port map (
	  clkin => event_clk_rx,
	  reset => recdcm_reset,
	  clkoutn => recclkp,
	  clkoutp => recclkn,
	  rx_link_ok => rx_link_ok,
	  dcm_run_out => recdcm_running,
	  init_ps_done => recdcm_init_ps_done,
	  init_ps => recdcm_init_ps,
	  psclk => OPB_clk,
	  psinc => recdcm_psinc,
	  psdec => recdcm_psdec,
	  psdone => recdcm_psdone);
    end generate;

  event_clock_monitoring:
    if C_EVG_USE_RESYNC generate
      i_evdcm: event_dcm
	port map (
	  recclkin => event_clk_rx,
	  event_clk => event_clk,
	  reset => evdcm_reset,
	  clkout => evdcm_clkout,
	  sample_reset => evdcm_sample_reset,
	  sample_run => evdcm_sample_run,
	  sample_low => evdcm_sample_low,
	  sample_high => evdcm_sample_high,
	  dcm_locked => evdcm_locked,
	  clkin_stopped => evdcm_stopped,
	  psclk => OPB_clk,
	  psinc => evdcm_psinc,
	  psdec => evdcm_psdec,
	  psdone => evdcm_psdone);
    end generate;

  sequence_rams:
    for i in 0 to C_EVG_SEQUENCE_RAMS-1 generate
      i_seqram: seq_ramb
	port map (
	  data_in => OPB_DBus(0 to 31),
	  data_out => seqram_data(i),
	  addr_in => OPB_ABus(0 to 31),
	  wr_en => seqram_we(i),
	  clk => OPB_Clk,

	  event_code => event_seqram_code(i),
	  event_pending => event_seqram_pend(i),
	  event_ack     => event_seqram_ack(i),
	  event_time_out => seqram_timeout(i),
	  seq_clk => event_clk,
	  seq_enable => seqram_enable(i),
	  seq_disable => seqram_disable(i),
	  seq_triggers => seqram_triggers,
	  seq_trig_sel => seqram_control(i)(C_EVG_SEQRAM_TRIGSEL_BITS-1 +
					    C_EVG_SQRC_TRIGSEL_LOW downto
					    C_EVG_SQRC_TRIGSEL_LOW),
	  seq_recycle => seqram_recycle(i),
	  seq_single => seqram_single(i),
	  seq_reset => seqram_reset(i),
	  seq_alternate => seqram_alternate(i),
	  seq_alt_in => seqram_altseq(i),

	  seq_enabled_out => seqram_ena_read(i),
	  seq_running_out => seqram_running(i),
	  seq_end_out => seqram_endseq(i),
          irq_start_out => seqram_start_irq(i),
          irq_stop_out => seqram_stop_irq(i),
          irq_start_reset => seqram_start_irq_reset(i), 
          irq_stop_reset => seqram_stop_irq_reset(i), 
          
	  reset => int_reset,
	  debug_out => seqram_debug(i*32+31 downto i*32));
    end generate;

  event_triggers:
    for i in 0 to C_EVG_EVENT_TRIGGERS-1 generate
      i_trigger: trigger
	port map ( trigger_input  => event_trig_trig(i),
		   trigger_enable => event_trig_ena(i),
		   event_pending  => event_trig_pend(i),
		   event_ack      => event_trig_ack(i),
		   event_clk      => event_clk);
    end generate;

  i_ac : ac_input
    port map (
      clk => event_clk,
      usec_enable => usec_enable,
      reset => int_reset,
      ac_divider => reg_ac_divider,
      shift => reg_ac_delay,
      ac_in => ACIN,
      bypass => reg_ac_bypass,
      trigger => ac_trig);

  i_transceiver : transceiver
    port map (
      brefclk => brefclk,
      brefclk2 => brefclk2,
      brefclk_dcm => brefclk_dcm,
      brefclk2_dcm => brefclk2_dcm,
      refclksel => refclksel,
      event_clk => event_clk,
      rxreset => gnd,
      rx_polarity => C_EVG_RXPOLARITY,
      tx_polarity => C_EVG_TXPOLARITY,
      rxclk_out => event_clk_rx,
      tx_event => event_tx_code,
      tx_dbus => dbus_tx,
      tx_databuf => databuf_tx_data,
      k_databuf => databuf_tx_k,
      ena_databuf => databuf_tx_ena,
      databuf_mode => databuf_mode,
      event_code_out => event_rx_code,
      event_pending => event_rx_pend,
      event_ack => event_rx_ack,
      dbus_upstream => dbus_rx,
      fifo_full => rx_fifo_full,
      fifo_reset => rx_fifo_reset,
      fifo_clear_full => rx_fifo_full_clr,
      rx_disable => rx_disable,
      rx_pwrdown => rx_pwrdown,
      rx_link_ok => rx_link_ok,
      rx_violation => rx_violation,
      rx_violation2 => rx_violation2,
      rx_clear_viol => reset_violation_irq,
      reset => mgt_reset,
      tx_reset => gnd,
      tx_powerdown => gnd,
      RXN1 => EVTX_unusedRXN,
      RXP1 => EVTX_unusedRXP,
      EVTXN => EVTXN,
      EVTXP => EVTXP,
      EVRXN => EVRXN,
      EVRXP => EVRXP,
      TXN2 => EVRX_unusedTXN,
      TXP2 => EVRX_unusedTXP);

  i_membuf : memory_buffer
    port map (
      data_in => OPB_DBus(0 to 31),
      data_out => databuf_data,
      addr_in => OPB_ABus(0 to 31),
      wr_en => databuf_we,
      clk => OPB_Clk,

      event_data_out => databuf_tx_data,
      event_k_out => databuf_tx_k,
      event_clk => event_clk,
      buf_clkena => databuf_tx_ena,

      enable => databuf_ena,
      trigger => databuf_trigger,
      bufsize_words => databuf_txsize(11 downto 2),
      tx_running => databuf_running,
      tx_complete => databuf_complete,
      reset => int_reset);

  multiplexed_counters:
    for i in 0 to C_EVG_MULTIPLEXED_COUNTERS-1 generate
      i_mxc: mx_counter
	port map (
	  reg_width => reg_mxc_count(i),
	  pulse_out => mxc_out(i),
	  ena_out => mxc_ena_out(i),
	  clk => event_clk,
	  sync => mxc_reset,
	  polarity => reg_mxc_polarity(i),
	  reset => int_reset);
    end generate;

  i_evan : event_analyser
    port map (
      event_clk => event_clk,
      event_code => event_tx_code,
      dbus_data => dbus_tx,
      count_reset => evan_count_reset,
      enable => evan_enable,
      rd_clk => OPB_Clk,
      event_time_out => evan_time_out,
      event_out => evan_event_out,
      dbus_out => evan_dbus_out,
      rd_en => evan_rd_en,
      overflow => evan_overflow,
      event_avail => evan_not_empty,
      clear_overflow => evan_clr_overflow,
      fifo_reset => evan_reset);

  i_micrel_load : micrel_load
    port map (
      clk => OPB_Clk,
      data => reg_micrel_word,
      reload => micrel_reload,
      reset => OPB_Rst,
      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS);

  i_refclock : refclock
    port map (
      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      reset => refdcm_reset,
      brefclk => brefclk,
      brefclk_dcm => brefclk_dcm);

  i_refclock2 : refclock
    port map (
      BREFCLKN => BREFCLK2N,
      BREFCLKP => BREFCLK2P,
      reset => refdcm_reset,
      brefclk => brefclk2,
      brefclk_dcm => brefclk2_dcm);

  seq_ram_signals:
    for i in 0 to C_EVG_SEQUENCE_RAMS-1 generate
      i_sync_ena: sync_domain
	port map (
	  clk_in => OPB_Clk,
	  trig_in => reg_seqram_enable(i),
	  clk_out => event_clk,
	  trig_out => seqram_enable(i));

      i_sync_dis: sync_domain
	port map (
	  clk_in => OPB_Clk,
	  trig_in => reg_seqram_disable(i),
	  clk_out => event_clk,
	  trig_out => seqram_disable(i));

      i_sync_trg: sync_domain
	port map (
	  clk_in => OPB_Clk,
	  trig_in => reg_seqram_trigger(i),
	  clk_out => event_clk,
	  trig_out => seqram_sw_trig(i));

      i_sync_res: sync_domain
	port map (
	  clk_in => OPB_Clk,
	  trig_in => reg_seqram_reset(i),
	  clk_out => event_clk,
	  trig_out => seqram_sw_reset(i));
    end generate;

  i_sync_mxcreset: sync_domain
    port map (
      clk_in => OPB_Clk,
      trig_in => reg_mxc_reset,
      clk_out => event_clk,
      trig_out => mxc_reset);

  i_sync_swtrig: sync_domain
    port map (
      clk_in => OPB_Clk,
      trig_in => reg_sw_trigger,
      clk_out => event_clk,
      trig_out => event_sw_trig);
  
  refdcm_reset <= not CGLOCK;

  RFSEL <= reg_ext_rfsel;
  
  reference_select: process (reg_ref_extrf)
  begin
    EXTRFSEL <= reg_ref_extrf;
    refclksel <= '1';
    if (C_EVG_REF_BREFCLK and reg_ref_extrf = '0') or
       (not C_EVG_REF_BREFCLK and reg_ref_extrf = '1') then
      refclksel <= '0';
    end if;
  end process;
  
  recovered_clock: process (brefclk, brefclk2)
  begin
    if C_EVG_USE_RESYNC then
      if C_EVG_REF_BREFCLK then
	brefrecclk <= brefclk2;
      else
	brefrecclk <= brefclk;
      end if;
    end if;
  end process;
  
  reset_generation : process (event_clk, OPB_Rst, CGLOCK)
  begin
    if OPB_Rst = '1' or CGLOCK = '0' then
      mgt_reset <= '1';
    elsif rising_edge(event_clk) then
      mgt_reset <= '0';
      if OPB_Rst = '1' or CGLOCK = '0' then
	mgt_reset <= '1';
      end if;
    end if;
  end process;

  int_reset_generation : process (event_clk, mgt_reset)
    variable reset_sync : std_logic;
  begin
    if event_clk'event and event_clk = '1' then
      int_reset <= reset_sync;
      reset_sync := mgt_reset;
    end if;
  end process;

  OPB_address_decoding : process (OPB_ABus, OPB_select)
  begin
    Sl_select <= '0';
    if (OPB_ABus and C_BASEMASK) = C_BASEADDR and
      OPB_select = '1' then
      Sl_select <= '1';
    end if;
  end process;

  transfer_acknowledge : process (opb_wr_ack, opb_rd_ack)
  begin
    if opb_wr_ack = '1' or opb_rd_ack = '1' then
      Sl_xferAck <= '1';
    else
      Sl_xferAck <= '0';
    end if;
    Sl_errAck <= '0';
    Sl_retry <= '0';
    Sl_toutSup <= '0';
  end process;

  opb_writing : process (OPB_Clk, OPB_ABus, OPB_DBus, Sl_select,
			 OPB_RNW, OPB_Rst, OPB_BE)
    variable sel           : integer;
    variable opb_wr_ack_sr : std_logic_vector(1 downto 0);
  begin
    if rising_edge(OPB_Clk) then
      opb_wr_ack <= '0';
      if Sl_select = '1' and OPB_RNW = '0' and OPB_BE /= "0000" then
	if opb_wr_ack_sr(opb_wr_ack_sr'high) = '1' then
	  opb_wr_ack <= '1';
	end if;
	opb_wr_ack_sr := opb_wr_ack_sr(opb_wr_ack_sr'high - 1 downto 0) & '0';
      else
	opb_wr_ack_sr := (others => '0');
	opb_wr_ack_sr(0) := '1';
      end if;

      reset_rxfifofull_irq <= '0';
      reset_violation_irq <= '0';
      seqram_start_irq_reset <= (others => '0');
      seqram_stop_irq_reset <= (others => '0');
      reg_sw_trigger <= '0';
      evan_count_reset <= '0';
      evan_clr_overflow <= '0';
      evan_reset <= '0';
      databuf_trigger <= '0';
      recdcm_psdec <= '0';
      recdcm_psinc <= '0';
      evdcm_psdec <= '0';
      evdcm_psinc <= '0';
      evdcm_sample_reset <= '0';
      micrel_reload <= '0';
      reg_mxc_reset <= '0';
      if Sl_select = '1' and OPB_RNW = '0' then
	case OPB_ABus(C_EVG_REG_CMP_LOW to 29) is
	  when C_EVG_REG_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	    -- Control Register
	    if OPB_BE(3 - C_EVG_CTRL_MASTER_ENABLE/8) = '1' then
	      master_enable <= OPB_DBus(31 - C_EVG_CTRL_MASTER_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVG_CTRL_RX_DISABLE/8) = '1' then
	      rx_disable <= OPB_DBus(31 - C_EVG_CTRL_RX_DISABLE);
	    end if;
	    if OPB_BE(3 - C_EVG_CTRL_RX_PWRDOWN/8) = '1' then
	      rx_pwrdown <= OPB_DBus(31 - C_EVG_CTRL_RX_PWRDOWN);
	    end if;
	    if OPB_BE(3 - C_EVG_CTRL_MXC_RESET/8) = '1' then
	      reg_mxc_reset <= OPB_DBus(31 - C_EVG_CTRL_MXC_RESET);
	    end if;
--	    if OPB_BE(3 - C_EVG_CTRL_SEQRAM_ALT/8) = '1' then
--	      reg_seqram_alt <= OPB_DBus(31 - C_EVG_CTRL_SEQRAM_ALT);
--	    end if;
	  when C_EVG_REG_IRQFLAG(C_EVG_REG_CMP_LOW to 29) =>
	    -- Interrupt Flag Register
	    if OPB_BE(3 - C_EVG_IRQFLAG_RXFIFOFULL/8) = '1' then
	      reset_rxfifofull_irq <= OPB_DBus(31 - C_EVG_IRQFLAG_RXFIFOFULL);
	    end if;
	    if OPB_BE(3 - C_EVG_IRQFLAG_VIOLATION/8) = '1' then
	      reset_violation_irq <= OPB_DBus(31 - C_EVG_IRQFLAG_VIOLATION);
	    end if;
            for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
              if OPB_BE(3 - (C_EVG_IRQFLAG_SEQSTART+i)/8) = '1' then
                seqram_start_irq_reset(i) <=
                  OPB_DBus(31 - C_EVG_IRQFLAG_SEQSTART - i);
              end if;
              if OPB_BE(3 - (C_EVG_IRQFLAG_SEQSTOP+i)/8) = '1' then
                seqram_stop_irq_reset(i) <=
                  OPB_DBus(31 - C_EVG_IRQFLAG_SEQSTOP - i);
              end if;
            end loop;
	  when C_EVG_REG_IRQENABLE(C_EVG_REG_CMP_LOW to 29) =>
	    -- Interrupt Enable Register
	    if OPB_BE(3 - C_EVG_IRQ_MASTER_ENABLE/8) = '1' then
	      master_irq_enable <= OPB_DBus(31 - C_EVG_IRQ_MASTER_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVG_IRQFLAG_DATABUF/8) = '1' then
	      irq_databuf_ena <= OPB_DBus(31 - C_EVG_IRQFLAG_DATABUF);
	    end if;
	    if OPB_BE(3 - C_EVG_IRQFLAG_RXFIFOFULL/8) = '1' then
	      irq_rxfifofull_ena <= OPB_DBus(31 - C_EVG_IRQFLAG_RXFIFOFULL);
	    end if;
	    if OPB_BE(3 - C_EVG_IRQFLAG_VIOLATION/8) = '1' then
	      irq_rxviol_ena <= OPB_DBus(31 - C_EVG_IRQFLAG_VIOLATION);
	    end if;
            for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
              if OPB_BE(3 - (C_EVG_IRQFLAG_SEQSTART+i)/8) = '1' then
                irq_seq_start_ena(i) <=
                  OPB_DBus(31 - C_EVG_IRQFLAG_SEQSTART - i);
              end if;
              if OPB_BE(3 - (C_EVG_IRQFLAG_SEQSTOP+i)/8) = '1' then
                irq_seq_stop_ena(i) <=
                  OPB_DBus(31 - C_EVG_IRQFLAG_SEQSTOP - i);
              end if;
            end loop;
	  when C_EVG_REG_SW_EVENT(C_EVG_REG_CMP_LOW to 29) =>
	    -- SW Event Register
	    if OPB_BE(3 - C_EVG_SWEVENT_ENABLE/8) = '1' then
	      event_sw_ena <= OPB_DBus(31 - C_EVG_SWEVENT_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVG_SWEVENT_CODE_HIGH/8) = '1' then
	      event_sw_code <= OPB_DBus(31 - C_EVG_SWEVENT_CODE_HIGH to
					31 - C_EVG_SWEVENT_CODE_LOW);
	      reg_sw_trigger <= '1';
	    end if;
	  when C_EVG_REG_AC_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	    -- AC Input Register
	    if OPB_BE(3 - C_EVG_ACCTRL_BYPASS/8) = '1' then
	      reg_ac_bypass <= OPB_DBus(31 - C_EVG_ACCTRL_BYPASS);
	    end if;
	    if OPB_BE(3 - C_EVG_ACCTRL_ACSYNC/8) = '1' then
	      reg_ac_sync <= OPB_DBus(31 - C_EVG_ACCTRL_ACSYNC);
	    end if;	
	    if OPB_BE(3 - C_EVG_ACCTRL_DIV_LOW/8) = '1' then
	      reg_ac_divider <= OPB_DBus(31 - C_EVG_ACCTRL_DIV_HIGH to
					 31 - C_EVG_ACCTRL_DIV_LOW);
	    end if;
	    if OPB_BE(3 - C_EVG_ACCTRL_DELAY_LOW/8) = '1' then
	      reg_ac_delay <= OPB_DBus(31 - C_EVG_ACCTRL_DELAY_HIGH to
				       31 - C_EVG_ACCTRL_DELAY_LOW);
	    end if;
	  when C_EVG_REG_AC_MAP(C_EVG_REG_CMP_LOW to 29) =>
	    -- AC Input Mapping Register
	    for i in 0 to C_EVG_EVENT_TRIGGERS-1 loop
	      if OPB_BE(3 - (C_EVG_ACMAP_TRIG_BASE+i)/8) = '1' then
		reg_ac_map(i) <= OPB_DBus(31 - C_EVG_ACMAP_TRIG_BASE - i);
	      end if;
	    end loop;
	  when C_EVG_REG_DATABUF_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	    -- Databuffer Control Register
	    if OPB_BE(3 - C_EVG_DATABUF_TRIGGER/8) = '1' then
	      databuf_trigger <= OPB_DBus(31 - C_EVG_DATABUF_TRIGGER);
	    end if;
	    if OPB_BE(3 - C_EVG_DATABUF_ENA/8) = '1' then
	      databuf_ena <= OPB_DBus(31 - C_EVG_DATABUF_ENA);
	    end if;
	    if OPB_BE(3 - C_EVG_DATABUF_MODE/8) = '1' then
	      databuf_mode <= OPB_DBus(31 - C_EVG_DATABUF_MODE);
	    end if;
	    -- Databuffer Transfer Size
	    for i in 2 to 3 loop
	      if OPB_BE(i) = '1' then
		databuf_txsize(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	    databuf_txsize(31 downto 12) <= X"00000";
	    databuf_txsize(1 downto 0) <= "00";
	  when C_EVG_REG_DBUS_MAP(C_EVG_REG_CMP_LOW to 29) =>
	    -- Distributed bus multiplexer register
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_dbus_map(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	    -- Clear unused mapping bits
	    for i in 0 to C_EVG_DBUS_BITS-1 loop
	      for j in 0 to C_EVG_DBUS_SEL_BITS-1 loop
		if C_EVG_DBUS_SEL_MASK(j) = '0' then
		  reg_dbus_map(i*C_EVG_DBUS_SEL_BITS+j) <= '0';
		end if;
	      end loop;
	    end loop;
	  when C_EVG_REG_USEC_DIVIDER(C_EVG_REG_CMP_LOW to 29) =>
	    -- uSec Divider Register
	    for i in 2 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_usec_divider(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	  when C_EVG_REG_DBUS_TSEVENTS(C_EVG_REG_CMP_LOW to 29) =>
	    -- DBUS Special Timestamping event bit enable register
	    if OPB_BE(3 - C_EVG_DBUS_TS_1HZ/8) = '1' then
	      reg_dbus_event(C_EVG_DBUS_TS_1HZ) <=
		OPB_DBus(31 - C_EVG_DBUS_TS_1HZ);
	    end if;
	    if OPB_BE(3 - C_EVG_DBUS_TS_0/8) = '1' then
	      reg_dbus_event(C_EVG_DBUS_TS_0) <=
		OPB_DBus(31 - C_EVG_DBUS_TS_0);
	    end if;
	    if OPB_BE(3 - C_EVG_DBUS_TS_1/8) = '1' then
	      reg_dbus_event(C_EVG_DBUS_TS_1) <=
		OPB_DBus(31 - C_EVG_DBUS_TS_1);
	    end if;
	  when C_EVG_REG_CLOCK_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	    -- Clock Control Register
	    if OPB_BE(3 - C_EVG_CLKCTRL_EXTRF/8) = '1' then
	      reg_ref_extrf <= OPB_DBus(31 - C_EVG_CLKCTRL_EXTRF);
	    end if;
	    if OPB_BE(3 - C_EVG_CLKCTRL_DIV_HIGH/8) = '1' then
	      reg_ext_rfsel <= OPB_DBus(31 - C_EVG_CLKCTRL_DIV_HIGH to
					31 - C_EVG_CLKCTRL_DIV_LOW);
	    end if;
	    if C_EVG_USE_RESYNC then
	      if OPB_BE(3 - C_EVG_CLKCTRL_RECDCM_PSDEC/8) = '1' then
		recdcm_psdec <= OPB_DBus(31 - C_EVG_CLKCTRL_RECDCM_PSDEC);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_RECDCM_PSINC/8) = '1' then
		recdcm_psinc <= OPB_DBus(31 - C_EVG_CLKCTRL_RECDCM_PSINC);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_RECDCM_RESET/8) = '1' then
		recdcm_reset <= OPB_DBus(31 - C_EVG_CLKCTRL_RECDCM_RESET);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_EVDCM_PSDEC/8) = '1' then
		evdcm_psdec <= OPB_DBus(31 - C_EVG_CLKCTRL_EVDCM_PSDEC);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_EVDCM_PSINC/8) = '1' then
		evdcm_psinc <= OPB_DBus(31 - C_EVG_CLKCTRL_EVDCM_PSINC);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_EVDCM_SRUN/8) = '1' then
		evdcm_sample_run <= OPB_DBus(31 - C_EVG_CLKCTRL_EVDCM_SRUN);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_EVDCM_SRES/8) = '1' then
		evdcm_sample_reset <= OPB_DBus(31 - C_EVG_CLKCTRL_EVDCM_SRES);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_EVDCM_RES/8) = '1' then
		evdcm_reset <= OPB_DBus(31 - C_EVG_CLKCTRL_EVDCM_RES);
	      end if;
	      if OPB_BE(3 - C_EVG_CLKCTRL_USE_RXRECCLK/8) = '1' then
		use_rxrecclk <= OPB_DBus(31 - C_EVG_CLKCTRL_USE_RXRECCLK);
	      end if;
	    end if;
	  when C_EVG_REG_EVAN_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	    -- Event Ananlyzer Control Register
	    if OPB_BE(3 - C_EVG_EVANCTRL_COUNTRES/8) = '1' then
	      evan_count_reset <= OPB_DBus(31 - C_EVG_EVANCTRL_COUNTRES);
	    end if;
	    if OPB_BE(3 - C_EVG_EVANCTRL_ENABLE/8) = '1' then
	      evan_enable <= OPB_DBus(31 - C_EVG_EVANCTRL_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVG_EVANCTRL_CLROVERFLOW/8) = '1' then
	      evan_clr_overflow <= OPB_DBus(31 - C_EVG_EVANCTRL_CLROVERFLOW);
	    end if;
	    if OPB_BE(3 - C_EVG_EVANCTRL_RESET/8) = '1' then
	      evan_reset <= OPB_DBus(31 - C_EVG_EVANCTRL_RESET);
	    end if;
	  when C_EVG_REG_FRACDIV(C_EVG_REG_CMP_LOW to 29) =>
	    -- Micrel SY87739 Fractional Divider Configuration Word
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_micrel_word(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	    if OPB_BE(3) = '1' then
	      micrel_reload <= '1';
	    end if;
	  when C_EVG_REG_RX_INIT_PS(C_EVG_REG_CMP_LOW to 29) =>
	    -- Initial Phase Shift for Event clock recovery DCM
	    if C_EVG_USE_RESYNC then
	      if OPB_BE(3) = '1' then
		recdcm_init_ps(7 downto 0) <=
		  OPB_DBus(24 to 31);
	      end if;
	    end if;
	  when others =>
	end case;
      end if;
      
      if OPB_Rst = '1' then
	master_enable <= '0';
	rx_disable <= '1';
	rx_pwrdown <= '1';
	master_irq_enable <= '0';
	irq_databuf_ena <= '0';
	irq_rxfifofull_ena <= '0';
	irq_rxviol_ena <= '0';
        irq_seq_start_ena <= (others => '0');
        irq_seq_stop_ena <= (others => '0');
        event_sw_ena <= '0';
	databuf_mode <= '0';
	databuf_ena <= '0';
	databuf_mode <= '0';
	reg_usec_divider <= X"003E";
	reg_ref_extrf <= '0';
	recdcm_reset <= '0';
	evdcm_sample_run <= '0';
	evdcm_reset <= '0';
	use_rxrecclk <= '0';
	reg_micrel_word <= C_EVG_MICREL_WORD;
      end if;

      if not C_EVG_USE_RESYNC then
	use_rxrecclk <= '1';
      end if;	
    end if;
  end process;

  seqram_control_wr: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
		      OPB_BE)
    variable sel : integer range 0 to C_EVG_SEQUENCE_RAMS-1;
  begin
    -- Sequence RAM control register access
    sel := conv_integer(OPB_ABus(C_EVG_REG_SQRC_INDEX_LOW to
				 C_EVG_REG_SQRC_INDEX_HIGH));
    if rising_edge(OPB_Clk) then
      for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
	reg_seqram_enable(i) <= '0';
	reg_seqram_disable(i) <= '0';
	reg_seqram_reset(i) <= '0';
	reg_seqram_trigger(i) <= '0';
      end loop;
      if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
				      C_EVG_REG_SQRC_CMP_HIGH) =
	C_EVG_REG_SQRC_BASE(C_EVG_REG_CMP_LOW to
			    C_EVG_REG_SQRC_CMP_HIGH) and OPB_RNW = '0' then
	if sel >= 0 and sel < C_EVG_SEQUENCE_RAMS then
	  if OPB_BE(3 - C_EVG_SQRC_ENABLE/8) = '1' then
	    reg_seqram_enable(sel) <= OPB_DBus(31 - C_EVG_SQRC_ENABLE);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_DISABLE/8) = '1' then
	    reg_seqram_disable(sel) <= OPB_DBus(31 - C_EVG_SQRC_DISABLE);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_RESET/8) = '1' then
	    reg_seqram_reset(sel) <= OPB_DBus(31 - C_EVG_SQRC_RESET);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_SINGLE/8) = '1' then
	    seqram_single(sel) <= OPB_DBus(31 - C_EVG_SQRC_SINGLE);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_RECYCLE/8) = '1' then
	    seqram_recycle(sel) <= OPB_DBus(31 - C_EVG_SQRC_RECYCLE);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_SWTRIGGER/8) = '1' then
	    reg_seqram_trigger(sel) <= OPB_DBus(31 - C_EVG_SQRC_SWTRIGGER);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_ALTERNATE/8) = '1' then
	    seqram_alternate(sel) <= OPB_DBus(31 - C_EVG_SQRC_ALTERNATE);
	  end if;
	  if OPB_BE(3 - C_EVG_SQRC_TRIGSEL_LOW/8) = '1' then
	    seqram_control(sel)(C_EVG_SQRC_TRIGSEL_LOW +
				C_EVG_SEQRAM_TRIGSEL_BITS - 1 downto
				C_EVG_SQRC_TRIGSEL_LOW) <=
	      OPB_DBus(31 - C_EVG_SQRC_TRIGSEL_LOW - C_EVG_SEQRAM_TRIGSEL_BITS + 1 to
		       31 - C_EVG_SQRC_TRIGSEL_LOW);
	  end if;
--	  if OPB_BE(3 - C_EVG_SQRC_CLKSEL_LOW/8) = '1' then
--	    seqram_control(sel)(C_EVG_SQRC_CLKSEL_LOW +
--				C_EVG_SEQRAM_CLKSEL_BITS - 1 downto
--				C_EVG_SQRC_CLKSEL_LOW) <=
--	      OPB_DBus(31 - C_EVG_SQRC_CLKSEL_LOW - C_EVG_SEQRAM_CLKSEL_BITS + 1 to
--		       31 - C_EVG_SQRC_CLKSEL_LOW);
--	  end if;
	end if;
      end if;
    end if;
  end process;
    
  seqram_wr: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
		      OPB_BE)
    variable sel : integer range 0 to C_EVG_SEQUENCE_RAMS-1;
  begin
    -- Sequence RAM access
    sel := conv_integer(OPB_ABus(C_EVG_REG_SEQRAM_INDEX_LOW to
				 C_EVG_REG_SEQRAM_INDEX_HIGH));
    for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
      seqram_we(i) <= (others => '0');
    end loop;
    if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
				    C_EVG_REG_SEQRAM_CMP_HIGH) =
      C_EVG_REG_SEQRAM_BASE(C_EVG_REG_CMP_LOW to
			    C_EVG_REG_SEQRAM_CMP_HIGH) and OPB_RNW = '0' then
      if sel >= 0 and sel < C_EVG_SEQUENCE_RAMS then
        seqram_we(sel)(3 downto 0) <= OPB_BE(0 to 3);
      end if;
    end if;
  end process;

  databuf_writing: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
                            OPB_BE)
  begin
    databuf_we <= (others => '0');
    if Sl_select = '1' and OPB_RNW = '0' and
      OPB_ABus(C_EVG_REG_CMP_LOW to C_EVG_DATABUF_CMP_HIGH) =
      C_EVG_DATABUF_BASE(C_EVG_REG_CMP_LOW to C_EVG_DATABUF_CMP_HIGH) then
      databuf_we(3 downto 0) <= OPB_BE(0 to 3);
    end if;
  end process;
  
  fp_input_map_register_wr:
    if C_EVG_TTL_INPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_TTL_INPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_FPIN_INDEX_LOW to
				       C_EVG_REG_FPIN_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_FPIN_CMP_HIGH) =
	    C_EVG_REG_FPIN_BASE(C_EVG_REG_CMP_LOW to
				C_EVG_REG_FPIN_CMP_HIGH) and OPB_RNW = '0' then
	    if sel >= 0 and sel < C_EVG_TTL_INPUTS then
	      for i in 0 to 3 loop
		if OPB_BE(i) = '1' then
		  reg_fp_input_map(sel)(31-i*8 downto 24-i*8) <=
		    OPB_DBus(i*8 to i*8+7);
		end if;
	      end loop;
	    end if;
	  end if;
	  for i in 0 to C_EVG_TTL_INPUTS-1 loop
	    if C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS <
	      C_EVG_INMAP_DBUS_BASE then
	      reg_fp_input_map(i)(C_EVG_INMAP_DBUS_BASE - 1 downto
				  C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS)
		<= gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
			 C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	    end if;
	    reg_fp_input_map(i)(31 downto C_EVG_INMAP_DBUS_BASE +
				C_EVG_DBUS_BITS) <=
	      gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
		    C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	  end loop;
	end if;
      end process;
    end generate;
  
  fp_output_map_register_wr:
    if C_EVG_TTL_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_TTL_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_FPOUT_INDEX_LOW to
				       C_EVG_REG_FPOUT_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_FPOUT_CMP_HIGH) =
	    C_EVG_REG_FPOUT_BASE(C_EVG_REG_CMP_LOW to
				 C_EVG_REG_FPOUT_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_fp_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_fp_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;
  
  univ_input_map_register_wr:
    if C_EVG_UNIV_INPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_UNIV_INPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_UNIVIN_INDEX_LOW to
				       C_EVG_REG_UNIVIN_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_UNIVIN_CMP_HIGH) =
	    C_EVG_REG_UNIVIN_BASE(C_EVG_REG_CMP_LOW to
				C_EVG_REG_UNIVIN_CMP_HIGH) and OPB_RNW = '0' then
	    if sel >= 0 and sel < C_EVG_UNIV_INPUTS then
	      for i in 0 to 3 loop
		if OPB_BE(i) = '1' then
		  reg_univ_input_map(sel)(31-i*8 downto 24-i*8) <=
		    OPB_DBus(i*8 to i*8+7);
		end if;
	      end loop;
	    end if;
	  end if;
	  for i in 0 to C_EVG_UNIV_INPUTS-1 loop
	    if C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS <
	      C_EVG_INMAP_DBUS_BASE then
	      reg_univ_input_map(i)(C_EVG_INMAP_DBUS_BASE - 1 downto
				  C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS)
		<= gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
			 C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	    end if;
	    reg_univ_input_map(i)(31 downto C_EVG_INMAP_DBUS_BASE +
				  C_EVG_DBUS_BITS) <=
	      gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
		    C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	  end loop;
	end if;
      end process;
    end generate;

  univ_output_map_register_wr:
    if C_EVG_UNIV_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_UNIV_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_UNIVOUT_INDEX_LOW to
				       C_EVG_REG_UNIVOUT_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_UNIVOUT_CMP_HIGH) =
	    C_EVG_REG_UNIVOUT_BASE(C_EVG_REG_CMP_LOW to
				C_EVG_REG_UNIVOUT_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_univ_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_univ_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;

  tb_input_map_register_wr:
    if C_EVG_TB_INPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_TB_INPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_TBIN_INDEX_LOW to
				       C_EVG_REG_TBIN_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_TBIN_CMP_HIGH) =
	    C_EVG_REG_TBIN_BASE(C_EVG_REG_CMP_LOW to
				C_EVG_REG_TBIN_CMP_HIGH) and OPB_RNW = '0' then
	    if sel >= 0 and sel < C_EVG_TB_INPUTS then
	      for i in 0 to 3 loop
		if OPB_BE(i) = '1' then
		  reg_tb_input_map(sel)(31-i*8 downto 24-i*8) <=
		    OPB_DBus(i*8 to i*8+7);
		end if;
	      end loop;
	    end if;
	  end if;
	  for i in 0 to C_EVG_TB_INPUTS-1 loop
	    if C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS <
	      C_EVG_INMAP_DBUS_BASE then
	      reg_tb_input_map(i)(C_EVG_INMAP_DBUS_BASE - 1 downto
				  C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS)
		<= gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
			 C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	    end if;
	    reg_tb_input_map(i)(31 downto C_EVG_INMAP_DBUS_BASE +
				C_EVG_DBUS_BITS) <=
	      gnd32(C_EVG_INMAP_DBUS_BASE - 1 downto
		    C_EVG_INMAP_TRIG_BASE+C_EVG_EVENT_TRIGGERS);
	  end loop;
	end if;
      end process;
    end generate;

  tb_map_register_wr:
    if C_EVG_TB_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_TB_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_TBOUT_INDEX_LOW to
				       C_EVG_REG_TBOUT_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_TBOUT_CMP_HIGH) =
	    C_EVG_REG_TBOUT_BASE(C_EVG_REG_CMP_LOW to
				 C_EVG_REG_TBOUT_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_tb_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_tb_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;

  event_trigger_wr:
    if C_EVG_EVENT_TRIGGERS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_EVENT_TRIGGERS - 1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_EVENT_INDEX_LOW to
				       C_EVG_REG_EVENT_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_EVENT_CMP_HIGH) =
	    	    C_EVG_REG_EVENT_TRIG_BASE(C_EVG_REG_CMP_LOW to
					 C_EVG_REG_EVENT_CMP_HIGH) and
	    OPB_RNW = '0' then
	    if OPB_BE(3 - C_EVG_EVENTTRIG_CODE_HIGH/8) = '1' then
	      event_trig_code(sel) <= OPB_DBus(31 - C_EVG_EVENTTRIG_CODE_HIGH
					       to 31 -C_EVG_EVENTTRIG_CODE_LOW);
	    end if;
	    if OPB_BE(3 - C_EVG_EVENTTRIG_ENABLE/8) = '1' then
	      event_trig_ena(sel) <= OPB_DBus(31 - C_EVG_EVENTTRIG_ENABLE);
	    end if;
	  end if;
	end if;
      end process;
    end generate;

  mx_counter_wr:
    if C_EVG_MULTIPLEXED_COUNTERS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,	       
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVG_MULTIPLEXED_COUNTERS - 1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVG_REG_MXC_INDEX_LOW to
				       C_EVG_REG_MXC_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVG_REG_CMP_LOW to
					  C_EVG_REG_MXC_CMP_HIGH) =
	    	    C_EVG_REG_MXC_BASE(C_EVG_REG_CMP_LOW to
				       C_EVG_REG_MXC_CMP_HIGH) and
	    OPB_RNW = '0' then
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
                if OPB_ABus(C_EVG_REG_MXC_CONTROL_SEL) = '0' then
                  reg_mxc_control(sel)(31-i*8 downto 24-i*8) <=
                    OPB_DBus(i*8 to i*8+7);
                else
                  reg_mxc_count(sel)(31-i*8 downto 24-i*8) <=
                    OPB_DBus(i*8 to i*8+7);
                end if;
	      end if;
	    end loop;
	  end if;
          for i in 0 to C_EVG_MULTIPLEXED_COUNTERS - 1 loop
            reg_mxc_control(i)(C_EVG_MXC_RESERVED_LIMIT downto C_EVG_EVENT_TRIGGERS) <=
              gnd32(C_EVG_MXC_RESERVED_LIMIT downto C_EVG_EVENT_TRIGGERS);
          end loop;
	end if;
        for i in 0 to C_EVG_MULTIPLEXED_COUNTERS - 1 loop
          reg_mxc_control(i)(C_EVG_MXC_READ) <= mxc_out(i);
          reg_mxc_polarity(i) <= reg_mxc_control(i)(C_EVG_MXC_POLARITY);
        end loop;
      end process;
    end generate;
  
  OPB_reading : process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			 rx_violation,
			 reg_clock_control, reg_usec_divider,
			 databuf_txsize, evdcm_locked, evdcm_stopped,
			 reg_univ_output_map)
    variable opb_rd_ack_sr     : std_logic_vector(3 downto 0);
    variable data_out          : std_logic_vector(31 downto 0);
    variable seqram_sel        : integer range 0 to C_EVG_SEQUENCE_RAMS-1;
    variable seqram_ctrl_sel   : integer range 0 to C_EVG_SEQUENCE_RAMS-1;
    variable seqram_ctrl_out   : std_logic_vector(31 downto 0);
    variable fp_input_sel      : integer range 0 to C_EVG_TTL_INPUTS-1;
    variable fp_input_data_out : std_logic_vector(31 downto 0);
    variable fp_map_sel        : integer range 0 to C_EVG_TTL_OUTPUTS-1;
    variable fp_map_data_out   : std_logic_vector(31 downto 0);
    variable univout_sel       : integer range 0 to C_EVG_UNIV_OUTPUTS-1;
    variable univout_data_out  : std_logic_vector(31 downto 0);
    variable univin_sel        : integer range 0 to C_EVG_UNIV_INPUTS-1;
    variable univin_data_out   : std_logic_vector(31 downto 0);
    variable tb_map_sel        : integer range 0 to C_EVG_TB_OUTPUTS-1;
    variable tb_map_data_out   : std_logic_vector(31 downto 0);
    variable tb_input_sel      : integer range 0 to C_EVG_TB_INPUTS-1;
    variable tb_input_data_out : std_logic_vector(31 downto 0);
    variable evtrig_sel        : integer range 0 to C_EVG_EVENT_TRIGGERS - 1;
    variable evtrig_data_out   : std_logic_vector(31 downto 0);
    variable OPB_BE_1          : std_logic_vector(0 to 3);
    variable mxc_sel           : integer range 0 to C_EVG_MULTIPLEXED_COUNTERS-1;
    variable mxc_data_out      : std_logic_vector(31 downto 0);
    variable trigger_sel       : integer range 0 to C_EVG_EVENT_TRIGGERS-1;
    variable trigger_data_out  : std_logic_vector(31 downto 0);
  begin
    evan_rd_en <= '0';
    if OPB_ABus(C_EVG_REG_CMP_LOW to 29) =
      C_EVG_REG_EVAN_EVENT(C_EVG_REG_CMP_LOW to 29) and
      Sl_select = '1' and OPB_RNW = '1' and OPB_BE(3) = '1' then
      evan_rd_en <= '1';
    end if;

    if rising_edge(OPB_Clk) then
      seqram_ctrl_sel := conv_integer(OPB_ABus(C_EVG_REG_SQRC_INDEX_LOW to
				 C_EVG_REG_SQRC_INDEX_HIGH));
      seqram_ctrl_out := seqram_control(seqram_ctrl_sel);
      seqram_ctrl_out(C_EVG_SQRC_ALTERNATE) := seqram_alternate(seqram_ctrl_sel);
      seqram_ctrl_out(C_EVG_SQRC_SINGLE) := seqram_single(seqram_ctrl_sel);
      seqram_ctrl_out(C_EVG_SQRC_RECYCLE) := seqram_recycle(seqram_ctrl_sel);
      seqram_ctrl_out(C_EVG_SQRC_RUNNING) := seqram_running(seqram_ctrl_sel);
      seqram_ctrl_out(C_EVG_SQRC_ENABLED) := seqram_ena_read(seqram_ctrl_sel);
      
      if C_EVG_TTL_INPUTS > 0 then
	fp_input_sel := conv_integer(OPB_ABus(C_EVG_REG_FPIN_INDEX_LOW to
					      C_EVG_REG_FPIN_INDEX_HIGH));
	fp_input_data_out := (others => '0');
	fp_input_data_out := reg_fp_input_map(fp_input_sel);
      end if;

      if C_EVG_TTL_OUTPUTS > 0 then
	fp_map_sel := conv_integer(OPB_ABus(C_EVG_REG_FPOUT_INDEX_LOW to
					    C_EVG_REG_FPOUT_INDEX_HIGH) & '0');
	fp_map_data_out := (others => '0');
	fp_map_data_out(C_SIGNAL_MAP_BITS+15 downto 16) :=
	  reg_fp_output_map(fp_map_sel);
	fp_map_data_out(C_SIGNAL_MAP_BITS-1 downto 0) :=
	  reg_fp_output_map(fp_map_sel+1);
      end if;

      if C_EVG_UNIV_OUTPUTS > 0 then
	univout_sel := conv_integer(OPB_ABus(C_EVG_REG_UNIVOUT_INDEX_LOW to
					     C_EVG_REG_UNIVOUT_INDEX_HIGH)
				    & '0');
	univout_data_out := (others => '0');
	univout_data_out(C_SIGNAL_MAP_BITS+15 downto 16) :=
	  reg_univ_output_map(univout_sel);
	univout_data_out(C_SIGNAL_MAP_BITS-1 downto 0) :=
	  reg_univ_output_map(univout_sel+1);
      end if;

      if C_EVG_UNIV_INPUTS > 0 then
	univin_sel := conv_integer(OPB_ABus(C_EVG_REG_UNIVIN_INDEX_LOW to
					      C_EVG_REG_UNIVIN_INDEX_HIGH));
	univin_data_out := (others => '0');
	univin_data_out := reg_univ_input_map(univin_sel);
      end if;

      if C_EVG_TB_OUTPUTS > 0 then
	tb_map_sel := conv_integer(OPB_ABus(C_EVG_REG_TBOUT_INDEX_LOW to
					    C_EVG_REG_TBOUT_INDEX_HIGH) & '0');
	tb_map_data_out := (others => '0');
	tb_map_data_out(C_SIGNAL_MAP_BITS+15 downto 16) := 
	  reg_tb_output_map(tb_map_sel);
	tb_map_data_out(C_SIGNAL_MAP_BITS-1 downto 0) := 
	  reg_tb_output_map(tb_map_sel+1);
      end if;

      if C_EVG_TB_INPUTS > 0 then
	tb_input_sel := conv_integer(OPB_ABus(C_EVG_REG_TBIN_INDEX_LOW to
					      C_EVG_REG_TBIN_INDEX_HIGH));
	tb_input_data_out := (others => '0');
	tb_input_data_out := reg_tb_input_map(tb_input_sel);
      end if;

      if C_EVG_MULTIPLEXED_COUNTERS > 0 then
	mxc_sel := conv_integer(OPB_ABus(C_EVG_REG_MXC_INDEX_LOW to
					      C_EVG_REG_MXC_INDEX_HIGH));
	mxc_data_out := (others => '0');
        if OPB_ABus(C_EVG_REG_MXC_CONTROL_SEL) = '0' then
          mxc_data_out := reg_mxc_control(mxc_sel);
        else
          mxc_data_out(C_EVG_MXC_WIDTH-1 downto 0) := reg_mxc_count(mxc_sel);
        end if;
      end if;

      if C_EVG_EVENT_TRIGGERS > 0 then
        trigger_sel := conv_integer(OPB_ABus(C_EVG_REG_EVENT_INDEX_LOW to
                                             C_EVG_REG_EVENT_INDEX_HIGH));
        trigger_data_out := (others => '0');
        trigger_data_out(C_EVG_EVENTTRIG_CODE_HIGH downto
                         C_EVG_EVENTTRIG_CODE_LOW) :=
          event_trig_code(trigger_sel);
        trigger_data_out(C_EVG_EVENTTRIG_ENABLE) := event_trig_ena(trigger_sel);
      end if;
      
      data_out := (others => '0');
      case OPB_ABus(C_EVG_REG_CMP_LOW to 29) is
	when C_EVG_REG_STATUS(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_STATUS_RXDBUS_HIGH downto C_EVG_STATUS_RXDBUS_LOW)
		   := dbus_rx;
	  data_out(C_EVG_STATUS_TXDBUS_HIGH downto C_EVG_STATUS_TXDBUS_LOW)
		   := dbus_tx;
	when C_EVG_REG_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_CTRL_MASTER_ENABLE) := master_enable;
	  data_out(C_EVG_CTRL_RX_DISABLE) := rx_disable;
          data_out(C_EVG_CTRL_RX_PWRDOWN) := rx_pwrdown;
	when C_EVG_REG_IRQFLAG(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_IRQFLAG_DATABUF) := databuf_complete;
	  data_out(C_EVG_IRQFLAG_RXFIFOFULL) := rx_fifo_full;
	  data_out(C_EVG_IRQFLAG_VIOLATION) := rx_violation;
          for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
            data_out(C_EVG_IRQFLAG_SEQSTART+i) := seqram_start_irq(i);
            data_out(C_EVG_IRQFLAG_SEQSTOP+i) := seqram_stop_irq(i);
          end loop;
	when C_EVG_REG_IRQENABLE(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_IRQ_MASTER_ENABLE) := master_irq_enable;
          for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
            data_out(C_EVG_IRQFLAG_SEQSTART+i) := irq_seq_start_ena(i);
            data_out(C_EVG_IRQFLAG_SEQSTOP+i) := irq_seq_stop_ena(i);
          end loop;
	  data_out(C_EVG_IRQFLAG_DATABUF) := irq_databuf_ena;
	  data_out(C_EVG_IRQFLAG_RXFIFOFULL) := irq_rxfifofull_ena;
	  data_out(C_EVG_IRQFLAG_VIOLATION) := irq_rxviol_ena;
	when C_EVG_REG_AC_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_ACCTRL_BYPASS) := reg_ac_bypass;
	  data_out(C_EVG_ACCTRL_ACSYNC) := reg_ac_sync;
	  data_out(C_EVG_ACCTRL_DIV_HIGH downto C_EVG_ACCTRL_DIV_LOW) := reg_ac_divider;
	  data_out(C_EVG_ACCTRL_DELAY_HIGH downto C_EVG_ACCTRL_DELAY_LOW) := reg_ac_delay;
	when C_EVG_REG_AC_MAP(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := reg_ac_map;
	when C_EVG_REG_SW_EVENT (C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_SWEVENT_PENDING) := event_sw_pend;
	  data_out(C_EVG_SWEVENT_ENABLE) := event_sw_ena;
	  data_out(C_EVG_SWEVENT_CODE_HIGH downto
		   C_EVG_SWEVENT_CODE_LOW) := event_sw_code;
	when C_EVG_REG_DATABUF_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_DATABUF_ENA) := databuf_ena;
	  data_out(C_EVG_DATABUF_RUNNING) := databuf_running;
	  data_out(C_EVG_DATABUF_COMPLETE) := databuf_complete;
	  data_out(C_EVG_DATABUF_MODE) := databuf_mode;
	  data_out(C_EVG_DATABUF_SIZEHIGH downto C_EVG_DATABUF_SIZELOW)
	    := databuf_txsize(C_EVG_DATABUF_SIZEHIGH downto
			      C_EVG_DATABUF_SIZELOW);
	when C_EVG_REG_DBUS_MAP(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := reg_dbus_map;
	when C_EVG_REG_DBUS_TSEVENTS(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_DBUS_BITS-1 downto 0) := reg_dbus_event;
	when C_EVG_REG_FW_VERSION(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := C_EVG_FW_VERSION;
	when C_EVG_REG_EVDCM_SAMPLE(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(31 downto 24) := evdcm_sample_high(15 downto 8);
	  data_out(23 downto 16) := evdcm_sample_low(15 downto 8);
	when C_EVG_REG_USEC_DIVIDER(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(15 downto 0) := reg_usec_divider;
	when C_EVG_REG_CLOCK_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_CLKCTRL_EXTRF) := reg_ref_extrf;
	  data_out(C_EVG_CLKCTRL_DIV_HIGH downto C_EVG_CLKCTRL_DIV_LOW) :=
	    reg_ext_rfsel;
	  data_out(C_EVG_CLKCTRL_RECDCM_RUN) := recdcm_running;
	  data_out(C_EVG_CLKCTRL_RECDCM_INITD) := recdcm_init_ps_done;
	  data_out(C_EVG_CLKCTRL_RECDCM_PSDONE) := recdcm_psdone;
	  data_out(C_EVG_CLKCTRL_EVDCM_STOPPED) := evdcm_stopped;
	  data_out(C_EVG_CLKCTRL_EVDCM_LOCKED) := evdcm_locked;
	  data_out(C_EVG_CLKCTRL_EVDCM_PSDONE) := evdcm_psdone;
	  data_out(C_EVG_CLKCTRL_CGLOCK) := CGLOCK;
	  data_out(C_EVG_CLKCTRL_RECDCM_RESET) := recdcm_reset;
	  data_out(C_EVG_CLKCTRL_EVDCM_SRUN) := evdcm_sample_run;
	  data_out(C_EVG_CLKCTRL_EVDCM_RES) := evdcm_reset;
	  data_out(C_EVG_CLKCTRL_USE_RXRECCLK) := use_rxrecclk;
	when C_EVG_REG_EVAN_CONTROL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVG_EVANCTRL_NOTEMPTY) := evan_not_empty;
	  data_out(C_EVG_EVANCTRL_OVERFLOW) := evan_overflow;
	  data_out(C_EVG_EVANCTRL_ENABLE) := evan_enable;
	when C_EVG_REG_EVAN_EVENT(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(C_EVENT_CODE_BITS-1 downto 0) := evan_event_out;
	  data_out(C_EVG_DBUS_BITS+C_EVENT_CODE_BITS-1 downto C_EVG_DBUS_BITS)
	    := evan_dbus_out;
	when C_EVG_REG_EVAN_TIMEH(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := evan_time_out(63 downto 32);
	when C_EVG_REG_EVAN_TIMEL(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := evan_time_out(31 downto 0);
	when C_EVG_REG_FRACDIV(C_EVG_REG_CMP_LOW to 29) =>
	  data_out := reg_micrel_word;
	when C_EVG_REG_RX_INIT_PS(C_EVG_REG_CMP_LOW to 29) =>
	  data_out(7 downto 0) := recdcm_init_ps;
	when others =>
      end case;

      seqram_sel := conv_integer(OPB_ABus(C_EVG_REG_SEQRAM_INDEX_LOW to
					  C_EVG_REG_SEQRAM_INDEX_HIGH));
      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_SEQRAM_CMP_HIGH) =
        C_EVG_REG_SEQRAM_BASE(C_EVG_REG_CMP_LOW to
			      C_EVG_REG_SEQRAM_CMP_HIGH) then
        data_out := seqram_data(seqram_sel);
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_FPIN_CMP_HIGH) =
	C_EVG_REG_FPIN_BASE(C_EVG_REG_CMP_LOW to
			    C_EVG_REG_FPIN_CMP_HIGH) then
        data_out := fp_input_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_FPOUT_CMP_HIGH) =
	C_EVG_REG_FPOUT_BASE(C_EVG_REG_CMP_LOW to
			     C_EVG_REG_FPOUT_CMP_HIGH) then
        data_out := fp_map_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_UNIVOUT_CMP_HIGH) =
	C_EVG_REG_UNIVOUT_BASE(C_EVG_REG_CMP_LOW to
			       C_EVG_REG_UNIVOUT_CMP_HIGH) then
        data_out := univout_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_UNIVIN_CMP_HIGH) =
	C_EVG_REG_UNIVIN_BASE(C_EVG_REG_CMP_LOW to
			       C_EVG_REG_UNIVIN_CMP_HIGH) then
        data_out := univin_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_TBOUT_CMP_HIGH) =
	C_EVG_REG_TBOUT_BASE(C_EVG_REG_CMP_LOW to
			     C_EVG_REG_TBOUT_CMP_HIGH) then
        data_out := tb_map_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_TBIN_CMP_HIGH) =
	C_EVG_REG_TBIN_BASE(C_EVG_REG_CMP_LOW to
			     C_EVG_REG_TBIN_CMP_HIGH) then
        data_out := tb_input_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
		  C_EVG_REG_SQRC_CMP_HIGH) =
	C_EVG_REG_SQRC_BASE(C_EVG_REG_CMP_LOW to
		  C_EVG_REG_SQRC_CMP_HIGH) then
	data_out := seqram_ctrl_out;
      end if;
      
      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_EVENT_CMP_HIGH) =
	C_EVG_REG_EVENT_TRIG_BASE(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_EVENT_CMP_HIGH) then
	data_out := evtrig_data_out;
      end if;
	
      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_DATABUF_CMP_HIGH) =
	C_EVG_DATABUF_BASE(C_EVG_REG_CMP_LOW to
                           C_EVG_DATABUF_CMP_HIGH) then
        data_out := databuf_data;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_MXC_CMP_HIGH) =
	C_EVG_REG_MXC_BASE(C_EVG_REG_CMP_LOW to
                       C_EVG_REG_MXC_CMP_HIGH) then
        data_out := mxc_data_out;
      end if;

      if OPB_ABus(C_EVG_REG_CMP_LOW to
                  C_EVG_REG_EVENT_CMP_HIGH) =
	C_EVG_REG_EVENT_TRIG_BASE(C_EVG_REG_CMP_LOW to
                       C_EVG_REG_EVENT_CMP_HIGH) then
        data_out := trigger_data_out;
      end if;

      opb_rd_ack <= '0';
      Sl_DBus <= (others => '0');
      if Sl_select = '1' and OPB_RNW = '1' then
	Sl_DBus <= data_out;
	if opb_rd_ack_sr(opb_rd_ack_sr'high) = '1' then
	  opb_rd_ack <= '1';
	end if;
	opb_rd_ack_sr := opb_rd_ack_sr(opb_rd_ack_sr'high - 1 downto 0) & '0';
      else
	opb_rd_ack_sr := (others => '0');
	opb_rd_ack_sr(0) := '1';
      end if;

      OPB_BE_1 := OPB_BE;
      
    end if;
  end process;
  
  host_interrupt: process (OPB_Clk, 
			   irq_rxfifofull_ena, rx_fifo_full,
			   irq_rxviol_ena, rx_violation,
			   master_irq_enable, master_enable,
			   irq_databuf_ena, databuf_complete,
                           irq_seq_start_ena, seqram_start_irq,
                           irq_seq_stop_ena, seqram_stop_irq)
    variable seq_irq_zeros : std_logic_vector(C_EVG_SEQUENCE_RAMS-1 downto 0)
      := (others => '0');
  begin
    if rising_edge(OPB_Clk) then
      ip_irq_out <= '0';
      if master_irq_enable = '1' and master_enable = '1' then
	if (irq_databuf_ena = '1' and databuf_complete = '1') or
	  (irq_rxfifofull_ena = '1' and rx_fifo_full = '1') or
	  (irq_rxviol_ena = '1' and rx_violation = '1') or
          ((irq_seq_start_ena and seqram_start_irq) /= seq_irq_zeros) or
          ((irq_seq_start_ena and seqram_start_irq) /= seq_irq_zeros) then
	  ip_irq_out <= '1';
	end if;
      end if;
    end if;
  end process;

  priority_encoder: process(event_seqram_pend,
			    event_sseq_pend,
                            event_rx_pend,
                            event_sw_pend,
                            event_trig_pend,
                            event_seqram_code,
                            event_rx_code,
			    event_dbus_pend,
                            event_clk)
    type event_priority is (EVENT_NONE,
			    EVENT_TRIG0,
			    EVENT_TRIG1,
			    EVENT_TRIG2,
			    EVENT_TRIG3,
			    EVENT_SEQ1,
			    EVENT_SEQ2,
			    EVENT_SSEQ,
			    EVENT_TRIG4,
			    EVENT_TRIG5,
			    EVENT_TRIG6,
			    EVENT_TRIG7,
			    EVENT_RX,
			    EVENT_SW,
			    EVENT_DBUS7,
			    EVENT_DBUS6,
			    EVENT_DBUS5);
    variable encoded_event : event_priority;
  begin
    event_trig_ack <= (others => '0');
    event_seqram_ack <= (others => '0');
    event_rx_ack <= '0';
    event_sw_ack <= '0';
    event_dbus_ack <= (others => '0');
    encoded_event := EVENT_NONE;
    if master_enable = '1' then
      if event_trig_pend(0) = '1' then
	event_trig_ack(0) <= '1';
	encoded_event := EVENT_TRIG0;
      elsif event_trig_pend(1) = '1' then
	event_trig_ack(1) <= '1';
	encoded_event := EVENT_TRIG1;
      elsif event_trig_pend(2) = '1' then
	event_trig_ack(2) <= '1';
	encoded_event := EVENT_TRIG2;
      elsif event_trig_pend(3) = '1' then
	event_trig_ack(3) <= '1';
	encoded_event := EVENT_TRIG3;
      elsif event_seqram_pend(0) = '1' and seqram_ena_read(0) = '1' then
	event_seqram_ack(0) <= '1';
	encoded_event := EVENT_SEQ1;
      elsif event_seqram_pend(1) = '1' and seqram_ena_read(1) = '1' then
	event_seqram_ack(1) <= '1';
	encoded_event := EVENT_SEQ2;
      elsif event_sseq_pend = '1' then
	event_sseq_ack <= '1';
	encoded_event := EVENT_SSEQ;
      elsif event_trig_pend(4) = '1' then
	event_trig_ack(4) <= '1';
	encoded_event := EVENT_TRIG4;
      elsif event_trig_pend(5) = '1' then
	event_trig_ack(5) <= '1';
	encoded_event := EVENT_TRIG5;
      elsif event_trig_pend(6) = '1' then
	event_trig_ack(6) <= '1';
	encoded_event := EVENT_TRIG6;
      elsif event_trig_pend(7) = '1' then
	event_trig_ack(7) <= '1';
	encoded_event := EVENT_TRIG7;
      elsif event_rx_pend = '1' then
	event_rx_ack <= '1';
	encoded_event := EVENT_RX;
      elsif event_sw_pend = '1' then
	event_sw_ack <= '1';
	encoded_event := EVENT_SW;
      elsif event_dbus_pend(7) = '1' then
	event_dbus_ack(7) <= '1';
	encoded_event := EVENT_DBUS7;
      elsif event_dbus_pend(6) = '1' then
	event_dbus_ack(6) <= '1';
	encoded_event := EVENT_DBUS6;
      elsif event_dbus_pend(5) = '1' then
	event_dbus_ack(5) <= '1';
	encoded_event := EVENT_DBUS5;
      end if;
    end if;

    if rising_edge(event_clk) then
      case encoded_event is
	when EVENT_TRIG0 => event_tx_code <= event_trig_code(0);
	when EVENT_TRIG1 => event_tx_code <= event_trig_code(1);
	when EVENT_TRIG2 => event_tx_code <= event_trig_code(2);
	when EVENT_TRIG3 => event_tx_code <= event_trig_code(3);
	when EVENT_SEQ1  => event_tx_code <= event_seqram_code(0);
	when EVENT_SEQ2  => event_tx_code <= event_seqram_code(1);
	when EVENT_SSEQ  => event_tx_code <= event_sseq_code;
	when EVENT_TRIG4 => event_tx_code <= event_trig_code(4);
	when EVENT_TRIG5 => event_tx_code <= event_trig_code(5);
	when EVENT_TRIG6 => event_tx_code <= event_trig_code(6);
	when EVENT_TRIG7 => event_tx_code <= event_trig_code(7);
	when EVENT_RX    => event_tx_code <= event_rx_code;
	when EVENT_SW    => event_tx_code <= event_sw_code;
	when EVENT_DBUS7 => event_tx_code <= X"71";
	when EVENT_DBUS6 => event_tx_code <= X"70";
	when EVENT_DBUS5 => event_tx_code <= X"7D";
	when others      => event_tx_code <= (others => '0');
      end case;
    end if;
  end process;
  
  SW_event_triggering : process (event_clk, event_sw_ack, event_sw_trig,
				 int_reset, event_sw_ena, master_enable)
    variable event_sw_trig_1 : std_logic;
  begin
    if rising_edge(event_clk) then
      if event_sw_trig = '1' and event_sw_trig_1 = '0' then
	event_sw_pend <= '1';
      end if;
      if event_sw_ack = '1' or int_reset = '1' or master_enable = '0' or
	event_sw_ena = '0' then
	event_sw_pend <= '0';
      end if;

      event_sw_trig_1 := event_sw_trig;
    end if;
  end process;

  -- Distributed bus clocks 7 downto 5 that are generated by multiplexed
  -- counters have unused transition board input that can be used to feed
  -- in timestamping signals from an external microprocessor:
  -- DBUS5: 1 Hz clock, event code 0x7D
  -- DBUS7: seconds '1', event code 0x71
  -- DBUS6: seconds '0', event code 0x70
  -- 1 MHz clock is distributed on bus bit 4 (DBUS4)
  -- It is expected that rising edges of the 1 Hz clock and 1 MHz clock
  -- are aligned, so the 1 Hz clock is sampled at the falling edge of
  -- 1 MHz clock to get the 1 Hz event about 500 ns off from the rising
  -- edge of the 1 MHz clock.
  
  dbus_timestamp_events : process (event_clk, reg_dbus_event,
				   event_dbus_ack, dbus_ext, int_reset)
    variable edge_1Hz                   : std_logic;
    variable dbus_rising                : std_logic_vector(7 downto 4);
    variable dbus7_sync                 : std_logic_vector(2 downto 0);
    variable dbus6_sync                 : std_logic_vector(2 downto 0);
    variable dbus5_sync                 : std_logic_vector(2 downto 0);
    variable dbus4_sync                 : std_logic_vector(2 downto 0);
  begin
    if rising_edge(event_clk) then
      -- Seconds '0' 0x70 and '1' 0x71 events
      if dbus_rising(6) = '1' then
	event_dbus_pend(6) <= '1';
      end if;
      if event_dbus_ack(6) = '1' then
	event_dbus_pend(6) <= '0';
      end if;
      if dbus_rising(7) = '1' then
	event_dbus_pend(7) <= '1';
      end if;
      if event_dbus_ack(7) = '1' then
	event_dbus_pend(7) <= '0';
      end if;

      -- Detect rising edges
      dbus_rising := (others => '0');
      if dbus5_sync(1 downto 0) = "10" then
	edge_1Hz := '1';
      end if;      
      if dbus7_sync(1 downto 0) = "10" then
	dbus_rising(7) := '1';
      end if;
      if dbus6_sync(1 downto 0) = "10" then
	dbus_rising(6) := '1';
      end if;
      -- sample 1Hz on 1 MHz falling edge
      if dbus4_sync(1 downto 0) = "01" and edge_1Hz = '1' then
	event_dbus_pend(5) <= '1';
	edge_1Hz := '0';
      end if;
      if event_dbus_ack(5) = '1' then
	event_dbus_pend(5) <= '0';
      end if;

      -- Synchronize unreloated signals to event clock
      dbus7_sync := dbus_ext(7) & dbus7_sync(2 downto 1);
      dbus6_sync := dbus_ext(6) & dbus6_sync(2 downto 1);
      dbus5_sync := dbus_ext(5) & dbus5_sync(2 downto 1);
      dbus4_sync := dbus_ext(4) & dbus4_sync(2 downto 1);

      -- Check if dbus events disabled
      if reg_dbus_event(5) = '0' then
	event_dbus_pend(5) <= '0';
	dbus5_sync := "111";
	edge_1Hz := '0';
      end if;
      if reg_dbus_event(6) = '0' then
	event_dbus_pend(6) <= '0';
	dbus6_sync := "111";
      end if;
      if reg_dbus_event(7) = '0' then
	event_dbus_pend(7) <= '0';
	dbus7_sync := "111";
      end if;
      if int_reset = '1' then
	dbus7_sync := "111";
	dbus6_sync := "111";
	dbus5_sync := "111";
	dbus4_sync := "111";
	event_dbus_pend <= (others => '0');
	edge_1Hz := '0';
      end if;
    end if;    
  end process;
    
  usec_generation : process (int_reset, event_clk, reg_usec_divider)
    variable counter_us : std_logic_vector(16 downto 0);
    variable enable_us  : std_logic;
  begin
    if rising_edge(event_clk) then
      usec_enable <= enable_us;
      enable_us := '0';
      if counter_us(16) = '1' then
	counter_us := ('0' & reg_usec_divider) - 2;
	enable_us := '1';
      else
	counter_us := counter_us - 1;
      end if;
    end if;
  end process;

  ac_triggering : process (event_clk, int_reset, ac_trig, reg_ac_sync,
			   mxc_out)
    variable mx_edge : std_logic_vector(1 downto 0);
    variable ac_pending : std_logic;
  begin
    if rising_edge(event_clk) then
      if ac_trig = '1' then
	ac_pending := '1';
      end if;
      if ac_trigger = '1' then
	ac_pending := '0';
      end if;

      ac_trigger <= '0';
      if ac_pending = '1' then
	if reg_ac_sync = '0' then
	  ac_trigger <= '1';
	elsif mx_edge = "01" then
	  ac_trigger <= '1';
	end if;
      end if;
	
      mx_edge(1) := mx_edge(0);
      mx_edge(0) := mxc_out(7);

      if int_reset = '1' then
	ac_pending := '0';
	mx_edge := "00";
	ac_trigger <= '0';
      end if;
    end if;
  end process;
   
  dbus : process (dbus_forward, dbus_rx, dbus_ext, event_clk,
		  reg_dbus_map, mxc_out)
    variable sel : std_logic_vector(C_EVG_DBUS_SEL_BITS-1 downto 0);
  begin
    if rising_edge(event_clk) then
      dbus_tx <= (others => '0');
      for i in 0 to 7 loop
	sel := reg_dbus_map(i*C_EVG_DBUS_SEL_BITS+C_EVG_DBUS_SEL_BITS-1 downto i*C_EVG_DBUS_SEL_BITS);
	case sel is
	  when C_EVG_DBUS_SEL_EXT =>
	    dbus_tx(i) <= dbus_ext(i);
	  when C_EVG_DBUS_SEL_MXC =>
	    dbus_tx(i) <= mxc_out(i);
	  when C_EVG_DBUS_SEL_FORWARD =>
	    dbus_tx(i) <= dbus_rx(i);
	  when others =>
	end case;
      end loop;
    end if;
  end process;

  sequence_ram_signals: process (mxc_out, ac_trigger, seqram_endseq)
  begin
    seqram_triggers <= (others => '0');
    for i in 0 to C_EVG_MULTIPLEXED_COUNTERS - 1 loop
      seqram_triggers(C_EVG_SEQTRIG_MXC_BASE + i) <= mxc_out(i);
    end loop;

    seqram_triggers(C_EVG_SEQTRIG_ACINPUT) <= ac_trigger;
    seqram_triggers(C_EVG_SEQTRIG_SWTRIGGER1) <= seqram_sw_trig(0);
    seqram_triggers(C_EVG_SEQTRIG_SWTRIGGER2) <= seqram_sw_trig(1);

    for i in 0 to C_EVG_SEQUENCE_RAMS-1 loop
      seqram_altseq((i + 1) mod C_EVG_SEQUENCE_RAMS) <= seqram_endseq(i);
    end loop;
  end process;

  
  LED_fail <= '1';
  LED_rx_link <= not rx_link_ok;
  LED_ena <= not master_enable;
  LED_error <= CGLOCK;
  
  event_leds : process (event_clk, event_rx_code, event_tx_code,
			rx_link_ok, rx_disable)
    variable tx_event        : std_logic;
    variable prescaler       : natural range 0 to COUNT_10US-1;
    variable clk_ena         : std_logic;
    variable count_event_in  : natural range 0 to 4999;
    variable count_event_out : natural range 0 to 4999;
  begin
    if rising_edge(event_clk) then
      clk_ena := '0';
      if prescaler /= 0 then
	prescaler := prescaler - 1;
      else
	prescaler := COUNT_10US-1;
	clk_ena := '1';
      end if;

      if clk_ena = '1' then
	LED_event_in <= '1';
	if count_event_in /= 0 then
	  count_event_in := count_event_in - 1;
	  LED_event_in <= '0';
	end if;

	LED_event_out <= '1';
	if count_event_out /= 0 then
	  count_event_out := count_event_out - 1;
	  LED_event_out <= '0';
	end if;
      end if;

      if event_rx_pend = '1' and rx_link_ok = '1' and rx_disable = '0' then
	count_event_in := 4999;
      end if;
      
      if tx_event = '1' then
	count_event_out := 4999;
      end if;

      tx_event := '0';
      if event_tx_code /= X"00" then
	tx_event := '1';
      end if;

    end if;
  end process;

  LED_rx_fail <= rx_disable or not rx_violation;

  input_mapping : process (event_clk, TTLIN, UNIVIN, TBIN)
    type event_trig_vector is array (integer range<>) of std_logic_vector(C_EVG_EVENT_TRIGGERS-1 downto 0);
    variable event_trig_sync : event_trig_vector(0 to 1);
    type event_dbus_vector is array (integer range<>) of std_logic_vector(C_EVG_DBUS_BITS-1 downto 0);
    variable event_dbus_sync : event_dbus_vector(0 to 1);
  begin
    for j in 0 to C_EVG_EVENT_TRIGGERS - 1 loop
      event_trig_sync(0)(j) := '0';
      -- Map front panel inputs
      for i in 0 to C_EVG_TTL_INPUTS - 1 loop
	if reg_fp_input_map(i)(C_EVG_INMAP_TRIG_BASE + j) = '1' and
	  TTLIN(i) = '1' then
	  event_trig_sync(0)(j) := '1';
	end if;
      end loop;
      -- Map universal I/O inputs
      for i in 0 to C_EVG_UNIV_INPUTS - 1 loop
	if reg_univ_input_map(i)(C_EVG_INMAP_TRIG_BASE + j) = '1' and
	  UNIVIN(i) = '1' then
	  event_trig_sync(0)(j) := '1';
	end if;
      end loop;
      -- Map TB inputs
      for i in 0 to C_EVG_TB_INPUTS - 1 loop
	if reg_tb_input_map(i)(C_EVG_INMAP_TRIG_BASE + j) = '1' and
	  TBIN(i) = '1' then
	  event_trig_sync(0)(j) := '1';
	end if;
      end loop;
      -- Map MXC signals
      for i in 0 to C_EVG_MULTIPLEXED_COUNTERS - 1 loop
        if reg_mxc_control(i)(C_EVG_MXCMAP_TRIG_BASE + j) = '1' and
          mxc_out(i) = '1' then
	  event_trig_sync(0)(j) := '1';
	end if;
      end loop;          
      -- Map AC trigger
      if reg_ac_map(C_EVG_ACMAP_TRIG_BASE + j) = '1' and
        ac_trigger = '1' then
	event_trig_sync(0)(j) := '1';
      end if;
    end loop;

    for j in 0 to C_EVG_DBUS_BITS - 1 loop
      event_dbus_sync(0)(j) := '0';
      -- Map front panel inputs
      for i in 0 to C_EVG_TTL_INPUTS - 1 loop
	if reg_fp_input_map(i)(C_EVG_INMAP_DBUS_BASE + j) = '1' and
	  TTLIN(i) = '1' then
	  event_dbus_sync(0)(j) := '1';
	end if;
      end loop;
      -- Map universal I/O inputs
      for i in 0 to C_EVG_UNIV_INPUTS - 1 loop
	if reg_univ_input_map(i)(C_EVG_INMAP_DBUS_BASE + j) = '1' and
	  UNIVIN(i) = '1' then
	  event_dbus_sync(0)(j) := '1';
	end if;
      end loop;
      -- Map TB inputs
      for i in 0 to C_EVG_TB_INPUTS - 1 loop
	if reg_tb_input_map(i)(C_EVG_INMAP_DBUS_BASE + j) = '1' and
	  TBIN(i) = '1' then
	  event_dbus_sync(0)(j) := '1';
	end if;
      end loop;
    end loop;

    if rising_edge(event_clk) then
      event_trig_trig <= event_trig_sync(1);
      event_trig_sync(1) := event_trig_sync(0);
	  
      dbus_ext <= event_dbus_sync(1);
      event_dbus_sync(1) := event_dbus_sync(0);
    end if;
  end process;
  
  output_mapping : process (event_clk, reg_fp_output_map,
			    dbus_tx, int_reset)
    variable fp_out   : std_logic_vector(0 to C_EVG_TTL_OUTPUTS-1);
    variable univ_out : std_logic_vector(0 to C_EVG_UNIV_OUTPUTS-1);
    variable tb_out   : std_logic_vector(0 to C_EVG_TB_OUTPUTS-1);
    variable sel      : integer range 0 to 2**C_SIGNAL_MAP_BITS-1;
    variable map_mux  : std_logic_vector(0 to 2**C_SIGNAL_MAP_BITS-1);
  begin
    TTLOUT <= fp_out(0 to C_EVG_TTL_OUTPUTS-1);
    UNIVOUT <= univ_out;
    TBOUT <= tb_out;
    if rising_edge(event_clk) then
      map_mux := (others => '0');
      map_mux(2**C_SIGNAL_MAP_BITS-2) := '1';
      for i in 0 to C_EVG_DBUS_BITS-1 loop
        map_mux(C_SIGNAL_MAP_DBUS+i) := dbus_tx(i);
      end loop;
      for i in 0 to C_EVG_MULTIPLEXED_COUNTERS-1 loop
        map_mux(C_SIGNAL_MAP_MXC+i) := mxc_out(i);
      end loop;
      -- Map front panel outputs
      for i in 0 to C_EVG_TTL_OUTPUTS - 1 loop
	sel := conv_integer(reg_fp_output_map(i));
	fp_out(i) := map_mux(sel);
      end loop;
      -- Map Universal I/O outputs
      for i in 0 to C_EVG_UNIV_OUTPUTS - 1 loop
	sel := conv_integer(reg_univ_output_map(i));
	univ_out(i) := map_mux(sel);
      end loop;
      -- Map transition board outputs
      for i in 0 to C_EVG_TB_OUTPUTS - 1 loop
	sel := conv_integer(reg_tb_output_map(i));
	tb_out(i) := map_mux(sel);
      end loop;
    end if;
  end process;

  event_sseq_pend <= '0';
  event_sseq_code <= (others => '0');
  
end structure;
