---------------------------------------------------------------------------
--
--  File        : seq_ramb.vhd
--
--  Title       : Sparse sequence RAM
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description :
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
LIBRARY UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity seq_ramb is
  generic (
    max_trigger_bits  : integer;
    max_trigsel_bits  : integer);
  port (
    -- Sequence RAMB write interface
    data_in           : in std_logic_vector(31 downto 0);
    addr_in           : in std_logic_vector(31 downto 0);
    data_out          : out std_logic_vector(31 downto 0);
    wr_en             : in std_logic_vector(3 downto 0);
    clk               : in std_logic;
    
    -- Event interface
    event_code        : out std_logic_vector(7 downto 0);
    event_pending     : out std_logic;
    event_ack         : in std_logic;
    event_time_out    : out std_logic_vector(31 downto 0);
    
    -- Sequence control interface
    seq_clk           : in std_logic;

    seq_enable        : in std_logic;
    seq_disable       : in std_logic;
    seq_triggers      : in std_logic_vector(max_trigger_bits-1 downto 0);
    seq_trig_sel      : in std_logic_vector(max_trigsel_bits-1 downto 0);
    seq_recycle       : in std_logic;
    seq_single        : in std_logic;
    seq_reset         : in std_logic;
    seq_alternate     : in std_logic;
    seq_alt_in        : in std_logic;

    seq_enabled_out   : out std_logic;
    seq_running_out   : out std_logic;
    seq_end_out       : out std_logic;
    irq_start_out     : out std_logic;
    irq_stop_out      : out std_logic;
    irq_start_reset   : in std_logic;
    irq_stop_reset    : in std_logic;
    
    reset             : in std_logic;
    debug_out         : out std_logic_vector(31 downto 0)
    );
end seq_ramb;

architecture implementation of seq_ramb is

  signal gnd8            : std_logic_vector(7 downto 0);
  signal gnd             : std_logic;
  signal vcc             : std_logic;
  
  signal event_code_i    : std_logic_vector(7 downto 0);
  signal event_address   : std_logic_vector(31 downto 0);
  signal event_address_i : std_logic_vector(31 downto 0);
  signal seq_running     : std_logic;
  signal seq_enabled     : std_logic;

  signal data_A_out      : std_logic_vector(39 downto 0);
  signal data_B_out      : std_logic_vector(39 downto 0);
  signal we_A            : std_logic_vector(4 downto 0);
  signal code_end        : std_logic;
  signal addr_B          : std_logic_vector(10 downto 0);
  signal en_B            : std_logic;
  
  signal fifo_wr         : std_logic;
  signal fifo_rd         : std_logic;
  signal event_buf_empty : std_logic;
  signal fifo_empty      : std_logic;

  signal seq_clk_ena     : std_logic;
  signal seq_trigger     : std_logic;
  
  component RAMB16_S9_S9 is
    port (
      DOA   : out std_logic_vector(7 downto 0);
      DOB   : out std_logic_vector(7 downto 0);
      DOPA  : out std_logic_vector(0 downto 0);
      DOPB  : out std_logic_vector(0 downto 0);
      ADDRA : in  std_logic_vector(10 downto 0);
      ADDRB : in  std_logic_vector(10 downto 0);
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      DIA   : in  std_logic_vector(7 downto 0);
      DIB   : in  std_logic_vector(7 downto 0);
      DIPA  : in  std_logic_vector(0 downto 0);
      DIPB  : in  std_logic_vector(0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic);
  end component;

begin

  ram_blocks:
    for i in 4 downto 0 generate
      i_ramb: RAMB16_S9_S9
	port map (
	  DOA => data_A_out(8*i+7 downto 8*i),
	  DOB => data_B_out(8*i+7 downto 8*i),
	  DOPA => open,
	  DOPB => open,
	  ADDRA => addr_in(13 downto 3),
	  ADDRB => addr_B,
	  CLKA => clk,
	  CLKB => seq_clk,
	  DIA => data_in(8*(i mod 4)+7 downto 8*(i mod 4)),
	  DIB => gnd8,
	  DIPA => gnd8(0 downto 0),
	  DIPB => gnd8(0 downto 0),
	  ENA => vcc,
	  ENB => en_B,
	  SSRA => gnd,
	  SSRB => gnd,
	  WEA => we_A(i),
	  WEB => gnd);
    end generate;

  gnd8 <= (others => '0');
  gnd <= '0';
  vcc <= '1';
  
  data_reading: process (addr_in, data_A_out)
  begin
    data_out <= data_A_out(31 downto 0);
    if addr_in(2) = '1' then
      data_out(31 downto 8) <= X"000000";
      data_out(7 downto 0) <= data_A_out(39 downto 32);
    end if;
  end process;

  data_writing: process (addr_in, wr_en)
  begin
    we_A(3 downto 0) <= wr_en;
    we_A(4) <= '0';
    if addr_in(2) = '1' then
      we_A(3 downto 0) <= "0000";
      we_A(4) <= wr_en(0);
    end if;
  end process;
  
  ramb_address_control : process (seq_clk, seq_clk_ena, event_address,
				  data_B_out, seq_enabled, seq_running,
				  code_end)
    variable wr_en_1 : std_logic_vector(4 downto 0);
  begin
    event_code_i <= data_B_out(39 downto 32);
    event_address_i <= data_B_out(31 downto 0);
    en_B <= '0';
    if seq_clk_ena = '1' and seq_running = '1' then
      if event_address_i <= event_address then
	en_B <= '1';
      end if;
    end if;
    if seq_clk_ena = '1' and code_end = '1' then
      en_B <= '1';
    end if;
    if seq_clk'event and seq_clk = '1' then
      if en_B = '1' then
	addr_B <= addr_B + 1;
      end if;
      if seq_clk_ena = '1' and seq_running = '1' then
	if code_end = '1' then
	  addr_B <= (others => '0');
	end if;
      end if;
      
      if reset = '1' or seq_reset = '1' then
	addr_B <= (others => '0');
      end if;
    end if;
  end process;
    
  address_generation : process (seq_clk, seq_clk_ena,
				seq_reset, reset, event_address,
				seq_running)
    variable addr_next : std_logic_vector(31 downto 0);
  begin
    event_time_out <= event_address;
    if seq_clk'event and seq_clk = '1' then
      addr_next := event_address + 1;

      if code_end = '1' then
	addr_next := X"FFFFFFFF";
      end if;

      if seq_clk_ena = '1' and seq_running = '1' then
	event_address <= addr_next;
      end if;
	
      if reset = '1' or seq_reset = '1' then
	event_address <= X"FFFFFFFF";
      end if;
    end if;
  end process;

  sequence_state: process (seq_clk, reset, seq_enabled, seq_running,
                           seq_enable, seq_disable,
                           code_end, seq_recycle, seq_single)
  begin
    seq_enabled_out <= seq_enabled;
    seq_running_out <= seq_running;

    if seq_clk'event and seq_clk = '1' then
      if seq_enable = '1' then
	seq_enabled <= '1';
      end if;

      if seq_enabled = '1' and seq_trigger = '1' then
        if seq_running = '0' then
          irq_start_out <= '1';
        end if;
	seq_running <= '1';
      end if;

      if seq_running = '1' then
        if code_end = '1' and seq_clk_ena = '1' then
          irq_stop_out <= '1';
        end if;
	if code_end = '1' and seq_clk_ena = '1' and seq_recycle = '0' then
	  seq_running <= '0';
	end if;
	if code_end = '1' and seq_clk_ena = '1' and seq_single = '1' then
	  seq_running <= '0';
	  seq_enabled <= '0';
	end if;
      end if;
      
      if seq_reset = '1' then
	seq_running <= '0';
      end if;

      if irq_start_reset = '1' then
        irq_start_out <= '0';
      end if;

      if irq_stop_reset = '1' then
        irq_stop_out <= '0';
      end if;
      
      if reset = '1' or seq_disable = '1' then
	seq_running <= '0';
	seq_enabled <= '0';
      end if;

      if reset = '1' then
        irq_start_out <= '0';
        irq_stop_out <= '0';
      end if;        
    end if;
  end process;
  
  events: process (seq_clk, event_ack, reset, event_code_i,
		   event_address, event_address_i, seq_clk_ena,
		   seq_running)
    variable i_pending : std_logic;
  begin
    fifo_wr <= '0';
    if seq_running = '1' then
      if event_address = event_address_i then
	if event_code_i /= X"00" and -- Null event code
	  event_code_i /= X"7E" and  -- Freeze sequence code
	  event_code_i /= X"7F" and  -- End sequence code
	  seq_clk_ena = '1' then
	  fifo_wr <= '1';
	end if;
      end if;
    end if;
    
    code_end <= '0';
    if event_address = event_address_i and event_code_i = X"7F" then
      code_end <= '1';
    end if;

    if seq_clk'event and seq_clk = '1' then
      if fifo_wr = '1' then
	event_code <= event_code_i;
      end if;
      if reset = '1' then
	event_code <= (others => '0');
      end if;
    end if;
  end process;

  fifo_event : process (seq_clk, fifo_wr, event_ack, event_buf_empty, reset)
  begin
    if seq_clk'event and seq_clk = '1' then
      if fifo_wr = '1' then
	event_pending <= '1';
	debug_out(21) <= '1';
      elsif event_ack = '1' then
	event_pending <= '0';
	debug_out(21) <= '0';
      end if;
      if reset = '1' then
	event_pending <= '0';
	debug_out(21) <= '0';
      end if;
    end if;
  end process;

--  seq_clock_selector: process (seq_clk, seq_clocks, seq_clock_sel)
--    variable sel : integer range 0 to 2**max_clock_bits-1;
--    variable clk : std_logic_vector(1 downto 0);
--  begin
--    sel := conv_integer(seq_clock_sel);
--    clk(0) := seq_clocks(sel);
--    if rising_edge(seq_clk) then
--      seq_clk_ena <= '0';
--      if clk = "01" then
	seq_clk_ena <= '1';
--      end if;
--      clk(1) := clk(0);
--    end if;
--  end process;

  trigger_selector: process (seq_clk, seq_triggers, seq_trig_sel)
    variable sel : integer range 0 to 2**max_trigsel_bits-1;
    variable trigger : std_logic_vector(1 downto 0);
  begin
    sel := conv_integer(seq_trig_sel);
    trigger(0) := seq_triggers(sel);
    if rising_edge(seq_clk) then
      seq_trigger <= '0';
      if trigger = "01" then
	seq_trigger <= '1';
      end if;
      trigger(1) := trigger(0);
    end if;
  end process;

--  debug_out(31 downto 24) <= seq_triggers(7 downto 0);
  debug_out(31 downto 24) <= event_address(7 downto 0);
  debug_out(23) <= seq_trigger;
  debug_out(22) <= event_ack;
  debug_out(20 downto 16) <= seq_trig_sel;
  debug_out(14) <= seq_enable;
  debug_out(13) <= seq_disable;
  debug_out(12) <= seq_recycle;
  debug_out(11) <= seq_single;
  debug_out(10) <= seq_reset;
  debug_out(9) <= seq_enabled;
  debug_out(8) <= seq_running;
  debug_out(7 downto 0) <= event_code_i;
  
end implementation;
      
