---------------------------------------------------------------------------
--
--  File        : transceiver.vhd
--
--  Title       : Event Generator Multi-Gigabit Transceiver
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evg_pkg.all;
library unisim;
use unisim.vcomponents.all;

entity transceiver is
  port (
    brefclk         : in std_logic;
    brefclk2        : in std_logic;
    brefclk_dcm     : in std_logic;
    brefclk2_dcm    : in std_logic;
    refclksel       : in std_logic; -- 0 - use brefclk, 1 -.use brefclk2
    event_clk       : out std_logic;
    rxreset         : in std_logic;
    rx_polarity     : in std_logic;
    tx_polarity     : in std_logic;
    
    rxclk_out       : out std_logic;
    
    -- Transmitter side connections
    tx_event        : in std_logic_vector(7 downto 0);
    tx_dbus         : in std_logic_vector(7 downto 0);
    tx_databuf      : in std_logic_vector(7 downto 0);
    k_databuf       : in std_logic;
    ena_databuf     : out std_logic;
    databuf_mode    : in std_logic;
    
    -- Receiver side connections
    event_code_out  : out std_logic_vector(7 downto 0);
    event_pending   : out std_logic;
    event_ack       : in std_logic;
    dbus_upstream   : out std_logic_vector(7 downto 0);

    fifo_full       : out   std_logic;  -- fifo full flag (latched)
    fifo_reset      : in    std_logic;  -- active high fifo reset
    fifo_clear_full : in    std_logic;  -- active high clear fifo full flag
    
    rx_disable      : in    std_logic;
    rx_pwrdown      : in    std_logic;
    rx_link_ok      : out   std_logic;
    rx_violation    : out   std_logic;
    rx_violation2   : out   std_logic;
    rx_clear_viol   : in    std_logic;

    reset           : in    std_logic;
    tx_reset        : in    std_logic;
    tx_powerdown    : in    std_logic;
    
    -- MGT I/O pins
    RXN1            : in std_logic;
    RXP1            : in std_logic;
    EVTXN           : out std_logic;
    EVTXP           : out std_logic;
    
    EVRXN           : in std_logic;
    EVRXP           : in std_logic;
    TXN2            : out std_logic;
    TXP2            : out std_logic);
end transceiver;

architecture structure of transceiver is

  signal vcc1 : std_logic;
  signal gnd1 : std_logic;
  signal gnd2 : std_logic_vector(1 downto 0);
  signal gnd16 : std_logic_vector(15 downto 0);

  signal refclk_dcm : std_logic;
  
  signal tx_mgt_pwd    : std_logic;
  signal tx_charisk    : std_logic_vector(1 downto 0);
  signal tx_data       : std_logic_vector(15 downto 0);

  signal rx_mgt_pwd    : std_logic;
  signal rx_bufstatus  : std_logic_vector(1 downto 0);
  signal rx_charisk    : std_logic_vector(1 downto 0);
  signal rx_rundisp    : std_logic_vector(1 downto 0);
  signal rx_commadet   : std_logic;
  signal rx_data       : std_logic_vector(15 downto 0);
  signal rx_disperr    : std_logic_vector(1 downto 0);
  signal rx_loss       : std_logic_vector(1 downto 0);
  signal rx_notintable : std_logic_vector(1 downto 0);
  signal rx_realign    : std_logic;
  signal rxclk         : std_logic;

  signal rx_disperr2    : std_logic_vector(1 downto 0);
  signal rx_loss2       : std_logic_vector(1 downto 0);
  signal rx_notintable2 : std_logic_vector(1 downto 0);

  signal link_ok       : std_logic;
  
  signal rx_fifo_data_in      : std_logic_vector(7 downto 0);
  signal rx_fifo_wr           : std_logic;
  signal rx_fifo_rd           : std_logic;
  signal rx_fifo_reset        : std_logic;
  signal rx_fifo_full         : std_logic;
  signal rx_fifo_empty        : std_logic;
  signal rx_buffer_empty      : std_logic;

  signal async_in   : std_logic_vector(31 downto 0);
  signal async_out  : std_logic_vector(7 downto 0);

  signal ila_data   : std_logic_vector(31 downto 0);
  signal ila_trig   : std_logic_vector(7 downto 0);

  component mgt_tx
    port (
      BREFCLK : in std_logic;
      BREFCLK2 : in std_logic;
      event_clk : in std_logic;
      REFCLKSEL : in std_logic;
      POWERDOWN : in std_logic;
      RXN : in std_logic;
      RXP : in std_logic;
      TXCHARISK : in std_logic_vector (1 downto 0);
      TXDATA : in std_logic_vector (15 downto 0);
      TXRESET : in std_logic;
      TXPOLARITY : in std_logic;
      TXN : out std_logic;
      TXP : out std_logic);
  end component;

  component mgt_rx
    port (
      BREFCLK : in std_logic;
      BREFCLK2 : in std_logic;
      REFCLKSEL : in std_logic;
      POWERDOWN : in std_logic;
      RXN : in std_logic;
      RXP : in std_logic;
      RXRESET : in std_logic;
      RXPOLARITY : in std_logic;
      RXBUFSTATUS : out std_logic_vector (1 downto 0);
      RXCHARISK : out std_logic_vector (1 downto 0);
      RXRUNDISP : out std_logic_vector (1 downto 0);
      RXCOMMADET : out std_logic;
      RXDATA : out std_logic_vector (15 downto 0);
      RXDISPERR : out std_logic_vector (1 downto 0);
      RXLOSSOFSYNC : out std_logic_vector (1 downto 0);
      RXNOTINTABLE : out std_logic_vector (1 downto 0);
      RXREALIGN : out std_logic;
      RXCLK : out std_logic;
      TXRESET : in std_logic;
      TXN : out std_logic;
      TXP : out std_logic);
  end component;
  
  component BUFG
    port (O : out STD_ULOGIC;
	  I : in  STD_ULOGIC);
  end component;

  component fifo8x511 is
    port (
      din: IN std_logic_VECTOR(7 downto 0);
      wr_en: IN std_logic;
      wr_clk: IN std_logic;
      rd_en: IN std_logic;
      rd_clk: IN std_logic;
      ainit: IN std_logic;
      dout: OUT std_logic_VECTOR(7 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;

  component decoder8b10b is
    port (
      din : in std_logic_vector(9 downto 0);
      dout : out std_logic_vector(7 downto 0);
      kout : out std_logic;
      invalid : out std_logic;
      clk  : in std_logic);
  end component;
  
  -------------------------------------------------------------------
  --
  --  VIO core component declaration
  --
  -------------------------------------------------------------------
  component vio
    port
    (
      control     : in    std_logic_vector(35 downto 0);
      async_in    : in    std_logic_vector(31 downto 0);
      async_out   :   out std_logic_vector(7 downto 0)
    );
  end component;

  -------------------------------------------------------------------
  --
  --  ILA core component declaration
  --
  -------------------------------------------------------------------
  component ila
    port
    (
      control     : in    std_logic_vector(35 downto 0);
      clk         : in    std_logic;
      data        : in    std_logic_vector(31 downto 0);
      trig0       : in    std_logic_vector(7 downto 0)
    );
  end component;

begin

  vcc1 <= '1';
  gnd1 <= '0';
  gnd2 <= "00";
  gnd16 <= X"0000";
  
  i_mgt_tx: mgt_tx
    port map(
      BREFCLK => brefclk,
      BREFCLK2 => brefclk2,
      event_clk => refclk_dcm,
      REFCLKSEL => refclksel,      
      POWERDOWN => tx_mgt_pwd,
      RXN => RXN1,
      RXP => RXP1,
      TXCHARISK => tx_charisk,
      TXDATA => tx_data,
      TXRESET => tx_reset,
      TXPOLARITY => tx_polarity,
      TXN => EVTXN,
      TXP => EVTXP);

  i_mgt_rx: mgt_rx
    port map(
      BREFCLK => brefclk,
      BREFCLK2 => brefclk2,
      REFCLKSEL => refclksel,
      POWERDOWN => rx_mgt_pwd,
      RXN => EVRXN,
      RXP => EVRXP,
      RXRESET => rxreset,
      RXPOLARITY => rx_polarity,
      RXBUFSTATUS => rx_bufstatus,
      RXCHARISK => rx_charisk,
      RXRUNDISP => rx_rundisp,
      RXCOMMADET => rx_commadet,
      RXDATA => rx_data,
      RXDISPERR => rx_disperr,
      RXLOSSOFSYNC => rx_loss,
      RXNOTINTABLE => rx_notintable,
      RXREALIGN => rx_realign,
      RXCLK => rxclk,
      TXRESET => tx_reset,
      TXN => TXN2,
      TXP => TXP2);

  i_rx_fifo : fifo8x511
    port map (
      din => rx_fifo_data_in,
      wr_en => rx_fifo_wr,
      wr_clk => rxclk,
      rd_en => rx_fifo_rd,
      rd_clk => refclk_dcm,
      ainit => rx_fifo_reset,
      dout => event_code_out,
      full => rx_fifo_full,
      empty => rx_fifo_empty
      );

  event_clock_source: process (refclksel, brefclk_dcm, brefclk2_dcm)
  begin
    event_clk <= refclk_dcm;
    if refclksel = '0' then
      refclk_dcm <= brefclk_dcm;
    else
      refclk_dcm <= brefclk2_dcm;
    end if;
  end process;
  
  tx_mgt_pwd <= tx_powerdown;
  rxclk_out <= rxclk;
  
  transmit_data : process (refclk_dcm, tx_event, tx_dbus, tx_databuf,
			   k_databuf, databuf_mode)
    variable even : std_logic_vector(1 downto 0) := "00";
  begin
    if rising_edge(refclk_dcm) then
      tx_charisk <= "00";
      tx_data(15 downto 8) <= tx_event;
      if tx_event = X"00" and even = "00" then
	tx_charisk <= "10";
	tx_data(15 downto 8) <= X"BC"; -- K28.5 character
      end if;
      tx_data(7 downto 0) <= tx_dbus;
      if even(0) = '1' and databuf_mode = '1' then
	tx_data(7 downto 0) <= tx_databuf;
	tx_charisk(0) <= k_databuf;
      end if;
      ena_databuf <= even(0);
      even := even + 1;
    end if;
  end process;

  process (refclk_dcm, reset, rx_loss, rx_charisk, link_ok, rx_disperr,
           rx_pwrdown)
    variable prescaler : natural range 0 to MGT_RX_PRESCALER - 1;
    variable count : natural range 0 to MGT_RX_TIMEOUT - 1;
    variable k_counter : natural range 0 to MGT_RX_K_COUNTER - 1;
    variable rx_error : std_logic;
    variable loss_lock : std_logic;
    variable rx_error_count : std_logic_vector(5 downto 0);
  begin
    rx_link_ok <= link_ok;

    if rising_edge(refclk_dcm) then
      if prescaler = 0 then
	link_ok <= '0';
	if count = 0 then
	  link_ok <= '1';
	end if;
      
	rx_mgt_pwd <= '0';
	if count > MGT_RX_TIMEOUT - 1 - MGT_RX_PWRDWN_TIME then
	  rx_mgt_pwd <= '1';
	end if;

	if count /= 0 then
	  count := count - 1;
	end if;

      end if;

      if count < MGT_RX_LOCK_ACQ then
	if loss_lock = '1' then
	  count := MGT_RX_TIMEOUT - 1;
	end if;
      end if;

      if link_ok = '0' then
	loss_lock := rx_error;
      else
	loss_lock := rx_error_count(5);
      end if;

      if link_ok = '1' then
	if rx_error = '1' then
	  if rx_error_count(5) = '0' then
	    rx_error_count := rx_error_count - 1;
	  end if;
	else
	  if prescaler = 0 and
	    (rx_error_count(5) = '1' or rx_error_count(4) = '0') then
	    rx_error_count := rx_error_count + 1;
	  end if;
	end if;
      else
	rx_error_count := "011111";
      end if;

      rx_error := '0';
      if rx_notintable /= "00" or
	rx_disperr /= "00" or
	(rx_charisk(0) = '1' and rx_data(7) = '1') then
	rx_error := '1';
      end if;

      if prescaler = MGT_RX_PRESCALER - 1 then
	prescaler := 0;
      else
	prescaler := prescaler + 1;
      end if;
      if reset = '1' or rx_pwrdown = '1' then
	prescaler := 0;
	count := MGT_RX_TIMEOUT - 1;
	rx_mgt_pwd <= '1';
      end if;
    end if;
  end process;

  dbus_data : process (rxclk, link_ok, rx_data)
  begin
    if rising_edge(rxclk) then
      dbus_upstream <= rx_data(7 downto 0);
      if link_ok = '0' then
	dbus_upstream <= (others => '0');
      end if;
    end if;
  end process;
    
  fifo_writing : process (rxclk, rx_disable, link_ok,
			  rx_charisk, reset)
    variable event_code : std_logic_vector(7 downto 0);
  begin
    if rising_edge(rxclk) then
      rx_fifo_wr <= '0';
      if event_code /= X"00" then
	rx_fifo_wr <= '1';
	rx_fifo_data_in <= event_code;
      end if;

      event_code := rx_data(15 downto 8);
      if link_ok = '0' or rx_charisk(1) = '1' or
	reset = '1' or rx_disable = '1' then
	event_code := (others => '0');
      end if;
    end if;
  end process;

  fifo_reseting : process (rx_disable, reset, fifo_reset)
  begin
    rx_fifo_reset <= '0';
    if rx_disable = '1' or reset = '1' or fifo_reset = '1' then
      rx_fifo_reset <= '1';
    end if;
  end process;

  rx_event : process (refclk_dcm, event_ack, rx_fifo_empty,
		      rx_buffer_empty)
  begin
    if rising_edge(refclk_dcm) then
      event_pending <= '0';
      if (rx_buffer_empty = '0' and event_ack = '0')
	or rx_fifo_rd = '1' then
	event_pending <= '1';
      end if;
    end if;
  end process;

  fifo_reading : process (refclk_dcm, event_ack, rx_fifo_empty)
  begin
    rx_fifo_rd <= '0';
    if rx_fifo_empty = '0' then
      if rx_buffer_empty = '1' then
	rx_fifo_rd <= '1';
      else
	if event_ack = '1' then
	  rx_fifo_rd <= '1';
	end if;
      end if;
    end if;

    if rising_edge(refclk_dcm) then
      if rx_fifo_rd = '1' or rx_fifo_empty = '0' then
	rx_buffer_empty <= '0';
      end if;

      if (rx_fifo_empty = '1' and event_ack = '1')
	or fifo_reset = '1' or reset = '1' then
	rx_buffer_empty <= '1';
      end if;
    end if;
  end process;

  fifo_full_flag : process (refclk_dcm, rx_fifo_full, fifo_clear_full,
			    fifo_reset, reset)
  begin
    if rising_edge(refclk_dcm) then
      if fifo_clear_full = '1' or fifo_reset = '1' or reset = '1' then
	fifo_full <= '0';
      end if;
      if rx_fifo_full = '1' then
	fifo_full <= '1';
      end if;
    end if;
  end process;

  violation_detect : process (refclk_dcm, reset, rx_clear_viol, rx_disable,
			      rx_disperr, rx_loss, rx_notintable, link_ok)
  begin
    if reset = '1' or rx_clear_viol = '1' or rx_disable = '1' then
      rx_violation <= '0';
    elsif rising_edge(refclk_dcm) then
      if rx_disperr /= "00" or rx_loss /= "00" or rx_notintable /= "00" then
	rx_violation <= '1';
      end if;
    end if;
  end process;

  violation2_detect : process (refclk_dcm, reset, rx_clear_viol, rx_disable,
			      rx_disperr2, rx_loss2, rx_notintable2)
  begin
    if reset = '1' or rx_clear_viol = '1' or rx_disable = '1' then
      rx_violation2 <= '0';
    elsif rising_edge(refclk_dcm) then
      if rx_disperr2 /= "00" or rx_loss2 /= "00" or
	rx_notintable2 /= "00" then
	rx_violation2 <= '1';
      end if;
    end if;
  end process;
	
end structure;
