------------------------------------------------------------------------------
--
--  File        : plbv46_ubicom_master
--
--  Title       : Ubicom IP2022 PLBV46 Bus master interface
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity plbv46_ubicom_master is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_FAMILY                       : string               := "virtex5";
    C_MPLB_AWIDTH                  : integer              := 32;
    C_MPLB_DWIDTH                  : integer              := 128;
    C_MPLB_NATIVE_DWIDTH           : integer              := 32;
    C_MPLB_P2P                     : integer              := 0;
    C_MPLB_SMALLEST_SLAVE          : integer              := 32;
    C_MPLB_CLK_PERIOD_PS           : integer              := 10000
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    -- Ubicom interface signals
    ULE          : in std_logic;
    nUBERR       : out std_logic;
    nUACK        : out std_logic;
    nUIRQ        : out std_logic;
    nURD         : in std_logic;
    nUWR         : in std_logic;
    UA           : in std_logic_vector(16 downto 0);
                                  -- Ubicom address bits A16-A9 and A0,
                                  -- A8-A1 are multiplexed with D7-D0
    UD           : inout std_logic_vector(7 downto 0);
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    MPLB_Clk                       : in  std_logic;
    MPLB_Rst                       : in  std_logic;
    MD_error                       : out std_logic;
    M_request                      : out std_logic;
    M_priority                     : out std_logic_vector(0 to 1);
    M_busLock                      : out std_logic;
    M_RNW                          : out std_logic;
    M_BE                           : out std_logic_vector(0 to C_MPLB_DWIDTH/8-1);
    M_MSize                        : out std_logic_vector(0 to 1);
    M_size                         : out std_logic_vector(0 to 3);
    M_type                         : out std_logic_vector(0 to 2);
    M_TAttribute                   : out std_logic_vector(0 to 15);
    M_lockErr                      : out std_logic;
    M_abort                        : out std_logic;
    M_UABus                        : out std_logic_vector(0 to 31);
    M_ABus                         : out std_logic_vector(0 to 31);
    M_wrDBus                       : out std_logic_vector(0 to C_MPLB_DWIDTH-1);
    M_wrBurst                      : out std_logic;
    M_rdBurst                      : out std_logic;
    PLB_MAddrAck                   : in  std_logic;
    PLB_MSSize                     : in  std_logic_vector(0 to 1);
    PLB_MRearbitrate               : in  std_logic;
    PLB_MTimeout                   : in  std_logic;
    PLB_MBusy                      : in  std_logic;
    PLB_MRdErr                     : in  std_logic;
    PLB_MWrErr                     : in  std_logic;
    PLB_MIRQ                       : in  std_logic;
    PLB_MRdDBus                    : in  std_logic_vector(0 to (C_MPLB_DWIDTH-1));
    PLB_MRdWdAddr                  : in  std_logic_vector(0 to 3);
    PLB_MRdDAck                    : in  std_logic;
    PLB_MRdBTerm                   : in  std_logic;
    PLB_MWrDAck                    : in  std_logic;
    PLB_MWrBTerm                   : in  std_logic;
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
    control0    :   in std_logic_vector(35 downto 0)
  );

  attribute SIGIS : string;
  attribute SIGIS of MPLB_Clk      : signal is "CLK";
  attribute SIGIS of MPLB_Rst      : signal is "RST";

end entity plbv46_ubicom_master;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of plbv46_ubicom_master is

  constant UBICOM_REG_BITS : natural := 2;
  constant UBICOM_REG_A32  : std_logic_vector := "00";
  constant UBICOM_REG_AM   : std_logic_vector := "01";
  constant UBICOM_REG_DATA : std_logic_vector := "10";  

  signal ubiLE       : std_logic;
  signal nubiRD      : std_logic;
  signal nubiWR      : std_logic;
  signal nubiBERR    : std_logic;
  signal nubiACK     : std_logic;
  signal nubiIRQ     : std_logic;
  signal ubiA        : std_logic_vector(16 downto 0);
  signal ubiD        : std_logic_vector(7 downto 0);
  signal mst_request : std_logic;
  signal mst_RNW     : std_logic;
  
  -- mst_A holds the 32 bit OPB address for Ubicom OPB accesses
  signal mst_A  : std_logic_vector(0 to 31);
  -- mst_D holds the 16 bit OPB write data for Ubicom OPB accesses
  signal mst_D  : std_logic_vector(0 to 15);
  -- slv_D holds the 16 bit OPB read data for Ubicom OPB accesses
  signal slv_D  : std_logic_vector(0 to 15);

  signal write_trigger : std_logic;
  signal read_trigger  : std_logic;

  signal ila_trig      : std_logic_vector(127 downto 0);
  
  component ila128
    port
    (
      control     : in    std_logic_vector(35 downto 0);
      clk         : in    std_logic;
      trig0       : in    std_logic_vector(127 downto 0)
    );
  end component;
  
begin

--  ila : ila128
--    port map (
--      control => control0,
--      clk => MPLB_Clk,
--      trig0 => ila_trig);

  ila_trig(16 downto 0) <= ubiA;
  ila_trig(17) <= ubiLE;
  ila_trig(18) <= nubiRD;
  ila_trig(19) <= nubiWR;
  ila_trig(27 downto 20) <= ubiD;
  ila_trig(28) <= nubiBERR; 
  ila_trig(29) <= nubiACK; 
  ila_trig(30) <= mst_request;
  ila_trig(31) <= '0';
  ila_trig(32) <= PLB_MAddrAck;
  ila_trig(34 downto 33) <= PLB_MSSize;
  ila_trig(35) <= PLB_MRearbitrate;
  ila_trig(36) <= PLB_MTimeout;
  ila_trig(37) <= PLB_MBusy;
  ila_trig(38) <= PLB_MRdErr;
  ila_trig(39) <= PLB_MWrErr;
  ila_trig(40) <= PLB_MIRQ;
  ila_trig(41) <= PLB_MRdDAck;
  ila_trig(42) <= PLB_MRdBTerm;
  ila_trig(43) <= PLB_MWrDAck;
  ila_trig(44) <= PLB_MWrBTerm;
  ila_trig(45) <= read_trigger;
  ila_trig(46) <= write_trigger;
  ila_trig(47) <= mst_RNW;
  ila_trig(63 downto 48) <= slv_D;
  ila_trig(95 downto 64) <= mst_A; 
  ila_trig(127 downto 96) <= PLB_MRdDBus(96 to 127);
  
  nUBERR <= nubiBERR;
  nUACK <= nubiACK;
  nUIRQ <= nubiIRQ;
  M_request <= mst_request;
  M_RNW <= mst_RNW;
  
  nubiIRQ <= '1';
  
  sync_ubi_to_local_clock : process (MPLB_Clk, ULE, nURD, nUWR, UA, UD)
  begin
    -- Synchronise all incoming signals from Ubicom to OPB clock (in IOBs)
    if rising_edge(MPLB_Clk) then
      ubiLE <= ULE;
      nubiRD <= nURD;
      nubiWR <= nUWR;
      ubiA(16 downto 9) <= UA(16 downto 9);
      ubiA(0) <= UA(0);
      -- Latch multiplexed address bits
      if ubiLE = '1' then
	ubiA(8 downto 1) <= ubiD;
      end if;
      ubiD <= UD;
    end if;
  end process;

  master_address_generation : process (MPLB_Clk, nubiWR, ubiA, ubiD,
				       mst_A)
  begin
    M_UABus <= (others => '0');
    M_ABus(0 to 29) <= mst_A(0 to 29);
    M_ABus(30 to 31) <= "00";

    mst_A(16 to 30) <= ubiA(15 downto 1);
    mst_A(31) <= '0';
    if rising_edge(MPLB_Clk) then
      if ubiA(16) = '1' then
	if ubiA(UBICOM_REG_BITS downto 1) = UBICOM_REG_A32 then
	  if nubiWR = '0' then
	    if ubiA(0) = '0' then
	      mst_A(0 to 7) <= ubiD;
	    else
	      mst_A(8 to 15) <= ubiD;
	    end if;
	  end if;
	end if;
      end if;
    end if;
  end process;

  ubicom_data_bus : process (MPLB_Clk, nubiRD, ubiA, slv_D)
  begin
    if rising_edge(MPLB_Clk) then
      UD <= (others => 'Z');
      if nubiRD = '0' then
	if ubiA(0) = '0' then
	  UD <= slv_D(0 to 7);
	else
	  UD <= slv_D(8 to 15);
	end if;
      end if;
    end if;
  end process;

  -- Master Data Bus Size 32 bits
  M_MSize <= "00";
  -- Only transfer 2 bytes at a time
  M_size <= "0000";
  -- Memory transfer
  M_type <= "000";
  -- Attributes
  M_TAttribute <= (others => '0');
  -- This interface will only perform one operation on one request,
  -- bus locking is not required.
  M_busLock <= '0';
  -- Bursting not supported.
  M_wrBurst <= '0';
  M_rdBurst <= '0';
  M_priority <= "00";
  M_abort <= '0';
  
  PLB_access_triggering : process (MPLB_Clk, nubiWR, nubiRD, ubiA, ubiD)
    variable nubiWR_1 : std_logic;
    variable nubiRD_1 : std_logic;
  begin
    if rising_edge(MPLB_Clk) then
      write_trigger <= '0';
      read_trigger <= '0';
      if ubiA(16) = '0' then
	if nubiWR = '0' and nubiWR_1 = '1' then
	  if ubiA(0) = '0' then
	    mst_D(0 to 7) <= ubiD;
	  else
	    mst_D(8 to 15) <= ubiD;
	    -- Bits 7--0 are written last, so this triggers the actual write
	    write_trigger <= '1';
	  end if;
	end if;
	if nubiRD = '0' and nubiRD_1 = '1' and ubiA(0) = '0' then
	  read_trigger <= '1';
	end if;
      end if;
      nubiWR_1 := nubiWR;
      nubiRD_1 := nubiRD;
    end if;
  end process;

  PLB_requesting : process (MPLB_Clk, write_trigger, read_trigger, MPLB_Rst)
  begin
    if rising_edge(MPLB_Clk) then
      if write_trigger = '1' or read_trigger = '1' then
	mst_request <= '1';
        if mst_A(30) = '0' then
          M_BE <= "1100";
        else
          M_BE <= "0011";
        end if;
      end if;
      if read_trigger = '1' then
        mst_RNW <= '1';
      end if;
      if PLB_MRearbitrate = '1' or PLB_MTimeout = '1' or
        PLB_MRdErr = '1' or PLB_MWrErr = '1' or PLB_MAddrAck = '1' or
        MPLB_Rst = '1' then
	mst_request <= '0';
        mst_RNW <= '0';
        M_BE <= (others => '0');
      end if;
    end if;
  end process;

  PLB_data_transfer : process (MPLB_Clk, 
			       write_trigger, read_trigger, MPLB_Rst)
    variable write_pend   : std_logic := '0';
    variable read_pend    : std_logic := '0';
    variable transfer_act : std_logic := '0';
  begin
    if rising_edge(MPLB_Clk) then
      if write_trigger = '1' or read_trigger = '1' then
	nubiACK <= '1';
	nubiBERR <= '1';
      end if;
      if write_trigger = '1' then
	write_pend := '1';
      end if;
      if read_trigger = '1' then
	read_pend := '1';
      end if;
      -- Data transfer starts with request
      if write_pend = '1' or read_pend = '1' then
	transfer_act := '1';
      end if;

      if PLB_MRdDack = '1' then
        if mst_A(30) = '0' then
          slv_D <= PLB_MRdDBus(96 to 111);
        else
          slv_D <= PLB_MRdDBus(112 to 127);
        end if;
      end if;

      if transfer_act = '1' then
        if PLB_MRdDack = '1' or PLB_MWrDack = '1' then
          nubiACK <= '0';
          write_pend := '0';
          read_pend := '0';
        end if;

	if PLB_MRearbitrate = '1' or PLB_MTimeout = '1' or
          PLB_MRdErr = '1' or PLB_MWrErr = '1' then
	  nubiBERR <= '0';
	  write_pend := '0';
	  read_pend := '0';
	end if;
      end if;
      
      M_wrDBus <= (others => '0');
      if write_pend = '1' then
	  M_wrDBus(112 to 127) <= mst_D;
	  M_wrDBus(96 to 111) <= mst_D;
      end if;

      if MPLB_Rst = '1' then
	nubiACK <= '1';
	nubiBERR <= '1';
	write_pend := '0';
	read_pend := '0';
      end if;
    end if;
  end process;

end IMP;
