---------------------------------------------------------------------------
--
--  File        : adi.vhd
--
--  Title       : ADN2812ACPZ serial configuration
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description:
--
-- Disable the harmonic detector in the ADN2812ACPZ
--
-- Write 0x80 into CTRLC register (address 0x11). This enables the
-- test register.
-- Write 0x40 to address 0x0a. This disables the harmonic detector.
--
--   SCK SDA
-- 3   1   1
-- 1   1   0  Start bit (SDA falling while SCK is high)
-- 0   0   0
-- 2   0   1  A6 (not data may change only while SCK is low)
-- 3   1   1
-- 2   0   1
-- 0   0   0  A5 (SADDR5 tied to GND)
-- 1   1   0
-- 0   0   0  A4
-- 1   1   0
-- 0   0   0  A3
-- 1   1   0
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0  A1
-- 1   1   0
-- 0   0   0  A0
-- 1   1   0
-- 0   0   0  WR (Write operation)
-- 1   1   0
-- 0   0   0
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 0   0   0  Sub address A7 (MSB of 0x11 CTRLC register)
-- 1   1   0
-- 0   0   0  A6
-- 1   1   0
-- 0   0   0  A5
-- 1   1   0
-- 0   0   0
-- 2   0   1  A4
-- 3   1   1
-- 2   0   1
-- 0   0   0  A3
-- 1   1   0
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0  A1
-- 1   1   0
-- 0   0   0
-- 2   0   1  A0
-- 3   1   1
-- 2   0   1
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 2   0   1  Data to test register (MSB of 0x80)
-- 3   1   1
-- 2   0   1
-- 0   0   0  A6
-- 1   1   0
-- 0   0   0  A5
-- 1   1   0
-- 0   0   0  A4
-- 1   1   0
-- 0   0   0  A3
-- 1   1   0
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0  A1
-- 1   1   0
-- 0   0   0  A0
-- 1   1   0
-- 0   0   0 
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 0   0   0
-- 1   1   0
-- 3   1   1  Stop bit

-- 3   1   1
-- 1   1   0  Start bit (SDA falling while SCK is high)
-- 0   0   0
-- 2   0   1  A6 (not data may change only while SCK is low)
-- 3   1   1
-- 2   0   1
-- 0   0   0  A5 (SADDR5 tied to GND)
-- 1   1   0
-- 0   0   0  A4
-- 1   1   0
-- 0   0   0  A3
-- 1   1   0
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0  A1
-- 1   1   0
-- 0   0   0  A0
-- 1   1   0
-- 0   0   0  WR (Write operation)
-- 1   1   0
-- 0   0   0
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 0   0   0  Sub address A7 (MSB of 0x0a)
-- 1   1   0
-- 0   0   0  A6
-- 1   1   0
-- 0   0   0  A5
-- 1   1   0
-- 0   0   0  A4
-- 1   1   0
-- 0   0   0
-- 2   0   1  A3
-- 3   1   1
-- 2   0   1
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0
-- 2   0   1  A1
-- 3   1   1
-- 2   0   1
-- 0   0   0  A0
-- 1   1   0
-- 0   0   0
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 0   0   0  Data to test register (MSB of 0x40)
-- 1   1   0
-- 0   0   0
-- 2   0   1  A6
-- 3   1   1
-- 2   0   1
-- 0   0   0  A5
-- 1   1   0
-- 0   0   0  A4
-- 1   1   0
-- 0   0   0  A3
-- 1   1   0
-- 0   0   0  A2
-- 1   1   0
-- 0   0   0  A1
-- 1   1   0
-- 0   0   0  A0
-- 1   1   0
-- 0   0   0 
-- 4   0   Z
-- 5   1   I  Slave acknowledge
-- 4   0   Z
-- 0   0   0
-- 1   1   0
-- 3   1   1  Stop bit

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;

entity adi_config is
  port (
    clk      : in std_logic;
    usec_ena : in std_logic;
    sck      : out std_logic;
    sda      : inout std_logic;
    status   : out std_logic
    );
end adi_config;

architecture rtl of adi_config is

  signal vcc         : std_logic;
  signal gnd         : std_logic;
  signal gnd_bus     : std_logic_vector(15 downto 0);
  
  signal rom_addr : std_logic_vector(10 downto 0) := "00000000000";
  signal rom_data : std_logic_vector(7 downto 0);

    component RAMB16_S9_S9
    generic (
      INIT_00 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_01 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_02 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_03 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_04 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_05 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_06 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_07 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_08 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_09 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_0B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_10 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_11 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_12 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_13 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_14 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_15 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_16 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_17 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_18 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_19 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_1B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_20 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_21 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_22 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_23 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_24 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_25 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_26 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_27 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_28 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_29 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2A : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_30 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_31 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_32 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_33 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_34 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_35 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_36 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_37 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_38 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_39 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3A : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_A : bit_vector  := X"000";
      INIT_B : bit_vector  := X"000";
      INITP_00 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_01 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_02 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_03 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_04 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_05 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_06 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_07 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      SRVAL_A : bit_vector  := X"000";
      SRVAL_B : bit_vector  := X"000";
      WRITE_MODE_A : string := "WRITE_FIRST";
      WRITE_MODE_B : string := "WRITE_FIRST"
      );

    port (
      DOA   : out std_logic_vector(7 downto 0);
      DOB   : out std_logic_vector(7 downto 0);
      DOPA  : out std_logic_vector(0 downto 0);
      DOPB  : out std_logic_vector(0 downto 0);
      ADDRA : in  std_logic_vector(10 downto 0);
      ADDRB : in  std_logic_vector(10 downto 0);
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      DIA   : in  std_logic_vector(7 downto 0);
      DIB   : in  std_logic_vector(7 downto 0);
      DIPA  : in  std_logic_vector(0 downto 0);
      DIPB  : in  std_logic_vector(0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic);
  end component;      

begin

  i_rom: RAMB16_S9_S9
    generic map (
      -- bit 0 : SCK
      -- bit 1 : SDA tri-state
      -- bit 2 : SDA state
      -- bit 3 : sample SDA
      INIT_00 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_01 =>
      X"0302000100010001000405040001000100010001000100010001000203020001",
      INIT_02 =>
      X"0001000100010001000100010001000203020405040203020001000100010002",
      INIT_03 =>
      X"0303030303030303030303030303030301000405040203020001030100040504",
      INIT_04 =>
      X"0001000100010001000405040001000100010001000100010001000203020001",
      INIT_05 =>
      X"0001000100010001000100020302000100040504000100020302000100020302",
      INIT_06 =>
      X"0303030303030303030303030303030303030303030303030301000405040001",
      INIT_07 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_08 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_09 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0A =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0B =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0C =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0D =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0E =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_0F =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_10 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_11 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_12 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_13 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_14 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_15 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_16 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_17 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_18 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_19 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1A =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1B =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1C =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1D =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1E =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_1F =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_20 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_21 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_22 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_23 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_24 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_25 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_26 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_27 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_28 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_29 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2A =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2B =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2C =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2D =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2E =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_2F =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_30 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_31 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_32 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_33 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_34 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_35 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_36 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_37 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_38 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_39 =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3A =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3B =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3C =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3D =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3E =>
      X"0303030303030303030303030303030303030303030303030303030303030303",
      INIT_3F =>
      X"0303030303030303030303030303030303030303030303030303030303030303")
    port map(
      DOA => rom_data,
      ADDRA => rom_addr,
      ADDRB => gnd_bus(10 downto 0),
      CLKA => clk,
      CLKB => gnd,
      DIA => gnd_bus(7 downto 0),
      DIB => gnd_bus(7 downto 0),
      DIPA => gnd_bus(0 downto 0),
      DIPB => gnd_bus(0 downto 0),
      ENA => vcc,
      ENB => gnd,
      SSRA => gnd,
      SSRB => gnd,
      WEA => gnd,
      WEB => gnd);

  gnd <= '0';
  gnd_bus <= (others => '0');
  vcc <= '1';

  playback: process (clk, usec_ena, rom_data)
    variable counter : std_logic_vector(8 downto 0) := "000000000";
  begin
    sda <= 'Z';
    if rom_data(2) = '0' then
      sda <= rom_data(1);
    end if;
    sck <= rom_data(0);
    
    if rising_edge(clk) then
      if usec_ena = '1' then
        if counter(counter'high) = '0' then
          counter := counter + 1;
        end if;
        rom_addr(counter'high downto 0) <= counter;
      end if;
    end if;
  end process;

end rtl;
