---------------------------------------------------------------------------
--
--  File        : cpcievr220.vhd
--
--  Title       : PMC-EVR Top Level
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_bs_pkg.all;
use work.evr_pkg.all;
 
entity cpcievr220 is
  port (
    -- Transceiver transmitter disable signal
    TXDISB : out std_logic;

    -- Crosspoint switch select signals
    -- '0' selects transceiver input
    -- '1' selects passes signal from MGT output further
    TXSEL  : out std_logic;
    RXSEL  : out std_logic;
    
    -- BREFCLKx is sourced by RF input (via AD9515BCPZ, EVG only)
    BREFCLKP : in std_logic;
    BREFCLKN : in std_logic;

    -- BREFCLK2x is sourced by Micrel SY87739L
    BREFCLK2P : in std_logic;
    BREFCLK2N : in std_logic;

    -- Local Bus Signals
    LCLK      : in std_logic;
    nREADY    : out std_logic;
    nLBE      : in std_logic_vector(3 downto 0);
    nBTERM    : out std_logic;
    LINTi     : out std_logic_vector(2 downto 1);
    LWnR      : in std_logic;
    nADS      : in std_logic;
    LAD       : inout std_logic_vector(31 downto 0);
    
    -- I/O connector pins

    TTLIN      : in std_logic_vector(0 to 1);
    UNIVOUT    : out std_logic_vector(0 to 9);
    UNIVIN     : in std_logic_vector(0 to 9);

    GPIO       : inout std_logic_vector(7 downto 0);

    -- PXI pins
    PXI_CLK10       : in std_logic;
    PXI_CLK10_IN_I  : in std_logic;
    PXI_CLK10_IN_O  : out std_logic;
    PXI_CLK10_DIR   : out std_logic; -- 0 - PXI CLK10_IN is output
                                     -- 1 - PXI CLK10_IN is input
    PXI_TRIG_DIR    : out std_logic_vector(7 downto 0);
                      -- PXI_TRIGx: 0 - output, 1 - input
    PXI_TRIG        : out std_logic_vector(7 downto 0); -- inout
    PXI_STAR        : out std_logic_vector(6 downto 0); -- inout
    nQSOE           : out std_logic; -- 0 - enable QuickSwitch
                                     -- 1 - disable QuickSwitch

    GA              : in std_logic_vector(4 downto 0);

    -- LEDS
    -- 1 - LNK (red)
    -- 2 - LNK (green)
    -- 3 - EVT (red)
    -- 4 - EVT (green)
    nLED       : out std_logic_vector(4 downto 1);
    
    -- Micrel SY87739L programming signals
    CGLOCK    : in std_logic;
    CGSK      : out std_logic;
    CGDI      : out std_logic;
    CGCS      : out std_logic;

    MOD_DEF0     : in std_logic;    -- Grounded on SFP module
    MOD_DEF1     : out std_logic;   -- SFP module clock
    MOD_DEF2     : inout std_logic; -- SPF module data
    
    TBON      : out std_logic_vector(2 downto 1);

    RXN       : in std_logic_vector(4 downto 1);
    RXP       : in std_logic_vector(4 downto 1);
    TXN       : out std_logic_vector(4 downto 1);
    TXP       : out std_logic_vector(4 downto 1)
    );

end cpcievr220;

architecture structure of cpcievr220 is

  signal OPB_Clk         : std_logic;

  signal reset           : std_logic;
  
  signal gnd             : std_logic := '0';
  signal vcc             : std_logic := '1';

  signal pci_OPB_Clk : std_logic;
  signal pci_OPB_Rst : std_logic;
  signal pci_M_DBus : std_logic_vector(0 to 31);
  signal pci_OPB_DBus : std_logic_vector(0 to 31);
  signal pci_M_ABus : std_logic_vector(0 to 31);
  signal pci_M_BE : std_logic_vector(0 to 3);
  signal pci_M_busLock : std_logic;
  signal pci_M_request : std_logic;
  signal pci_M_RNW : std_logic;
  signal pci_M_select : std_logic;
  signal pci_M_seqAddr : std_logic;
  signal pci_OPB_errAck : std_logic;
  signal pci_OPB_MGrant : std_logic;
  signal pci_OPB_retry : std_logic;
  signal pci_OPB_timeout : std_logic;
  signal pci_OPB_xferAck : std_logic;

  signal evr_OPB_Clk : std_logic;
  signal evr_OPB_Rst : std_logic;
  signal evr_Sl_DBus : std_logic_vector(0 to 31);
  signal evr_Sl_errAck : std_logic;
  signal evr_Sl_retry : std_logic;
  signal evr_Sl_toutSup : std_logic;
  signal evr_Sl_xferAck : std_logic;
  signal evr_OPB_ABus : std_logic_vector(0 to 31);
  signal evr_OPB_BE : std_logic_vector(0 to 3);
  signal evr_OPB_DBus : std_logic_vector(0 to 31);
  signal evr_OPB_RNW : std_logic;
  signal evr_OPB_select : std_logic;
  signal evr_OPB_seqAddr : std_logic;
  signal evr_OPB_debug : std_logic_vector(39 downto 0);

  signal CMLRXN : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal CMLRXP : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

  signal LED_rx_link : std_logic;
  signal LED_event_in : std_logic;
  signal LED_event_out : std_logic;
  signal LED_rx_fail : std_logic;

  signal TXLOOPBACK : std_logic;
  signal RXLOOPBACK : std_logic;

  component bufgdll is
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component system is
    port (
      sys_clk : in std_logic;
      sys_rst : in std_logic;
      opb_m_0_OPB_Clk : out std_logic;
      opb_m_0_OPB_Rst : out std_logic;
      opb_m_0_M_DBus : in std_logic_vector(0 to 31);
      opb_m_0_OPB_DBus : out std_logic_vector(0 to 31);
      opb_m_0_M_ABus : in std_logic_vector(0 to 31);
      opb_m_0_M_BE : in std_logic_vector(0 to 3);
      opb_m_0_M_busLock : in std_logic;
      opb_m_0_M_request : in std_logic;
      opb_m_0_M_RNW : in std_logic;
      opb_m_0_M_select : in std_logic;
      opb_m_0_M_seqAddr : in std_logic;
      opb_m_0_OPB_errAck : out std_logic;
      opb_m_0_OPB_MGrant : out std_logic;
      opb_m_0_OPB_retry : out std_logic;
      opb_m_0_OPB_timeout : out std_logic;
      opb_m_0_OPB_xferAck : out std_logic;
      opb_s_1_OPB_Clk : out std_logic;
      opb_s_1_OPB_Rst : out std_logic;
      opb_s_1_Sl_DBus : in std_logic_vector(0 to 31);
      opb_s_1_Sl_errAck : in std_logic;
      opb_s_1_Sl_retry : in std_logic;
      opb_s_1_Sl_toutSup : in std_logic;
      opb_s_1_Sl_xferAck : in std_logic;
      opb_s_1_OPB_ABus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_BE : out std_logic_vector(0 to 3);
      opb_s_1_OPB_DBus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_RNW : out std_logic;
      opb_s_1_OPB_select : out std_logic;
      opb_s_1_OPB_seqAddr : out std_logic);
--      chipscope_opb_iba_0_iba_trig_in_pin : in std_logic_vector(39 downto 0));
  end component;

  component opb_pci9030 is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_OPB_AWIDTH    : integer               := 32;
      C_OPB_DWIDTH    : integer               := 32;
      C_FAMILY        : string                := "virtex2p"
      );
    port (
      -- PLX PCI9030 Local Bus Signals
      nREADY    : out std_logic;
      nLBE      : in std_logic_vector(3 downto 0);
      nBTERM    : out std_logic;
      LWnR      : in std_logic;
      nADS      : in std_logic;
      LAD       : inout std_logic_vector(31 downto 0);
      
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      M_DBus          : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
      M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
      M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      M_busLock       : out   std_logic;
      M_request       : out   std_logic;
      M_RNW           : out   std_logic;
      M_select        : out   std_logic;
      M_seqAddr       : out   std_logic;
      OPB_errAck      : in    std_logic;
      OPB_MGrant      : in    std_logic;
      OPB_retry       : in    std_logic;
      OPB_timeout     : in    std_logic;
      OPB_xferAck     : in    std_logic;
      debug           : out   std_logic_vector(39 downto 0));
  end component;
  
  component opb_evr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
      UNIVIN          : in std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

      GPIO            : inout std_logic_vector(C_EVR_GPIOS-1 downto 0);

      TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic;
      BREFCLKP        : in std_logic;
      BREFCLK2N       : in std_logic;  -- Reference clock input from 
      BREFCLK2P       : in std_logic;  -- fractional synthesizer

      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;
      
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXN
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXN
      CMLRXN          : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
      CMLRXP          : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
      CMLTXN          : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
      CMLTXP          : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic;

      -- SFP transceiver serial interface signals
      sfp_moddef0 : in std_logic;
      sfp_sda     : inout std_logic;
      sfp_scl     : out std_logic;

      TXLOOPBACK  : out std_logic; -- when '1' loop back FPGA TX to FPGA RX
      RXLOOPBACK  : out std_logic; -- when '1' loop back optical RX signal
                                   -- to transceiver TX port
 
      debug       : out std_logic_vector(39 downto 0)
      );
  end component;

begin
  
  i_bufgdll : bufgdll
    port map (
      I => lclk,
      O => OPB_Clk);

  i_opb : system
    port map (
      sys_clk => OPB_Clk,
      sys_rst => reset,
      opb_m_0_OPB_Clk => pci_OPB_Clk,
      opb_m_0_OPB_Rst => pci_OPB_Rst,
      opb_m_0_M_DBus => pci_M_DBus,
      opb_m_0_OPB_DBus => pci_OPB_DBus,
      opb_m_0_M_ABus => pci_M_ABus,
      opb_m_0_M_BE => pci_M_BE,
      opb_m_0_M_busLock => pci_M_busLock,
      opb_m_0_M_request => pci_M_request,
      opb_m_0_M_RNW => pci_M_RNW,
      opb_m_0_M_select => pci_M_select,
      opb_m_0_M_seqAddr => pci_M_seqAddr,
      opb_m_0_OPB_errAck => pci_OPB_errAck,
      opb_m_0_OPB_MGrant => pci_OPB_MGrant,
      opb_m_0_OPB_retry => pci_OPB_retry,
      opb_m_0_OPB_timeout => pci_OPB_timeout,
      opb_m_0_OPB_xferAck => pci_OPB_xferAck,
      opb_s_1_OPB_Clk => evr_OPB_Clk,
      opb_s_1_OPB_Rst => evr_OPB_Rst,
      opb_s_1_Sl_DBus => evr_Sl_DBus,
      opb_s_1_Sl_errAck => evr_Sl_errAck,
      opb_s_1_Sl_retry => evr_Sl_retry,
      opb_s_1_Sl_toutSup => evr_Sl_toutSup,
      opb_s_1_Sl_xferAck => evr_Sl_xferAck,
      opb_s_1_OPB_ABus => evr_OPB_ABus,
      opb_s_1_OPB_BE => evr_OPB_BE,
      opb_s_1_OPB_DBus => evr_OPB_DBus,
      opb_s_1_OPB_RNW => evr_OPB_RNW,
      opb_s_1_OPB_select => evr_OPB_select,
      opb_s_1_OPB_seqAddr => evr_OPB_seqAddr);
--      chipscope_opb_iba_0_iba_trig_in_pin => evr_OPB_debug);
  
  i_opb_pci : opb_pci9030
    port map (
      nREADY => nREADY,
      nLBE => nLBE,
      nBTERM => nBTERM,
      LWnR => LWnR,
      nADS => nADS,
      LAD => LAD,
      OPB_Clk => OPB_Clk,
      OPB_Rst => pci_OPB_Rst,
      M_DBus => pci_M_DBus,
      OPB_DBus => pci_OPB_DBus,
      M_ABus => pci_M_ABus,
      M_BE => pci_M_BE,
      M_busLock => pci_M_busLock,
      M_request => pci_M_request,
      M_RNW => pci_M_RNW,
      M_select => pci_M_select,
      M_seqAddr => pci_M_seqAddr,
      OPB_errAck => pci_OPB_errAck,
      OPB_MGrant => pci_OPB_MGrant,
      OPB_retry => pci_OPB_retry,
      OPB_timeout => pci_OPB_timeout,
      OPB_xferAck => pci_OPB_xferAck,
      debug => evr_OPB_debug);
  
  i_opb_evr : opb_evr
    generic map (
      C_BASEADDR => EVR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => OPB_Clk,
      OPB_Rst => evr_OPB_Rst,
      Sl_DBus => evr_Sl_DBus,
      Sl_errAck => evr_Sl_errAck,
      Sl_retry => evr_Sl_retry,
      Sl_toutSup => evr_Sl_toutSup,
      Sl_xferAck => evr_Sl_xferAck,
      OPB_ABus => evr_OPB_ABus,
      OPB_BE => evr_OPB_BE,
      OPB_DBus => evr_OPB_DBus,
      OPB_RNW => evr_OPB_RNW,
      OPB_select => evr_OPB_select,
      OPB_seqAddr => evr_OPB_seqAddr,

      ip_irq_out => LINTi(1),
    
      TTLIN => TTLIN,
      UNIVIN => UNIVIN,
      UNIVOUT => UNIVOUT,
      GPIO => GPIO,
      
      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      BREFCLK2N => BREFCLK2N,
      BREFCLK2P => BREFCLK2P,

      LED_rx_link => LED_rx_link,
      LED_event_in => LED_event_in,
      LED_event_out => LED_event_out,
      LED_rx_fail => LED_rx_fail,

      EVRXN => RXN(2),
      EVRXP => RXP(2),
      EVTXN => TXN(2),
      EVTXP => TXP(2),

      CMLRXN => CMLRXN,
      CMLRXP => CMLRXP,
      
      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS,
      CGLOCK => CGLOCK,

      sfp_moddef0 => MOD_DEF0,
      sfp_sda => MOD_DEF2,
      sfp_scl => MOD_DEF1,
      
      TXLOOPBACK => TXLOOPBACK,
      RXLOOPBACK => RXLOOPBACK,

      debug => open);

  LINTi(2) <= '0';
  
  -- Always enable transmitter
  TXDISB <= '0';

  -- Do not power on TB
  TBON <= "00";

  crosspoint_switch: process
  begin
    if C_EVR_ENABLE_BACKWARD_CHANNEL = TRUE then
      -- Get gigabit signal from transmitter (if not in loopback mode)
      TXSEL <= not RXLOOPBACK;
    else
      -- Get gigabit signal from transceiver RX input
      TXSEL <= '0';
    end if;
    RXSEL <= TXLOOPBACK;
  end process;

  leds: process (LED_rx_link, LED_event_in, LED_event_out, LED_rx_fail)
  begin
    nLED(2) <= '1'; -- green link led
    if LED_rx_fail = '1' and LED_rx_link = '0' then
      nLED(2) <= '0';
    end if;
    nLED(1) <= '1'; -- red link led
    if LED_rx_fail = '0' then
      nLED(1) <= '0';
    end if;
    nLED(4) <= '1'; -- green evt led
    if LED_rx_link = '0' and LED_event_in = '1' then
      nLED(4) <= '0';
    end if;
    nLED(3) <= '1'; -- red evt led
    if LED_event_out = '0' then
      nLED(3) <= '0';
    end if;
  end process;
  
  reset_generation : process (OPB_Clk)
    variable reset_cnt : std_logic_vector(1 downto 0) := "01";
  begin
    if rising_edge(OPB_Clk) then
      reset <= '1';
      if reset_cnt(1) = '1' then
	reset <= '0';
      end if;
      
      if reset_cnt(1) = '0' then
	reset_cnt := reset_cnt - 1;
      end if;
    end if;
  end process;
  
  PXI_CLK10_IN_O <= '0';
  PXI_CLK10_DIR <= '1';
  PXI_TRIG_DIR <= (others => '1');
  nQSOE <= '1';
  PXI_TRIG <= (others => 'Z');
  PXI_STAR <= (others => 'Z');

end structure;
