---------------------------------------------------------------------------
--
--  File        : cpcievrtg_cml_gtx.vhd
--
--  Title       : Virtex 5 FX - GTX Transceiver CML output block
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.evr_pkg.all;
use work.evr_bs_pkg.all;
library unisim;
use unisim.vcomponents.all;

entity cml_gtx is
  port (
    REFCLK_out   : out std_logic;
    gtxreset     : in std_logic;
    tx_data      : in gtx_data(C_EVR_CML_OUTPUTS-1 downto 0);
    tx_clk_out   : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
    txinhibit    : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
    plllkdet_out : out std_logic;
    
    rx_data      : out gtx_data(C_EVR_CML_INPUTS-1 downto 0);
    rx_clk_out   : out std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);
    rxreset      : in std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);
    
    -- External pins
    RXREFCLKN    : in std_logic;
    RXREFCLKP    : in std_logic;
    
    RDN  : in std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
    RDP  : in std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
    TDN  : out std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
    TDP  : out std_logic_vector(C_EVR_CML_OUTPUTS downto 1));
end entity;

architecture struct of cml_gtx is

  signal gnd : std_logic;
  signal vcc : std_logic;
  signal gnd_vec : std_logic_vector(31 downto 0);
  signal vcc_vec : std_logic_vector(31 downto 0);
  
  signal RXREFCLK  : std_logic; 
  signal REFCLKOUT : std_logic; -- Reference clock (EVR event clock) to FPGA fabric

  signal rxrecclk  : std_logic_vector(1 downto 0);
  signal plllkdet  : std_logic_vector(3 downto 0);
  signal txoutclk  : std_logic_vector(7 downto 0);
  
  component GTX 
    generic
      (
	-- Simulation attributes
	WRAPPER_SIM_GTXRESET_SPEEDUP    : integer   := 0; -- Set to 1 to speed up sim reset
	WRAPPER_SIM_PLL_PERDIV2         : bit_vector:= x"0c8" -- Set to the VCO Unit Interval time
	);
    port
      (
	
	--_________________________________________________________________________
	--_________________________________________________________________________
	--TILE0  (Location)
	
	------------------------ Loopback and Powerdown Ports ----------------------
	TILE0_LOOPBACK0_IN                      : in   std_logic_vector(2 downto 0);
	TILE0_LOOPBACK1_IN                      : in   std_logic_vector(2 downto 0);
	------------------- Receive Ports - RX Data Path interface -----------------
	TILE0_RXDATA0_OUT                       : out  std_logic_vector(39 downto 0);
	TILE0_RXDATA1_OUT                       : out  std_logic_vector(39 downto 0);
	TILE0_RXRECCLK0_OUT                     : out  std_logic;
	TILE0_RXRECCLK1_OUT                     : out  std_logic;
	TILE0_RXRESET0_IN                       : in   std_logic;
	TILE0_RXRESET1_IN                       : in   std_logic;
	TILE0_RXUSRCLK0_IN                      : in   std_logic;
	TILE0_RXUSRCLK1_IN                      : in   std_logic;
	TILE0_RXUSRCLK20_IN                     : in   std_logic;
	TILE0_RXUSRCLK21_IN                     : in   std_logic;
	------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
	TILE0_RXEQMIX0_IN                       : in   std_logic_vector(1 downto 0);
	TILE0_RXEQMIX1_IN                       : in   std_logic_vector(1 downto 0);
	TILE0_RXN0_IN                           : in   std_logic;
	TILE0_RXN1_IN                           : in   std_logic;
	TILE0_RXP0_IN                           : in   std_logic;
	TILE0_RXP1_IN                           : in   std_logic;
	-------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
	TILE0_RXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE0_RXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE0_RXPMASETPHASE0_IN	    : in  std_logic;
	TILE0_RXPMASETPHASE1_IN	    : in  std_logic;
	----------------- Receive Ports - RX Polarity Control Ports ----------------
	TILE0_RXPOLARITY0_IN	    : in  std_logic;
	TILE0_RXPOLARITY1_IN	    : in  std_logic;
	--------------------- Shared Ports - Tile and PLL Ports --------------------
	TILE0_CLKIN_IN		    : in  std_logic;
	TILE0_GTXRESET_IN	    : in  std_logic;
	TILE0_PLLLKDET_OUT	    : out std_logic;
	TILE0_REFCLKOUT_OUT	    : out std_logic;
	TILE0_RESETDONE0_OUT	    : out std_logic;
	TILE0_RESETDONE1_OUT	    : out std_logic;
	------------------ Transmit Ports - TX Data Path interface -----------------
	TILE0_TXDATA0_IN	    : in  std_logic_vector(39 downto 0);
	TILE0_TXDATA1_IN	    : in  std_logic_vector(39 downto 0);
	TILE0_TXOUTCLK0_OUT	    : out std_logic;
	TILE0_TXOUTCLK1_OUT	    : out std_logic;
	TILE0_TXUSRCLK0_IN	    : in  std_logic;
	TILE0_TXUSRCLK1_IN	    : in  std_logic;
	TILE0_TXUSRCLK20_IN	    : in  std_logic;
	TILE0_TXUSRCLK21_IN	    : in  std_logic;
	--------------- Transmit Ports - TX Driver and OOB signalling --------------
	TILE0_TXDIFFCTRL0_IN	    : in  std_logic_vector(2 downto 0);
	TILE0_TXDIFFCTRL1_IN	    : in  std_logic_vector(2 downto 0);
	TILE0_TXINHIBIT0_IN	    : in  std_logic;
	TILE0_TXINHIBIT1_IN	    : in  std_logic;
	TILE0_TXN0_OUT		    : out std_logic;
	TILE0_TXN1_OUT		    : out std_logic;
	TILE0_TXP0_OUT		    : out std_logic;
	TILE0_TXP1_OUT		    : out std_logic;
	TILE0_TXPREEMPHASIS0_IN	    : in  std_logic_vector(2 downto 0);
	TILE0_TXPREEMPHASIS1_IN	    : in  std_logic_vector(2 downto 0);
	-------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
	TILE0_TXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE0_TXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE0_TXPMASETPHASE0_IN	    : in  std_logic;
	TILE0_TXPMASETPHASE1_IN	    : in  std_logic;
	-------------------- Transmit Ports - TX Polarity Control ------------------
	TILE0_TXPOLARITY0_IN	    : in  std_logic;
	TILE0_TXPOLARITY1_IN	    : in  std_logic;



	--_________________________________________________________________________
	--_________________________________________________________________________
	--TILE1	 (Location)

	------------------------ Loopback and Powerdown Ports ----------------------
	TILE1_LOOPBACK0_IN	    : in  std_logic_vector(2 downto 0);
	TILE1_LOOPBACK1_IN	    : in  std_logic_vector(2 downto 0);
	------------------- Receive Ports - RX Data Path interface -----------------
	TILE1_RXDATA0_OUT	    : out std_logic_vector(39 downto 0);
	TILE1_RXDATA1_OUT	    : out std_logic_vector(39 downto 0);
	TILE1_RXRECCLK0_OUT	    : out std_logic;
	TILE1_RXRECCLK1_OUT	    : out std_logic;
	TILE1_RXRESET0_IN	    : in  std_logic;
	TILE1_RXRESET1_IN	    : in  std_logic;
	TILE1_RXUSRCLK0_IN	    : in  std_logic;
	TILE1_RXUSRCLK1_IN	    : in  std_logic;
	TILE1_RXUSRCLK20_IN	    : in  std_logic;
	TILE1_RXUSRCLK21_IN	    : in  std_logic;
	------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
	TILE1_RXEQMIX0_IN	    : in  std_logic_vector(1 downto 0);
	TILE1_RXEQMIX1_IN	    : in  std_logic_vector(1 downto 0);
	TILE1_RXN0_IN		    : in  std_logic;
	TILE1_RXN1_IN		    : in  std_logic;
	TILE1_RXP0_IN		    : in  std_logic;
	TILE1_RXP1_IN		    : in  std_logic;
	-------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
	TILE1_RXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE1_RXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE1_RXPMASETPHASE0_IN	    : in  std_logic;
	TILE1_RXPMASETPHASE1_IN	    : in  std_logic;
	----------------- Receive Ports - RX Polarity Control Ports ----------------
	TILE1_RXPOLARITY0_IN	    : in  std_logic;
	TILE1_RXPOLARITY1_IN	    : in  std_logic;
	--------------------- Shared Ports - Tile and PLL Ports --------------------
	TILE1_CLKIN_IN		    : in  std_logic;
	TILE1_GTXRESET_IN	    : in  std_logic;
	TILE1_PLLLKDET_OUT	    : out std_logic;
	TILE1_REFCLKOUT_OUT	    : out std_logic;
	TILE1_RESETDONE0_OUT	    : out std_logic;
	TILE1_RESETDONE1_OUT	    : out std_logic;
	------------------ Transmit Ports - TX Data Path interface -----------------
	TILE1_TXDATA0_IN	    : in  std_logic_vector(39 downto 0);
	TILE1_TXDATA1_IN	    : in  std_logic_vector(39 downto 0);
	TILE1_TXOUTCLK0_OUT	    : out std_logic;
	TILE1_TXOUTCLK1_OUT	    : out std_logic;
	TILE1_TXUSRCLK0_IN	    : in  std_logic;
	TILE1_TXUSRCLK1_IN	    : in  std_logic;
	TILE1_TXUSRCLK20_IN	    : in  std_logic;
	TILE1_TXUSRCLK21_IN	    : in  std_logic;
	--------------- Transmit Ports - TX Driver and OOB signalling --------------
	TILE1_TXDIFFCTRL0_IN	    : in  std_logic_vector(2 downto 0);
	TILE1_TXDIFFCTRL1_IN	    : in  std_logic_vector(2 downto 0);
	TILE1_TXINHIBIT0_IN	    : in  std_logic;
	TILE1_TXINHIBIT1_IN	    : in  std_logic;
	TILE1_TXN0_OUT		    : out std_logic;
	TILE1_TXN1_OUT		    : out std_logic;
	TILE1_TXP0_OUT		    : out std_logic;
	TILE1_TXP1_OUT		    : out std_logic;
	TILE1_TXPREEMPHASIS0_IN	    : in  std_logic_vector(2 downto 0);
	TILE1_TXPREEMPHASIS1_IN	    : in  std_logic_vector(2 downto 0);
	-------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
	TILE1_TXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE1_TXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE1_TXPMASETPHASE0_IN	    : in  std_logic;
	TILE1_TXPMASETPHASE1_IN	    : in  std_logic;
	-------------------- Transmit Ports - TX Polarity Control ------------------
	TILE1_TXPOLARITY0_IN	    : in  std_logic;
	TILE1_TXPOLARITY1_IN	    : in  std_logic;



	--_________________________________________________________________________
	--_________________________________________________________________________
	--TILE2	 (Location)

	------------------------ Loopback and Powerdown Ports ----------------------
	TILE2_LOOPBACK0_IN	    : in  std_logic_vector(2 downto 0);
	TILE2_LOOPBACK1_IN	    : in  std_logic_vector(2 downto 0);
	------------------- Receive Ports - RX Data Path interface -----------------
	TILE2_RXDATA0_OUT	    : out std_logic_vector(39 downto 0);
	TILE2_RXDATA1_OUT	    : out std_logic_vector(39 downto 0);
	TILE2_RXRECCLK0_OUT	    : out std_logic;
	TILE2_RXRECCLK1_OUT	    : out std_logic;
	TILE2_RXRESET0_IN	    : in  std_logic;
	TILE2_RXRESET1_IN	    : in  std_logic;
	TILE2_RXUSRCLK0_IN	    : in  std_logic;
	TILE2_RXUSRCLK1_IN	    : in  std_logic;
	TILE2_RXUSRCLK20_IN	    : in  std_logic;
	TILE2_RXUSRCLK21_IN	    : in  std_logic;
	------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
	TILE2_RXEQMIX0_IN	    : in  std_logic_vector(1 downto 0);
	TILE2_RXEQMIX1_IN	    : in  std_logic_vector(1 downto 0);
	TILE2_RXN0_IN		    : in  std_logic;
	TILE2_RXN1_IN		    : in  std_logic;
	TILE2_RXP0_IN		    : in  std_logic;
	TILE2_RXP1_IN		    : in  std_logic;
	-------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
	TILE2_RXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE2_RXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE2_RXPMASETPHASE0_IN	    : in  std_logic;
	TILE2_RXPMASETPHASE1_IN	    : in  std_logic;
	----------------- Receive Ports - RX Polarity Control Ports ----------------
	TILE2_RXPOLARITY0_IN	    : in  std_logic;
	TILE2_RXPOLARITY1_IN	    : in  std_logic;
	--------------------- Shared Ports - Tile and PLL Ports --------------------
	TILE2_CLKIN_IN		    : in  std_logic;
	TILE2_GTXRESET_IN	    : in  std_logic;
	TILE2_PLLLKDET_OUT	    : out std_logic;
	TILE2_REFCLKOUT_OUT	    : out std_logic;
	TILE2_RESETDONE0_OUT	    : out std_logic;
	TILE2_RESETDONE1_OUT	    : out std_logic;
	------------------ Transmit Ports - TX Data Path interface -----------------
	TILE2_TXDATA0_IN	    : in  std_logic_vector(39 downto 0);
	TILE2_TXDATA1_IN	    : in  std_logic_vector(39 downto 0);
	TILE2_TXOUTCLK0_OUT	    : out std_logic;
	TILE2_TXOUTCLK1_OUT	    : out std_logic;
	TILE2_TXUSRCLK0_IN	    : in  std_logic;
	TILE2_TXUSRCLK1_IN	    : in  std_logic;
	TILE2_TXUSRCLK20_IN	    : in  std_logic;
	TILE2_TXUSRCLK21_IN	    : in  std_logic;
	--------------- Transmit Ports - TX Driver and OOB signalling --------------
	TILE2_TXDIFFCTRL0_IN	    : in  std_logic_vector(2 downto 0);
	TILE2_TXDIFFCTRL1_IN	    : in  std_logic_vector(2 downto 0);
	TILE2_TXINHIBIT0_IN	    : in  std_logic;
	TILE2_TXINHIBIT1_IN	    : in  std_logic;
	TILE2_TXN0_OUT		    : out std_logic;
	TILE2_TXN1_OUT		    : out std_logic;
	TILE2_TXP0_OUT		    : out std_logic;
	TILE2_TXP1_OUT		    : out std_logic;
	TILE2_TXPREEMPHASIS0_IN	    : in  std_logic_vector(2 downto 0);
	TILE2_TXPREEMPHASIS1_IN	    : in  std_logic_vector(2 downto 0);
	-------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
	TILE2_TXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE2_TXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE2_TXPMASETPHASE0_IN	    : in  std_logic;
	TILE2_TXPMASETPHASE1_IN	    : in  std_logic;
	-------------------- Transmit Ports - TX Polarity Control ------------------
	TILE2_TXPOLARITY0_IN	    : in  std_logic;
	TILE2_TXPOLARITY1_IN	    : in  std_logic;



	--_________________________________________________________________________
	--_________________________________________________________________________
	--TILE3	 (Location)

	------------------------ Loopback and Powerdown Ports ----------------------
	TILE3_LOOPBACK0_IN	    : in  std_logic_vector(2 downto 0);
	TILE3_LOOPBACK1_IN	    : in  std_logic_vector(2 downto 0);
	------------------- Receive Ports - RX Data Path interface -----------------
	TILE3_RXDATA0_OUT	    : out std_logic_vector(39 downto 0);
	TILE3_RXDATA1_OUT	    : out std_logic_vector(39 downto 0);
	TILE3_RXRECCLK0_OUT	    : out std_logic;
	TILE3_RXRECCLK1_OUT	    : out std_logic;
	TILE3_RXRESET0_IN	    : in  std_logic;
	TILE3_RXRESET1_IN	    : in  std_logic;
	TILE3_RXUSRCLK0_IN	    : in  std_logic;
	TILE3_RXUSRCLK1_IN	    : in  std_logic;
	TILE3_RXUSRCLK20_IN	    : in  std_logic;
	TILE3_RXUSRCLK21_IN	    : in  std_logic;
	------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
	TILE3_RXEQMIX0_IN	    : in  std_logic_vector(1 downto 0);
	TILE3_RXEQMIX1_IN	    : in  std_logic_vector(1 downto 0);
	TILE3_RXN0_IN		    : in  std_logic;
	TILE3_RXN1_IN		    : in  std_logic;
	TILE3_RXP0_IN		    : in  std_logic;
	TILE3_RXP1_IN		    : in  std_logic;
	-------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
	TILE3_RXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE3_RXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE3_RXPMASETPHASE0_IN	    : in  std_logic;
	TILE3_RXPMASETPHASE1_IN	    : in  std_logic;
	----------------- Receive Ports - RX Polarity Control Ports ----------------
	TILE3_RXPOLARITY0_IN	    : in  std_logic;
	TILE3_RXPOLARITY1_IN	    : in  std_logic;
	--------------------- Shared Ports - Tile and PLL Ports --------------------
	TILE3_CLKIN_IN		    : in  std_logic;
	TILE3_GTXRESET_IN	    : in  std_logic;
	TILE3_PLLLKDET_OUT	    : out std_logic;
	TILE3_REFCLKOUT_OUT	    : out std_logic;
	TILE3_RESETDONE0_OUT	    : out std_logic;
	TILE3_RESETDONE1_OUT	    : out std_logic;
	------------------ Transmit Ports - TX Data Path interface -----------------
	TILE3_TXDATA0_IN	    : in  std_logic_vector(39 downto 0);
	TILE3_TXDATA1_IN	    : in  std_logic_vector(39 downto 0);
	TILE3_TXOUTCLK0_OUT	    : out std_logic;
	TILE3_TXOUTCLK1_OUT	    : out std_logic;
	TILE3_TXUSRCLK0_IN	    : in  std_logic;
	TILE3_TXUSRCLK1_IN	    : in  std_logic;
	TILE3_TXUSRCLK20_IN	    : in  std_logic;
	TILE3_TXUSRCLK21_IN	    : in  std_logic;
	--------------- Transmit Ports - TX Driver and OOB signalling --------------
	TILE3_TXDIFFCTRL0_IN	    : in  std_logic_vector(2 downto 0);
	TILE3_TXDIFFCTRL1_IN	    : in  std_logic_vector(2 downto 0);
	TILE3_TXINHIBIT0_IN	    : in  std_logic;
	TILE3_TXINHIBIT1_IN	    : in  std_logic;
	TILE3_TXN0_OUT		    : out std_logic;
	TILE3_TXN1_OUT		    : out std_logic;
	TILE3_TXP0_OUT		    : out std_logic;
	TILE3_TXP1_OUT		    : out std_logic;
	TILE3_TXPREEMPHASIS0_IN	    : in  std_logic_vector(2 downto 0);
	TILE3_TXPREEMPHASIS1_IN	    : in  std_logic_vector(2 downto 0);
	-------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
	TILE3_TXENPMAPHASEALIGN0_IN : in  std_logic;
	TILE3_TXENPMAPHASEALIGN1_IN : in  std_logic;
	TILE3_TXPMASETPHASE0_IN	    : in  std_logic;
	TILE3_TXPMASETPHASE1_IN	    : in  std_logic;
	-------------------- Transmit Ports - TX Polarity Control ------------------
	TILE3_TXPOLARITY0_IN	    : in  std_logic;
	TILE3_TXPOLARITY1_IN	    : in  std_logic


	);
  end component;

begin

  ----------------------------- The GTX Wrapper -----------------------------

  gtx_i : GTX
    generic map
    (
      WRAPPER_SIM_GTXRESET_SPEEDUP => 1,
      WRAPPER_SIM_PLL_PERDIV2	   => x"0c8"
      )
    port map
    (
      ----------------------------------------------------------------------------
      -- TILE0  (X0Y2)
      -- MGT0 - CH1 (GUN-TX)
      -- MGT1 - CH2 (GUN-TX)
      
      ------------------------ Loopback and Powerdown Ports ----------------------
      TILE0_LOOPBACK0_IN	  => gnd_vec(2 downto 0),
      TILE0_LOOPBACK1_IN	  => gnd_vec(2 downto 0),
      ------------------- Receive Ports - RX Data Path interface -----------------
      TILE0_RXDATA0_OUT		  => rx_data(0),
      TILE0_RXDATA1_OUT		  => rx_data(1),
      TILE0_RXRECCLK0_OUT	  => rxrecclk(0),
      TILE0_RXRECCLK1_OUT	  => rxrecclk(1),
      TILE0_RXRESET0_IN		  => rxreset(0),
      TILE0_RXRESET1_IN		  => rxreset(1),
      TILE0_RXUSRCLK0_IN	  => rxrecclk(0),
      TILE0_RXUSRCLK1_IN	  => rxrecclk(1),
      TILE0_RXUSRCLK20_IN	  => rxrecclk(0),
      TILE0_RXUSRCLK21_IN	  => rxrecclk(1),
      ------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
      TILE0_RXEQMIX0_IN		  => gnd_vec(1 downto 0),
      TILE0_RXEQMIX1_IN		  => gnd_vec(1 downto 0),
      TILE0_RXN0_IN		  => RDN(2),
      TILE0_RXN1_IN		  => RDP(1),
      TILE0_RXP0_IN		  => RDP(2),
      TILE0_RXP1_IN		  => RDN(1),
      -------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
      TILE0_RXENPMAPHASEALIGN0_IN => gnd,
      TILE0_RXENPMAPHASEALIGN1_IN => gnd,
      TILE0_RXPMASETPHASE0_IN	  => gnd,
      TILE0_RXPMASETPHASE1_IN	  => gnd,
      ----------------- Receive Ports - RX Polarity Control Ports ----------------
      TILE0_RXPOLARITY0_IN	  => gnd,
      TILE0_RXPOLARITY1_IN	  => vcc,
      --------------------- Shared Ports - Tile and PLL Ports --------------------
      TILE0_CLKIN_IN		  => RXREFCLK,
      TILE0_GTXRESET_IN		  => gtxreset,
      TILE0_PLLLKDET_OUT	  => plllkdet(0),
      TILE0_REFCLKOUT_OUT	  => open,
      TILE0_RESETDONE0_OUT	  => open,
      TILE0_RESETDONE1_OUT	  => open,
      ------------------ Transmit Ports - TX Data Path interface -----------------
      TILE0_TXDATA0_IN		  => tx_data(0),
      TILE0_TXDATA1_IN		  => tx_data(1),
      TILE0_TXOUTCLK0_OUT	  => txoutclk(0),
      TILE0_TXOUTCLK1_OUT	  => txoutclk(1),
      TILE0_TXUSRCLK0_IN	  => txoutclk(0),
      TILE0_TXUSRCLK1_IN	  => txoutclk(1),
      TILE0_TXUSRCLK20_IN	  => txoutclk(0),
      TILE0_TXUSRCLK21_IN	  => txoutclk(1),
      --------------- Transmit Ports - TX Driver and OOB signalling --------------
      TILE0_TXDIFFCTRL0_IN	  => gnd_vec(2 downto 0),
      TILE0_TXDIFFCTRL1_IN	  => gnd_vec(2 downto 0),
      TILE0_TXINHIBIT0_IN	  => txinhibit(0),
      TILE0_TXINHIBIT1_IN	  => txinhibit(1),
      TILE0_TXN0_OUT		  => TDP(2),
      TILE0_TXN1_OUT		  => TDN(1),
      TILE0_TXP0_OUT		  => TDN(2),
      TILE0_TXP1_OUT		  => TDP(1),
      TILE0_TXPREEMPHASIS0_IN	  => gnd_vec(2 downto 0),
      TILE0_TXPREEMPHASIS1_IN	  => gnd_vec(2 downto 0),
      -------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
      TILE0_TXENPMAPHASEALIGN0_IN => gnd,
      TILE0_TXENPMAPHASEALIGN1_IN => gnd,
      TILE0_TXPMASETPHASE0_IN	  => gnd,
      TILE0_TXPMASETPHASE1_IN	  => gnd,
      -------------------- Transmit Ports - TX Polarity Control ------------------
      TILE0_TXPOLARITY0_IN	  => vcc,
      TILE0_TXPOLARITY1_IN	  => gnd,

      ----------------------------------------------------------------------------
      -- TILE1  (X0Y3)
      -- MGT0 - LVPECL0
      -- MGT1 - LVPECL1
      -- This tile has the reference clock RXREFCLK input
      
      ------------------------ Loopback and Powerdown Ports ----------------------
      TILE1_LOOPBACK0_IN	  => gnd_vec(2 downto 0),
      TILE1_LOOPBACK1_IN	  => gnd_vec(2 downto 0),
      ------------------- Receive Ports - RX Data Path interface -----------------
      TILE1_RXDATA0_OUT		  => open,
      TILE1_RXDATA1_OUT		  => open,
      TILE1_RXRECCLK0_OUT	  => open,
      TILE1_RXRECCLK1_OUT	  => open,
      TILE1_RXRESET0_IN		  => vcc,
      TILE1_RXRESET1_IN		  => vcc,
      TILE1_RXUSRCLK0_IN	  => gnd,
      TILE1_RXUSRCLK1_IN	  => gnd,
      TILE1_RXUSRCLK20_IN	  => gnd,
      TILE1_RXUSRCLK21_IN	  => gnd,
      ------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
      TILE1_RXEQMIX0_IN		  => gnd_vec(1 downto 0),
      TILE1_RXEQMIX1_IN		  => gnd_vec(1 downto 0),
      TILE1_RXN0_IN		  => RDN(4),
      TILE1_RXN1_IN		  => RDN(3),
      TILE1_RXP0_IN		  => RDP(4),
      TILE1_RXP1_IN		  => RDP(3),
      -------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
      TILE1_RXENPMAPHASEALIGN0_IN => gnd,
      TILE1_RXENPMAPHASEALIGN1_IN => gnd,
      TILE1_RXPMASETPHASE0_IN	  => gnd,
      TILE1_RXPMASETPHASE1_IN	  => gnd,
      ----------------- Receive Ports - RX Polarity Control Ports ----------------
      TILE1_RXPOLARITY0_IN	  => gnd,
      TILE1_RXPOLARITY1_IN	  => gnd,
      --------------------- Shared Ports - Tile and PLL Ports --------------------
      TILE1_CLKIN_IN		  => RXREFCLK,
      TILE1_GTXRESET_IN		  => gtxreset,
      TILE1_PLLLKDET_OUT	  => plllkdet(1),
      TILE1_REFCLKOUT_OUT	  => REFCLKOUT,
      TILE1_RESETDONE0_OUT	  => open,
      TILE1_RESETDONE1_OUT	  => open,
      ------------------ Transmit Ports - TX Data Path interface -----------------
      TILE1_TXDATA0_IN		  => tx_data(2),
      TILE1_TXDATA1_IN		  => tx_data(3),
      TILE1_TXOUTCLK0_OUT	  => txoutclk(2),
      TILE1_TXOUTCLK1_OUT	  => txoutclk(3),
      TILE1_TXUSRCLK0_IN	  => txoutclk(2),
      TILE1_TXUSRCLK1_IN	  => txoutclk(3),
      TILE1_TXUSRCLK20_IN	  => txoutclk(2),
      TILE1_TXUSRCLK21_IN	  => txoutclk(3),
      --------------- Transmit Ports - TX Driver and OOB signalling --------------
      TILE1_TXDIFFCTRL0_IN	  => gnd_vec(2 downto 0),
      TILE1_TXDIFFCTRL1_IN	  => gnd_vec(2 downto 0),
      TILE1_TXINHIBIT0_IN	  => txinhibit(2),
      TILE1_TXINHIBIT1_IN	  => txinhibit(3),
      TILE1_TXN0_OUT		  => TDP(4),
      TILE1_TXN1_OUT		  => TDN(3),
      TILE1_TXP0_OUT		  => TDN(4),
      TILE1_TXP1_OUT		  => TDP(3),
      TILE1_TXPREEMPHASIS0_IN	  => gnd_vec(2 downto 0),
      TILE1_TXPREEMPHASIS1_IN	  => gnd_vec(2 downto 0),
      -------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
      TILE1_TXENPMAPHASEALIGN0_IN => gnd,
      TILE1_TXENPMAPHASEALIGN1_IN => gnd,
      TILE1_TXPMASETPHASE0_IN	  => gnd,
      TILE1_TXPMASETPHASE1_IN	  => gnd,
      -------------------- Transmit Ports - TX Polarity Control ------------------
      TILE1_TXPOLARITY0_IN	  => vcc,
      TILE1_TXPOLARITY1_IN	  => gnd,

      ----------------------------------------------------------------------------
      -- TILE2  (X0Y4)
      -- MGT0 - UNIV2
      -- MGT1 - UNIV3
      
      ------------------------ Loopback and Powerdown Ports ----------------------
      TILE2_LOOPBACK0_IN	  => gnd_vec(2 downto 0),
      TILE2_LOOPBACK1_IN	  => gnd_vec(2 downto 0),
      ------------------- Receive Ports - RX Data Path interface -----------------
      TILE2_RXDATA0_OUT		  => open,
      TILE2_RXDATA1_OUT		  => open,
      TILE2_RXRECCLK0_OUT	  => open,
      TILE2_RXRECCLK1_OUT	  => open,
      TILE2_RXRESET0_IN		  => vcc,
      TILE2_RXRESET1_IN		  => vcc,
      TILE2_RXUSRCLK0_IN	  => gnd,
      TILE2_RXUSRCLK1_IN	  => gnd,
      TILE2_RXUSRCLK20_IN	  => gnd,
      TILE2_RXUSRCLK21_IN	  => gnd,
      ------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
      TILE2_RXEQMIX0_IN		  => gnd_vec(1 downto 0),
      TILE2_RXEQMIX1_IN		  => gnd_vec(1 downto 0),
      TILE2_RXN0_IN		  => RDN(6),
      TILE2_RXN1_IN		  => RDN(5),
      TILE2_RXP0_IN		  => RDP(6),
      TILE2_RXP1_IN		  => RDP(5),
      -------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
      TILE2_RXENPMAPHASEALIGN0_IN => gnd,
      TILE2_RXENPMAPHASEALIGN1_IN => gnd,
      TILE2_RXPMASETPHASE0_IN	  => gnd,
      TILE2_RXPMASETPHASE1_IN	  => gnd,
      ----------------- Receive Ports - RX Polarity Control Ports ----------------
      TILE2_RXPOLARITY0_IN	  => gnd,
      TILE2_RXPOLARITY1_IN	  => gnd,
      --------------------- Shared Ports - Tile and PLL Ports --------------------
      TILE2_CLKIN_IN		  => RXREFCLK,
      TILE2_GTXRESET_IN		  => gtxreset,
      TILE2_PLLLKDET_OUT	  => plllkdet(2),
      TILE2_REFCLKOUT_OUT	  => open,
      TILE2_RESETDONE0_OUT	  => open,
      TILE2_RESETDONE1_OUT	  => open,
      ------------------ Transmit Ports - TX Data Path interface -----------------
      TILE2_TXDATA0_IN		  => tx_data(4),
      TILE2_TXDATA1_IN		  => tx_data(5),
      TILE2_TXOUTCLK0_OUT	  => txoutclk(4),
      TILE2_TXOUTCLK1_OUT	  => txoutclk(5),
      TILE2_TXUSRCLK0_IN	  => txoutclk(4),
      TILE2_TXUSRCLK1_IN	  => txoutclk(5),
      TILE2_TXUSRCLK20_IN	  => txoutclk(4),
      TILE2_TXUSRCLK21_IN	  => txoutclk(5),
      --------------- Transmit Ports - TX Driver and OOB signalling --------------
      TILE2_TXDIFFCTRL0_IN	  => gnd_vec(2 downto 0),
      TILE2_TXDIFFCTRL1_IN	  => gnd_vec(2 downto 0),
      TILE2_TXINHIBIT0_IN	  => txinhibit(4),
      TILE2_TXINHIBIT1_IN	  => txinhibit(5),
      TILE2_TXN0_OUT		  => TDP(6),
      TILE2_TXN1_OUT		  => TDN(5),
      TILE2_TXP0_OUT		  => TDN(6),
      TILE2_TXP1_OUT		  => TDP(5),
      TILE2_TXPREEMPHASIS0_IN	  => gnd_vec(2 downto 0),
      TILE2_TXPREEMPHASIS1_IN	  => gnd_vec(2 downto 0),
      -------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
      TILE2_TXENPMAPHASEALIGN0_IN => gnd,
      TILE2_TXENPMAPHASEALIGN1_IN => gnd,
      TILE2_TXPMASETPHASE0_IN	  => gnd,
      TILE2_TXPMASETPHASE1_IN	  => gnd,
      -------------------- Transmit Ports - TX Polarity Control ------------------
      TILE2_TXPOLARITY0_IN	  => gnd,
      TILE2_TXPOLARITY1_IN	  => gnd,

      ----------------------------------------------------------------------------
      -- TILE3  (X0Y5)
      -- MGT0 - UNIV0
      -- MGT1 - UNIV1
      
      ------------------------ Loopback and Powerdown Ports ----------------------
      TILE3_LOOPBACK0_IN	  => gnd_vec(2 downto 0),
      TILE3_LOOPBACK1_IN	  => gnd_vec(2 downto 0),
      ------------------- Receive Ports - RX Data Path interface -----------------
      TILE3_RXDATA0_OUT		  => open,
      TILE3_RXDATA1_OUT		  => open,
      TILE3_RXRECCLK0_OUT	  => open,
      TILE3_RXRECCLK1_OUT	  => open,
      TILE3_RXRESET0_IN		  => vcc,
      TILE3_RXRESET1_IN		  => vcc,
      TILE3_RXUSRCLK0_IN	  => gnd,
      TILE3_RXUSRCLK1_IN	  => gnd,
      TILE3_RXUSRCLK20_IN	  => gnd,
      TILE3_RXUSRCLK21_IN	  => gnd,
      ------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
      TILE3_RXEQMIX0_IN		  => gnd_vec(1 downto 0),
      TILE3_RXEQMIX1_IN		  => gnd_vec(1 downto 0),
      TILE3_RXN0_IN		  => RDN(8),
      TILE3_RXN1_IN		  => RDN(7),
      TILE3_RXP0_IN		  => RDP(8),
      TILE3_RXP1_IN		  => RDP(7),
      -------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
      TILE3_RXENPMAPHASEALIGN0_IN => gnd,
      TILE3_RXENPMAPHASEALIGN1_IN => gnd,
      TILE3_RXPMASETPHASE0_IN	  => gnd,
      TILE3_RXPMASETPHASE1_IN	  => gnd,
      ----------------- Receive Ports - RX Polarity Control Ports ----------------
      TILE3_RXPOLARITY0_IN	  => gnd,
      TILE3_RXPOLARITY1_IN	  => gnd,
      --------------------- Shared Ports - Tile and PLL Ports --------------------
      TILE3_CLKIN_IN		  => RXREFCLK,
      TILE3_GTXRESET_IN		  => gtxreset,
      TILE3_PLLLKDET_OUT	  => plllkdet(3),
      TILE3_REFCLKOUT_OUT	  => open,
      TILE3_RESETDONE0_OUT	  => open,
      TILE3_RESETDONE1_OUT	  => open,
      ------------------ Transmit Ports - TX Data Path interface -----------------
      TILE3_TXDATA0_IN		  => tx_data(6),
      TILE3_TXDATA1_IN		  => tx_data(7),
      TILE3_TXOUTCLK0_OUT	  => txoutclk(6),
      TILE3_TXOUTCLK1_OUT	  => txoutclk(7),
      TILE3_TXUSRCLK0_IN	  => txoutclk(6),
      TILE3_TXUSRCLK1_IN	  => txoutclk(7),
      TILE3_TXUSRCLK20_IN	  => txoutclk(6),
      TILE3_TXUSRCLK21_IN	  => txoutclk(7),
      --------------- Transmit Ports - TX Driver and OOB signalling --------------
      TILE3_TXDIFFCTRL0_IN	  => gnd_vec(2 downto 0),
      TILE3_TXDIFFCTRL1_IN	  => gnd_vec(2 downto 0),
      TILE3_TXINHIBIT0_IN	  => txinhibit(6),
      TILE3_TXINHIBIT1_IN	  => txinhibit(7),
      TILE3_TXN0_OUT		  => TDP(8),
      TILE3_TXN1_OUT		  => TDN(7),
      TILE3_TXP0_OUT		  => TDN(8),
      TILE3_TXP1_OUT		  => TDP(7),
      TILE3_TXPREEMPHASIS0_IN	  => gnd_vec(2 downto 0),
      TILE3_TXPREEMPHASIS1_IN	  => gnd_vec(2 downto 0),
      -------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
      TILE3_TXENPMAPHASEALIGN0_IN => gnd,
      TILE3_TXENPMAPHASEALIGN1_IN => gnd,
      TILE3_TXPMASETPHASE0_IN	  => gnd,
      TILE3_TXPMASETPHASE1_IN	  => gnd,
      -------------------- Transmit Ports - TX Polarity Control ------------------
      TILE3_TXPOLARITY0_IN	  => vcc,
      TILE3_TXPOLARITY1_IN	  => gnd
      );

  -----------------------Dedicated GTX Reference Clock Inputs ---------------
  -- Each dedicated refclk you are using in your design will need its own IBUFDS instance
  
  tile1_refclk_ibufds_i : IBUFDS
    port map
    (
      O	 => RXREFCLK,
      I	 => RXREFCLKP,			-- Connect to package pin Y4
      IB => RXREFCLKN			-- Connect to package pin Y3
      );

  gnd <= '0';
  vcc <= '1';
  gnd_vec <= (others => '0');
  vcc_vec <= (others => '1');
  
  REFCLK_out <= REFCLKOUT;

  tx_clk_out <= txoutclk;
  rx_clk_out <= rxrecclk;

  process (plllkdet)
  begin
    plllkdet_out <= '1';
    if plllkdet = "0000" then
      plllkdet_out <= '0';
    end if;
  end process;

end struct;

