---------------------------------------------------------------------------
--
--  File        : databuf_rx.vhd
--
--  Title       : Data memory buffer to receive a configurable size
--                data buffer from EVG
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description :
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- LIBRARY UNISIM;
-- use UNISIM.VCOMPONENTS.ALL;

entity databuf_rx is
  port (
    -- Memory buffer RAMB read interface
    data_out          : out std_logic_vector(31 downto 0);
    addr_in           : in std_logic_vector(10 downto 2);
    clk               : in std_logic;
    
    -- Data stream interface
    databuf_data      : in std_logic_vector(7 downto 0);
    databuf_k         : in std_logic;
    databuf_ena       : in std_logic;
    event_clk         : in std_logic;
    
    -- Control interface
    enable            : in std_logic;
    disable           : in std_logic;
    bufsize_words     : out std_logic_vector(11 downto 2);
    rx_enable         : out std_logic;
    rx_complete       : out std_logic;
    checksum_error    : out std_logic;
    
    reset             : in std_logic
    );
end databuf_rx;

architecture implementation of databuf_rx is

  signal rx_addr         : std_logic_vector(10 downto 0);
  signal rx_data         : std_logic_vector(7 downto 0);
  
  signal we_A            : std_logic;

  component buf_bsram IS
    port (
      addra: IN std_logic_VECTOR(10 downto 0);
      addrb: IN std_logic_VECTOR(8 downto 0);
      clka: IN std_logic;
      clkb: IN std_logic;
      dina: IN std_logic_VECTOR(7 downto 0);
      doutb: OUT std_logic_VECTOR(31 downto 0);
      wea: IN std_logic);
  end component;

begin

  i_ramb0 : buf_bsram
    port map (
      addra => rx_addr,
      addrb => addr_in,
      clka => event_clk,
      clkb => clk,
      dina => rx_data,
      doutb => data_out,
      wea => we_A);

  reception : process (event_clk, databuf_data, databuf_k, databuf_ena,
		       enable, reset)
    variable data_ena     : std_logic;
    variable enabled      : std_logic;
    variable running      : std_logic;
    variable address      : std_logic_vector(11 downto 0);
    variable sync_enable  : std_logic_vector(2 downto 0);
    variable sync_disable : std_logic_vector(2 downto 0);
    variable wordcnt      : std_logic_vector(11 downto 2);
    variable checksum     : std_logic_vector(15 downto 0);
    variable rx_checksum  : std_logic_vector(15 downto 0);
    variable addr_decode  : std_logic_vector(2 downto 0);
  begin
    bufsize_words <= wordcnt;
    rx_enable <= enabled;
    if event_clk'event and event_clk = '1' then
      we_A <= '0';
      if databuf_ena = '1' then
	rx_data <= databuf_data;
	if databuf_k = '1' then
	  if databuf_data = X"1C" then -- K28.0 start data
	    address := (others => '0');
	    checksum := (others => '1');
	    running := '1';
	    data_ena := '1';
	  end if;
	  if databuf_data = X"3C" then -- K28.1 end data
	    wordcnt := address(11 downto 2);
	    address := "100000000000";
	    data_ena := '0';
	  end if;
	else
	  if running = '1' then
	    rx_addr <= address(10 downto 0);
	    if data_ena = '1' then
	      we_A <= '1';
	      checksum := checksum - databuf_data;
	    end if;
	    addr_decode := address(11) & address(1 downto 0);
	    case addr_decode is
	      when "100"  => rx_checksum(15 downto 8) := databuf_data;
	      when "101"  => rx_checksum(7 downto 0) := databuf_data;
	      when "110"  => running := '0';
			     enabled := '0';
			     rx_complete <= '1';
			     if rx_checksum /= checksum then
			       checksum_error <= '1';
			     end if;
	      when others =>
	    end case;
	    address := address + 1;
	  end if;
	end if;

	if running = '0' and sync_enable(1 downto 0) = "10" then
	  enabled := '1';
	  rx_complete <= '0';
	  checksum_error <= '0';
	end if;
	if sync_disable(1 downto 0) = "10" then
	  running := '0';
	  enabled := '0';
	  rx_complete <= '0';
	  checksum_error <= '0';
	end if;
	sync_enable := enable & sync_enable(2 downto 1);
	sync_disable := disable & sync_disable(2 downto 1);
      end if;
      if reset = '1' then
	running := '0';
	enabled := '0';
	data_ena := '0';
	rx_complete <= '0';
	checksum_error <= '0';
      end if;
    end if;
  end process;

end implementation;
      
