---------------------------------------------------------------------------
--                                                                       --
--  File        : event_dcm.vhd                                          --
--                                                                       --
--  Title       : Event clock DCM lock detector
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
-- LIBRARY UNISIM;
-- use UNISIM.VCOMPONENTS.ALL;

entity event_dcm is
  port (
    recclkin      : in std_logic;
    event_clk     : in std_logic;
    reset         : in std_logic;
    sample_reset  : in std_logic;
    sample_run    : in std_logic;
    clkout        : out std_logic;
    sample_low    : out std_logic_vector(15 downto 0);
    sample_high   : out std_logic_vector(15 downto 0);
    dcm_locked    : out std_logic;
    clkin_stopped : out std_logic;
    psclk         : in std_logic;
    psinc         : in std_logic;
    psdec         : in std_logic;
    psdone        : out std_logic
    );
end event_dcm;

architecture structure of event_dcm is

  signal CLKFB_IN : std_logic;
  signal CLK0_BUF : std_logic;
  signal CLK180_BUF : std_logic;
  signal GND : std_logic;
  signal status : std_logic_vector(7 downto 0);
  signal psen : std_logic;
  signal psincdec : std_logic;
  signal dcm_psdone : std_logic;
  
  component bufg
    port (
      O : out std_logic;
      I : in  std_logic
      );
  end component;
  
  component DCM
    generic (
      CLKDV_DIVIDE : real := 2.0;
      CLKFX_DIVIDE : integer := 1;
      CLKFX_MULTIPLY : integer := 4;
      CLKIN_DIVIDE_BY_2 : boolean := false;
      CLKIN_PERIOD : real := 0.0;
      CLKOUT_PHASE_SHIFT : string := "NONE";
      CLK_FEEDBACK : string := "1X";
      DESKEW_ADJUST : string := "SYSTEM_SYNCHRONOUS";
      DFS_FREQUENCY_MODE : string := "LOW";
      DLL_FREQUENCY_MODE : string := "LOW";
      DSS_MODE : string := "NONE";
      DUTY_CYCLE_CORRECTION : boolean := true;
      FACTORY_JF : bit_vector := X"C080";                 
      MAXPERCLKIN : time := 1000000 ps;                   
      MAXPERPSCLK : time := 100000000 ps;                 
      PHASE_SHIFT : integer := 0;
      SIM_CLKIN_CYCLE_JITTER : time := 300 ps;            
      SIM_CLKIN_PERIOD_JITTER : time := 1000 ps;          
      STARTUP_WAIT : boolean := false);
    port (
      CLKIN : in STD_LOGIC;
      CLKFB : in STD_LOGIC;
      RST : in STD_LOGIC;
      PSEN : in STD_LOGIC;
      PSINCDEC : in STD_LOGIC;
      PSCLK : in STD_LOGIC;
      DSSEN : in STD_LOGIC;
      CLK0 : out STD_LOGIC;
      CLK90 : out STD_LOGIC;
      CLK180 : out STD_LOGIC;
      CLK270 : out STD_LOGIC;
      CLKDV : out STD_LOGIC;
      CLK2X : out STD_LOGIC;
      CLK2X180 : out STD_LOGIC;
      CLKFX : out STD_LOGIC;
      CLKFX180 : out STD_LOGIC;
      STATUS : out STD_LOGIC_VECTOR (7 downto 0);
      LOCKED : out STD_LOGIC;
      PSDONE : out STD_LOGIC);
  end component;

begin

  DCM_local : DCM
    generic map(
      CLK_FEEDBACK => "1X",
      CLKDV_DIVIDE => 2.0,
      CLKFX_DIVIDE => 1,
      CLKFX_MULTIPLY => 4,
      CLKIN_DIVIDE_BY_2 => FALSE,
      CLKIN_PERIOD => 8.0,
      CLKOUT_PHASE_SHIFT => "VARIABLE",
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
      DFS_FREQUENCY_MODE => "HIGH",
      DLL_FREQUENCY_MODE => "HIGH",
      DUTY_CYCLE_CORRECTION => FALSE,
      PHASE_SHIFT => 0,
      STARTUP_WAIT => FALSE)
    port map (
      CLKIN => recclkin,
      CLKFB => clkfb_in,
      RST => reset,
      PSEN => psen,
      PSINCDEC => psincdec,
      PSCLK => psclk,
      DSSEN => GND,
      CLK0 => CLK0_BUF,
      STATUS => status,
      LOCKED => dcm_locked,
      PSDONE => dcm_psdone);

  CLK0_BUFG_INST : BUFG
     port map (
      I => CLK0_BUF,
      O => clkfb_in);

  clkout <= clkfb_in;
  
  clkin_stopped <= status(1);

  process (clkfb_in, event_clk, sample_reset)
    variable sample_l : std_logic_vector(16 downto 0);
    variable sample_h : std_logic_vector(16 downto 0);
    variable sample_bit : std_logic_vector(1 downto 0);
    variable sample_reset_1 : std_logic;
    variable sample_run_1 : std_logic;
  begin
    sample_low <= sample_l(15 downto 0);
    sample_high <= sample_h(15 downto 0);
    
    if rising_edge(clkfb_in) then
      if sample_run_1 = '1' then
	if sample_bit(0) = '1' then
	  sample_h := sample_h + 1;
	end if;
	if sample_bit(0) = '0' then
	  sample_l := sample_l + 1;
	end if;
      end if;
      if sample_h(16) = '1' or sample_l(16) = '1' then
	sample_h := '0' & sample_h(16 downto 1);
	sample_l := '0' & sample_l(16 downto 1);
      end if;
      sample_bit := event_clk & sample_bit(1);
      if sample_reset_1 = '1' then
	sample_l := (others => '0');
	sample_h := (others => '0');
      end if;
      sample_reset_1 := sample_reset;
      sample_run_1 := sample_run;
    end if;
  end process;

  phase_shifting : process (psclk, psinc, psdec, reset)
    variable psinc_1 : std_logic;
    variable psdec_1 : std_logic;
  begin
    if psclk'event and psclk = '1' then
      psincdec <= '0';
      psen <= '0';
      if psinc = '1' and psinc_1 = '0' then
        psincdec <= '1';
        psen <= '1';
	psdone <= '0';
      end if;
      if psdec = '1' and psdec_1 = '0' then
        psen <= '1';
	psdone <= '0';
      end if;
      psinc_1 := psinc;
      psdec_1 := psdec;
      if dcm_psdone = '1' then
	psdone <= '1';
      end if;
      if reset = '1' then
        psincdec <= '0';
        psen <= '0';
	psdone <= '1';
      end if;
    end if;
  end process;

end structure;
  
