---------------------------------------------------------------------------
--
--  File        : event_map_ram.vhd
--
--  Title       : Event receiver mapping RAM design file
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : A mapping RAM converts the received event codes to
--                mapped event signals (or triggers). The event codes
--                are applied to the address lines of event mapping
--                RAMs. The contents of the memory location addressed
--                determines which actions will be taken.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity event_map_ram is
  generic (
    code_width     : integer;
    map_code_abits : integer);
  port (
    clk            : in    std_logic;
    data_in        : in    std_logic_vector(31 downto 0);
    data_out       : out   std_logic_vector(31 downto 0);
    addr           : in    std_logic_vector(31 downto 0);
    we             : in    std_logic_vector(3 downto 0);
    event_clk      : in    std_logic;
    event_code     : in    std_logic_vector(code_width - 1 downto 0);
    event_map      : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0);
    debug          : out   std_logic_vector(39 downto 0));
end event_map_ram;

architecture structure of event_map_ram is

  signal bram_addr_A     : std_logic_vector(8 downto 0);
  signal bram_addr_B     : std_logic_vector(8 downto 0);
  signal bram_data_A_in  : std_logic_vector(31 downto 0);         
  signal bram_data_A_out : std_logic_vector(8*(2**map_code_abits) - 1 downto 0);
  signal bram_we         : std_logic_vector(2**(map_code_abits-2)-1 downto 0);
  signal gnd1            : std_logic;
  signal vcc1            : std_logic;
  signal gnd32           : std_logic_vector(31 downto 0);

  component map_ram is
  generic (
    map_code_abits : integer);
  port (
    bram_addr_A     : in    std_logic_vector(8 downto 0);
    bram_addr_B     : in    std_logic_vector(8 downto 0);    
    clk             : in    std_logic;
    event_clk       : in    std_logic;
    bram_data_A_in  : in    std_logic_vector(31 downto 0); 
    bram_data_A_out : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0);
    bram_we         : in    std_logic_vector(2**(map_code_abits-2) - 1 downto 0);
    event_map       : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0));
  end component;

begin

  i_ram: map_ram
    generic map (
      map_code_abits => map_code_abits)
    port map (
      bram_addr_A => bram_addr_A,
      bram_addr_B => bram_addr_B,
      clk => clk,
      event_clk => event_clk,
      bram_data_A_in => bram_data_A_in,
      bram_data_A_out => bram_data_A_out,
      bram_we => bram_we,
      event_map => event_map);
  
  gnd1 <= '0';
  vcc1 <= '1';
  gnd32 <= (others => '0');
  bram_addr_B <= '0' & event_code;
  
  -- Accessing the RAM takes several cycles, during the write cycles data,
  -- address and write enables have to remain stable:
  -- Reading
  -- 1. cycle: assign address to BRAM blocks
  -- 2. cycle: valid data out of BRAM blocks
  -- 3. cycle: data presented on bus
  -- Writing
  -- 1. cycle: assign address to BRAM blocks
  -- 2. cycle: valid data out of BRAM blocks
  -- 3. cycle: write old/new data (masked with byte enables) into BRAM blocks
  ram_access: process (clk, addr)
    variable long_offset : integer range 0 to (2**(map_code_abits-2)-1);
    variable wr_sr       : std_logic_vector(2 downto 0);
    variable data32      : std_logic_vector(31 downto 0);
  begin
    -- long_offset = 0 for address 0x???C
    -- long_offset = 1 for address 0x???8
    -- long_offset = 2 for address 0x???4
    -- long_offset = 3 for address 0x???0
    long_offset := 2**(map_code_abits-2)-1 - conv_integer(addr(map_code_abits-1 downto 2));
    data32 := bram_data_A_out(long_offset*32+31 downto long_offset*32);
    data_out <= data32;
    debug(31 downto 24) <= data32(15 downto 8);
    if rising_edge(clk) then
      bram_addr_A <= '0' & addr(code_width - 1 +
				map_code_abits downto map_code_abits);
      bram_data_A_in <= data32;
      for i in 3 downto 0 loop
        if we(i) = '1' then
          bram_data_A_in(i*8+7 downto i*8) <= data_in(i*8+7 downto i*8);
        end if;
      end loop;
      bram_we <= (others => '0');
      if wr_sr(1 downto 0) = "01" then
        bram_we(long_offset) <= '1';
      end if;
      wr_sr := wr_sr(1 downto 0) & '0';
      if we /= "0000" then
        wr_sr(0) := '1';
      end if;
    end if;
  end process;  

  debug(39 downto 36) <= "0000";
  debug(35 downto 32) <= bram_we;
  debug(23 downto 16) <= bram_data_A_out(15 downto 8);
  debug(15 downto 8) <= bram_data_A_in(15 downto 8);
  debug(7 downto 0) <= bram_addr_A(7 downto 0);
  
end structure;
    
