---------------------------------------------------------------------------
--
--  File        : map_ram_v5.vhd
--
--  Title       : Event receiver mapping RAM
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;

entity map_ram is
  generic (
    map_code_abits : integer);
  port (
    bram_addr_A     : in    std_logic_vector(8 downto 0);
    bram_addr_B     : in    std_logic_vector(8 downto 0);    
    clk             : in    std_logic;
    event_clk       : in    std_logic;
    bram_data_A_in  : in    std_logic_vector(31 downto 0); 
    bram_data_A_out : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0);
    bram_we         : in    std_logic_vector(2**(map_code_abits-2) - 1 downto 0);
    event_map       : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0));
end map_ram;

architecture structure of map_ram is

  signal gnd1            : std_logic;
  signal vcc1            : std_logic;
  signal gnd32           : std_logic_vector(31 downto 0);
  
begin

  gnd1 <= '0';
  vcc1 <= '1';
  gnd32 <= (others => '0');
  
  -- This RAM block maps internal events, default values are set here for
  -- the following event codes:
  -- code bit Event
  -- 0x70 0   Seconds '0'
  -- 0x71 1   Seconds '1'
  -- 0x79 27  Stop Event Log
  -- 0x7A 5   Heartbeat
  -- 0x7B 4   Reset prescalers
  -- 0x7C 2   Timestamp event counter clock
  -- 0x7D 3   Timestamp Event coutner reset
  i_ram0: RAMB36
    generic map (
      DOA_REG => 0,
      DOB_REG => 0,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 36,
      READ_WIDTH_B => 36,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 36,
      WRITE_WIDTH_B => 36,
      -- INIT_0E contains mapping bits for event codes 0x77, x076, ... 0x70
      INIT_0E =>
      X"0000000000000000000000000000000000000000000000000000000200000001",
      -- INIT_0F contains mapping bits for event codes 0x7F, 0x7E, ... 0x78
      INIT_0F =>
      X"0000000000000000000000080000000400000010000000200800000000000000")
    port map (
      DOA => bram_data_A_out(8*(2**map_code_abits)-1 downto
			     8*(2**map_code_abits)-32),
      DOB => event_map(8*(2**map_code_abits)-1 downto
		       8*(2**map_code_abits)-32),
      DOPA => open,
      DOPB => open,
      ADDRA(15 downto 14) => gnd32(1 downto 0),
      ADDRA(13 downto 5) => bram_addr_A,
      ADDRA(4 downto 0) => gnd32(4 downto 0),
      ADDRB(15 downto 14) => gnd32(1 downto 0),
      ADDRB(13 downto 5) => bram_addr_B,
      ADDRB(4 downto 0) => gnd32(4 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA => bram_data_A_in,
      DIB => gnd32,
      DIPA => gnd32(3 downto 0),
      DIPB => gnd32(3 downto 0),
      ENA => vcc1,
      ENB => vcc1,
      SSRA => gnd1,
      SSRB => gnd1,
      WEA(0) => bram_we(2**(map_code_abits-2)-1),
      WEA(1) => bram_we(2**(map_code_abits-2)-1),
      WEA(2) => bram_we(2**(map_code_abits-2)-1),
      WEA(3) => bram_we(2**(map_code_abits-2)-1),
      WEB => gnd32(3 downto 0));
  
  ram_blocks:
    for i in 1 to (2**(map_code_abits-2))-1 generate
      i_ram: RAMB36
	generic map (
	  DOA_REG => 0,
	  DOB_REG => 0,
	  INIT_A => X"00000",
	  INIT_B => X"00000",
	  READ_WIDTH_A => 36,
	  READ_WIDTH_B => 36,
	  SIM_COLLISION_CHECK => "ALL",
	  SRVAL_A => X"00000",
	  SRVAL_B => X"00000",
	  WRITE_MODE_A => "WRITE_FIRST",
	  WRITE_MODE_B => "WRITE_FIRST",
	  WRITE_WIDTH_A => 36,
	  WRITE_WIDTH_B => 36)
        port map (
          DOA => bram_data_A_out(8*(2**map_code_abits)-1-i*32 downto
				 8*(2**map_code_abits)-32-i*32),
          DOB => event_map(8*(2**map_code_abits)-1-i*32 downto
			   8*(2**map_code_abits)-32-i*32),
          DOPA => open,
          DOPB => open,
	  ADDRA(15 downto 14) => gnd32(1 downto 0),
          ADDRA(13 downto 5) => bram_addr_A,
	  ADDRA(4 downto 0) => gnd32(4 downto 0),
	  ADDRB(15 downto 14) => gnd32(1 downto 0),
          ADDRB(13 downto 5) => bram_addr_B,
	  ADDRB(4 downto 0) => gnd32(4 downto 0),
          CLKA => clk,
          CLKB => event_clk,
          DIA => bram_data_A_in,
          DIB => gnd32,
          DIPA => gnd32(3 downto 0),
          DIPB => gnd32(3 downto 0),
          ENA => vcc1,
          ENB => vcc1,
          SSRA => gnd1,
          SSRB => gnd1,
          WEA(0) => bram_we(2**(map_code_abits-2)-1-i),
          WEA(1) => bram_we(2**(map_code_abits-2)-1-i),
          WEA(2) => bram_we(2**(map_code_abits-2)-1-i),
          WEA(3) => bram_we(2**(map_code_abits-2)-1-i),
          WEB => gnd32(3 downto 0));
    end generate;
  
end structure;
    
