---------------------------------------------------------------------------
--
--  File        : memory_buffer.vhd
--
--  Title       : Memory buffer to transfer a configurable size data buffer
--                from EVG to EVR
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description :
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- LIBRARY UNISIM;
-- use UNISIM.VCOMPONENTS.ALL;

entity memory_buffer is
  port (
    -- Memory buffer RAMB read/write interface
    data_in           : in std_logic_vector(31 downto 0);
    data_out          : out std_logic_vector(31 downto 0);
    addr_in           : in std_logic_vector(31 downto 0);
    wr_en             : in std_logic_vector(3 downto 0);
    clk               : in std_logic;
    
    -- Data stream interface
    event_data_out    : out std_logic_vector(7 downto 0);
    event_k_out       : out std_logic;
    event_clk         : in std_logic;
    buf_clkena        : in std_logic;
    
    -- Control interface
    enable            : in std_logic;
    trigger           : in std_logic;
    bufsize_words     : in std_logic_vector(11 downto 2);
    tx_running        : out std_logic;
    tx_complete       : out std_logic;

    reset             : in std_logic
    );
end memory_buffer;

architecture implementation of memory_buffer is

  signal tx_addr         : std_logic_vector(10 downto 0);
  signal tx_data         : std_logic_vector(7 downto 0);
  
  signal data_B_in       : std_logic_vector(31 downto 0);
  signal data_B_out      : std_logic_vector(31 downto 0);
  signal we_B            : std_logic;
  
  component membuf_bsram IS
    port (
      addra: IN std_logic_VECTOR(10 downto 0);
      addrb: IN std_logic_VECTOR(8 downto 0);
      clka: IN std_logic;
      clkb: IN std_logic;
      douta: OUT std_logic_VECTOR(7 downto 0);
      dinb: IN std_logic_VECTOR(31 downto 0);
      doutb: OUT std_logic_VECTOR(31 downto 0);
      web: IN std_logic);
  end component;

begin

  i_ramb0 : membuf_bsram
    port map (
      addra => tx_addr,
      addrb => addr_in(10 downto 2),
      clka => event_clk,
      clkb => clk,
      douta => tx_data,
      dinb => data_B_in,
      doutb => data_B_out,
      web => we_B);

  data_out <= data_B_out;
  
  process (clk, wr_en, data_in, data_B_out)
    variable ena : std_logic_vector(1 downto 0) := "00";
  begin
    if rising_edge(clk) then
      data_B_in <= data_B_out;
      if wr_en(3) = '1' then
        data_B_in(31 downto 24) <= data_in(31 downto 24);
      end if;
      if wr_en(2) = '1' then
        data_B_in(23 downto 16) <= data_in(23 downto 16);
      end if;
      if wr_en(1) = '1' then
        data_B_in(15 downto 8) <= data_in(15 downto 8);
      end if;
      if wr_en(0) = '1' then
        data_B_in(7 downto 0) <= data_in(7 downto 0);
      end if;

      we_B <= '0';
      if wr_en /= "0000" and ena = "01" then
        we_B <= '1';
      end if;

      ena(1) := ena(0);
      ena(0) := '0';
      if wr_en /= "0000" then
        ena(0) := '1';
      end if;
    end if;
  end process;

  transmission : process (event_clk, enable, trigger, reset)
    variable sync_trigger : std_logic_vector(2 downto 0);
    variable sync_enable  : std_logic_vector(1 downto 0);
    variable address      : std_logic_vector(11 downto 0);
    variable running      : std_logic;
    variable checksum     : std_logic_vector(15 downto 0);
    variable addr_decode  : std_logic_vector(2 downto 0);
  begin
    tx_addr <= address(10 downto 0);
    tx_running <= running;
    tx_complete <= not running;
    if rising_edge(event_clk) then
      if buf_clkena = '1' then
	event_k_out <= '0';
	event_data_out <= (others => '0');
	if running = '1' then
	  event_data_out <= tx_data;
	  addr_decode := address(10) & address(1 downto 0);
	  if address(11) = '1' then
	    case addr_decode is
	      when "111"  => event_k_out <= '1';
			     event_data_out <= X"1C"; -- K28.0 start data
	      when "000"  => event_k_out <= '1';
			     event_data_out <= X"3C"; -- K28.1 end data
	      when "001"  => event_data_out <= checksum(15 downto 8);
	      when "010"  => event_data_out <= checksum(7 downto 0);
	      when others => running := '0';
			     event_data_out <= (others => '0');
	    end case;
	  else
	    checksum := checksum - tx_data;
	  end if;
	  address := address + 1;
	  if address(11 downto 2) = bufsize_words and address(11) = '0' then
	    address := "100000000000";
	  end if;
	end if;
	if sync_trigger(1 downto 0) = "10" then
	  running := '1';
	  address := (others => '1');
	  checksum := (others => '1');
	end if;
	if reset = '1' or sync_enable(0) = '0' then
	  running := '0';
	end if;
	sync_trigger := trigger & sync_trigger(2 downto 1);
	sync_enable := enable & sync_enable(1);
      end if;
    end if;
  end process;

end implementation;
      
