---------------------------------------------------------------------------
--
--  File        : opb_evr.vhd
--
--  Title       : OPB Event Receiver IP
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : OPB Event Receiver is the base for all event receivers.
--                It connect to the OPB (IBM Core Connect On-Board
--                Peripheral Bus.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
use work.evr_bs_pkg.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity opb_evr is
  generic (
    -- Bus protocol parameters, do not add to or delete
    C_BASEADDR         : std_logic_vector      := X"7A000000";
    C_BASEMASK         : std_logic_vector      := X"FF000000";
    C_OPB_AWIDTH       : integer               := 32;
    C_OPB_DWIDTH       : integer               := 32;
    C_FAMILY           : string                := "virtex2p");
  port (
    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    Sl_DBus         : out   std_logic_vector(0 to 31);
    Sl_errAck       : out   std_logic;
    Sl_retry        : out   std_logic;
    Sl_toutSup      : out   std_logic;
    Sl_xferAck      : out   std_logic;
    OPB_ABus        : in    std_logic_vector(0 to 31);
    OPB_BE          : in    std_logic_vector(0 to 3);
    OPB_DBus        : in    std_logic_vector(0 to 31);
    OPB_RNW         : in    std_logic;
    OPB_select      : in    std_logic;
    OPB_seqAddr     : in    std_logic;
    
    ip_irq_out      : out std_logic;
    
    TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
    TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
    UNIVIN          : in std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
    UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

    GPIO            : inout std_logic_vector(C_EVR_GPIOS-1 downto 0);

    TBIN            : in std_logic_vector(0 to C_EVR_TB_INPUTS-1);
    TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);

    -- Differential clocks from special brefclkx and brefclk2x pins
    -- Either pair may provide reference from fractional synthesizer
    BREFCLKN        : in std_logic; -- Reference clock input from
    BREFCLKP        : in std_logic; -- fractional synthesizer
    BREFCLK2N       : in std_logic; -- Resynchronised clock input
    BREFCLK2P       : in std_logic; -- from FPGA external FF

    refclk_out      : out std_logic; -- reference clock output
    brefclk_out     : out std_logic; -- reference clock output for MGT
    refclk2_out     : out std_logic; -- reference clock output
    brefclk2_out    : out std_logic; -- reference clock output for MGT
    
    RECCLKN         : out std_logic; -- Recovered clock output
    RECCLKP         : out std_logic;

    event_clk_out   : out std_logic;
    
    -- Front panel LEDs
    LED_fail        : out std_logic;
    LED_rx_link     : out std_logic;
    LED_ena         : out std_logic;
    LED_event_in    : out std_logic;
    LED_event_out   : out std_logic;
    LED_rx_fail     : out std_logic;
    LED_error       : out std_logic;
    
    -- MGT I/O pins
    EVRXN       : in std_logic;  -- Event Link RXN
    EVRXP       : in std_logic;  -- Event Link RXN
    EVTXN       : out std_logic; -- Event Link TXN
    EVTXP       : out std_logic; -- Event Link TXN
    FBRXN       : in std_logic;  -- Event Link RXN
    FBRXP       : in std_logic;  -- Event Link RXN
    FBTXN       : out std_logic; -- Event Link TXN
    FBTXP       : out std_logic; -- Event Link TXN
    CMLRXN      : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
    CMLRXP      : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
    CMLTXN      : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
    CMLTXP      : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);

    -- Micrel SY87729L serial bus signals
    CGSK        : out std_logic;
    CGDI        : out std_logic;
    CGCS        : out std_logic;
    CGLOCK      : in std_logic;

    -- SFP transceiver serial interface signals
    sfp_moddef0 : in std_logic;
    sfp_sda     : inout std_logic;
    sfp_scl     : out std_logic;
    
    TXLOOPBACK  : out std_logic; -- when '1' loop back FPGA TX to FPGA RX
    RXLOOPBACK  : out std_logic; -- when '1' loop back optical RX signal
                                 -- to transceiver TX port
    
    debug       : out std_logic_vector(39 downto 0);
    CONTROL0    : inout std_logic_vector(35 downto 0);
      
    -- Lattice ECP2M dedicated transceiver pins
    refclkp         : in std_logic;
    refclkn         : in std_logic;
    hdinp0          : in std_logic;
    hdinn0          : in std_logic;
    hdinp1          : in std_logic;
    hdinn1          : in std_logic;
    hdinp2          : in std_logic;
    hdinn2          : in std_logic;
    hdinp3          : in std_logic;
    hdinn3          : in std_logic;

    hdoutp0         : out std_logic;
    hdoutn0         : out std_logic;
    hdoutp1         : out std_logic;
    hdoutn1         : out std_logic;
    hdoutp2         : out std_logic;
    hdoutn2         : out std_logic;
    hdoutp3         : out std_logic;
    hdoutn3         : out std_logic
    );
end opb_evr;

architecture structure of opb_evr is

  constant C_EVR_FW_VERSION : std_logic_vector(31 downto 0) := X"1" &
                                                               C_FORM_FACTOR &
                                                               X"000003";  
  
  signal gnd                : std_logic;
  signal vcc                : std_logic;
  signal gnd32              : std_logic_vector(31 downto 0);

  signal Sl_select          : std_logic;
  signal opb_rd_ack         : std_logic;
  signal opb_wr_ack         : std_logic;
  
  signal int_reset          : std_logic;
  signal mgt_reset          : std_logic;
  signal event_clk          : std_logic;
  signal use_rxrecclk       : std_logic; -- for EVRs with low jitter
                                         -- clock resynchronizing force
                                         -- use of internal recovered clock
  signal event_clk_rx       : std_logic;
  signal event_clk_rx_nobuf : std_logic;
  signal event_code_rx      : std_logic_vector(7 downto 0);
  signal event_code         : std_logic_vector(7 downto 0);
  signal dbus_data          : std_logic_vector(7 downto 0);
  signal databuf_data       : std_logic_vector(7 downto 0);
  signal databuf_k          : std_logic;
  signal databuf_ena        : std_logic;
  signal databuf_mode       : std_logic;
  signal databuf_rxsize     : std_logic_vector(11 downto 2);    

  signal evdcm_reset        : std_logic;
  signal evdcm_clkout       : std_logic;
  signal evdcm_locked       : std_logic;
  signal evdcm_stopped      : std_logic;
  signal evdcm_sample_reset : std_logic;
  signal evdcm_sample_run   : std_logic;
  signal evdcm_sample_low   : std_logic_vector(15 downto 0);
  signal evdcm_sample_high  : std_logic_vector(15 downto 0);
  signal evdcm_psinc        : std_logic;
  signal evdcm_psdec        : std_logic;
  signal evdcm_psdone       : std_logic;

  signal recdcm_init_ps_done : std_logic;
  signal recdcm_init_ps     : std_logic_vector(7 downto 0);
  signal recdcm_running     : std_logic;
  signal recdcm_reset       : std_logic;
  signal recdcm_psinc       : std_logic;
  signal recdcm_psdec       : std_logic;
  signal recdcm_psdone      : std_logic;

  signal int_pulse          : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);

  signal pulse_presc        : data_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_delay        : data_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_width        : data_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_reset_ena    : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_set_ena      : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_trig_ena     : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_enable       : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_polarity     : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_trigger      : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_sw_set       : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_sw_reset     : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_set          : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_reset        : std_logic_vector(C_EVR_PULSE_GENS-1 downto 0);
  signal pulse_force_low    : std_logic;
  
  signal reg_control        : std_logic_vector(31 downto 0);
  signal control_bset       : std_logic_vector(31 downto 0);
  signal reg_status         : std_logic_vector(31 downto 0);
  signal reg_irq_flag       : std_logic_vector(31 downto 0);
  signal reg_evcnt_presc    : std_logic_vector(15 downto 0);
  signal reg_fp_input_map   : data_vector(C_EVR_TTL_INPUTS-1 downto 0);
  signal reg_fp_output_map  : signal_map_vector(C_EVR_TTL_OUTPUTS+
						C_EVR_CML_OUTPUTS-1 downto 0);
  signal reg_univ_output_map : signal_map_vector(C_EVR_UNIV_OUTPUTS-1 downto 0);
  signal reg_tb_output_map  : signal_map_vector(C_EVR_TB_OUTPUTS-1 downto 0);
  signal reg_pulse_irq_map  : std_logic_vector(C_SIGNAL_MAP_BITS-1 downto 0);
  signal reg_usec_divider   : std_logic_vector(15 downto 0);
  signal reg_clock_control  : std_logic_vector(15 downto 0);
  signal usec_enable        : std_logic;
  signal reg_prescaler      : count_vector(C_EVR_PRESCALERS-1 downto 0);
  signal prescaler_out      : std_logic_vector(C_EVR_PRESCALERS-1 downto 0);
  signal prescaler_ena_out  : std_logic_vector(C_EVR_PRESCALERS-1 downto 0);
  signal prescaler_polarity : std_logic_vector(C_EVR_PRESCALERS-1 downto 0);
  signal reg_micrel_word    : std_logic_vector(31 downto 0);
  signal micrel_reload      : std_logic;

  signal reg_fp_univ_gpdir  : std_logic_vector(31 downto 0);
  signal reg_fp_univ_gpin   : std_logic_vector(31 downto 0);
  signal reg_fp_univ_gpout  : std_logic_vector(31 downto 0);
  
  signal map_ram_data       : data_vector(C_EVR_MAP_RAMS-1 downto 0);
  signal map_ram_we         : we_vector(C_EVR_MAP_RAMS-1 downto 0);
  type debug_vector is array (natural range <>) of std_logic_vector(39 downto 0);
  signal map_ram_debug      : debug_vector(0 to C_EVR_MAP_RAMS-1);
  
  signal master_enable      : std_logic;
  signal TXLOOPBACK_int     : std_logic;
  signal RXLOOPBACK_int     : std_logic;
  signal sw_reset_timestamp : std_logic;
  signal sw_latch_timestamp : std_logic;
  signal map_ram_enable     : std_logic;
  signal map_ram_select     : std_logic;
  signal start_nullfill     : std_logic;
  signal host_ram_select    : std_logic;
  signal map_ram_autoinc    : std_logic;
  signal reset_ram_addr     : std_logic;
  signal reset_event_fifo   : std_logic;
  signal ram1_access_ena    : std_logic;
  signal ram2_access_ena    : std_logic;
  signal timestamp_clk_dbus : std_logic;

  signal reset_linkchg_irq   : std_logic;
  signal reset_pulse_irq     : std_logic;
  signal reset_heartbeat_irq : std_logic;
  signal reset_fifofull_irq  : std_logic;
  signal reset_violation_irq : std_logic;
  
  signal flag_lostheartbeat : std_logic;
  signal flag_event_irq     : std_logic;
  signal flag_fifofull      : std_logic;
  signal flag_fifonotempty  : std_logic;
  signal flag_lostlock      : std_logic;
  signal flag_pulse_irq     : std_logic;
  signal flag_linkchg_irq   : std_logic;
  
  signal master_irq_enable  : std_logic;
  signal irq_event_ena      : std_logic;
  signal irq_heartbeat_ena  : std_logic;
  signal irq_fifofull_ena   : std_logic;
  signal irq_rxviol_ena     : std_logic;  
  signal irq_pulse_ena      : std_logic;
  signal irq_databuf_ena    : std_logic;
  signal irq_linkchg_ena    : std_logic;
  
  signal irq_out            : std_logic;
  
  signal host_read_fifo     : std_logic;
  signal event_counter_rd   : std_logic_vector(31 downto 0);
  signal timestamp_rd       : std_logic_vector(31 downto 0);
  signal event_log_rd       : std_logic_vector(79 downto 0);
  signal event_log_addr     : std_logic_vector(8 downto 0);
  signal event_log_wr_pnt   : std_logic_vector(8 downto 0);
  signal event_log_overflow : std_logic;
  signal event_log_reset    : std_logic;
  signal event_fifo_rd      : std_logic_vector(79 downto 0);
  signal event_fifo_in      : std_logic_vector(79 downto 0);
  signal fifofull           : std_logic;
  signal fifoempty          : std_logic;
  signal save_event         : std_logic;
  signal log_event          : std_logic;
  signal latch_timestamp_event         : std_logic;
  signal led_event                     : std_logic;
  signal heartbeat_event               : std_logic;
  signal reset_prescalers_event        : std_logic;
  signal timestamp_counter_reset_event : std_logic;
  signal timestamp_counter_clk_event   : std_logic;
  signal seconds_counter_1_event       : std_logic;
  signal seconds_counter_0_event       : std_logic;
  signal forward_event                 : std_logic;
  signal stop_log_event               : std_logic;

  signal stop_log_event_enable        : std_logic;
  signal log_disable_sw               : std_logic;
  signal log_enable_sw                : std_logic;
  signal log_stopped                  : std_logic;
  
  signal rx_link_ok         : std_logic;
  signal rx_violation       : std_logic;
  signal rx_violation2      : std_logic;
  
  signal evcnt_clk_ena      : std_logic;
  signal sync_code_reset    : std_logic;

  signal reg_seconds        : std_logic_vector(31 downto 0);
  signal reg_seconds_sr     : std_logic_vector(31 downto 0);
  signal timestamp_sec_rd   : std_logic_vector(31 downto 0);
  
  signal event_map          : event_map_vector(0 to C_EVR_MAP_RAMS-1);

  signal databuf_data_out   : std_logic_vector(31 downto 0);
  signal databuf_load       : std_logic;
  signal databuf_stop       : std_logic;
  signal databuf_receiving  : std_logic;
  signal databuf_rx_ready   : std_logic;
  signal databuf_checksum   : std_logic;

  signal refdcm_reset       : std_logic;
  signal brefclk            : std_logic;
  signal brefclk2           : std_logic;
  signal brefrecclk         : std_logic;
  signal refclk_dcm         : std_logic;
  signal brefclk_dcm        : std_logic;
  signal brefclk2_dcm       : std_logic;

  signal mgt_tx_charisk     : std_logic_vector(1 downto 0);
  signal mgt_tx_data        : std_logic_vector(15 downto 0);
  
  signal cml_data           : data_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_addr           : std_logic_vector(13 downto 2);
  signal cml_we             : cml_we_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_pulse          : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal cml_enable         : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_pwd            : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_txreset        : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_refclksel      : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);

  signal cml_freq_mode      : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_high_period    : short_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_low_period     : short_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_trig_pos       : short_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_trig_level     : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  
  -- cPCI-EVRTG-300 CML signals
  signal gtx_cs             : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_gtxreset       : std_logic;
  signal cml_tx_data        : gtx_data(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_tx_clk         : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_txinhibit      : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_rx_data        : gtx_data(C_EVR_CML_INPUTS-1 downto 0);
  signal cml_rx_clk         : std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);
  signal cml_rxreset        : std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);

  signal cml_pattern_mode   : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_recycle        : std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  signal cml_samples        : cml_samples_vector(C_EVR_CML_OUTPUTS-1 downto 0);
  
  signal tx_clk             : std_logic;
  signal tx_event_clk       : std_logic;
  signal tx_event           : std_logic_vector(7 downto 0);
  signal tx_dbus            : std_logic_vector(7 downto 0);
  signal tx_databuf_data    : std_logic_vector(7 downto 0);
  signal tx_databuf_k       : std_logic;
  signal tx_databuf_txena   : std_logic; -- pulse enable from transmitter

  signal tx_databuf_we      : std_logic_vector(3 downto 0); -- write to databuffer memory
  signal tx_databuf_ena     : std_logic; -- enable data buffer
  signal tx_databuf_trigger : std_logic;
  signal tx_databuf_mode    : std_logic; -- data buffer mode
  signal tx_databuf_data_out : std_logic_vector(31 downto 0); -- readback data
  signal tx_databuf_size    : std_logic_vector(31 downto 0);
  signal tx_databuf_running : std_logic;
  signal tx_databuf_complete : std_logic;

  type count_vector is array (integer range <>) of std_logic_vector(C_DIAG_COUNTER_BITS-1 downto 0);
  signal pulse_counter_cnt  : count_vector(C_DIAG_COUNTERS-1 downto 0);
  signal pulse_counter_rst  : std_logic_vector(31 downto 0);
  signal pulse_counter_ena  : std_logic_vector(31 downto 0);

  signal fwd_event_ena      : std_logic;
  
  signal fwd_fifo_data_in   : std_logic_vector(7 downto 0);
  signal fwd_fifo_wr        : std_logic;
  signal fwd_fifo_rd        : std_logic;
  signal fwd_fifo_reset     : std_logic;
  signal fwd_fifo_event_out : std_logic_vector(7 downto 0);
  signal fwd_fifo_full      : std_logic;
  signal fwd_fifo_empty     : std_logic;

  signal confRAM_data       : std_logic_vector(31 downto 0);
  signal confRAM_we         : std_logic_vector(3 downto 0);
  
  component bufg is
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component bufgmux is
    port (
      O : out std_ulogic;
      I0 : in std_ulogic;
      I1 : in std_ulogic;
      S : in std_ulogic);
  end component;

  component ibufgds
  port (O  : out std_ulogic;
        I  : in std_ulogic;
        IB : in std_ulogic);
  end component;

  component event_map_ram
    generic (
      code_width     : integer;
      map_code_abits : integer);
    port (
      clk            : in std_logic;
      data_in        : in std_logic_vector(31 downto 0);
      data_out       : out std_logic_vector(31 downto 0);
      addr           : in std_logic_vector(31 downto 0);
      we             : in std_logic_vector(3 downto 0);
      event_clk      : in std_logic;
      event_code     : in std_logic_vector(C_EVENT_CODE_BITS-1 downto 0);
      event_map      : out std_logic_vector(8*(2**C_EVR_MAP_CODE_ABITS) - 1
					    downto 0);
      debug          : out std_logic_vector(39 downto 0));
  end component;
      
  component pulse_generator
    generic (
      prescaler_range : integer;
      delay_range     : integer;
      width_range     : integer);
    port (
      prescaler       : in   std_logic_vector(31 downto 0);
      delay           : in   std_logic_vector(31 downto 0);
      width           : in   std_logic_vector(31 downto 0);
      polarity        : in   std_logic;
      sw_set          : in    std_logic;
      sw_reset        : in    std_logic;
      clk             : in    std_logic; -- control interface clock
                                         -- sw_set and sw_reset and synchronous
                                         -- to this clock
      pulse_out       : out  std_logic;
      event_clk       : in   std_logic;
      enable          : in   std_logic;
      trigger         : in   std_logic;
      set             : in   std_logic;
      reset           : in   std_logic;
      force_low       : in   std_logic);
  end component;
    
  component event_fifo
    port (
      din: IN std_logic_VECTOR(79 downto 0);
      wr_en: IN std_logic;
      wr_clk: IN std_logic;
      rd_en: IN std_logic;
      rd_clk: IN std_logic;
      ainit: IN std_logic;
      dout: OUT std_logic_VECTOR(79 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;

  component event_fifo_v5
    port (
      rst: IN std_logic;
      wr_clk: IN std_logic;
      rd_clk: IN std_logic;
      din: IN std_logic_VECTOR(79 downto 0);
      wr_en: IN std_logic;
      rd_en: IN std_logic;
      dout: OUT std_logic_VECTOR(79 downto 0);
      full: OUT std_logic;
      empty: OUT std_logic);
  end component;

  component event_log
    port (
      log_in      : in std_logic_vector(79 downto 0);
      log_wr_en   : in std_logic;
      log_wr_clk  : in std_logic;
      
      clk         : in std_logic;
      data_out    : out std_logic_vector(79 downto 0);
      addr        : in std_logic_vector(8 downto 0); 
      wr_pnt      : out std_logic_vector(8 downto 0);
      wr_overflow : out std_logic;
      reset       : in std_logic);
  end component;
  
  component receiver is
    generic (
      ref_on_brefclk  : boolean;
      separate_tx_rx  : boolean);
    port (
      brefclk         : in std_logic;
      brefclk_dcm     : in std_logic;
      brefclk2        : in std_logic;
      brefclk2_dcm    : in std_logic;
      refclk_dcm_out  : out std_logic;
      rxclk_nobuf_out : out std_logic;
      rxclk_out       : out std_logic;
      event_clk       : in std_logic;
      rxpolarity      : in std_logic;

      -- Receiver side connections
      event_code      : out std_logic_vector(7 downto 0);
      dbus_data       : out std_logic_vector(7 downto 0);
      databuf_data    : out std_logic_vector(7 downto 0);
      databuf_k       : out std_logic;
      databuf_ena     : out std_logic;
      databuf_mode    : in std_logic;

      rx_link_ok      : out   std_logic;
      rx_violation    : out   std_logic;
      rx_clear_viol   : in    std_logic;
      
      reset           : in    std_logic;
      
      -- Transmitter side connections
      tx_charisk      : in    std_logic_vector(1 downto 0);
      tx_data         : in    std_logic_vector(15 downto 0);
      tx_reset        : in    std_logic;
      tx_polarity     : in    std_logic;
      tx_event_clk    : in    std_logic;
      
      -- MGT I/O pins
      RXN             : in std_logic;
      RXP             : in std_logic;
      TXN             : out std_logic;
      TXP             : out std_logic);
  end component;

  component receiver_ecp2m
    port (
      refclkp         : in std_logic;
      refclkn         : in std_logic;
      refclk_out      : out std_logic;
      event_clk_out   : out std_logic;
      rx_polarity     : in std_logic;
      
      -- Receiver side connections
      event_code      : out std_logic_vector(7 downto 0);
      dbus_data       : out std_logic_vector(7 downto 0);
      databuf_data    : out std_logic_vector(7 downto 0);
      databuf_k       : out std_logic;
      databuf_ena     : out std_logic;
      databuf_mode    : in std_logic;
      
      rx_link_ok      : out   std_logic;
      rx_violation    : out   std_logic;
      rx_clear_viol   : in    std_logic;
      
      reset           : in    std_logic;
      
      -- Transmitter side connections
      tx_charisk      : in    std_logic_vector(1 downto 0);
      tx_data         : in    std_logic_vector(15 downto 0);
      tx_reset        : in    std_logic;
      tx_polarity     : in    std_logic;
      tx_event_clk    : out   std_logic;    
      
      -- PCS I/O pins
      hdinp0          : in std_logic;
      hdinn0          : in std_logic;
      hdinp1          : in std_logic;
      hdinn1          : in std_logic;
      hdinp2          : in std_logic;
      hdinn2          : in std_logic;
      hdinp3          : in std_logic;
      hdinn3          : in std_logic;
      
      hdoutp0         : out std_logic;
      hdoutn0         : out std_logic;
      hdoutp1         : out std_logic;
      hdoutn1         : out std_logic;
      hdoutp2         : out std_logic;
      hdoutn2         : out std_logic;
      hdoutp3         : out std_logic;
      hdoutn3         : out std_logic
      );
  end component;

  component pcs_rx is
  port (
    refclkp         : in std_logic;
    refclkn         : in std_logic;
    refclk_out      : out std_logic;
    
    RXPOLARITY      : in std_logic;

    RXPOWERDOWN     : in std_logic;
    RXRESET         : in std_logic;

    TXCHARISK       : in std_logic_vector (1 downto 0);
    TXDATA          : in std_logic_vector (15 downto 0);
    TXRESET         : in std_logic;
    TXPOLARITY      : in std_logic;
    TXPOWERDOWN     : in std_logic;

    RXLOS           : out std_logic;
    RXSYNC          : out std_logic;
    RXCDRLOS        : out std_logic;
    
    RXCHARISK       : out std_logic_vector (1 downto 0);
    RXDATA          : out std_logic_vector (15 downto 0);
    RXDISPERR       : out std_logic_vector (1 downto 0);
    RXNOTINTABLE    : out std_logic_vector (1 downto 0);

    RXCLK           : out std_logic;

    PCSRESET        : in std_logic;
    
    -- PCS I/O pins
    hdinp0          : in std_logic;
    hdinn0          : in std_logic;
    hdinp1          : in std_logic;
    hdinn1          : in std_logic;
    hdinp2          : in std_logic;
    hdinn2          : in std_logic;
    hdinp3          : in std_logic;
    hdinn3          : in std_logic;

    hdoutp0         : out std_logic;
    hdoutn0         : out std_logic;
    hdoutp1         : out std_logic;
    hdoutn1         : out std_logic;
    hdoutp2         : out std_logic;
    hdoutn2         : out std_logic;
    hdoutp3         : out std_logic;
    hdoutn3         : out std_logic);
  end component;

  component receiver_v5fxt is
    generic
      (
	-- Simulation attributes
	TILE_SIM_MODE                : string    := "FAST"; -- Set to Fast Functional Simulation Model
	TILE_SIM_GTXRESET_SPEEDUP    : integer   := 0; -- Set to 1 to speed up sim reset
	TILE_SIM_PLL_PERDIV2         : bit_vector:= x"0c8"; -- Set to the VCO Unit Interval time 
	
	-- Channel bonding attributes
	TILE_CHAN_BOND_MODE_0        : string    := "OFF";  -- "MASTER", "SLAVE", or "OFF"
	TILE_CHAN_BOND_LEVEL_0       : integer   := 0;     -- 0 to 7. See UG for details
	
	TILE_CHAN_BOND_MODE_1        : string    := "OFF";  -- "MASTER", "SLAVE", or "OFF"
	TILE_CHAN_BOND_LEVEL_1       : integer   := 0      -- 0 to 7. See UG for details
	);
    port (
      REFCLKP         : in std_logic;
      REFCLKN         : in std_logic;
      REFCLK_OUT      : out std_logic;
      event_clk_out   : out std_logic;
      rx_polarity     : in std_logic;
      
      -- Receiver side connections
      event_code      : out std_logic_vector(7 downto 0);
      dbus_data       : out std_logic_vector(7 downto 0);
      databuf_data    : out std_logic_vector(7 downto 0);
      databuf_k       : out std_logic;
      databuf_ena     : out std_logic;
      databuf_mode    : in std_logic;
      
      rx_link_ok      : out   std_logic;
      rx_violation    : out   std_logic;
      rx_clear_viol   : in    std_logic;
      
      reset           : in    std_logic;

      cdrres          : in    std_logic;
      cdrphas         : in    std_logic;
 
      -- Transmitter side connections
      tx_charisk      : in    std_logic_vector(1 downto 0);
      tx_data         : in    std_logic_vector(15 downto 0);
      tx_reset        : in    std_logic;
      tx_polarity     : in    std_logic;
      tx_event_clk    : out   std_logic;    
      
      RXN0            : in    std_logic;
      RXP0            : in    std_logic;
      RXN1            : in    std_logic;
      RXP1            : in    std_logic;
      
      TXN0            : out   std_logic;
      TXP0            : out   std_logic;
      TXN1            : out   std_logic;
      TXP1            : out   std_logic;
      CONTROL0        : inout std_logic_vector(35 downto 0)
      );
  end component;
  
  component transmitter is
    generic (
      ref_on_brefclk  : boolean);
    port (
      brefclk_dcm     : in std_logic;
      brefclk2_dcm    : in std_logic;
      
      tx_clk          : out std_logic;
      
      -- Transmitter side connections
      tx_event        : in std_logic_vector(7 downto 0);
      tx_dbus         : in std_logic_vector(7 downto 0);
      tx_databuf      : in std_logic_vector(7 downto 0);
      k_databuf       : in std_logic;
      ena_databuf     : out std_logic;
      databuf_mode    : in std_logic;
      
      tx_charisk      : out   std_logic_vector(1 downto 0);
      tx_data         : out   std_logic_vector(15 downto 0));
  end component;
  
  component mgt_tx is
    generic (
      ref_on_brefclk  : boolean);
    port (
      BREFCLK : in std_logic;
      BREFCLK2 : in std_logic;
      tx_clk : in std_logic;
      POWERDOWN : in std_logic;
      RXN : in std_logic;
      RXP : in std_logic;
      TXCHARISK : in std_logic_vector (1 downto 0);
      TXDATA : in std_logic_vector (15 downto 0);
      TXRESET : in std_logic;
      TXPOLARITY : in std_logic;
      TXN : out std_logic;
      TXP : out std_logic);
  end component;

  component mx_counter is
    generic (
      counter_range  : integer := 16);
    port (
      reg_width      : in    std_logic_vector(31 downto 0);
      pulse_out      : out   std_logic;
      ena_out        : out   std_logic;
      clk            : in    std_logic;
      sync           : in    std_logic;
      polarity       : in    std_logic;
      reset          : in    std_logic);
  end component;
  
  component databuf_rx is
    port (
      -- Memory buffer RAMB read interface
      data_out          : out std_logic_vector(31 downto 0);
      addr_in           : in std_logic_vector(10 downto 2);
      clk               : in std_logic;
      
      -- Data stream interface
      databuf_data      : in std_logic_vector(7 downto 0);
      databuf_k         : in std_logic;
      databuf_ena       : in std_logic;
      event_clk         : in std_logic;
      
      -- Control interface
      enable            : in std_logic;
      disable           : in std_logic;
      bufsize_words     : out std_logic_vector(11 downto 2);
      rx_enable         : out std_logic;
      rx_complete       : out std_logic;
      checksum_error    : out std_logic;
      
      reset             : in std_logic);
  end component;

  component memory_buffer is
    port (
      -- Memory buffer RAMB read/write interface
      data_in           : in std_logic_vector(31 downto 0);
      data_out          : out std_logic_vector(31 downto 0);
      addr_in           : in std_logic_vector(31 downto 0);
      wr_en             : in std_logic_vector(3 downto 0);
      clk               : in std_logic;
      
      -- Data stream interface
      event_data_out    : out std_logic_vector(7 downto 0);
      event_k_out       : out std_logic;
      event_clk         : in std_logic;
      buf_clkena        : in std_logic;
      
      -- Control interface
      enable            : in std_logic;
      trigger           : in std_logic;
      bufsize_words     : in std_logic_vector(11 downto 2);
      tx_running        : out std_logic;
      tx_complete       : out std_logic;
      
      reset             : in std_logic
      );
  end component;

  component micrel_load is
    port (
      clk           : in std_logic;
      data          : in std_logic_vector(31 downto 0);
      reload        : in std_logic;
      reset         : in std_logic;
      
      -- Micrel SY87729L serial bus signals
      CGSK          : out std_logic;
      CGDI          : out std_logic;
      CGCS          : out std_logic);
  end component;

  component refclock is
    port (
      BREFCLKN     : in std_logic; -- Reference clock
      BREFCLKP     : in std_logic;
      brefclk      : out std_logic;
      brefclk_dcm  : out std_logic);
  end component;

  component event_dcm is
    port (
      recclkin      : in std_logic;
      event_clk     : in std_logic;
      reset         : in std_logic;
      sample_reset  : in std_logic;
      sample_run    : in std_logic;
      clkout        : out std_logic;
      sample_low    : out std_logic_vector(15 downto 0);
      sample_high   : out std_logic_vector(15 downto 0);
      dcm_locked    : out std_logic;
      clkin_stopped : out std_logic;
      psclk         : in std_logic;
      psinc         : in std_logic;
      psdec         : in std_logic;
      psdone        : out std_logic
      );
  end component;

  component recclk_out is
    port (
      clkin         : in std_logic;
      reset         : in std_logic;
      clkoutn       : out std_logic;
      clkoutp       : out std_logic;
      rx_link_ok    : in std_logic;
      dcm_run_out   : out std_logic;
      init_ps_done  : out std_logic;
      init_ps       : in std_logic_vector(7 downto 0);
      psclk         : in std_logic;
      psinc         : in std_logic;
      psdec         : in std_logic;
      psdone        : out std_logic
      );
  end component;

  component mgt_cmlout_bram is
    port (
      clk          : in std_logic;
      data_in      : in std_logic_vector(23 downto 0);
      data_out     : out std_logic_vector(31 downto 0);
      addr         : in std_logic_vector(1 downto 0);
      we           : in std_logic_vector(2 downto 0);
      event_clk    : in std_logic;
      brefclk      : in std_logic;
      brefclk2     : in std_logic;
      refclksel    : in std_logic;
      pulse        : in std_logic;
      enable       : in std_logic;
      powerdown    : in std_logic;
      freq_mode    : in std_logic;
      high_period  : in std_logic_vector(15 downto 0);
      low_period   : in std_logic_vector(15 downto 0);
      trig_pos     : in std_logic_vector(15 downto 0);
      trig_level   : in std_logic;
      RXN          : in std_logic;
      RXP          : in std_logic;
      txreset      : in std_logic;
      TXN          : out std_logic;
      TXP          : out std_logic);
  end component;

  component cml_gtx is
    port (
      REFCLK_out   : out std_logic;
      gtxreset     : in std_logic;
      tx_data      : in gtx_data(C_EVR_CML_OUTPUTS-1 downto 0);
      tx_clk_out   : out std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
      txinhibit    : in std_logic_vector(C_EVR_CML_OUTPUTS-1 downto 0);
      plllkdet_out : out std_logic;
      
      rx_data      : out gtx_data(C_EVR_CML_INPUTS-1 downto 0);
      rx_clk_out   : out std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);
      rxreset      : in std_logic_vector(C_EVR_CML_INPUTS-1 downto 0);
      
      -- External pins
      RXREFCLKN    : in std_logic;
      RXREFCLKP    : in std_logic;

      RDN  : in std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
      RDP  : in std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
      TDN  : out std_logic_vector(C_EVR_CML_OUTPUTS downto 1);
      TDP  : out std_logic_vector(C_EVR_CML_OUTPUTS downto 1));
  end component;

  component gtx_cmlout_bram is
    port (
      clk          : in std_logic;
      data_in      : in std_logic_vector(31 downto 0);
      data_out     : out std_logic_vector(31 downto 0);
      addr         : in std_logic_vector(13 downto 2);
      we           : in std_logic_vector(3 downto 0);
      cs           : in std_logic;
      
      event_clk    : in std_logic;
      tx_data      : out std_logic_vector(39 downto 0);
      pulse        : in std_logic;
      enable       : in std_logic;
      freq_mode    : in std_logic;
      high_period  : in std_logic_vector(15 downto 0);
      low_period   : in std_logic_vector(15 downto 0);
      trig_pos     : in std_logic_vector(15 downto 0);
      trig_level   : in std_logic;
      pattern_mode : in std_logic;
      recycle      : in std_logic;
      samples      : in std_logic_vector(10 downto 0));
  end component;
  
  component fifo_128x8 is
    port (
      din: IN std_logic_VECTOR(7 downto 0);
      rd_clk: IN std_logic;
      rd_en: IN std_logic;
      rst: IN std_logic;
      wr_clk: IN std_logic;
      wr_en: IN std_logic;
      dout: OUT std_logic_VECTOR(7 downto 0);
      empty: OUT std_logic;
      full: OUT std_logic);
  end component;

  component config_RAM is
    port (
      clk  : IN std_logic;
      addr : IN std_logic_VECTOR(10 downto 2);
      din  : IN std_logic_VECTOR(31 downto 0);
      dout : OUT std_logic_VECTOR(31 downto 0);
      we   : IN std_logic_vector(3 downto 0);
      sda  : inout std_logic;
      scl  : out std_logic);
  end component;
  
begin

  gnd <= '0';
  vcc <= '1';
  gnd32 <= X"00000000";

  TXLOOPBACK <= TXLOOPBACK_int;
  RXLOOPBACK <= RXLOOPBACK_int;


  ecp2m_clocks:
  if ECP2M = TRUE generate
    brefclk_dcm <= tx_event_clk;
    brefclk2_dcm <= tx_event_clk;
  end generate;

  v5fct_clocks:
  if VIRTEX5FXT = TRUE generate
    brefclk <= refclk_dcm;
    brefclk_dcm <= refclk_dcm;
    brefclk2_dcm <= brefclk2;
  end generate;
  
  event_clk_selection:
    if VIRTEX2PRO = TRUE and C_EVR_USE_RESYNC = TRUE generate
      i_evclk: bufgmux
	port map (
	  O => event_clk,
	  I0 => brefrecclk,
	  I1 => event_clk_rx,
	  S => use_rxrecclk);
    end generate;

  event_rx_clk_selection:
    if VIRTEX2PRO = TRUE and C_EVR_USE_RESYNC = FALSE generate
      i_evclk: bufg
	port map (
	  O => event_clk,
	  I => event_clk_rx);
    end generate;

  event_clock_resynchronizing:
    if VIRTEX2PRO = TRUE and C_EVR_USE_RESYNC = TRUE generate
      i_recclk : recclk_out
	port map (
	  clkin => event_clk_rx,
	  reset => recdcm_reset,
	  clkoutn => recclkp,
	  clkoutp => recclkn,
	  rx_link_ok => rx_link_ok,
	  dcm_run_out => recdcm_running,
	  init_ps_done => recdcm_init_ps_done,
	  init_ps => recdcm_init_ps,
	  psclk => OPB_clk,
	  psinc => recdcm_psinc,
	  psdec => recdcm_psdec,
	  psdone => recdcm_psdone);
    end generate;

  event_clock_monitoring:
    if VIRTEX2PRO = TRUE and C_EVR_USE_RESYNC = TRUE generate
      i_evdcm: event_dcm
	port map (
	  recclkin => event_clk_rx,
	  event_clk => event_clk,
	  reset => evdcm_reset,
	  clkout => evdcm_clkout,
	  sample_reset => evdcm_sample_reset,
	  sample_run => evdcm_sample_run,
	  sample_low => evdcm_sample_low,
	  sample_high => evdcm_sample_high,
	  dcm_locked => evdcm_locked,
	  clkin_stopped => evdcm_stopped,
	  psclk => OPB_clk,
	  psinc => evdcm_psinc,
	  psdec => evdcm_psdec,
	  psdone => evdcm_psdone);
    end generate;

  map_rams:
    for i in 0 to C_EVR_MAP_RAMS-1 generate
      i_map: event_map_ram
	generic map (
	  code_width => C_EVENT_CODE_BITS,
	  map_code_abits => C_EVR_MAP_CODE_ABITS)
	port map (
	  clk => OPB_Clk,
	  data_in(31 downto 0) => OPB_DBus(0 to 31),
	  data_out(31 downto 0) => map_ram_data(i),
	  addr(31 downto 0) => OPB_ABus(0 to 31),
	  we => map_ram_we(i),
	  event_clk => event_clk,
	  event_code => event_code,
	  event_map => event_map(i)(0 to 8*(2**C_EVR_MAP_CODE_ABITS) - 1),
	  debug => map_ram_debug(i)(39 downto 0));
    end generate;

--  debug <= map_ram_debug(0)(39 downto 0);

  internal_pulses:
    for i in 0 to C_EVR_PULSE_GENS-1 generate
      i_pulse: pulse_generator
	generic map (
	  prescaler_range => C_EVR_PULSE_PRESC_RANGE(i),
	  delay_range => C_EVR_PULSE_DELAY_RANGE(i),
	  width_range => C_EVR_PULSE_WIDTH_RANGE(i))
	port map (
	  prescaler => pulse_presc(i),
	  delay => pulse_delay(i),
	  width => pulse_width(i),
	  polarity => pulse_polarity(i),
          sw_set => pulse_sw_set(i),
          sw_reset => pulse_sw_reset(i),
          clk => OPB_Clk,
	  pulse_out => int_pulse(i),
	  event_clk => event_clk,
	  enable => pulse_enable(i),
	  trigger => pulse_trigger(i),
	  set => pulse_set(i),
	  reset => pulse_reset(i),
	  force_low => pulse_force_low);
    end generate;

  v2p_evfifo :
  if VIRTEX2PRO = TRUE generate
    ev_fifo: event_fifo
      port map ( din => event_fifo_in,
		 wr_en => save_event,
		 wr_clk => event_clk,
		 rd_en => host_read_fifo,
		 rd_clk => OPB_Clk,
		 ainit => reset_event_fifo,
		 dout => event_fifo_rd,
		 full => fifofull,
		 empty => fifoempty);
  end generate;

  v5_evfifo :
    if VIRTEX5FXT = TRUE generate
      ev_fifo: event_fifo_v5
	port map (
	  rst => reset_event_fifo,
	  wr_clk => event_clk,
	  rd_clk => OPB_Clk,
	  din => event_fifo_in,
	  wr_en => save_event,
	  rd_en => host_read_fifo,
	  dout => event_fifo_rd,
	  full => fifofull,
	  empty => fifoempty);
    end generate;

  ev_log : event_log
    port map (
      log_in => event_fifo_in,
      log_wr_en => log_event,
      log_wr_clk => event_clk,
      clk => OPB_Clk,
      data_out => event_log_rd,
      addr => OPB_ABus(19 to 27),
      wr_pnt => event_log_wr_pnt,
      wr_overflow => event_log_overflow,
      reset => event_log_reset);
  
  v2p_receiver :
  if VIRTEX2PRO = TRUE generate
    i_receiver : receiver
      generic map (
	ref_on_brefclk => C_EVR_REF_BREFCLK,
	separate_tx_rx => C_EVR_SEPARATE_TX_RX)
      port map (
	brefclk => brefclk,
	brefclk_dcm => brefclk_dcm,
	brefclk2 => brefclk2,
	brefclk2_dcm => brefclk2_dcm,
	refclk_dcm_out => refclk_dcm,
	rxclk_nobuf_out => event_clk_rx_nobuf,
	rxclk_out => event_clk_rx,
	event_clk => event_clk,
	rxpolarity => C_EVR_RXPOLARITY,
	event_code => event_code_rx,
	dbus_data => dbus_data,
	databuf_data => databuf_data,
	databuf_k => databuf_k,
	databuf_ena => databuf_ena,
	databuf_mode => databuf_mode,
	rx_link_ok => rx_link_ok,
	rx_violation => rx_violation,
	rx_clear_viol => reset_violation_irq,
	reset => mgt_reset,
	tx_charisk => mgt_tx_charisk,
	tx_data => mgt_tx_data,
	tx_reset => gnd,
	tx_polarity => C_EVR_TXPOLARITY,
	tx_event_clk => tx_clk,
	RXN => EVRXN,
	RXP => EVRXP,
	TXN => EVTXN,
	TXP => EVTXP);
  end generate;

  ecp2m_receiver :
  if ECP2M = TRUE generate
    i_receiver : receiver_ecp2m
      port map (
	refclkp => refclkp,
	refclkn => refclkn,
	refclk_out => refclk_dcm,
	event_clk_out => event_clk,
	rx_polarity => C_EVR_RXPOLARITY,
	event_code => event_code_rx,
	dbus_data => dbus_data,
	databuf_data => databuf_data,
	databuf_k => databuf_k,
	databuf_ena => databuf_ena,
	databuf_mode => databuf_mode,
	rx_link_ok => rx_link_ok,
	rx_violation => rx_violation,
	rx_clear_viol => reset_violation_irq,
	reset => mgt_reset,
	tx_charisk => mgt_tx_charisk,
	tx_data => mgt_tx_data,
	tx_reset => gnd,
	tx_polarity => C_EVR_TXPOLARITY,
	tx_event_clk => tx_event_clk,
	hdinp0 => hdinp0,
	hdinn0 => hdinn0,
	hdoutp0 => hdoutp0,
	hdoutn0 => hdoutn0,
	hdinp1 => hdinp1,
	hdinn1 => hdinn1,
	hdoutp1 => hdoutp1,
	hdoutn1 => hdoutn1,
	hdinp2 => hdinp2,
	hdinn2 => hdinn2,
	hdoutp2 => hdoutp2,
	hdoutn2 => hdoutn2,
	hdinp3 => hdinp3,
	hdinn3 => hdinn3,
	hdoutp3 => hdoutp3,
	hdoutn3 => hdoutn3
	);
  end generate;

  v5fxt_receiver :
  if VIRTEX5FXT = TRUE generate
    i_receiver : receiver_v5fxt
      port map (
	REFCLKP => BREFCLKP,
	REFCLKN => BREFCLKN,
	REFCLK_OUT => refclk_dcm,
	event_clk_out => event_clk,
	rx_polarity => C_EVR_RXPOLARITY,
	event_code => event_code_rx,
	dbus_data => dbus_data,
	databuf_data => databuf_data,
	databuf_k => databuf_k,
	databuf_ena => databuf_ena,
	databuf_mode => databuf_mode,
	rx_link_ok => rx_link_ok,
	rx_violation => rx_violation,
	rx_clear_viol => reset_violation_irq,
	reset => mgt_reset,
        cdrres => GPIO(6),
        cdrphas => GPIO(7),
	tx_charisk => mgt_tx_charisk,
	tx_data => mgt_tx_data,
	tx_reset => gnd,
	tx_polarity => C_EVR_TXPOLARITY,
	tx_event_clk => tx_event_clk,
	RXN0 => EVRXP,
	RXP0 => EVRXN,
	RXN1 => FBRXN,
	RXP1 => FBRXP,
	TXN0 => EVTXP,
	TXP0 => EVTXN,
	TXN1 => FBTXN,
	TXP1 => FBTXP,
        CONTROL0 => CONTROL0
	);
  end generate;
  
  transmitter_generation:
    if C_EVR_USE_TRANSMITTER = TRUE generate
      i_transmitter : transmitter
        generic map (
          ref_on_brefclk => C_EVR_REF_BREFCLK)
        port map (
          brefclk_dcm => brefclk_dcm,
          brefclk2_dcm => brefclk2_dcm,
          tx_clk => tx_clk,
          tx_event => tx_event,
          tx_dbus => tx_dbus,
          tx_databuf => tx_databuf_data,
          k_databuf => tx_databuf_k,
          ena_databuf => tx_databuf_txena,
          databuf_mode => tx_databuf_mode,
          tx_charisk => mgt_tx_charisk,
          tx_data => mgt_tx_data);
    end generate;

  transmitter_mgt_generation:
    if VIRTEX2PRO = TRUE and C_EVR_USE_TRANSMITTER = TRUE and
      C_EVR_SEPARATE_TX_RX = TRUE generate
      i_mgt_tx : mgt_tx
        generic map (
          ref_on_brefclk => C_EVR_REF_BREFCLK)
        port map(
          BREFCLK => brefclk,
          BREFCLK2 => brefclk2,
          tx_clk => tx_clk,
          POWERDOWN => gnd,
          RXN => FBRXN,
          RXP => FBRXP,
          TXCHARISK => mgt_tx_charisk,
          TXDATA => mgt_tx_data,
          TXRESET => gnd,
          TXPOLARITY => C_EVR_TXPOLARITY,
          TXN => FBTXN,
          TXP => FBTXP);
    end generate;
  
  i_prescalers:
    for i in 0 to C_EVR_PRESCALERS-1 generate
      i_presc: mx_counter
	generic map (
	  counter_range => C_EVR_PRESC_RANGE(i))
	port map (
	  reg_width => reg_prescaler(i)(31 downto 0),
	  pulse_out => prescaler_out(i),
	  ena_out => prescaler_ena_out(i),
	  clk => event_clk,
	  sync => reset_prescalers_event,
	  polarity => prescaler_polarity(i),
	  reset => OPB_Rst);
    end generate;

  i_databuf : databuf_rx
    port map (
      data_out => databuf_data_out,
      addr_in(10 downto 2) => OPB_ABus(21 to 29),
      clk => OPB_Clk,

      databuf_data => databuf_data,
      databuf_k => databuf_k,
      databuf_ena => databuf_ena,
      event_clk => event_clk,
      
      enable => databuf_load,
      disable => databuf_stop,
      bufsize_words => databuf_rxsize,
      rx_enable => databuf_receiving,
      rx_complete => databuf_rx_ready,
      checksum_error => databuf_checksum,

      reset => int_reset);

  i_membuf : memory_buffer
    port map (
      data_in => OPB_DBus(0 to 31),
      data_out => tx_databuf_data_out,
      addr_in => OPB_ABus(0 to 31),
      wr_en => tx_databuf_we,
      clk => OPB_Clk,

      event_data_out => tx_databuf_data,
      event_k_out => tx_databuf_k,
      event_clk => tx_clk,
      buf_clkena => tx_databuf_txena,

      enable => tx_databuf_ena,
      trigger => tx_databuf_trigger,
      bufsize_words => tx_databuf_size(11 downto 2),
      tx_running => tx_databuf_running,
      tx_complete => tx_databuf_complete,
      reset => int_reset);

  i_micrel_load : micrel_load
    port map (
      clk => OPB_Clk,
      data => reg_micrel_word,
      reload => micrel_reload,
      reset => OPB_Rst,
      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS);

  v2p_ref :
  if VIRTEX2PRO = TRUE generate
    i_refclock : refclock
      port map (
	BREFCLKN => BREFCLKN,
	BREFCLKP => BREFCLKP,
	brefclk => brefclk,
	brefclk_dcm => brefclk_dcm);
  end generate;
  
  refclk_out <= brefclk_dcm;
  brefclk_out <= brefclk;

  event_clk_out <= event_clk;
  
  v2p_ref2 :
  if VIRTEX2PRO = TRUE generate
    i_refclock2 : refclock
      port map (
	BREFCLKN => BREFCLK2N,
	BREFCLKP => BREFCLK2P,
	brefclk => brefclk2,
	brefclk_dcm => brefclk2_dcm);
  end generate;

  refclk2_out <= brefclk2_dcm;
  brefclk2_out <= brefclk2;

  cml:
  if VIRTEX2PRO = TRUE generate
  i_mgt_cml:
    for i in 0 to C_EVR_CML_OUTPUTS-1 generate
      i_mgt_cmlout : mgt_cmlout_bram
	port map (
	  clk => OPB_Clk,
	  data_in(23 downto 0) => OPB_DBus(8 to 31),
	  data_out => cml_data(i),
	  addr(1 downto 0) => OPB_ABus(28 to 29),
	  we => cml_we(i),
	  event_clk => event_clk,
	  brefclk => brefclk,
	  brefclk2 => brefclk2,
          refclksel => not cml_refclksel(i),
	  pulse => cml_pulse(i),
	  enable => cml_enable(i),
	  powerdown => cml_pwd(i),
	  freq_mode => cml_freq_mode(i),
	  high_period => cml_high_period(i)(15 downto 0),
	  low_period => cml_low_period(i)(15 downto 0),
	  trig_pos => cml_trig_pos(i)(15 downto 0),
	  trig_level => cml_trig_level(i),
	  RXN => CMLRXN(i),
	  RXP => CMLRXP(i),
	  txreset => cml_txreset(i),
	  TXN => CMLTXN(i),
	  TXP => CMLTXP(i));
    end generate;
  end generate;

  v5fxt_cml :
  if VIRTEX5FXT = TRUE generate
    i_gtx : cml_gtx
      port map (
	REFCLK_out => brefclk2,
	gtxreset => cml_gtxreset,
	tx_data => cml_tx_data,
	tx_clk_out => cml_tx_clk,
	txinhibit => cml_txinhibit,
	plllkdet_out => open,
	rx_data => cml_rx_data,
	rx_clk_out => cml_rx_clk,
	rxreset => cml_rxreset,
	RXREFCLKN => BREFCLK2N,
	RXREFCLKP => BREFCLK2P,
	RDN => CMLRXN,
	RDP => CMLRXP,
	TDN => CMLTXN,
	TDP => CMLTXP);
    i_cml: for i in 0 to C_EVR_CML_OUTPUTS-1 generate
      i_cmlout_bram : gtx_cmlout_bram
	port map (
	  clk => OPB_Clk,
	  data_in => OPB_DBus(0 to 31),
	  data_out => cml_data(i),
	  addr => cml_addr,
	  we => OPB_BE,
	  cs => gtx_cs(i),
	  event_clk => event_clk,
	  tx_data => cml_tx_data(i),
	  pulse => cml_pulse(i),
	  enable => cml_enable(i),
	  freq_mode => cml_freq_mode(i),
	  high_period => cml_high_period(i)(15 downto 0),
	  low_period => cml_low_period(i)(15 downto 0),
	  trig_pos => cml_trig_pos(i)(15 downto 0),
	  trig_level => cml_trig_level(i),
	  pattern_mode => cml_pattern_mode(i),
	  recycle => cml_recycle(i),
	  samples => cml_samples(i));
    end generate;
  end generate;
  
  i_fwd_fifo : fifo_128x8
    port map (
      din => fwd_fifo_data_in,
      rd_clk => tx_clk,
      rd_en => fwd_fifo_rd,
      rst => not fwd_event_ena,
      wr_clk => event_clk,
      wr_en => forward_event,
      dout => fwd_fifo_event_out,
      empty => fwd_fifo_empty,
      full => fwd_fifo_full
      );

  i_CR : config_RAM
    port map (
      clk => OPB_Clk,
      addr => OPB_ABus(21 to 29),
      din => OPB_DBus,
      dout => confRAM_data,
      we => confRAM_we,
      sda => sfp_sda,
      scl => sfp_scl);
  
  refdcm_reset <= not CGLOCK;

  recovered_clock: process (brefclk, brefclk2)
  begin
    if C_EVR_USE_RESYNC then
      if C_EVR_REF_BREFCLK then
	brefrecclk <= brefclk2;
      else
	brefrecclk <= brefclk;
      end if;
    end if;
  end process;
  
  reset_generation : process (refclk_dcm, OPB_Rst, CGLOCK)
  begin
    if OPB_Rst = '1' or CGLOCK = '0' then
      mgt_reset <= '1';
    elsif rising_edge(refclk_dcm) then
      mgt_reset <= '0';
      if OPB_Rst = '1' or CGLOCK = '0' then
	mgt_reset <= '1';
      end if;
    end if;
  end process;

  int_reset_generation : process (event_clk, mgt_reset)
    variable reset_sync : std_logic;
  begin
    if event_clk'event and event_clk = '1' then
      int_reset <= reset_sync;
      reset_sync := mgt_reset;
    end if;
  end process;

  OPB_address_decoding : process (OPB_ABus, OPB_select)
  begin
    Sl_select <= '0';
    if (OPB_ABus and C_BASEMASK) = C_BASEADDR and
      OPB_select = '1' then
      Sl_select <= '1';
    end if;
  end process;

  transfer_acknowledge : process (opb_wr_ack, opb_rd_ack)
  begin
    if opb_wr_ack = '1' or opb_rd_ack = '1' then
      Sl_xferAck <= '1';
    else
      Sl_xferAck <= '0';
    end if;
    Sl_errAck <= '0';
    Sl_retry <= '0';
    Sl_toutSup <= '0';
  end process;

  opb_writing : process (OPB_Clk, OPB_ABus, OPB_DBus, Sl_select,
			 OPB_RNW, OPB_Rst, OPB_BE)
    variable sel                : integer;
    variable opb_wr_ack_sr      : std_logic_vector(1 downto 0);
    variable var_fp_univ_gpdir  : std_logic_vector(31 downto 0) := X"00000000";
    variable var_fp_univ_gpout  : std_logic_vector(31 downto 0) := X"00000000";
  begin
    if rising_edge(OPB_Clk) then
      opb_wr_ack <= '0';
      if Sl_select = '1' and OPB_RNW = '0' and OPB_BE /= "0000" then
	if opb_wr_ack_sr(opb_wr_ack_sr'high) = '1' then
	  opb_wr_ack <= '1';
	end if;
	opb_wr_ack_sr := opb_wr_ack_sr(opb_wr_ack_sr'high - 1 downto 0) & '0';
      else
	opb_wr_ack_sr := (others => '0');
	opb_wr_ack_sr(0) := '1';
      end if;

      reset_event_fifo <= '0';
      sw_reset_timestamp <= '0';
      sw_latch_timestamp <= '0';
      reset_linkchg_irq <= '0';
      reset_pulse_irq <= '0';
      reset_heartbeat_irq <= '0';
      reset_fifofull_irq <= '0';
      reset_violation_irq <= '0';
      databuf_load <= '0';
      databuf_stop <= '0';
      recdcm_psdec <= '0';
      recdcm_psinc <= '0';
      evdcm_psdec <= '0';
      evdcm_psinc <= '0';
      evdcm_sample_reset <= '0';
      micrel_reload <= '0';
      pulse_counter_rst <= (others => '0');
      log_enable_sw <= '0';
      log_disable_sw <= '0';
      event_log_reset <= '0';
      if Sl_select = '1' and OPB_RNW = '0' then
	case OPB_ABus(C_EVR_REG_CMP_LOW to 29) is
	  when C_EVR_REG_STATUS(C_EVR_REG_CMP_LOW to 29) =>
	    -- Status Register
            if OPB_BE(3 - C_EVR_STATUS_LEGACY_VIO/8) = '1' then
	      reset_violation_irq <= OPB_DBus(31 - C_EVR_STATUS_LEGACY_VIO);
	    end if;
	  when C_EVR_REG_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	    -- Control Register
	    if OPB_BE(3 - C_EVR_CTRL_MASTER_ENABLE/8) = '1' then
	      master_enable <= OPB_DBus(31 - C_EVR_CTRL_MASTER_ENABLE);
	    end if;
            if OPB_BE(3 - C_EVR_CTRL_EVENT_FWD_ENA/8) = '1' then
              fwd_event_ena <= OPB_DBus(31 - C_EVR_CTRL_EVENT_FWD_ENA);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_TXLOOPBACK/8) = '1' then
              TXLOOPBACK_int <= OPB_DBus(31 - C_EVR_CTRL_TXLOOPBACK);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_RXLOOPBACK/8) = '1' then
              RXLOOPBACK_int <= OPB_DBus(31 - C_EVR_CTRL_RXLOOPBACK);
            end if;
	    if OPB_BE(3 - C_EVR_CTRL_RESET_TIMESTAMP/8) = '1' then
	      sw_reset_timestamp <= OPB_DBus(31 - C_EVR_CTRL_RESET_TIMESTAMP);
	    end if;
	    if OPB_BE(3 - C_EVR_CTRL_LATCH_TIMESTAMP/8) = '1' then
	      sw_latch_timestamp <= OPB_DBus(31 - C_EVR_CTRL_LATCH_TIMESTAMP);
	    end if;
	    if OPB_BE(3 - C_EVR_CTRL_MAP_RAM_ENABLE/8) = '1' then
	      map_ram_enable <= OPB_DBus(31 - C_EVR_CTRL_MAP_RAM_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVR_CTRL_MAP_RAM_SELECT/8) = '1' then
	      map_ram_select <= OPB_DBus(31 - C_EVR_CTRL_MAP_RAM_SELECT);
	    end if;
	    if OPB_BE(3 - C_EVR_CTRL_RESET_EVENTFIFO/8) = '1' then
	      reset_event_fifo <= OPB_DBus(31 - C_EVR_CTRL_RESET_EVENTFIFO);
	    end if;
            if OPB_BE(3 - C_EVR_CTRL_TS_CLOCK_DBUS/8) = '1' then
              timestamp_clk_dbus <= OPB_DBus(31 - C_EVR_CTRL_TS_CLOCK_DBUS);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_LOG_RESET/8) = '1' then
              event_log_reset <= OPB_DBus(31 - C_EVR_CTRL_LOG_RESET);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_LOG_ENABLE/8) = '1' then
              log_enable_sw <= OPB_DBus(31 - C_EVR_CTRL_LOG_ENABLE);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_LOG_DISABLE/8) = '1' then
              log_disable_sw <= OPB_DBus(31 - C_EVR_CTRL_LOG_DISABLE);
            end if;
            if OPB_BE(3 - C_EVR_CTRL_LOG_STOP_EV_EN/8) = '1' then
              stop_log_event_enable <=
                OPB_DBus(31 - C_EVR_CTRL_LOG_STOP_EV_EN);
            end if;
	  when C_EVR_REG_IRQFLAG(C_EVR_REG_CMP_LOW to 29) =>
	    -- Interrupt Flag Register
	    if OPB_BE(3 - C_EVR_IRQFLAG_LINKCHG/8) = '1' then
	      reset_linkchg_irq <= OPB_DBUS(31 - C_EVR_IRQFLAG_LINKCHG);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_PULSE/8) = '1' then
	      reset_pulse_irq <= OPB_DBus(31 - C_EVR_IRQFLAG_PULSE);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_HEARTBEAT/8) = '1' then
	      reset_heartbeat_irq <= OPB_DBus(31 - C_EVR_IRQFLAG_HEARTBEAT);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_FIFOFULL/8) = '1' then
	      reset_fifofull_irq <= OPB_DBus(31 - C_EVR_IRQFLAG_FIFOFULL);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_VIOLATION/8) = '1' then
	      reset_violation_irq <= OPB_DBus(31 - C_EVR_IRQFLAG_VIOLATION);
	    end if;
	  when C_EVR_REG_IRQENABLE(C_EVR_REG_CMP_LOW to 29) =>
	    -- Interrupt Enable Register
	    if OPB_BE(3 - C_EVR_IRQ_MASTER_ENABLE/8) = '1' then
	      master_irq_enable <= OPB_DBus(31 - C_EVR_IRQ_MASTER_ENABLE);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_LINKCHG/8) = '1' then
	      irq_linkchg_ena <= OPB_DBUS(31 - C_EVR_IRQFLAG_LINKCHG);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_DATABUF/8) = '1' then
	      irq_databuf_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_DATABUF);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_PULSE/8) = '1' then
	      irq_pulse_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_PULSE);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_EVENT/8) = '1' then
	      irq_event_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_EVENT);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_HEARTBEAT/8) = '1' then
	      irq_heartbeat_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_HEARTBEAT);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_FIFOFULL/8) = '1' then
	      irq_fifofull_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_FIFOFULL);
	    end if;
	    if OPB_BE(3 - C_EVR_IRQFLAG_VIOLATION/8) = '1' then
	      irq_rxviol_ena <= OPB_DBus(31 - C_EVR_IRQFLAG_VIOLATION);
	    end if;
          when C_EVR_REG_PULSE_IRQ_MAP(C_EVR_REG_CMP_LOW to 29) =>
            -- Pulse Interrupt Map Register
            if OPB_BE(3) = '1' then
              reg_pulse_irq_map <= OPB_DBus(32 - C_SIGNAL_MAP_BITS to 31);
            end if;
	  when C_EVR_REG_DATABUF_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	    -- Databuffer Control Register
	    if OPB_BE(3 - C_EVR_DATABUF_LOAD/8) = '1' then
	      databuf_load <= OPB_DBus(31 - C_EVR_DATABUF_LOAD);
	    end if;
	    if OPB_BE(3 - C_EVR_DATABUF_STOP/8) = '1' then
	      databuf_stop <= OPB_DBus(31 - C_EVR_DATABUF_STOP);
	    end if;
	    if OPB_BE(3 - C_EVR_DATABUF_MODE/8) = '1' then
	      databuf_mode <= OPB_DBus(31 - C_EVR_DATABUF_MODE);
	    end if;
	  when C_EVR_REG_TXDATABUF_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	    -- Transmit Databuffer Control Register
	    if OPB_BE(3 - C_EVR_TXDATABUF_TRIGGER/8) = '1' then
	      tx_databuf_trigger <= OPB_DBus(31 - C_EVR_TXDATABUF_TRIGGER);
	    end if;
	    if OPB_BE(3 - C_EVR_TXDATABUF_ENA/8) = '1' then
	      tx_databuf_ena <= OPB_DBus(31 - C_EVR_TXDATABUF_ENA);
	    end if;
	    if OPB_BE(3 - C_EVR_TXDATABUF_MODE/8) = '1' then
	      tx_databuf_mode <= OPB_DBus(31 - C_EVR_TXDATABUF_MODE);
	    end if;
	    -- Databuffer Transfer Size
	    for i in 2 to 3 loop
	      if OPB_BE(i) = '1' then
		tx_databuf_size(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	    tx_databuf_size(31 downto 12) <= X"00000";
	    tx_databuf_size(1 downto 0) <= "00";
	  when C_EVR_REG_EVCNT_PRESC(C_EVR_REG_CMP_LOW to 29) =>
	    -- Timestamp event counter prescaler
	    for i in 2 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_evcnt_presc(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
--	  when C_EVR_REG_EVCNT_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
--	    -- Timestamp event counter control register
	  when C_EVR_REG_USEC_DIVIDER(C_EVR_REG_CMP_LOW to 29) =>
	    -- uSec Divider Register
	    for i in 2 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_usec_divider(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	  when C_EVR_REG_CLOCK_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	    -- Clock Control Register
	    if C_EVR_USE_RESYNC then
	      if OPB_BE(3 - C_EVR_CLKCTRL_RECDCM_PSDEC/8) = '1' then
		recdcm_psdec <= OPB_DBus(31 - C_EVR_CLKCTRL_RECDCM_PSDEC);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_RECDCM_PSINC/8) = '1' then
		recdcm_psinc <= OPB_DBus(31 - C_EVR_CLKCTRL_RECDCM_PSINC);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_RECDCM_RESET/8) = '1' then
		recdcm_reset <= OPB_DBus(31 - C_EVR_CLKCTRL_RECDCM_RESET);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_EVDCM_PSDEC/8) = '1' then
		evdcm_psdec <= OPB_DBus(31 - C_EVR_CLKCTRL_EVDCM_PSDEC);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_EVDCM_PSINC/8) = '1' then
		evdcm_psinc <= OPB_DBus(31 - C_EVR_CLKCTRL_EVDCM_PSINC);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_EVDCM_SRUN/8) = '1' then
		evdcm_sample_run <= OPB_DBus(31 - C_EVR_CLKCTRL_EVDCM_SRUN);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_EVDCM_SRES/8) = '1' then
		evdcm_sample_reset <= OPB_DBus(31 - C_EVR_CLKCTRL_EVDCM_SRES);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_EVDCM_RES/8) = '1' then
		evdcm_reset <= OPB_DBus(31 - C_EVR_CLKCTRL_EVDCM_RES);
	      end if;
	      if OPB_BE(3 - C_EVR_CLKCTRL_USE_RXRECCLK/8) = '1' then
		use_rxrecclk <= OPB_DBus(31 - C_EVR_CLKCTRL_USE_RXRECCLK);
	      end if;
	    end if;
	  when C_EVR_REG_FRACDIV(C_EVR_REG_CMP_LOW to 29) =>
	    -- Micrel SY87739 Fractional Divider Configuration Word
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		reg_micrel_word(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	    if OPB_BE(3) = '1' then
	      micrel_reload <= '1';
	    end if;
	  when C_EVR_REG_RX_INIT_PS(C_EVR_REG_CMP_LOW to 29) =>
	    -- Initial Phase Shift for Event clock recovery DCM
	    if C_EVR_USE_RESYNC then
	      if OPB_BE(3) = '1' then
		recdcm_init_ps(7 downto 0) <=
		  OPB_DBus(24 to 31);
	      end if;
	    end if;
          when C_EVR_REG_GPIO_DIR(C_EVR_REG_CMP_LOW to 29) =>
            -- Universal I/O GPIO direction (0 = input, 1 = output)
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		var_fp_univ_gpdir(31-i*8 downto 24-i*8) :=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
          when C_EVR_REG_GPIO_OUT(C_EVR_REG_CMP_LOW to 29) =>
            -- Universal I/O GPIO output
	    for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		var_fp_univ_gpout(31-i*8 downto 24-i*8) :=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
          when C_DIAG_REG_CNT_RESET(C_EVR_REG_CMP_LOW to 29) =>
            for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		pulse_counter_rst(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
          when C_DIAG_REG_CNT_ENA(C_EVR_REG_CMP_LOW to 29) =>
            for i in 0 to 3 loop
	      if OPB_BE(i) = '1' then
		pulse_counter_ena(31-i*8 downto 24-i*8) <=
		  OPB_DBus(i*8 to i*8+7);
	      end if;
	    end loop;
	  when others =>
	end case;
      end if;
      
      if OPB_Rst = '1' then
	master_enable <= '0';
        fwd_event_ena <= '0';
        TXLOOPBACK_int <= '0';
        RXLOOPBACK_int <= '0';
	map_ram_enable <= '0';
	map_ram_select <= '0';
	master_irq_enable <= '0';
	irq_linkchg_ena <= '0';
	irq_databuf_ena <= '0';
	irq_pulse_ena <= '0';
	irq_event_ena <= '0';
	irq_heartbeat_ena <= '0';
	irq_fifofull_ena <= '0';
	irq_rxviol_ena <= '0';
	databuf_mode <= '0';
	reg_evcnt_presc <= (others => '0');
	reg_usec_divider <= C_EVR_USEC_DIVIDER;
	recdcm_reset <= '0';
	evdcm_sample_run <= '0';
	evdcm_reset <= '0';
	use_rxrecclk <= '0';
	reg_micrel_word <= C_EVR_MICREL_WORD;
        var_fp_univ_gpdir := (others => '0');
        var_fp_univ_gpout := (others => '0');
        pulse_counter_ena <= (others => '0');
        stop_log_event_enable <= '0';
        event_log_reset <= '1';
      end if;

      if OPB_Rst = '1' then
	reset_event_fifo <= '1';
      end if;
    end if;

    if C_EVR_GPIOS < 32 then
      var_fp_univ_gpdir(31 downto C_EVR_GPIOS) := (others => '0');
      var_fp_univ_gpout(31 downto C_EVR_GPIOS) := (others => '0');
    end if;

    reg_fp_univ_gpdir <= var_fp_univ_gpdir;
    reg_fp_univ_gpout <= var_fp_univ_gpout;

    if C_DIAG_COUNTERS < 32 then
      pulse_counter_rst(31 downto C_DIAG_COUNTERS) <= (others => '0');
      pulse_counter_ena(31 downto C_DIAG_COUNTERS) <= (others => '0');
    end if;
  end process;

  cml_register_wr:
    if C_EVR_CML_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_BE, OPB_Rst)
	variable sel    : integer range 0 to C_EVR_CML_OUTPUTS-1;
	variable patsel : integer range 0 to C_EVR_CML_OUTPUTS-1; 
     begin
	-- CML Pattern Registers
	sel := conv_integer(OPB_ABus(C_EVR_REG_CML_INDEX_LOW to
				     C_EVR_REG_CML_INDEX_HIGH));
	patsel := conv_integer(OPB_ABus(C_EVR_REG_CMLPAT_INDEX_LOW to
				     C_EVR_REG_CMLPAT_INDEX_HIGH));
	for i in 0 to C_EVR_CML_OUTPUTS-1 loop
	  cml_we(i) <= "000";
	end loop;
	gtx_cs <= (others => '0');
	cml_addr <= OPB_ABUS(17 to 28);
	if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					C_EVR_REG_CML_CMP_HIGH) =
	  C_EVR_REG_CML_BASE(C_EVR_REG_CMP_LOW to C_EVR_REG_CML_CMP_HIGH) and
	  OPB_RNW = '0' and OPB_ABus(C_EVR_REG_CML_CTRL_SEL) = '0' then
	  if sel >= 0 and sel < C_EVR_CML_OUTPUTS then
	    cml_we(sel)(2 downto 0) <= OPB_BE(1 to 3);
	    gtx_cs(sel) <= '1';
	    cml_addr(13 downto 4) <= (others => '0');
	  end if;
	end if;
	if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					C_EVR_REG_CMLPAT_CMP_HIGH) =
	  C_EVR_REG_CMLPAT_BASE(C_EVR_REG_CMP_LOW to
				C_EVR_REG_CMLPAT_CMP_HIGH) then
	  gtx_cs(patsel) <= '1';
	end if;
	if rising_edge(OPB_Clk) then
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_CML_CMP_HIGH) =
	    C_EVR_REG_CML_BASE(C_EVR_REG_CMP_LOW to C_EVR_REG_CML_CMP_HIGH) and
	    OPB_RNW = '0' then
	    if sel >= 0 and sel < C_EVR_CML_OUTPUTS then
	      case OPB_ABus(C_EVR_REG_CML_CTRL_SEL to 29) is
		when C_EVR_REG_CML_CTRL_ADDR =>
		  if OPB_BE(0) = '1' then
		    cml_trig_pos(sel)(15 downto 8) <= OPB_DBus(0 to 7);
		  end if;
		  if OPB_BE(1) = '1' then
		    cml_trig_pos(sel)(7 downto 0) <= OPB_DBus(8 to 15);
		  end if;
		  if OPB_BE(3 - C_EVR_CMLCTRL_REFCLKSEL/8) = '1' then
		    cml_refclksel(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_REFCLKSEL);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_RESET/8) = '1' then
		    cml_txreset(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_RESET);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_POWERDOWN/8) = '1' then
		    cml_pwd(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_POWERDOWN);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_ENABLE/8) = '1' then
		    cml_enable(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_ENABLE);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_FREQ_MODE/8) = '1' then
		    cml_freq_mode(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_FREQ_MODE);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_TRIG_LEVEL/8) = '1' then
		    cml_trig_level(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_TRIG_LEVEL);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_PATTERN_MODE/8) = '1' then
		    cml_pattern_mode(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_PATTERN_MODE);
		  end if;	
		  if OPB_BE(3 - C_EVR_CMLCTRL_RECYCLE/8) = '1' then
		    cml_recycle(sel) <= OPB_DBus(31 - C_EVR_CMLCTRL_RECYCLE);
		  end if;
		when C_EVR_REG_CML_PERIOD_ADDR =>
		  if OPB_BE(0) = '1' then
		    cml_high_period(sel)(15 downto 8) <= OPB_DBus(0 to 7);
		  end if;
		  if OPB_BE(1) = '1' then
		    cml_high_period(sel)(7 downto 0) <= OPB_DBus(8 to 15);
		  end if;
		  if OPB_BE(2) = '1' then
		    cml_low_period(sel)(15 downto 8) <= OPB_DBus(16 to 23);
		  end if;
		  if OPB_BE(3) = '1' then
		    cml_low_period(sel)(7 downto 0) <= OPB_DBus(24 to 31);
		  end if;
		when C_EVR_REF_CML_SAMPL_ADDR =>
		  if OPB_BE(2) = '1' then
		    cml_samples(sel)(10 downto 8) <= OPB_DBus(21 to 23);
		  end if;
		  if OPB_BE(3) = '1' then
		    cml_samples(sel)(7 downto 0) <= OPB_DBus(24 to 31);
		  end if;
		when others =>
	      end case;
	    end if;
	  end if;
	  if OPB_Rst = '1' then
	    for i in 0 to C_EVR_CML_OUTPUTS-1 loop
	      cml_refclksel(i) <= '0';
	      cml_enable(i) <= '0';
	      cml_freq_mode(i) <= '0';
	      cml_trig_level(i) <= '0';
	      cml_pattern_mode(i) <= '0';
	      cml_recycle(i) <= '0';
	      cml_pwd(i) <= '1';
	      cml_txreset(i) <= '1';
	      cml_high_period(i) <= (others => '0');
	      cml_low_period(i) <= (others => '0');
	      cml_trig_pos(i) <= (others => '0');
	      cml_samples(i) <= (others => '0');
	    end loop;
	  end if;
	end if;
      end process;
    end generate;

  map_ram_wr: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			    OPB_BE)
    variable sel : integer range 0 to C_EVR_MAP_RAMS-1;
  begin
    -- Map RAM access
    sel := conv_integer(OPB_ABus(C_EVR_REG_MAP_INDEX_LOW to
				 C_EVR_REG_MAP_INDEX_HIGH));
    for i in 0 to C_EVR_MAP_RAMS-1 loop
      map_ram_we(i) <= (others => '0');
    end loop;
    if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
				    C_EVR_REG_MAP_CMP_HIGH) =
      C_EVR_REG_MAP_BASE(C_EVR_REG_CMP_LOW to
			 C_EVR_REG_MAP_CMP_HIGH) and OPB_RNW = '0' then
      if sel >= 0 and sel < C_EVR_MAP_RAMS then
	map_ram_we(sel)(3 downto 0) <= OPB_BE(0 to 3);
      end if;
    end if;
  end process;

  fp_input_map_register_wr:
    if C_EVR_TTL_INPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVR_TTL_INPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVR_REG_FPIN_INDEX_LOW to
				       C_EVR_REG_FPIN_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_FPIN_CMP_HIGH) =
	    C_EVR_REG_FPIN_BASE(C_EVR_REG_CMP_LOW to
				C_EVR_REG_FPIN_CMP_HIGH) and OPB_RNW = '0' then
	    if sel >= 0 and sel < C_EVR_TTL_INPUTS then
	      for i in 0 to 3 loop
		if OPB_BE(i) = '1' then
		  reg_fp_input_map(sel)(31-i*8 downto 24-i*8) <=
		    OPB_DBus(i*8 to i*8+7);
		end if;
	      end loop;
	    end if;
	  end if;
	end if;
      end process;
    end generate;
  
  fp_map_register_wr:
    if C_EVR_TTL_OUTPUTS+C_EVR_CML_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVR_TTL_OUTPUTS+C_EVR_CML_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVR_REG_FPOUT_INDEX_LOW to
				       C_EVR_REG_FPOUT_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_FPOUT_CMP_HIGH) =
	    C_EVR_REG_FPOUT_BASE(C_EVR_REG_CMP_LOW to
				 C_EVR_REG_FPOUT_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_fp_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_fp_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;
  
  univ_map_register_wr:
    if C_EVR_UNIV_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVR_UNIV_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVR_REG_UNIV_INDEX_LOW to
				       C_EVR_REG_UNIV_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_UNIV_CMP_HIGH) =
	    C_EVR_REG_UNIV_BASE(C_EVR_REG_CMP_LOW to
				C_EVR_REG_UNIV_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_univ_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_univ_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;

  tb_map_register_wr:
    if C_EVR_TB_OUTPUTS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVR_TB_OUTPUTS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVR_REG_TBOUT_INDEX_LOW to
				       C_EVR_REG_TBOUT_INDEX_HIGH) & '0');
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_TBOUT_CMP_HIGH) =
	    C_EVR_REG_TBOUT_BASE(C_EVR_REG_CMP_LOW to
				 C_EVR_REG_TBOUT_CMP_HIGH) and OPB_RNW = '0' then
	    if OPB_BE(1) = '1' then
	      reg_tb_output_map(sel)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(16-C_SIGNAL_MAP_BITS to 15);
	    end if;
	    if OPB_BE(3) = '1' then
	      reg_tb_output_map(sel+1)(C_SIGNAL_MAP_BITS-1 downto 0) <=
		OPB_DBus(32-C_SIGNAL_MAP_BITS to 31);
	    end if;
	  end if;
	end if;
      end process;
    end generate;
  
  pulse_registers_wr: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			       OPB_DBus, OPB_BE)
    variable sel : integer range 0 to C_EVR_PULSE_GENS-1;
  begin
    if rising_edge(OPB_Clk) then
      for i in 0 to C_EVR_PULSE_GENS-1 loop
        pulse_sw_set(i) <= '0';
        pulse_sw_reset(i) <= '0';
      end loop;
      sel := conv_integer(OPB_ABus(C_EVR_REG_PULSE_INDEX_LOW to
				   C_EVR_REG_PULSE_INDEX_HIGH));
      if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
				      C_EVR_REG_PULSE_CMP_HIGH) =
	C_EVR_REG_PULSE_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_REG_PULSE_CMP_HIGH) and OPB_RNW = '0' then
	for byte in 0 to 3 loop
	  if OPB_BE(byte) = '1' then
	    case OPB_ABus(28 to 29) is
	      when "00" =>
                if OPB_BE(3 - C_EVR_PULSE_SW_SET/8) = '1' then
                  pulse_sw_set(sel) <= OPB_DBus(31 - C_EVR_PULSE_SW_SET);
                end if;	
                if OPB_BE(3 - C_EVR_PULSE_SW_RESET/8) = '1' then
                  pulse_sw_reset(sel) <= OPB_DBus(31 - C_EVR_PULSE_SW_RESET);
                end if;         
                if OPB_BE(3 - C_EVR_PULSE_POLARITY/8) = '1' then
                  pulse_polarity(sel) <= OPB_DBus(31 - C_EVR_PULSE_POLARITY);
                end if;         
                if OPB_BE(3 - C_EVR_PULSE_MAP_RESET_ENA/8) = '1' then
                  pulse_reset_ena(sel) <= OPB_DBus(31 - C_EVR_PULSE_MAP_RESET_ENA);
                end if;         
                if OPB_BE(3 - C_EVR_PULSE_MAP_SET_ENA/8) = '1' then
                  pulse_set_ena(sel) <= OPB_DBus(31 - C_EVR_PULSE_MAP_SET_ENA);
                end if;         
                if OPB_BE(3 - C_EVR_PULSE_MAP_TRIG_ENA/8) = '1' then
                  pulse_trig_ena(sel) <= OPB_DBus(31 - C_EVR_PULSE_MAP_TRIG_ENA);
                end if;         
                if OPB_BE(3 - C_EVR_PULSE_ENA/8) = '1' then
                  pulse_enable(sel) <= OPB_DBus(31 - C_EVR_PULSE_ENA);
                end if;         
	      when "01" => pulse_presc(sel)(31-byte*8 downto 24-byte*8)
			     <= OPB_DBus(byte*8 to (byte+1)*8-1);
	      when "10" => pulse_delay(sel)(31-byte*8 downto 24-byte*8)
			     <= OPB_DBus(byte*8 to (byte+1)*8-1);
	      when others => pulse_width(sel)(31-byte*8 downto 24-byte*8)
			     <= OPB_DBus(byte*8 to (byte+1)*8-1);
	    end case;
	  end if;
	end loop;
      end if;
      -- Clear unused bits to save FPGA resources
      for i in 0 to C_EVR_PULSE_GENS-1 loop
        if C_EVR_PULSE_PRESC_RANGE(i) < 32 then
          pulse_presc(i)(31 downto C_EVR_PULSE_PRESC_RANGE(i))
            <= gnd32(31 downto C_EVR_PULSE_PRESC_RANGE(i));
        end if;
        if C_EVR_PULSE_DELAY_RANGE(i) < 32 then
          pulse_delay(i)(31 downto C_EVR_PULSE_DELAY_RANGE(i))
            <= gnd32(31 downto C_EVR_PULSE_DELAY_RANGE(i));
        end if;
        if C_EVR_PULSE_WIDTH_RANGE(i) < 32 then
          pulse_width(i)(31 downto C_EVR_PULSE_WIDTH_RANGE(i))
            <= gnd32(31 downto C_EVR_PULSE_WIDTH_RANGE(i));
        end if;
      end loop;
    end if;
  end process;

  prescaler_register:
    if C_EVR_PRESCALERS > 0 generate
      process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
	       OPB_DBus, OPB_BE)
	variable sel : integer range 0 to C_EVR_PRESCALERS-1;
      begin
	if rising_edge(OPB_Clk) then
	  sel := conv_integer(OPB_ABus(C_EVR_REG_PRESC_INDEX_LOW to
				       C_EVR_REG_PRESC_INDEX_HIGH));
	  if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
					  C_EVR_REG_PRESC_CMP_HIGH) =
	    C_EVR_REG_PRESCALER_BASE(C_EVR_REG_CMP_LOW to
				     C_EVR_REG_PRESC_CMP_HIGH) and
	    OPB_RNW = '0' then
	    for byte in 0 to 3 loop
	      if OPB_BE(byte) = '1' then
		reg_prescaler(sel)(31-byte*8 downto 24-byte*8) <=
		  OPB_DBus(byte*8 to byte*8+7);
	      end if;
	    end loop;
	  end if;
	  
	  -- Clear unused bits to save FPGA resources
	  for i in 0 to C_EVR_PRESCALERS-1 loop
	    if C_EVR_PRESC_RANGE(i) < 32 then
	      reg_prescaler(sel)(31 downto C_EVR_PRESC_RANGE(i))
		<= gnd32(31 downto C_EVR_PRESC_RANGE(i));
	    end if;
	  end loop;
	end if;
      end process;

    end generate;
  
  tx_databuf_writing: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
                               OPB_BE)
  begin
    tx_databuf_we <= (others => '0');
    if Sl_select = '1' and OPB_RNW = '0' and
      OPB_ABus(C_EVR_REG_CMP_LOW to C_EVR_TXDATABUF_CMP_HIGH) =
      C_EVR_TXDATABUF_BASE(C_EVR_REG_CMP_LOW to C_EVR_TXDATABUF_CMP_HIGH) then
      tx_databuf_we(3 downto 0) <= OPB_BE(0 to 3);
    end if;
  end process;

  config_RAM_wr: process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			  OPB_BE)
  begin
    confRAM_we <= (others => '0');
    if Sl_select = '1' and OPB_ABus(C_EVR_REG_CMP_LOW to
				    C_EVR_REG_CONFRAM_CMP_HIGH) =
      C_EVR_REG_CONFRAM_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_REG_CONFRAM_CMP_HIGH) and OPB_RNW = '0' then
      confRAM_we(3 downto 0) <= OPB_BE(0 to 3);
    end if;
  end process;

  OPB_reading : process (OPB_Clk, OPB_ABus, Sl_select, OPB_RNW,
			 reg_control, flag_lostheartbeat,
			 flag_event_irq, map_ram_enable, map_ram_select,
			 host_ram_select, map_ram_autoinc,
			 flag_pulse_irq, flag_lostlock, flag_fifofull,
			 flag_fifonotempty, rx_violation,
			 reg_clock_control, reg_usec_divider,
			 reg_prescaler, sfp_moddef0,
			 databuf_data_out, databuf_receiving,
			 databuf_rx_ready, databuf_checksum,
			 databuf_rxsize, evdcm_locked, evdcm_stopped,
			 cml_enable, cml_pwd, cml_txreset, cml_data,
			 cml_freq_mode, cml_high_period, cml_low_period,
			 cml_trig_pos, cml_trig_level, cml_pattern_mode,
			 cml_recycle, cml_samples,
			 reg_univ_output_map,
                         int_pulse, pulse_polarity, pulse_reset_ena,
                         pulse_set_ena, pulse_trig_ena, pulse_enable,
                         reg_fp_univ_gpin, pulse_counter_cnt,
                         pulse_counter_rst, pulse_counter_ena,
                         tx_databuf_data_out, tx_databuf_ena,
                         tx_databuf_mode, tx_databuf_size,
                         tx_databuf_running, tx_databuf_complete,
                         stop_log_event_enable, log_stopped,
                         fwd_event_ena, TXLOOPBACK_int, RXLOOPBACK_int)
    variable opb_rd_ack_sr     : std_logic_vector(3 downto 0);
    variable data_out          : std_logic_vector(31 downto 0);
    variable cml_sel           : integer range 0 to C_EVR_CML_OUTPUTS-1;
    variable cml_patsel        : integer range 0 to C_EVR_CML_OUTPUTS-1;
    variable cml_data_out      : std_logic_vector(31 downto 0);
    variable map_ram_sel       : integer range 0 to C_EVR_MAP_RAMS-1;
    variable fp_input_sel      : integer range 0 to C_EVR_TTL_INPUTS-1;
    variable fp_input_data_out : std_logic_vector(31 downto 0);
    variable fp_map_sel        : integer range 0 to C_EVR_TTL_OUTPUTS+
      C_EVR_CML_OUTPUTS-1;
    variable fp_map_data_out   : std_logic_vector(31 downto 0);
    variable fp_univ_sel       : integer range 0 to C_EVR_UNIV_OUTPUTS-1;
    variable fp_univ_data_out  : std_logic_vector(31 downto 0);
    variable tb_map_sel        : integer range 0 to C_EVR_TB_OUTPUTS-1;
    variable tb_map_data_out   : std_logic_vector(31 downto 0);
    variable pulse_sel         : integer range 0 to C_EVR_PULSE_GENS-1;
    variable pulse_data_out    : std_logic_vector(31 downto 0);
    variable presc_sel         : integer range 0 to C_EVR_PRESCALERS-1;
    variable presc_data_out    : std_logic_vector(31 downto 0);
    variable cnt_sel           : integer range 0 to C_DIAG_COUNTERS-1;
    variable cnt_data_out      : std_logic_vector(31 downto 0);
    variable pulse_int         : std_logic_vector(C_DIAG_COUNTERS-1 downto 0);
    variable OPB_BE_1          : std_logic_vector(0 to 3);
    variable log_data_out      : std_logic_vector(31 downto 0);
  begin
    if rising_edge(OPB_Clk) then
      if C_EVR_CML_OUTPUTS > 0 then
	cml_sel := conv_integer(OPB_ABus(C_EVR_REG_CML_INDEX_LOW to
					 C_EVR_REG_CML_INDEX_HIGH));
	cml_patsel := conv_integer(OPB_ABus(C_EVR_REG_CMLPAT_INDEX_LOW to
					    C_EVR_REG_CMLPAT_INDEX_HIGH));
	cml_data_out := (others => '0');
	case OPB_ABus(C_EVR_REG_CML_CTRL_SEL to 29) is
	  when "000" | "001" | "010" | "011" =>
	    cml_data_out := cml_data(cml_sel);
	  when C_EVR_REG_CML_CTRL_ADDR =>
	    cml_data_out(31 downto 16) := cml_trig_pos(cml_sel)(15 downto 0);
            cml_data_out(C_EVR_CMLCTRL_RECYCLE) := cml_recycle(cml_sel);
            cml_data_out(C_EVR_CMLCTRL_TRIG_LEVEL) := cml_trig_level(cml_sel);
            cml_data_out(C_EVR_CMLCTRL_PATTERN_MODE) := cml_pattern_mode(cml_sel);
            cml_data_out(C_EVR_CMLCTRL_FREQ_MODE) := cml_freq_mode(cml_sel);
	    cml_data_out(C_EVR_CMLCTRL_REFCLKSEL) := cml_refclksel(cml_sel);
	    cml_data_out(C_EVR_CMLCTRL_RESET) := cml_txreset(cml_sel);
	    cml_data_out(C_EVR_CMLCTRL_POWERDOWN) := cml_pwd(cml_sel);
	    cml_data_out(C_EVR_CMLCTRL_ENABLE) := cml_enable(cml_sel);
	  when C_EVR_REG_CML_PERIOD_ADDR =>
	    cml_data_out(31 downto 16) := cml_high_period(cml_sel)(15 downto 0);
	    cml_data_out(15 downto 0) := cml_low_period(cml_sel)(15 downto 0);
	  when C_EVR_REF_CML_SAMPL_ADDR =>
	    cml_data_out(10 downto 0) := cml_samples(cml_sel);
	  when others =>
	end case;
	if OPB_ABus(C_EVR_REG_CMP_LOW to C_EVR_REG_CMLPAT_CMP_HIGH) =
	  C_EVR_REG_CMLPAT_BASE(C_EVR_REG_CMP_LOW to
				C_EVR_REG_CMLPAT_CMP_HIGH) then
	  cml_data_out := cml_data(cml_patsel);
	end if;
      end if;

      if C_EVR_TTL_INPUTS > 0 then
	fp_input_sel := conv_integer(OPB_ABus(C_EVR_REG_FPIN_INDEX_LOW to
					      C_EVR_REG_FPIN_INDEX_HIGH));
	fp_input_data_out := (others => '0');
	fp_input_data_out := reg_fp_input_map(fp_input_sel);
      end if;

      if C_EVR_TTL_OUTPUTS > 0 then
	fp_map_sel := conv_integer(OPB_ABus(C_EVR_REG_FPOUT_INDEX_LOW to
					    C_EVR_REG_FPOUT_INDEX_HIGH) & '0');
	fp_map_data_out := (others => '0');
	fp_map_data_out(C_SIGNAL_MAP_BITS+15 downto 16) :=
	  reg_fp_output_map(fp_map_sel);
	fp_map_data_out(C_SIGNAL_MAP_BITS-1 downto 0) :=
	  reg_fp_output_map(fp_map_sel+1);
      end if;

      if C_EVR_UNIV_OUTPUTS > 0 then
	fp_univ_sel := conv_integer(OPB_ABus(C_EVR_REG_UNIV_INDEX_LOW to
					     C_EVR_REG_UNIV_INDEX_HIGH) & '0');
	fp_univ_data_out := (others => '0');
	fp_univ_data_out(C_SIGNAL_MAP_BITS+15 downto 16) :=
	  reg_univ_output_map(fp_univ_sel);
	fp_univ_data_out(C_SIGNAL_MAP_BITS-1 downto 0) :=
	  reg_univ_output_map(fp_univ_sel+1);
      end if;

      if C_EVR_TB_OUTPUTS > 0 then
	tb_map_sel := conv_integer(OPB_ABus(C_EVR_REG_TBOUT_INDEX_LOW to
					    C_EVR_REG_TBOUT_INDEX_HIGH) & '0');
	tb_map_data_out := (others => '0');
	tb_map_data_out(C_SIGNAL_MAP_BITS+15 downto 16) := 
	  reg_tb_output_map(tb_map_sel);
	tb_map_data_out(C_SIGNAL_MAP_BITS-1 downto 0) := 
	  reg_tb_output_map(tb_map_sel+1);
      end if;

      pulse_sel := conv_integer(OPB_ABus(C_EVR_REG_PULSE_INDEX_LOW to
                                         C_EVR_REG_PULSE_INDEX_HIGH));
      pulse_data_out := (others => '0');
      case OPB_ABus(28 to 29) is
        when "00" =>
          pulse_data_out(C_EVR_PULSE_OUT) := int_pulse(pulse_sel);
          pulse_data_out(C_EVR_PULSE_POLARITY) := pulse_polarity(pulse_sel);
          pulse_data_out(C_EVR_PULSE_MAP_RESET_ENA) :=
            pulse_reset_ena(pulse_sel);
          pulse_data_out(C_EVR_PULSE_MAP_SET_ENA) :=
            pulse_set_ena(pulse_sel);
          pulse_data_out(C_EVR_PULSE_MAP_TRIG_ENA) :=
            pulse_trig_ena(pulse_sel);
          pulse_data_out(C_EVR_PULSE_ENA) := pulse_enable(pulse_sel);
        when "01" => pulse_data_out := pulse_presc(pulse_sel);
        when "10" => pulse_data_out := pulse_delay(pulse_sel);
        when others => pulse_data_out := pulse_width(pulse_sel);
      end case;

      if C_EVR_PRESCALERS > 0 then
	presc_sel := conv_integer(OPB_ABus(C_EVR_REG_PRESC_INDEX_LOW to
					   C_EVR_REG_PRESC_INDEX_HIGH));
	presc_data_out := reg_prescaler(presc_sel);
      end if;

      if C_DIAG_COUNTERS > 0 then
        cnt_sel := conv_integer(OPB_ABus(C_DIAG_COUNTERS_INDEX_LOW to
					 C_DIAG_COUNTERS_INDEX_HIGH));
        cnt_data_out := (others => '0');
        cnt_data_out(C_DIAG_COUNTER_BITS-1 downto 0) :=
          pulse_counter_cnt(cnt_sel);
      end if;

      log_data_out := (others => '0');
      case OPB_ABus(28 to 29) is
        when "00" => log_data_out := event_log_rd(79 downto 48);
        when "01" => log_data_out := event_log_rd(47 downto 16);
        when "10" => log_data_out(15 downto 0) := event_log_rd(15 downto 0);
        when others => log_data_out := (others => '0');
      end case;
      
      data_out := (others => '0');
      host_read_fifo <= '0';
      case OPB_ABus(C_EVR_REG_CMP_LOW to 29) is
	when C_EVR_REG_STATUS(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_STATUS_DBUS_HIGH downto C_EVR_STATUS_DBUS_HIGH -
		   C_EVR_DBUS_BITS + 1) := dbus_data;
          data_out(C_EVR_STATUS_LEGACY_VIO) := rx_violation;
	  data_out(C_EVR_STATUS_MODDEF0) := sfp_moddef0;
	  data_out(C_EVR_STATUS_LINK_UP) := rx_link_ok;
          data_out(C_EVR_STATUS_LOG_STOPPED) := log_stopped;
	when C_EVR_REG_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_CTRL_MASTER_ENABLE) := master_enable;
          data_out(C_EVR_CTRL_EVENT_FWD_ENA) := fwd_event_ena;
          data_out(C_EVR_CTRL_TXLOOPBACK) := TXLOOPBACK_int;
          data_out(C_EVR_CTRL_RXLOOPBACK) := RXLOOPBACK_int;
          data_out(C_EVR_CTRL_TS_CLOCK_DBUS) := timestamp_clk_dbus;
	  data_out(C_EVR_CTRL_MAP_RAM_ENABLE) := map_ram_enable;
	  data_out(C_EVR_CTRL_MAP_RAM_SELECT) := map_ram_select;
          data_out(C_EVR_CTRL_LOG_STOP_EV_EN) := stop_log_event_enable;
	when C_EVR_REG_IRQFLAG(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_IRQFLAG_LINKCHG) := flag_linkchg_irq;
	  data_out(C_EVR_IRQFLAG_DATABUF) := databuf_rx_ready;
	  data_out(C_EVR_IRQFLAG_PULSE) := flag_pulse_irq;
	  data_out(C_EVR_IRQFLAG_EVENT) := flag_event_irq;
	  data_out(C_EVR_IRQFLAG_HEARTBEAT) := flag_lostheartbeat;
	  data_out(C_EVR_IRQFLAG_FIFOFULL) := flag_fifofull;
	  data_out(C_EVR_IRQFLAG_VIOLATION) := rx_violation;
	when C_EVR_REG_IRQENABLE(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_IRQ_MASTER_ENABLE) := master_irq_enable;
	  data_out(C_EVR_IRQFLAG_LINKCHG) := irq_linkchg_ena;
	  data_out(C_EVR_IRQFLAG_DATABUF) := irq_databuf_ena;
	  data_out(C_EVR_IRQFLAG_PULSE) := irq_pulse_ena;
	  data_out(C_EVR_IRQFLAG_EVENT) := irq_event_ena;
	  data_out(C_EVR_IRQFLAG_HEARTBEAT) := irq_heartbeat_ena;
	  data_out(C_EVR_IRQFLAG_FIFOFULL) := irq_fifofull_ena;
	  data_out(C_EVR_IRQFLAG_VIOLATION) := irq_rxviol_ena;
        when C_EVR_REG_PULSE_IRQ_MAP(C_EVR_REG_CMP_LOW to 29) =>
          data_out(C_SIGNAL_MAP_BITS-1 downto 0) := reg_pulse_irq_map;
	when C_EVR_REG_DATABUF_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_DATABUF_RECEIVING) := databuf_receiving;
	  data_out(C_EVR_DATABUF_RXREADY) := databuf_rx_ready;
	  data_out(C_EVR_DATABUF_CHECKSUM) := databuf_checksum;
	  data_out(C_EVR_DATABUF_MODE) := databuf_mode;
	  data_out(C_EVR_DATABUF_SIZEHIGH downto C_EVR_DATABUF_SIZELOW)
	    := databuf_rxsize;
	when C_EVR_REG_TXDATABUF_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_TXDATABUF_ENA) := tx_databuf_ena;
	  data_out(C_EVR_TXDATABUF_RUNNING) := tx_databuf_running;
	  data_out(C_EVR_TXDATABUF_COMPLETE) := tx_databuf_complete;
	  data_out(C_EVR_TXDATABUF_MODE) := tx_databuf_mode;
	  data_out(C_EVR_TXDATABUF_SIZEHIGH downto C_EVR_TXDATABUF_SIZELOW)
	    := tx_databuf_size(C_EVR_TXDATABUF_SIZEHIGH downto
                               C_EVR_TXDATABUF_SIZELOW);
	when C_EVR_REG_FW_VERSION(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := C_EVR_FW_VERSION;
	when C_EVR_REG_EVDCM_SAMPLE(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(31 downto 24) := evdcm_sample_high(15 downto 8);
	  data_out(23 downto 16) := evdcm_sample_low(15 downto 8);
	when C_EVR_REG_EVCNT_PRESC(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(15 downto 0) := reg_evcnt_presc;
	when C_EVR_REG_USEC_DIVIDER(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(15 downto 0) := reg_usec_divider;
	when C_EVR_REG_CLOCK_CONTROL(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(C_EVR_CLKCTRL_RECDCM_RUN) := recdcm_running;
	  data_out(C_EVR_CLKCTRL_RECDCM_INITD) := recdcm_init_ps_done;
	  data_out(C_EVR_CLKCTRL_RECDCM_PSDONE) := recdcm_psdone;
	  data_out(C_EVR_CLKCTRL_EVDCM_STOPPED) := evdcm_stopped;
	  data_out(C_EVR_CLKCTRL_EVDCM_LOCKED) := evdcm_locked;
	  data_out(C_EVR_CLKCTRL_EVDCM_PSDONE) := evdcm_psdone;
	  data_out(C_EVR_CLKCTRL_CGLOCK) := CGLOCK;
	  data_out(C_EVR_CLKCTRL_RECDCM_RESET) := recdcm_reset;
	  data_out(C_EVR_CLKCTRL_EVDCM_SRUN) := evdcm_sample_run;
	  data_out(C_EVR_CLKCTRL_EVDCM_RES) := evdcm_reset;
	  data_out(C_EVR_CLKCTRL_USE_RXRECCLK) := use_rxrecclk;
	when C_EVR_REG_SECONDS_SHIFT(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := reg_seconds_sr;
	when C_EVR_REG_SECONDS_COUNTER(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := reg_seconds;
	when C_EVR_REG_TSEVENT_COUNTER(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := event_counter_rd;
	when C_EVR_REG_SECONDS_LATCH(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := timestamp_sec_rd;
	when C_EVR_REG_TSEVENT_LATCH(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := timestamp_rd;
	when C_EVR_REG_EVFIFO_SECONDS(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := event_fifo_rd(79 downto 48);
	when C_EVR_REG_EVFIFO_TSEVENT(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := event_fifo_rd(47 downto 16);
	when C_EVR_REG_EVFIFO_CODE(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(7 downto 0) := event_fifo_rd(7 downto 0);
	  if Sl_select = '1' and OPB_RNW = '1' and
	    OPB_BE(3) = '1' and OPB_BE_1(3) = '0' then
	    host_read_fifo <= '1';
	  end if;
        when C_EVR_REG_LOG_STATUS(C_EVR_REG_CMP_LOW to 29) =>
          data_out(31) := event_log_overflow;
          data_out(8 downto 0) := event_log_wr_pnt;
	when C_EVR_REG_FRACDIV(C_EVR_REG_CMP_LOW to 29) =>
	  data_out := reg_micrel_word;
	when C_EVR_REG_RX_INIT_PS(C_EVR_REG_CMP_LOW to 29) =>
	  data_out(7 downto 0) := recdcm_init_ps;
        when C_EVR_REG_GPIO_DIR(C_EVR_REG_CMP_LOW to 29) =>
          data_out := reg_fp_univ_gpdir;
        when C_EVR_REG_GPIO_IN(C_EVR_REG_CMP_LOW to 29) =>
          data_out := reg_fp_univ_gpin;
        when C_EVR_REG_GPIO_OUT(C_EVR_REG_CMP_LOW to 29) =>
          data_out := reg_fp_univ_gpout;
        when C_DIAG_REG_CNT_ENA(C_EVR_REG_CMP_LOW to 29) =>
          data_out := pulse_counter_ena;
        when C_DIAG_REG_CNT_INPUT(C_EVR_REG_CMP_LOW to 29) =>
          data_out(C_DIAG_COUNTERS-1 downto 0) := pulse_int;
	when others =>
      end case;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_CML_CMP_HIGH) =
        C_EVR_REG_CML_BASE(C_EVR_REG_CMP_LOW to C_EVR_REG_CML_CMP_HIGH) or
	OPB_ABus(C_EVR_REG_CMP_LOW to C_EVR_REG_CMLPAT_CMP_HIGH) =
	C_EVR_REG_CMLPAT_BASE(C_EVR_REG_CMP_LOW to
			      C_EVR_REG_CMLPAT_CMP_HIGH)
      then
        data_out := cml_data_out;
      end if;

      map_ram_sel := conv_integer(OPB_ABus(C_EVR_REG_MAP_INDEX_LOW to
                                           C_EVR_REG_MAP_INDEX_HIGH));
      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_MAP_CMP_HIGH) =
        C_EVR_REG_MAP_BASE(C_EVR_REG_CMP_LOW to
                           C_EVR_REG_MAP_CMP_HIGH) then
        data_out := map_ram_data(map_ram_sel);
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_FPIN_CMP_HIGH) =
	C_EVR_REG_FPIN_BASE(C_EVR_REG_CMP_LOW to
			    C_EVR_REG_FPIN_CMP_HIGH) then
        data_out := fp_input_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_FPOUT_CMP_HIGH) =
	C_EVR_REG_FPOUT_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_REG_FPOUT_CMP_HIGH) then
        data_out := fp_map_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_UNIV_CMP_HIGH) =
	C_EVR_REG_UNIV_BASE(C_EVR_REG_CMP_LOW to
			    C_EVR_REG_UNIV_CMP_HIGH) then
        data_out := fp_univ_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_TBOUT_CMP_HIGH) =
	C_EVR_REG_TBOUT_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_REG_TBOUT_CMP_HIGH) then
        data_out := tb_map_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_REG_PULSE_CMP_HIGH) =
	C_EVR_REG_PULSE_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_REG_PULSE_CMP_HIGH) then
        data_out := pulse_data_out;
      end if;
      
      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_DATABUF_CMP_HIGH) =
	C_EVR_DATABUF_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_DATABUF_CMP_HIGH) then
        data_out := databuf_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
                  C_EVR_TXDATABUF_CMP_HIGH) =
	C_EVR_TXDATABUF_BASE(C_EVR_REG_CMP_LOW to
			     C_EVR_TXDATABUF_CMP_HIGH) then
        data_out := tx_databuf_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to
		  C_EVR_REG_PRESC_CMP_HIGH) =
	C_EVR_REG_PRESCALER_BASE(C_EVR_REG_CMP_LOW to
				 C_EVR_REG_PRESC_CMP_HIGH) then
	data_out := presc_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to C_DIAG_REG_CNT_CMP_HIGH) =
        C_DIAG_REG_CNT_BASE(C_EVR_REG_CMP_LOW to C_DIAG_REG_CNT_CMP_HIGH) then
        data_out := cnt_data_out;
      end if;

      if OPB_ABus(C_EVR_REG_CMP_LOW to C_EVR_REG_LOG_CMP_HIGH) =
        C_EVR_REG_LOG_BASE(C_EVR_REG_CMP_LOW to C_EVR_REG_LOG_CMP_HIGH) then
        data_out := log_data_out;
      end if;
        
      opb_rd_ack <= '0';
      Sl_DBus <= (others => '0');
      if Sl_select = '1' and OPB_RNW = '1' then
	Sl_DBus <= data_out;
	if opb_rd_ack_sr(opb_rd_ack_sr'high) = '1' then
	  opb_rd_ack <= '1';
	end if;
	opb_rd_ack_sr := opb_rd_ack_sr(opb_rd_ack_sr'high - 1 downto 0) & '0';
      else
	opb_rd_ack_sr := (others => '0');
	opb_rd_ack_sr(0) := '1';
      end if;

      OPB_BE_1 := OPB_BE;

      for i in 0 to C_DIAG_COUNTERS-1 loop
        pulse_int(i) := '0';
        if i < C_EVR_TB_INPUTS then
          if TBIN(i) = '1' then
            pulse_int(i) := '1';
          end if;
        else
          if i < C_EVR_UNIV_OUTPUTS then
            if UNIVIN(i) = '1' then
              pulse_int(i) := '1';
            end if;
          end if;
        end if;
      end loop;

    end if;
  end process;
  
  map_ram_mux: process (event_clk, event_map, map_ram_select,
			map_ram_enable, master_enable, log_stopped)
  begin
    forward_event <= '0';
    if map_ram_enable = '1' and master_enable = '1' and
      fwd_event_ena = '1' then
      if map_ram_select = '0' then
        forward_event <= event_map(0)(31 - C_EVR_MAP_FORWARD_EVENT);
      else
        forward_event <= event_map(1)(31 - C_EVR_MAP_FORWARD_EVENT);
      end if;
    end if;
    
    if rising_edge(event_clk) then
      fwd_fifo_data_in <= event_code;
      pulse_trigger <= (others => '0');
      pulse_set <= (others => '0');
      pulse_reset <= (others => '0');
      save_event <= '0';
      log_event <= '0';
      latch_timestamp_event <= '0';
      heartbeat_event <= '0';
      reset_prescalers_event <= '0';
      timestamp_counter_reset_event <= '0';
      timestamp_counter_clk_event <= '0';
      seconds_counter_1_event <= '0';
      seconds_counter_0_event <= '0';
      stop_log_event <= '0';
      if map_ram_enable = '1' and master_enable = '1' then
	if map_ram_select = '0' then
	  pulse_trigger(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(0)(64-C_EVR_PULSE_GENS to 63);
	  pulse_set(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(0)(96-C_EVR_PULSE_GENS to 95);
	  pulse_reset(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(0)(128-C_EVR_PULSE_GENS to 127);
          save_event <= event_map(0)(31 - C_EVR_MAP_SAVE_EVENT);
          log_event <= event_map(0)(31 - C_EVR_MAP_LOG_EVENT);
          latch_timestamp_event <= event_map(0)(31 - C_EVR_MAP_LATCH_TIMESTAMP);
	  led_event <= event_map(0)(31 - C_EVR_MAP_LED_EVENT);
          heartbeat_event <= event_map(0)(31 - C_EVR_MAP_HEARTBEAT_EVENT);
          reset_prescalers_event <= event_map(0)(31 - C_EVR_MAP_RESETPRESC_EVENT);
          timestamp_counter_reset_event <= event_map(0)(31 - C_EVR_MAP_TIMESTAMP_RESET);
          timestamp_counter_clk_event <= event_map(0)(31 - C_EVR_MAP_TIMESTAMP_CLK);
          seconds_counter_1_event <= event_map(0)(31 - C_EVR_MAP_SECONDS_1);
          seconds_counter_0_event <= event_map(0)(31 - C_EVR_MAP_SECONDS_0);
          stop_log_event <= event_map(0)(31 - C_EVR_MAP_STOP_LOG);
	else
	  pulse_trigger(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(1)(64-C_EVR_PULSE_GENS to 63);
	  pulse_set(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(1)(96-C_EVR_PULSE_GENS to 95);
	  pulse_reset(C_EVR_PULSE_GENS-1 downto 0) <=
	    event_map(1)(128-C_EVR_PULSE_GENS to 127);
          save_event <= event_map(1)(31 - C_EVR_MAP_SAVE_EVENT);
          log_event <= event_map(1)(31 - C_EVR_MAP_LOG_EVENT);
          latch_timestamp_event <= event_map(1)(31 - C_EVR_MAP_LATCH_TIMESTAMP);
	  led_event <= event_map(1)(31 - C_EVR_MAP_LED_EVENT);
          heartbeat_event <= event_map(1)(31 - C_EVR_MAP_HEARTBEAT_EVENT);
          reset_prescalers_event <= event_map(1)(31 - C_EVR_MAP_RESETPRESC_EVENT);
          timestamp_counter_reset_event <= event_map(1)(31 - C_EVR_MAP_TIMESTAMP_RESET);
          timestamp_counter_clk_event <= event_map(1)(31 - C_EVR_MAP_TIMESTAMP_CLK);
          seconds_counter_1_event <= event_map(1)(31 - C_EVR_MAP_SECONDS_1);
          seconds_counter_0_event <= event_map(1)(31 - C_EVR_MAP_SECONDS_0);
          stop_log_event <= event_map(1)(31 - C_EVR_MAP_STOP_LOG);
	end if;
      end if;
      if log_stopped = '1' then
        log_event <= '0';
      end if;
    end if;
  end process;

  heartbeat: process (event_clk, heartbeat_event, reset_heartbeat_irq,
		      reg_usec_divider, usec_enable, int_reset)
    variable counter_us : std_logic_vector(16 downto 0);
    variable enable_us  : std_logic;
    variable heartbeat_cnt : std_logic_vector(21 downto 0);
    variable reset_hb_sync : std_logic_vector(1 downto 0);
  begin
    if event_clk'event and event_clk = '1' then
      if reset_hb_sync(1) = '1' or int_reset = '1' or
	heartbeat_event = '1' then
	heartbeat_cnt := conv_std_logic_vector(1599999, 22);
      end if;

      if reset_hb_sync(1) = '1' or int_reset = '1' then
	flag_lostheartbeat <= '0';
      end if;
      
      if usec_enable = '1' then
	if heartbeat_cnt(21) = '1' then
	  flag_lostheartbeat <= '1';
	else
	  heartbeat_cnt := heartbeat_cnt - 1;
	end if;
      end if;

      reset_hb_sync := reset_hb_sync(0) & reset_heartbeat_irq;

      usec_enable <= enable_us;
      enable_us := '0';
      if counter_us(16) = '1' then
	counter_us := ('0' & reg_usec_divider) - 2;
	enable_us := '1';
      else
	counter_us := counter_us - 1;
      end if;
    end if;
  end process;

  link_status_change: process (event_clk, rx_link_ok, reset_linkchg_irq,
			       irq_linkchg_ena)
    variable link_status : std_logic_vector(1 downto 0);
  begin
    if rising_edge(event_clk) then
      link_status := link_status(0) & rx_link_ok;
      if link_status = "01" or link_status = "10" then
	flag_linkchg_irq <= '1';
      end if;
      if reset_linkchg_irq = '1' or irq_linkchg_ena = '0' then
	flag_linkchg_irq <= '0';
      end if;
    end if;
  end process;
  
  event_counter_clk: process (event_clk, timestamp_counter_clk_event,
                              timestamp_clk_dbus,
                              reg_evcnt_presc, timestamp_counter_reset_event)
    variable prescaler : std_logic_vector(16 downto 0);
    variable counter : std_logic_vector(16 downto 0);
    variable cnt_enable : std_logic;
    variable dbus_clk_ena : std_logic_vector(1 downto 0);
  begin
    prescaler := '0' & reg_evcnt_presc;
    if event_clk'event and event_clk = '1' then
      evcnt_clk_ena <= cnt_enable;
      if prescaler = 0 then
	evcnt_clk_ena <= timestamp_counter_clk_event;
	if timestamp_clk_dbus = '1' then
	  evcnt_clk_ena <= '0';
	  if dbus_clk_ena = "10" then
	    evcnt_clk_ena <= '1';
	  end if;
	end if;
      end if;

      dbus_clk_ena := dbus_data(4) & dbus_clk_ena(1);
      
      cnt_enable := '0';
      if counter(16) = '1' then
	cnt_enable := '1';
      end if;
      
      if timestamp_counter_reset_event = '1' or counter(16) = '1' then
	counter := prescaler - 2;
      else
	counter := counter - 1;
      end if;
    end if;
  end process;

  
  event_counter: process (event_clk, evcnt_clk_ena,
                          timestamp_counter_reset_event,
			  sw_reset_timestamp, sw_latch_timestamp,
			  int_reset)
    variable event_counter : unsigned(31 downto 0);
  begin
    event_counter_rd <= std_logic_vector(event_counter);

    if event_clk'event and event_clk = '1' then
      if sync_code_reset = '1' and evcnt_clk_ena = '1' then
	event_counter := (others => '0');
      elsif evcnt_clk_ena = '1' then
	event_counter := event_counter + 1;
      end if;

      if sw_reset_timestamp = '1' or int_reset = '1' then
	event_counter := (others => '0');
      end if;
    end if;
  end process;

  timestamp_latch: process (event_clk, sw_reset_timestamp, sw_latch_timestamp,
			    event_counter_rd, reg_seconds,
			    latch_timestamp_event, int_reset)
    variable timestamp     : std_logic_vector(31 downto 0);
    variable timestamp_sec : std_logic_vector(31 downto 0);
    variable latch_ts      : std_logic_vector(1 downto 0) := "00";
  begin
    timestamp_rd <= timestamp;
    timestamp_sec_rd <= timestamp_sec;
    
    if event_clk'event and event_clk = '1' then
      if latch_ts(1) = '1' or latch_timestamp_event = '1' then
	timestamp := event_counter_rd;
	timestamp_sec := reg_seconds;
      end if;

      if sw_reset_timestamp = '1' or int_reset = '1' then
	timestamp := (others => '0');
	timestamp_sec := (others => '0');
      end if;

      -- synchronize to event clock
      latch_ts := latch_ts(0) & sw_latch_timestamp;
    end if;
  end process;

  seconds_shiftreg : process (event_clk, seconds_counter_0_event,
                              seconds_counter_1_event,
			      int_reset, timestamp_counter_reset_event)
    variable int_seconds : std_logic_vector(31 downto 0);
  begin
    reg_seconds_sr <= int_seconds;
    if event_clk'event and event_clk = '1' then
      if sync_code_reset = '1' and evcnt_clk_ena = '1' then
	reg_seconds <= int_seconds;
	sync_code_reset <= '0';
      end if;
      if seconds_counter_0_event = '1' or seconds_counter_1_event = '1' then
	int_seconds(31 downto 1) := int_seconds(30 downto 0);
	int_seconds(0) := '0';
	if seconds_counter_1_event = '1' then
	  int_seconds(0) := '1';
	end if;
      end if;
      if timestamp_counter_reset_event = '1' then
	sync_code_reset <= '1';
      end if;
      if int_reset = '1' then
	int_seconds := (others => '0');
	reg_seconds <= (others => '0');
	sync_code_reset <= '0';
      end if;
    end if;
  end process;

  event_fifo_control: process (event_clk, log_disable_sw,
                               log_enable_sw, stop_log_event,
                               stop_log_event_enable)
    variable log_disabled : std_logic := '0';
  begin
    if rising_edge(event_clk) then
      log_stopped <= log_disabled;
      if log_enable_sw = '1' then
        log_disabled := '0';
      end if;
      if log_disable_sw = '1' or
        (stop_log_event_enable = '1' and stop_log_event = '1') then
        log_disabled := '1';
      end if;
    end if;
  end process;  
  
  event_fifo_status: process (OPB_Clk, fifofull, fifoempty,
			      reset_fifofull_irq, OPB_Rst)
    variable fifofull_sync  : std_logic;
    variable latch_fifofull : std_logic;
  begin
    flag_fifofull <= latch_fifofull;
    if OPB_Clk'event and OPB_Clk = '1' then
      if fifofull_sync = '1' then
	latch_fifofull := '1';
      elsif reset_fifofull_irq = '1' or OPB_Rst = '1' then
	latch_fifofull := '0';
      end if;
      -- Synchronize between clock domains
      fifofull_sync := fifofull;
    end if;

    flag_fifonotempty <= not fifoempty;
  end process;

  -- Removed logic for event interrupt
  -- now event interrupt comes directly from fifo not empty flag
  event_interrupt: process (flag_fifonotempty)
  begin
    flag_event_irq <= flag_fifonotempty;
  end process;
  
  event_fifo_write_control: process (event_clk, event_code,
				     event_counter_rd)
    variable fifo_dbus_data : std_logic_vector(7 downto 0);
    variable fifo_event_code : std_logic_vector(7 downto 0);
  begin
    if event_clk'event and event_clk = '1' then
      event_fifo_in(79 downto 48) <= reg_seconds;
      event_fifo_in(47 downto 16) <= event_counter_rd;
      event_fifo_in(15 downto 8) <= fifo_dbus_data;
      event_fifo_in(7 downto 0) <= fifo_event_code;
      fifo_dbus_data := dbus_data;
      fifo_event_code := event_code;
    end if;
  end process;

  host_interrupt: process (OPB_Clk, irq_event_ena, flag_event_irq,
			   irq_heartbeat_ena, flag_lostheartbeat,
			   irq_fifofull_ena, flag_fifofull,
			   irq_rxviol_ena, rx_violation,
			   master_irq_enable, master_enable,
			   irq_pulse_ena, flag_pulse_irq,
			   irq_databuf_ena, databuf_rx_ready,
			   irq_linkchg_ena, flag_linkchg_irq)
  begin
    if OPB_Clk'event and OPB_Clk = '1' then
      ip_irq_out <= '0';
      if master_irq_enable = '1' and master_enable = '1' then
	if (irq_linkchg_ena = '1' and flag_linkchg_irq = '1') or 
	  (irq_databuf_ena = '1' and databuf_rx_ready = '1') or
	  (irq_pulse_ena = '1' and flag_pulse_irq = '1') or
	  (irq_event_ena = '1' and flag_event_irq = '1') or
	  (irq_heartbeat_ena = '1' and flag_lostheartbeat = '1') or
	  (irq_fifofull_ena = '1' and flag_fifofull = '1') or
	  (irq_rxviol_ena = '1' and rx_violation = '1') then
	  ip_irq_out <= '1';
	end if;
      end if;
    end if;
  end process;

  LED_fail <= '1';
  LED_rx_link <= not rx_link_ok;
  LED_ena <= not master_enable;
  LED_error <= CGLOCK;
  
  event_leds : process (event_clk, event_code, led_event,
			rx_link_ok, usec_enable, int_reset)
    variable rx_event        : std_logic;
    variable count_event_in  : std_logic_vector(16 downto 0);
    variable count_event_out : std_logic_vector(16 downto 0);
  begin
    if event_clk'event and event_clk = '1' then
      if usec_enable = '1' then
	LED_event_in <= '1';
	if count_event_in(16) = '0' then
	  count_event_in := count_event_in - 1;
	  LED_event_in <= '0';
	end if;

	LED_event_out <= '1';
	if count_event_out(16) = '0' then
	  count_event_out := count_event_out - 1;
	  LED_event_out <= '0';
	end if;
      end if;

      if rx_event = '1' and rx_link_ok = '1' then
	count_event_in := '0' & X"C34F";
      end if;
      
      if led_event = '1' then
	count_event_out := '0' & X"C34F";
      end if;

      rx_event := '0';
      if event_code /= X"00" then
	rx_event := '1';
      end if;

      if int_reset = '1' then
	count_event_in := (others => '1');
	count_event_out := (others => '1');
      end if;
      
    end if;
  end process;

  LED_rx_fail <= not rx_violation;
  
  external_event : process (event_clk, event_code_rx, TTLIN,
			    reg_fp_input_map, int_reset)
    type external_event_vector is array (integer range <>) of
      std_logic_vector(2 downto 0);
    variable external_event   : external_event_vector(1 downto 0);
    variable ext_trigger      : std_logic_vector(1 downto 0) := "00";
    variable ext_trig_ack     : std_logic_vector(1 downto 0) := "00";
  begin
    if event_clk'event and event_clk = '1' then
      ext_trig_ack := (others => '0');
      if event_code_rx = "00000000" then
        if ext_trigger(0) = '1' then
          ext_trig_ack(0) := '1';
        elsif ext_trigger(1) = '1' then
          ext_trig_ack(1) := '1';
        end if;
      end if;

      event_code <= event_code_rx;

      for i in 0 to C_EVR_TTL_INPUTS-1 loop
        if ext_trig_ack(i) = '1' and
          (reg_fp_input_map(i)(C_EVR_FPIN_EXT_ENABLE) = '1' or
          reg_fp_input_map(i)(C_EVR_FPIN_EXTLEV_ENABLE) = '1') then
          event_code <= reg_fp_input_map(i)(C_EVR_FPIN_EXTEVENT_BASE+7
                                            downto C_EVR_FPIN_EXTEVENT_BASE);
        end if;

        if external_event(i)(2) =
          reg_fp_input_map(i)(C_EVR_FPIN_EXT_EDGE) and
          external_event(i)(1) /=
          reg_fp_input_map(i)(C_EVR_FPIN_EXT_EDGE) then
          ext_trigger(i) := '1';
        end if;

        if usec_enable = '1' then
          if external_event(i)(1) /=
            reg_fp_input_map(i)(C_EVR_FPIN_EXTLEV_ACT) and
            reg_fp_input_map(i)(C_EVR_FPIN_EXTLEV_ENABLE) = '1' then
            ext_trigger(i) := '1';
          end if;
        end if;

        external_event(i) := external_event(i)(1 downto 0) & TTLIN(i);
        if int_reset = '1' or ext_trig_ack(i) = '1' then
          ext_trigger(i) := '0';
        end if;
      end loop;
	
    end if;
  end process;

  output_mapping : process (event_clk, reg_fp_output_map,
			int_pulse, dbus_data, prescaler_out,
			    reset_pulse_irq, int_reset)
    variable fp_out   : std_logic_vector(0 to C_EVR_TTL_OUTPUTS+
					 C_EVR_CML_OUTPUTS-1);
    variable univ_out : std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);
    variable tb_out   : std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
    variable sel      : integer range 0 to 2**C_SIGNAL_MAP_BITS-1;
    variable map_mux  : std_logic_vector(2**C_SIGNAL_MAP_BITS-1 downto 0);
    variable irq_trigger : std_logic;
    variable reset_pulse_irq_sync : std_logic;
  begin
    TTLOUT <= fp_out(0 to C_EVR_TTL_OUTPUTS-1);
    cml_pulse <= fp_out(C_EVR_TTL_OUTPUTS to C_EVR_TTL_OUTPUTS+
                        C_EVR_CML_OUTPUTS-1);
    UNIVOUT <= univ_out;
    TBOUT <= tb_out;
    if rising_edge(event_clk) then
      map_mux := (others => '0');
      map_mux(2**C_SIGNAL_MAP_BITS-2) := '1';
      map_mux(C_EVR_PULSE_GENS-1 downto 0) := int_pulse;
      map_mux(C_SIGNAL_MAP_DBUS+C_EVR_DBUS_BITS-1 downto C_SIGNAL_MAP_DBUS) :=
	dbus_data(7 downto 0);
      map_mux(C_SIGNAL_MAP_PRESC+C_EVR_PRESCALERS-1 downto C_SIGNAL_MAP_PRESC)
	:= prescaler_out;
      -- Map front panel and CML outputs
      for i in 0 to C_EVR_TTL_OUTPUTS + C_EVR_CML_OUTPUTS - 1 loop
	sel := conv_integer(reg_fp_output_map(i));
	fp_out(i) := map_mux(sel);
      end loop;
      -- Map Universal I/O outputs
      for i in 0 to C_EVR_UNIV_OUTPUTS - 1 loop
	sel := conv_integer(reg_univ_output_map(i));
	univ_out(i) := map_mux(sel);
      end loop;
      -- Map transition board outputs
      for i in 0 to C_EVR_TB_OUTPUTS - 1 loop
	sel := conv_integer(reg_tb_output_map(i));
	tb_out(i) := map_mux(sel);
      end loop;

      sel := conv_integer(reg_pulse_irq_map);
      irq_trigger := map_mux(sel);
      if irq_trigger = '1' then
        flag_pulse_irq <= '1';
      end if;
      if reset_pulse_irq_sync = '1' or int_reset = '1' then
        flag_pulse_irq <= '0';
      end if;

      reset_pulse_irq_sync := reset_pulse_irq;
    end if;
  end process;

  pulse_force_low <= '0';
  
  GPIO_outputs: process (OPB_Clk, reg_fp_univ_gpdir, reg_fp_univ_gpout,
                         GPIO)
  begin
    if rising_edge(OPB_Clk) then
      for i in 0 to C_EVR_GPIOS-1 loop
        reg_fp_univ_gpin(i) <= GPIO(i);
        GPIO(i) <= 'Z';
        if reg_fp_univ_gpdir(i) = '1' then
          GPIO(i) <= reg_fp_univ_gpout(i);
        end if;
      end loop;
    end if;
  end process;

  backstream_events: process (tx_clk, TTLIN, reg_fp_input_map, int_reset)
    type external_event_vector is array (integer range <>) of
      std_logic_vector(2 downto 0);
    variable external_event   : external_event_vector(C_EVR_TTL_INPUTS-1 downto 0);
    variable ext_trigger      : std_logic_vector(C_EVR_TTL_INPUTS-1 downto 0);
    variable ext_trig_ack     : std_logic_vector(C_EVR_TTL_INPUTS-1 downto 0);
    variable event_code_tx    : std_logic_vector(7 downto 0) := X"00";
    variable fwd_event_ack    : std_logic;
    variable usec_resync      : std_logic_vector(1 downto 0);
  begin
    fwd_fifo_rd <= '1';
    if rising_edge(tx_clk) then
      fwd_event_ack := '0';
      ext_trig_ack := (others => '0');

      if fwd_fifo_empty = '0' then
        fwd_event_ack := '1';        
      else
        for i in 0 to C_EVR_TTL_INPUTS-1 loop
          if ext_trigger(i) = '1' then
            ext_trig_ack(i) := '1';
            exit;
          end if;
        end loop;
      end if;

      event_code_tx := (others => '0');

      if fwd_event_ack = '1' then
        event_code_tx := fwd_fifo_event_out;
      else
        for i in 0 to C_EVR_TTL_INPUTS-1 loop
          if ext_trig_ack(i) = '1' and
            (reg_fp_input_map(i)(C_EVR_FPIN_BACKEV_ENABLE) = '1' or
             reg_fp_input_map(i)(C_EVR_FPIN_BACKLEV_ENABLE) = '1') then
            event_code_tx := reg_fp_input_map(i)(C_EVR_FPIN_BACKEVENT_BASE+7
                                                 downto C_EVR_FPIN_BACKEVENT_BASE);
          end if;
        end loop;
      end if;

      for i in 0 to C_EVR_TTL_INPUTS-1 loop
        if external_event(i)(2) =
          reg_fp_input_map(i)(C_EVR_FPIN_EXT_EDGE) and
          external_event(i)(1) /=
          reg_fp_input_map(i)(C_EVR_FPIN_EXT_EDGE) then
          ext_trigger(i) := '1';
        end if;

        if usec_resync(1) = '1' then
          if external_event(i)(1) /=
            reg_fp_input_map(i)(C_EVR_FPIN_EXTLEV_ACT) and
            reg_fp_input_map(i)(C_EVR_FPIN_BACKLEV_ENABLE) = '1' then
            ext_trigger(i) := '1';
          end if;
        end if;
        
        external_event(i) := external_event(i)(1 downto 0) & TTLIN(i);
        if int_reset = '1' or ext_trig_ack(i) = '1' then
          ext_trigger(i) := '0';
        end if;
      end loop;

      usec_resync := usec_resync(0) & usec_enable;
    end if;

    tx_event <= event_code_tx;
  end process;

  backstream_dbus: process (tx_clk, TTLIN, reg_fp_input_map)
    type external_dbus_vector is array (integer range <>) of
      std_logic_vector(1 downto 0);
    variable external_dbus   : external_dbus_vector(C_EVR_TTL_INPUTS-1 downto 0);
    variable dbus_tx         : std_logic_vector(7 downto 0) := X"00";
  begin
    tx_dbus <= dbus_tx;
    if rising_edge(tx_clk) then
      dbus_tx := (others => '0');
      
      for i in 0 to C_EVR_TTL_INPUTS-1 loop
        for j in 0 to 7 loop
          if external_dbus(i)(1) = '1' and
            reg_fp_input_map(i)(j+C_EVR_FPIN_BACKDBUS_BASE) = '1' then
            dbus_tx(j) := '1';
          end if;
        end loop;
        external_dbus(i) := external_dbus(i)(0) & TTLIN(i);
      end loop;
    end if;
  end process;

  counters: process (OPB_Clk, UNIVIN, pulse_counter_rst, pulse_counter_ena)
    variable cnt_1 : std_logic_vector(0 to C_DIAG_COUNTERS-1);
    variable cnt_2 : std_logic_vector(0 to C_DIAG_COUNTERS-1);
    variable cnt_3 : std_logic_vector(0 to C_DIAG_COUNTERS-1);
  begin
    if rising_edge(OPB_Clk) then
      for i in 0 to C_DIAG_COUNTERS-1 loop
        if pulse_counter_ena(i) = '1' then
          if cnt_3(i) = '0' and cnt_2(i) = '1' then
            -- rising edge and counter enabled
            pulse_counter_cnt(i) <= pulse_counter_cnt(i) + 1;
          end if;
        end if;
        
        if pulse_counter_rst(i) = '1' then
          pulse_counter_cnt(i) <= (others => '0');
        end if;
      end loop;
      
      -- Input flip-flops to synchronize between clock domains and find
      -- rising edge
      cnt_3 := cnt_2;
      cnt_2 := cnt_1;
      for j in 0 to C_DIAG_COUNTERS-1 loop
        cnt_1(j) := '0';
        if j < C_EVR_TB_INPUTS then
          if TBIN(j) = '1' then
            cnt_1(j) := '1';
          end if;
        else
          if j < C_EVR_UNIV_OUTPUTS then
            if UNIVIN(j) = '1' then
              cnt_1(j) := '1';
            end if;
          end if;
        end if;
      end loop;
    end if;
  end process;
  
end structure;
