---------------------------------------------------------------------------
--
--  File        : pmcevr200.vhd
--
--  Title       : PMC-EVR Top Level
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_bs_pkg.all;
use work.evr_pkg.all;
 
entity pmcevr200 is
  port (
        -- Transceiver control signals, trailing letter is replaced with
    -- number in vector index: 4 = D, 3 = C, 2 = B, 1 = A
    TXDIS : out std_logic_vector(4 downto 1);

    -- BREFCLKx is sourced by Micrel SY87739L
    BREFCLKP : in std_logic;
    BREFCLKN : in std_logic;

    -- BREFCLK2x is sourced by optional LVPECL oscillator 
    BREFCLK2P : in std_logic;
    BREFCLK2N : in std_logic;

    -- Local Bus Signals
    LCLK      : in std_logic;
    nREADY    : out std_logic;
    nLBE      : in std_logic_vector(3 downto 0);
    nBTERM    : out std_logic;
    LINTi     : out std_logic_vector(2 downto 1);
    LWnR      : in std_logic;
    nADS      : in std_logic;
    LAD       : inout std_logic_vector(31 downto 0);
    
    -- I/O connector pins
    PMCIO      : out std_logic_vector(31 downto 0);

    EXT_IN     : in std_logic;
    TTLOUT     : out std_logic_vector(3 downto 1);
    
    -- LEDS
    nLED       : out std_logic_vector(3 downto 0);
    
    -- Micrel SY87739L programming signals
    CGLOCK    : in std_logic;
    CGSK      : out std_logic;
    CGDI      : out std_logic;
    CGCS      : out std_logic;
    
    -- SDRAM signals
    DQ        : inout std_logic_vector(15 downto 0);
    A         : out std_logic_vector(12 downto 0);
    DQM       : out std_logic_vector(1 downto 0);
    nCAS      : out std_logic;
    nWE       : out std_logic;
    nRAS      : out std_logic;
    nSDCS     : out std_logic;
    BA        : out std_logic_vector(1 downto 0);
    SDCLK     : out std_logic;
    SDCLK2    : in std_logic;
    CKE       : out std_logic;

    RXN       : in std_logic_vector(4 downto 1);
    RXP       : in std_logic_vector(4 downto 1);
    TXN       : out std_logic_vector(4 downto 1);
    TXP       : out std_logic_vector(4 downto 1)
    );

end pmcevr200;

architecture structure of pmcevr200 is

  signal OPB_Clk         : std_logic;

  signal reset           : std_logic;
  
  signal gnd             : std_logic := '0';
  signal vcc             : std_logic := '1';

  signal TTLIN           : std_logic_vector(0 to 0);
  
  signal pci_OPB_Clk : std_logic;
  signal pci_OPB_Rst : std_logic;
  signal pci_M_DBus : std_logic_vector(0 to 31);
  signal pci_OPB_DBus : std_logic_vector(0 to 31);
  signal pci_M_ABus : std_logic_vector(0 to 31);
  signal pci_M_BE : std_logic_vector(0 to 3);
  signal pci_M_busLock : std_logic;
  signal pci_M_request : std_logic;
  signal pci_M_RNW : std_logic;
  signal pci_M_select : std_logic;
  signal pci_M_seqAddr : std_logic;
  signal pci_OPB_errAck : std_logic;
  signal pci_OPB_MGrant : std_logic;
  signal pci_OPB_retry : std_logic;
  signal pci_OPB_timeout : std_logic;
  signal pci_OPB_xferAck : std_logic;

  signal evr_OPB_Clk : std_logic;
  signal evr_OPB_Rst : std_logic;
  signal evr_Sl_DBus : std_logic_vector(0 to 31);
  signal evr_Sl_errAck : std_logic;
  signal evr_Sl_retry : std_logic;
  signal evr_Sl_toutSup : std_logic;
  signal evr_Sl_xferAck : std_logic;
  signal evr_OPB_ABus : std_logic_vector(0 to 31);
  signal evr_OPB_BE : std_logic_vector(0 to 3);
  signal evr_OPB_DBus : std_logic_vector(0 to 31);
  signal evr_OPB_RNW : std_logic;
  signal evr_OPB_select : std_logic;
  signal evr_OPB_seqAddr : std_logic;

  signal CMLRXN : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal CMLRXP : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

  component bufgdll is
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component system is
    port (
      sys_clk : in std_logic;
      sys_rst : in std_logic;
      opb_m_0_OPB_Clk : out std_logic;
      opb_m_0_OPB_Rst : out std_logic;
      opb_m_0_M_DBus : in std_logic_vector(0 to 31);
      opb_m_0_OPB_DBus : out std_logic_vector(0 to 31);
      opb_m_0_M_ABus : in std_logic_vector(0 to 31);
      opb_m_0_M_BE : in std_logic_vector(0 to 3);
      opb_m_0_M_busLock : in std_logic;
      opb_m_0_M_request : in std_logic;
      opb_m_0_M_RNW : in std_logic;
      opb_m_0_M_select : in std_logic;
      opb_m_0_M_seqAddr : in std_logic;
      opb_m_0_OPB_errAck : out std_logic;
      opb_m_0_OPB_MGrant : out std_logic;
      opb_m_0_OPB_retry : out std_logic;
      opb_m_0_OPB_timeout : out std_logic;
      opb_m_0_OPB_xferAck : out std_logic;
      opb_s_1_OPB_Clk : out std_logic;
      opb_s_1_OPB_Rst : out std_logic;
      opb_s_1_Sl_DBus : in std_logic_vector(0 to 31);
      opb_s_1_Sl_errAck : in std_logic;
      opb_s_1_Sl_retry : in std_logic;
      opb_s_1_Sl_toutSup : in std_logic;
      opb_s_1_Sl_xferAck : in std_logic;
      opb_s_1_OPB_ABus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_BE : out std_logic_vector(0 to 3);
      opb_s_1_OPB_DBus : out std_logic_vector(0 to 31);
      opb_s_1_OPB_RNW : out std_logic;
      opb_s_1_OPB_select : out std_logic;
      opb_s_1_OPB_seqAddr : out std_logic);
  end component;

  component opb_pci9030 is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_OPB_AWIDTH    : integer               := 32;
      C_OPB_DWIDTH    : integer               := 32;
      C_FAMILY        : string                := "virtex2p"
      );
    port (
      -- PLX PCI9030 Local Bus Signals
      nREADY    : out std_logic;
      nLBE      : in std_logic_vector(3 downto 0);
      nBTERM    : out std_logic;
      LWnR      : in std_logic;
      nADS      : in std_logic;
      LAD       : inout std_logic_vector(31 downto 0);
      
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      M_DBus          : out   std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_DBus        : in    std_logic_vector(0 to C_OPB_DWIDTH-1);
      M_ABus          : out   std_logic_vector(0 to C_OPB_AWIDTH-1);
      M_BE            : out   std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      M_busLock       : out   std_logic;
      M_request       : out   std_logic;
      M_RNW           : out   std_logic;
      M_select        : out   std_logic;
      M_seqAddr       : out   std_logic;
      OPB_errAck      : in    std_logic;
      OPB_MGrant      : in    std_logic;
      OPB_retry       : in    std_logic;
      OPB_timeout     : in    std_logic;
      OPB_xferAck     : in    std_logic);
  end component;
  
  component opb_evr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

      TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic; -- Reference clock input from
      BREFCLKP        : in std_logic; -- fractional synthesizer
      BREFCLK2N       : in std_logic; -- Resynchronised clock input
      BREFCLK2P       : in std_logic; -- from FPGA external FF

      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;
      
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXN
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXN
      CMLRXN          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLRXP          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXN          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXP          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic
      );
  end component;
  
begin
  
  i_bugdll : bufgdll
    port map (
      I => lclk,
      O => OPB_Clk);

  i_opb : system
    port map (
      sys_clk => OPB_Clk,
      sys_rst => reset,
      opb_m_0_OPB_Clk => pci_OPB_Clk,
      opb_m_0_OPB_Rst => pci_OPB_Rst,
      opb_m_0_M_DBus => pci_M_DBus,
      opb_m_0_OPB_DBus => pci_OPB_DBus,
      opb_m_0_M_ABus => pci_M_ABus,
      opb_m_0_M_BE => pci_M_BE,
      opb_m_0_M_busLock => pci_M_busLock,
      opb_m_0_M_request => pci_M_request,
      opb_m_0_M_RNW => pci_M_RNW,
      opb_m_0_M_select => pci_M_select,
      opb_m_0_M_seqAddr => pci_M_seqAddr,
      opb_m_0_OPB_errAck => pci_OPB_errAck,
      opb_m_0_OPB_MGrant => pci_OPB_MGrant,
      opb_m_0_OPB_retry => pci_OPB_retry,
      opb_m_0_OPB_timeout => pci_OPB_timeout,
      opb_m_0_OPB_xferAck => pci_OPB_xferAck,
      opb_s_1_OPB_Clk => evr_OPB_Clk,
      opb_s_1_OPB_Rst => evr_OPB_Rst,
      opb_s_1_Sl_DBus => evr_Sl_DBus,
      opb_s_1_Sl_errAck => evr_Sl_errAck,
      opb_s_1_Sl_retry => evr_Sl_retry,
      opb_s_1_Sl_toutSup => evr_Sl_toutSup,
      opb_s_1_Sl_xferAck => evr_Sl_xferAck,
      opb_s_1_OPB_ABus => evr_OPB_ABus,
      opb_s_1_OPB_BE => evr_OPB_BE,
      opb_s_1_OPB_DBus => evr_OPB_DBus,
      opb_s_1_OPB_RNW => evr_OPB_RNW,
      opb_s_1_OPB_select => evr_OPB_select,
      opb_s_1_OPB_seqAddr => evr_OPB_seqAddr);

  i_opb_pci : opb_pci9030
    port map (
      nREADY => nREADY,
      nLBE => nLBE,
      nBTERM => nBTERM,
      LWnR => LWnR,
      nADS => nADS,
      LAD => LAD,
      OPB_Clk => OPB_Clk,
      OPB_Rst => pci_OPB_Rst,
      M_DBus => pci_M_DBus,
      OPB_DBus => pci_OPB_DBus,
      M_ABus => pci_M_ABus,
      M_BE => pci_M_BE,
      M_busLock => pci_M_busLock,
      M_request => pci_M_request,
      M_RNW => pci_M_RNW,
      M_select => pci_M_select,
      M_seqAddr => pci_M_seqAddr,
      OPB_errAck => pci_OPB_errAck,
      OPB_MGrant => pci_OPB_MGrant,
      OPB_retry => pci_OPB_retry,
      OPB_timeout => pci_OPB_timeout,
      OPB_xferAck => pci_OPB_xferAck);
  
  i_opb_evr : opb_evr
    generic map (
      C_BASEADDR => EVR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => OPB_Clk,
      OPB_Rst => evr_OPB_Rst,
      Sl_DBus => evr_Sl_DBus,
      Sl_errAck => evr_Sl_errAck,
      Sl_retry => evr_Sl_retry,
      Sl_toutSup => evr_Sl_toutSup,
      Sl_xferAck => evr_Sl_xferAck,
      OPB_ABus => evr_OPB_ABus,
      OPB_BE => evr_OPB_BE,
      OPB_DBus => evr_OPB_DBus,
      OPB_RNW => evr_OPB_RNW,
      OPB_select => evr_OPB_select,
      OPB_seqAddr => evr_OPB_seqAddr,

      ip_irq_out => LINTi(1),
    
      TTLIN => TTLIN,
      TTLOUT => TTLOUT(3 downto 1),
      
      TBOUT => PMCIO,

      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      BREFCLK2N => BREFCLK2N,
      BREFCLK2P => BREFCLK2P,

      LED_rx_link => nLED(0),
      LED_event_in => nLED(1),
      LED_event_out => nLED(3),
      LED_rx_fail => nLED(2),
    
      EVRXN => RXN(2),
      EVRXP => RXP(2),
      EVTXN => TXN(2),
      EVTXP => TXP(2),

      CMLRXN => CMLRXN,
      CMLRXP => CMLRXP,
      
      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS,
      CGLOCK => CGLOCK);

  TTLIN(0) <= EXT_IN;
  LINTi(2) <= '0';
  
  DQ <= (others => '0');
  A <= (others => '0');
  DQM <= "00";
  nCAS <= '1';
  nWE <= '1';
  nRAS <= '1';
  nSDCS <= '1';
  BA <= "00";
  SDCLK <= '0';
  CKE <= '0';

  TXDIS <= "0000";
  
  reset_generation : process (OPB_Clk)
    variable reset_cnt : std_logic_vector(1 downto 0) := "01";
  begin
    if rising_edge(OPB_Clk) then
      reset <= '1';
      if reset_cnt(1) = '1' then
	reset <= '0';
      end if;
      
      if reset_cnt(1) = '0' then
	reset_cnt := reset_cnt - 1;
      end if;
    end if;
  end process;
  
end structure;
