---------------------------------------------------------------------------
--
--  File        : pulse_generator.vhd
--
--  Title       : Event receiver pulse generator
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : The delayed pulse generator generates pulses with a
--                programmable pulse delay and width.
--                The prescaler, delay and width registers are configurable.
--                It also support set/reset level inputs.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity pulse_generator is
  generic (
    prescaler_range : integer;
    delay_range     : integer;
    width_range     : integer);
  port (
    prescaler       : in    std_logic_vector(31 downto 0);
    delay           : in    std_logic_vector(31 downto 0);
    width           : in    std_logic_vector(31 downto 0);
    polarity        : in    std_logic;
    sw_set          : in    std_logic;
    sw_reset        : in    std_logic;
    clk             : in    std_logic; -- control interface clock
                                       -- sw_set and sw_reset and synchronous
                                       -- to this clock
    pulse_out       : out   std_logic;
    event_clk       : in    std_logic;
    enable          : in    std_logic;
    trigger         : in    std_logic;
    set             : in    std_logic;
    reset           : in    std_logic;
    force_low       : in    std_logic);
end pulse_generator;

architecture structure of pulse_generator is

  signal sw_set_sync        : std_logic;
  signal sw_reset_sync      : std_logic;
  signal cnt_enable         : std_logic;
  signal delay_trigger      : std_logic;
  signal delay_ok           : std_logic;
  signal pulse_cnt_enable   : std_logic;
  
  component sync_domain is
    port (
      clk_in   : in std_logic;
      trig_in  : in std_logic;
      clk_out  : in std_logic;
      trig_out : out std_logic);
  end component;

begin

  i_sync_sw_set: sync_domain
    port map (
      clk_in => clk,
      trig_in => sw_set,
      clk_out => event_clk,
      trig_out => sw_set_sync);

  i_sync_sw_reset: sync_domain
    port map (
      clk_in => clk,
      trig_in => sw_reset,
      clk_out => event_clk,
      trig_out => sw_reset_sync);

  delay_generation : process (event_clk, cnt_enable, delay_trigger, enable)
    variable delay_counter : std_logic_vector(delay_range downto 0);
  begin
    if event_clk'event and event_clk = '1' then
      if cnt_enable = '1' then
	delay_ok <= '0';
	if delay_counter(delay_range) = '1' then
	  delay_ok <= '1';
	else
	  delay_counter := delay_counter - 1;
	end if;
      end if;
	
      if delay_trigger = '1' or enable = '0' then
	delay_ok <= '0';
	delay_counter := ('0' & delay(delay_range - 1 downto 0)) - 1;
      end if;
      pulse_cnt_enable <= cnt_enable;
    end if;
  end process;
  
  pulse_generation : process (event_clk, delay_trigger, delay_ok, width,
			      pulse_cnt_enable, enable, set, sw_set_sync,
                              reset, sw_reset_sync, force_low)
    variable pulse_counter : std_logic_vector(width_range downto 0);
    variable pdw_pulse     : std_logic_vector(1 downto 0);
    variable pulse_int     : std_logic;
  begin
    pulse_out <= polarity;
    if pulse_int = '1' then
      pulse_out <= not polarity;
    end if;
    
    if event_clk'event and event_clk = '1' then
      pdw_pulse(1) := pdw_pulse(0);
      if pulse_cnt_enable = '1' then
	pdw_pulse(0) := '0';
	if pulse_counter(width_range) = '0' and delay_ok = '1' then
	  pdw_pulse(0) := '1';
	  pulse_counter := pulse_counter - 1;
	end if;
      end if;

      if delay_trigger = '1' then
	pulse_counter := ('0' & width(width_range - 1 downto 0)) - 1;
      end if;

      if pdw_pulse = "01" then
	pulse_int := '1';
      end if;
      if pdw_pulse = "10" then
	pulse_int := '0';
      end if;
      if set = '1' or sw_set_sync = '1' then
        pulse_int := '1';
      end if;
      if reset = '1' or sw_reset_sync = '1' then
        pulse_int := '0';
      end if;
      if enable = '0' then
	pulse_counter(width_range) := '1';
	pulse_int := '0';
      end if;
      if force_low = '1' then
        pulse_int := '0';
      end if;
    end if;
  end process;
  
  prescaling: process (event_clk, trigger, prescaler, enable)
    variable prescale_counter : std_logic_vector(prescaler_range downto 0);
  begin
    if event_clk'event and event_clk = '1' then
      if prescaler_range > 0 then 
        cnt_enable <= '0';	
        if prescale_counter(prescaler_range) = '1' then
	  cnt_enable <= '1';
	  prescale_counter := ('0' &
                              prescaler(prescaler_range - 1 downto 0)) - 1;
        end if;
        if trigger = '1' or enable = '0' then
	  prescale_counter := (others => '0');
        end if;
        prescale_counter := prescale_counter - 1;
      else
        cnt_enable <= '1';
      end if; 
      delay_trigger <= trigger;
    end if;
  end process;
  
end structure;
