---------------------------------------------------------------------------
--                                                                       --
--  File        : recclk_out.vhd                                          --
--                                                                       --
--  Title       : Recovered clock DCM output
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
-- LIBRARY UNISIM;
-- use UNISIM.VCOMPONENTS.ALL;

entity recclk_out is
  port (
    clkin         : in std_logic;
    reset         : in std_logic;
    clkoutn       : out std_logic;
    clkoutp       : out std_logic;
    rx_link_ok    : in std_logic;
    dcm_run_out   : out std_logic;
    init_ps_done  : out std_logic;
    init_ps       : in std_logic_vector(7 downto 0);
    psclk         : in std_logic;
    psinc         : in std_logic;
    psdec         : in std_logic;
    psdone        : out std_logic
    );
end recclk_out;

architecture structure of recclk_out is

  signal CLKFB_IN : std_logic;
  signal CLK0_BUF : std_logic;
  signal CLK180_BUF : std_logic;
  signal gnd : std_logic;
  signal vcc : std_logic;
  signal status : std_logic_vector(7 downto 0);
  signal psen : std_logic;
  signal psincdec : std_logic;
  signal recclk : std_logic;
  signal dcm_locked : std_logic;
  signal dcm_psdone : std_logic;
  signal dcm_psinc  : std_logic;
  signal dcm_psdec  : std_logic;
  signal dcm_reset  : std_logic;
  signal int_reset  : std_logic;
  
  component bufg
    port (
      O : out std_logic;
      I : in  std_logic
      );
  end component;

  component FDDRRSE
    port (
      Q : out std_ulogic;
      C0 : in std_ulogic;
      C1 : in std_ulogic;
      CE : in std_ulogic;
      D0 : in std_ulogic;
      D1 : in std_ulogic;
      R : in std_ulogic;
      S : in std_ulogic);
  end component;

  component OBUFDS
    port (
      I : in std_ulogic;
      O : out std_ulogic;
      OB : out std_ulogic);
  end component;
  
  component DCM
    generic (
      CLKDV_DIVIDE : real := 2.0;
      CLKFX_DIVIDE : integer := 1;
      CLKFX_MULTIPLY : integer := 4;
      CLKIN_DIVIDE_BY_2 : boolean := false;
      CLKIN_PERIOD : real := 0.0;
      CLKOUT_PHASE_SHIFT : string := "NONE";
      CLK_FEEDBACK : string := "1X";
      DESKEW_ADJUST : string := "SYSTEM_SYNCHRONOUS";
      DFS_FREQUENCY_MODE : string := "LOW";
      DLL_FREQUENCY_MODE : string := "LOW";
      DSS_MODE : string := "NONE";
      DUTY_CYCLE_CORRECTION : boolean := true;
      FACTORY_JF : bit_vector := X"C080";                 
      MAXPERCLKIN : time := 1000000 ps;                   
      MAXPERPSCLK : time := 100000000 ps;                 
      PHASE_SHIFT : integer := 0;
      SIM_CLKIN_CYCLE_JITTER : time := 300 ps;            
      SIM_CLKIN_PERIOD_JITTER : time := 1000 ps;          
      STARTUP_WAIT : boolean := false);
    port (
      CLKIN : in STD_LOGIC;
      CLKFB : in STD_LOGIC;
      RST : in STD_LOGIC;
      PSEN : in STD_LOGIC;
      PSINCDEC : in STD_LOGIC;
      PSCLK : in STD_LOGIC;
      DSSEN : in STD_LOGIC;
      CLK0 : out STD_LOGIC;
      CLK90 : out STD_LOGIC;
      CLK180 : out STD_LOGIC;
      CLK270 : out STD_LOGIC;
      CLKDV : out STD_LOGIC;
      CLK2X : out STD_LOGIC;
      CLK2X180 : out STD_LOGIC;
      CLKFX : out STD_LOGIC;
      CLKFX180 : out STD_LOGIC;
      STATUS : out STD_LOGIC_VECTOR (7 downto 0);
      LOCKED : out STD_LOGIC;
      PSDONE : out STD_LOGIC);
  end component;

begin

  gnd <= '0';
  vcc <= '1';

  i_ddr : FDDRRSE
    port map (
      Q => recclk,
      C0 => clkfb_in,
      C1 => not clkfb_in,
      CE => vcc,
      D0 => gnd,
      D1 => vcc,
      R => gnd,
      S => gnd);

  i_obufds : OBUFDS
    port map (
      I => recclk,
      O => clkoutp,
      OB => clkoutn);
  
  DCM_local : DCM
    generic map(
      CLK_FEEDBACK => "1X",
      CLKDV_DIVIDE => 2.0,
      CLKFX_DIVIDE => 1,
      CLKFX_MULTIPLY => 4,
      CLKIN_DIVIDE_BY_2 => FALSE,
      CLKIN_PERIOD => 8.0,
      CLKOUT_PHASE_SHIFT => "VARIABLE",
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
      DFS_FREQUENCY_MODE => "HIGH",
      DLL_FREQUENCY_MODE => "HIGH",
      DUTY_CYCLE_CORRECTION => FALSE,
      PHASE_SHIFT => 0,
      STARTUP_WAIT => FALSE)
    port map (
      CLKIN => clkin,
      CLKFB => clkfb_in,
      RST => dcm_reset,
      PSEN => psen,
      PSINCDEC => psincdec,
      PSCLK => psclk,
      DSSEN => GND,
      CLK0 => CLK0_BUF,
      STATUS => status,
      LOCKED => dcm_locked,
      PSDONE => dcm_psdone);
  
  CLK0_BUFG_INST : BUFG
     port map (
      I => CLK0_BUF,
      O => clkfb_in);

  process (psclk, init_ps, dcm_psdone, dcm_locked, int_reset)
    variable ps_counter          : std_logic_vector(8 downto 0);
    variable dcm_ready_for_shift : std_logic;
    variable dcm_running         : std_logic;
    variable dcm_locked_1        : std_logic;
    variable rx_link_ok_1        : std_logic;
  begin
    if rising_edge(psclk) then
      dcm_psinc <= '0';
      dcm_psdec <= '0';
      init_ps_done <= '0';
      if dcm_running = '1' then
	if dcm_ready_for_shift = '1' then
	  if ps_counter(ps_counter'high) = '0' then
	    if init_ps(init_ps'high) = '0' then
	      dcm_psinc <= '1';
	    else
	      dcm_psdec <= '1';
	    end if;
	    dcm_ready_for_shift := '0';
	    ps_counter := ps_counter - 1;
	  end if;
	  if ps_counter(ps_counter'high) = '1' then
	    init_ps_done <= '1';
	  end if;
	end if;
      end if;
      if dcm_psdone = '1' then
	dcm_ready_for_shift := '1';
      end if;
      if dcm_locked_1 = '0' then
	dcm_running := dcm_locked;
      end if;

      dcm_reset <= '0';      
      if int_reset = '1' then
	if init_ps(init_ps'high) = '0' then
	  ps_counter := '0' & init_ps - 1;
	else
	  ps_counter := '0' & (0 - init_ps);
	end if;
	dcm_reset <= '1';
	dcm_running := '0';
	dcm_ready_for_shift := '1';
      end if;
      dcm_locked_1 := dcm_locked;
      rx_link_ok_1 := rx_link_ok;
      dcm_run_out <= dcm_running;
    end if;
  end process;
  
  phase_shifting : process (psclk, dcm_psinc, dcm_psdec, psinc, psdec,
			    dcm_reset)
    variable psinc_1 : std_logic_vector(1 downto 0);
    variable psdec_1 : std_logic_vector(1 downto 0);
  begin
    if psclk'event and psclk = '1' then
      psincdec <= '0';
      psen <= '0';
      if psinc_1 = "01" then
        psincdec <= '1';
        psen <= '1';
	psdone <= '0';
      end if;
      if psdec_1 = "01" then
        psen <= '1';
	psdone <= '0';
      end if;
      psinc_1 := psinc_1(0) & (psinc or dcm_psinc);
      psdec_1 := psdec_1(0) & (psdec or dcm_psdec);
      if dcm_psdone = '1' then
	psdone <= '1';
      end if;
      if dcm_reset = '1' then
        psincdec <= '0';
        psen <= '0';
	psdone <= '1';
      end if;
    end if;
  end process;

  resetting : process (psclk, reset, rx_link_ok)
    variable delay_cnt      : std_logic_vector(7 downto 0);
    variable rx_link_resync : std_logic_vector(3 downto 0) := "0000";
  begin
    if rising_edge(psclk) then
      int_reset <= '0';
      if reset = '1' or
	(rx_link_resync(3 downto 2) = "01" and
	 delay_cnt(delay_cnt'high) = '1') then
	int_reset <= '1';
      end if;

      if delay_cnt(delay_cnt'high) = '1' then
	rx_link_resync := rx_link_resync(2 downto 0) & rx_link_ok;
	delay_cnt := (others => '0');
      else
	delay_cnt := delay_cnt + 1;
      end if;
    end if;
  end process;
  
end structure;
  
