---------------------------------------------------------------------------
--
--  File        : receiver.vhd
--
--  Title       : Event Receiver Multi-Gigabit Receiver
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  		
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity receiver is
  generic (
    ref_on_brefclk  : boolean;
    separate_tx_rx  : boolean);
  port (
    brefclk         : in std_logic;
    brefclk_dcm     : in std_logic;
    brefclk2        : in std_logic;
    brefclk2_dcm    : in std_logic;
    refclk_dcm_out  : out std_logic;
    rxclk_nobuf_out : out std_logic;
    rxclk_out       : out std_logic;
    event_clk       : in std_logic;
    rxpolarity      : in std_logic;
    
    -- Receiver side connections
    event_code      : out std_logic_vector(7 downto 0);
    dbus_data       : out std_logic_vector(7 downto 0);
    databuf_data    : out std_logic_vector(7 downto 0);
    databuf_k       : out std_logic;
    databuf_ena     : out std_logic;
    databuf_mode    : in std_logic;
    
    rx_link_ok      : out   std_logic;
    rx_violation    : out   std_logic;
    rx_clear_viol   : in    std_logic;

    reset           : in    std_logic;

    -- Transmitter side connections
    tx_charisk      : in    std_logic_vector(1 downto 0);
    tx_data         : in    std_logic_vector(15 downto 0);
    tx_reset        : in    std_logic;
    tx_polarity     : in    std_logic;
    tx_event_clk    : in    std_logic;
    
    -- MGT I/O pins
    RXN             : in std_logic;
    RXP             : in std_logic;
    TXN             : out std_logic;
    TXP             : out std_logic
    );
end receiver;

architecture structure of receiver is

  signal vcc1  : std_logic;
  signal gnd1  : std_logic;
  signal gnd2  : std_logic_vector(1 downto 0);
  signal gnd16 : std_logic_vector(15 downto 0);

  signal refclk_dcm    : std_logic;
  signal refclksel     : std_logic;
  
  signal rx_mgt_pwd    : std_logic;
  signal rx_reset      : std_logic;
  signal rx_bufstatus  : std_logic_vector(1 downto 0);
  signal rx_charisk    : std_logic_vector(1 downto 0);
  signal rx_rundisp    : std_logic_vector(1 downto 0);
  signal rx_commadet   : std_logic;
  signal rx_data       : std_logic_vector(15 downto 0);
  signal rx_disperr    : std_logic_vector(1 downto 0);
  signal rx_loss       : std_logic_vector(1 downto 0);
  signal rx_notintable : std_logic_vector(1 downto 0);
  signal rx_realign    : std_logic;
  signal rxclk         : std_logic;
  signal rxclk_nobuf   : std_logic;

  signal link_ok       : std_logic;
  signal align_error   : std_logic;

  signal mgt_tx_charisk : std_logic_vector(1 downto 0);
  signal mgt_tx_data    : std_logic_vector(15 downto 0);
  signal mgt_tx_reset   : std_logic;
  
  component mgt_rx
    port (
      BREFCLK : in std_logic;
      BREFCLK2 : in std_logic;
      REFCLKSEL : in std_logic;
      POWERDOWN : in std_logic;
      RXN : in std_logic;
      RXP : in std_logic;
      RXRESET : in std_logic;
      RXPOLARITY : in std_logic;
      TXCHARISK : in std_logic_vector (1 downto 0);
      TXDATA : in std_logic_vector (15 downto 0);
      TXPOLARITY : in std_logic;
      TXRESET : in std_logic;
      RXBUFSTATUS : out std_logic_vector (1 downto 0);
      RXCHARISK : out std_logic_vector (1 downto 0);
      RXRUNDISP : out std_logic_vector (1 downto 0);
      RXCOMMADET : out std_logic;
      RXDATA : out std_logic_vector (15 downto 0);
      RXDISPERR : out std_logic_vector (1 downto 0);
      RXLOSSOFSYNC : out std_logic_vector (1 downto 0);
      RXNOTINTABLE : out std_logic_vector (1 downto 0);
      RXREALIGN : out std_logic;
      RXCLK_NOBUF : out std_logic;
      RXCLK : out std_logic;
      event_clk : in std_logic;
      tx_event_clk : in std_logic;
      TXN : out std_logic;
      TXP : out std_logic);
  end component;
  
  component BUFG
    port (O : out STD_ULOGIC;
	  I : in  STD_ULOGIC);
  end component;

begin

  vcc1 <= '1';
  gnd1 <= '0';
  gnd2 <= "00";
  gnd16 <= X"0000";
  rx_reset <= '0';
  
  i_mgt_rx: mgt_rx
    port map(
      BREFCLK => brefclk,
      BREFCLK2 => brefclk2,
      REFCLKSEL => refclksel,
      POWERDOWN => rx_mgt_pwd,
      RXN => RXN,
      RXP => RXP,
      RXRESET => rx_reset,
      RXPOLARITY => rxpolarity,
      TXCHARISK => mgt_tx_charisk,
      TXDATA => mgt_tx_data,
      TXRESET => mgt_tx_reset,
      TXPOLARITY => tx_polarity,
      RXBUFSTATUS => rx_bufstatus,
      RXCHARISK => rx_charisk,
      RXRUNDISP => rx_rundisp,
      RXCOMMADET => rx_commadet,
      RXDATA => rx_data,
      RXDISPERR => rx_disperr,
      RXLOSSOFSYNC => rx_loss,
      RXNOTINTABLE => rx_notintable,
      RXREALIGN => rx_realign,
      RXCLK_NOBUF => rxclk_nobuf,
      RXCLK => rxclk,
      event_clk => event_clk,
      tx_event_clk => tx_event_clk,
      TXN => TXN,
      TXP => TXP);

  refclock_source_1:
    if ref_on_brefclk generate
      refclk_dcm <= brefclk_dcm;
      refclksel <= '0';
    end generate;

  refclock_source_2:
    if not ref_on_brefclk generate
      refclk_dcm <= brefclk2_dcm;
      refclksel <= '1';
    end generate;

  refclk_dcm_out <= refclk_dcm;
  
  rxclk_out <= rxclk;
  rxclk_nobuf_out <= rxclk_nobuf;

  process
  begin
    mgt_tx_charisk <= tx_charisk;
    mgt_tx_data <= tx_data;
    mgt_tx_reset <= tx_reset;
    if separate_tx_rx = TRUE then
      mgt_tx_charisk <= (others => '0');
      mgt_tx_data <= (others => '0');
      mgt_tx_reset <= '1';
    end if;
  end process;
  
  process (refclk_dcm, reset, rx_loss, rx_charisk, link_ok, rx_disperr)
    variable prescaler : natural range 0 to MGT_RX_PRESCALER - 1;
    variable count : natural range 0 to MGT_RX_TIMEOUT - 1;
    variable k_counter : natural range 0 to MGT_RX_K_COUNTER - 1;
    variable rx_error : std_logic;
    variable align_err_sync : std_logic;
    variable loss_lock : std_logic;
    variable rx_error_count : std_logic_vector(5 downto 0);
  begin
    rx_link_ok <= link_ok;

    if reset = '1' then
      prescaler := 0;
      count := MGT_RX_TIMEOUT - 1;
    elsif rising_edge(refclk_dcm) then
      if prescaler = 0 then
	link_ok <= '0';
	if count = 0 then
	  link_ok <= '1';
	end if;
      
	rx_mgt_pwd <= '0';
--	rx_mgt_pwd <= async_out(0);
	if count > MGT_RX_TIMEOUT - 1 - MGT_RX_PWRDWN_TIME then
	  rx_mgt_pwd <= '1';
	end if;

	if count /= 0 then
	  count := count - 1;
	end if;

      end if;

      if count < MGT_RX_LOCK_ACQ then
	if loss_lock = '1' then
	  count := MGT_RX_TIMEOUT - 1;
	end if;
      end if;

      if link_ok = '0' then
	loss_lock := rx_error;
      else
	loss_lock := rx_error_count(5);
      end if;
      
      if link_ok = '1' then
	if rx_error = '1' then
	  if rx_error_count(5) = '0' then
	    rx_error_count := rx_error_count - 1;
	  end if;
	else
	  if prescaler = 0 and
	    (rx_error_count(5) = '1' or rx_error_count(4) = '0') then
	    rx_error_count := rx_error_count + 1;
	  end if;
	end if;
      else
	rx_error_count := "011111";
      end if;
            
      rx_error := '0';
      if rx_notintable /= "00" or
	rx_disperr /= "00" or
	align_err_sync = '1' then
	rx_error := '1';
      end if;

      if prescaler = MGT_RX_PRESCALER - 1 then
	prescaler := 0;
      else
	prescaler := prescaler + 1;
      end if;

      align_err_sync := align_error;
    end if;
  end process;

  reg_dbus_data : process (event_clk, link_ok, rx_data)
    variable even : std_logic;
  begin
    if event_clk'event and event_clk = '1' then
      if databuf_mode = '0' or even = '0' then
	dbus_data <= rx_data(7 downto 0);
      end if;
      if databuf_mode = '1' then
	if even = '1' then
	  databuf_data <= rx_data(7 downto 0);
	  databuf_k <= rx_charisk(0);
	end if;
      else
	databuf_data <= (others => '0');
	databuf_k <= '0';
      end if;

      databuf_ena <= even;
      
      if link_ok = '0' then
	databuf_data <= (others => '0');
	databuf_k <= '0';
	dbus_data <= (others => '0');
      end if;

      even := not even;
      event_code <= rx_data(15 downto 8);
      if link_ok = '0' or rx_charisk(1) = '1' or
	reset = '1' then
	event_code <= (others => '0');
	even := '1';
      end if;
    end if;
  end process;

  align_detect : process (event_clk, reset, rx_charisk, rx_data)
  begin
    if rising_edge(event_clk) then
      align_error <= '0';
      if rx_charisk(0) = '1' and rx_data(7) = '1' then
	align_error <= '1';
      end if;
    end if;
  end process;

  violation_detect : process (event_clk, reset, rx_clear_viol,
			      rx_disperr, rx_loss, rx_notintable, link_ok)
  begin
    if reset = '1' or rx_clear_viol = '1' then
      rx_violation <= '0';
    elsif rising_edge(event_clk) then
      if rx_disperr /= "00" or rx_loss /= "00" or rx_notintable /= "00" then
	rx_violation <= '1';
      end if;
    end if;
  end process;
	
end structure;
