---------------------------------------------------------------------------
--
--  File        : receiver_ecp2m.vhd
--
--  Title       : Event Receiver Multi-Gigabit Receiver for Lattice ECP2M
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  		
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;

entity receiver_ecp2m is
  port (
    refclkp         : in std_logic;
    refclkn         : in std_logic;
    refclk_out      : out std_logic;
    event_clk_out   : out std_logic;
    rx_polarity     : in std_logic;
    
    -- Receiver side connections
    event_code      : out std_logic_vector(7 downto 0);
    dbus_data       : out std_logic_vector(7 downto 0);
    databuf_data    : out std_logic_vector(7 downto 0);
    databuf_k       : out std_logic;
    databuf_ena     : out std_logic;
    databuf_mode    : in std_logic;
    
    rx_link_ok      : out   std_logic;
    rx_violation    : out   std_logic;
    rx_clear_viol   : in    std_logic;

    reset           : in    std_logic;

    -- Transmitter side connections
    tx_charisk      : in    std_logic_vector(1 downto 0);
    tx_data         : in    std_logic_vector(15 downto 0);
    tx_reset        : in    std_logic;
    tx_polarity     : in    std_logic;
    tx_event_clk    : out   std_logic;    
    
    -- PCS I/O pins
    hdinp0          : in std_logic;
    hdinn0          : in std_logic;
    hdinp1          : in std_logic;
    hdinn1          : in std_logic;
    hdinp2          : in std_logic;
    hdinn2          : in std_logic;
    hdinp3          : in std_logic;
    hdinn3          : in std_logic;

    hdoutp0         : out std_logic;
    hdoutn0         : out std_logic;
    hdoutp1         : out std_logic;
    hdoutn1         : out std_logic;
    hdoutp2         : out std_logic;
    hdoutn2         : out std_logic;
    hdoutp3         : out std_logic;
    hdoutn3         : out std_logic
    );
end receiver_ecp2m;

architecture structure of receiver_ecp2m is

  signal vcc1  : std_logic;
  signal gnd1  : std_logic;
  signal gnd2  : std_logic_vector(1 downto 0);
  signal gnd16 : std_logic_vector(15 downto 0);

  signal event_clk     : std_logic;
  signal refclk        : std_logic;
  
  signal rx_mgt_pwd    : std_logic;
  signal rx_reset      : std_logic;
  signal rx_bufstatus  : std_logic_vector(1 downto 0);
  signal rx_charisk    : std_logic_vector(1 downto 0);
  signal rx_rundisp    : std_logic_vector(1 downto 0);
  signal rx_commadet   : std_logic;
  signal rx_data       : std_logic_vector(15 downto 0);
  signal rx_disperr    : std_logic_vector(1 downto 0);
  signal rx_loss       : std_logic_vector(1 downto 0);
  signal rx_notintable : std_logic_vector(1 downto 0);
  signal rx_realign    : std_logic;
  signal rxclk         : std_logic;
  signal txclk         : std_logic;
  signal rxclk_nobuf   : std_logic;
  signal rx_sync       : std_logic;
  
  signal link_ok       : std_logic;
  signal align_error   : std_logic;

  signal mgt_tx_charisk : std_logic_vector(1 downto 0);
  signal mgt_tx_data    : std_logic_vector(15 downto 0);
  signal mgt_tx_reset   : std_logic;

  signal rx_powerdown   : std_logic;
  signal tx_powerdown   : std_logic;
  
  component pcs_rx is
    port (
      refclkp         : in std_logic;
      refclkn         : in std_logic;
      refclk_out      : out std_logic;
      
      RXPOLARITY      : in std_logic;
      
      RXPOWERDOWN     : in std_logic;
      RXRESET         : in std_logic;
      
      TXCHARISK       : in std_logic_vector (1 downto 0);
      TXDATA          : in std_logic_vector (15 downto 0);
      TXRESET         : in std_logic;
      TXPOLARITY      : in std_logic;
      TXPOWERDOWN     : in std_logic;
      
      RXLOSS          : out std_logic_vector (1 downto 0);
      RXSYNC          : out std_logic;
      
      RXCHARISK       : out std_logic_vector (1 downto 0);
      RXDATA          : out std_logic_vector (15 downto 0);
      RXDISPERR       : out std_logic_vector (1 downto 0);
      RXNOTINTABLE    : out std_logic_vector (1 downto 0);
      
      RXCLK           : out std_logic;
      TXCLK           : out std_logic;
      
      PCSRESET        : in std_logic;
      
      -- PCS I/O pins
      hdinp0          : in std_logic;
      hdinn0          : in std_logic;
      hdinp1          : in std_logic;
      hdinn1          : in std_logic;
      hdinp2          : in std_logic;
      hdinn2          : in std_logic;
      hdinp3          : in std_logic;
      hdinn3          : in std_logic;
      
      hdoutp0         : out std_logic;
      hdoutn0         : out std_logic;
      hdoutp1         : out std_logic;
      hdoutn1         : out std_logic;
      hdoutp2         : out std_logic;
      hdoutn2         : out std_logic;
      hdoutp3         : out std_logic;
      hdoutn3         : out std_logic);
  end component;
  
begin

  vcc1 <= '1';
  gnd1 <= '0';
  gnd2 <= "00";
  gnd16 <= X"0000";
  
  i_pcs_rx: pcs_rx
    port map(
      refclkp => refclkp,
      refclkn => refclkn,
      refclk_out => refclk,
      RXPOLARITY => rx_polarity,
      RXPOWERDOWN => rx_powerdown,
      RXRESET => rx_reset,

      TXCHARISK => tx_charisk,
      TXDATA => tx_data,
      TXRESET => tx_reset,
      TXPOLARITY => tx_polarity,
      TXPOWERDOWN => tx_powerdown,

      RXLOSS => rx_loss,
      RXSYNC => rx_sync,

      RXCHARISK => rx_charisk,
      RXDATA => rx_data,
      RXDISPERR => rx_disperr,
      RXNOTINTABLE => rx_notintable,

      RXCLK => rxclk,
      TXCLK => txclk,
      PCSRESET => RESET,

      hdinp0 => hdinp0,
      hdinn0 => hdinn0,
      hdoutp0 => hdoutp0,
      hdoutn0 => hdoutn0,
      hdinp1 => hdinp1,
      hdinn1 => hdinn1,
      hdoutp1 => hdoutp1,
      hdoutn1 => hdoutn1,
      hdinp2 => hdinp2,
      hdinn2 => hdinn2,
      hdoutp2 => hdoutp2,
      hdoutn2 => hdoutn2,
      hdinp3 => hdinp3,
      hdinn3 => hdinn3,
      hdoutp3 => hdoutp3,
      hdoutn3 => hdoutn3);

  refclk_out <= refclk;
  tx_event_clk <= txclk;
  
  event_clk_out <= rxclk;
  event_clk <= rxclk;

  rx_powerdown <= '0';
  tx_powerdown <= '0';
  
  process (refclk, reset, rx_loss, rx_charisk, link_ok, rx_disperr)
    variable prescaler : natural range 0 to MGT_RX_PRESCALER - 1;
    variable count : natural range 0 to MGT_RX_TIMEOUT - 1;
    variable k_counter : natural range 0 to MGT_RX_K_COUNTER - 1;
    variable rx_error : std_logic;
    variable align_err_sync : std_logic;
    variable loss_lock : std_logic;
    variable rx_error_count : std_logic_vector(5 downto 0);
  begin
    rx_link_ok <= link_ok;

    if reset = '1' then
      prescaler := 0;
      count := MGT_RX_TIMEOUT - 1;
    elsif rising_edge(refclk) then
      if prescaler = 0 then
	link_ok <= '0';
	if count = 0 then
	  link_ok <= '1';
	end if;
      
	rx_reset <= '0';
	if count > MGT_RX_TIMEOUT - 1 - MGT_RX_PWRDWN_TIME then
	  rx_reset <= '1';
	end if;

	if count /= 0 then
	  count := count - 1;
	end if;

      end if;

      if count < MGT_RX_LOCK_ACQ then
	if loss_lock = '1' then
	  count := MGT_RX_TIMEOUT - 1;
	end if;
      end if;

      if link_ok = '0' then
	loss_lock := rx_error;
      else
	loss_lock := rx_error_count(5);
      end if;
      
      if link_ok = '1' then
	if rx_error = '1' then
	  if rx_error_count(5) = '0' then
	    rx_error_count := rx_error_count - 1;
	  end if;
	else
	  if prescaler = 0 and
	    (rx_error_count(5) = '1' or rx_error_count(4) = '0') then
	    rx_error_count := rx_error_count + 1;
	  end if;
	end if;
      else
	rx_error_count := "011111";
      end if;
            
      rx_error := '0';
      if rx_notintable /= "00" or
	rx_disperr /= "00" or
	align_err_sync = '1' then
	rx_error := '1';
      end if;

      if prescaler = MGT_RX_PRESCALER - 1 then
	prescaler := 0;
      else
	prescaler := prescaler + 1;
      end if;

      align_err_sync := align_error;
    end if;
  end process;

  reg_dbus_data : process (event_clk, link_ok, rx_data)
    variable even : std_logic;
  begin
    if event_clk'event and event_clk = '1' then
      if databuf_mode = '0' or even = '0' then
	dbus_data <= rx_data(7 downto 0);
      end if;
      if databuf_mode = '1' then
	if even = '1' then
	  databuf_data <= rx_data(7 downto 0);
	  databuf_k <= rx_charisk(0);
	end if;
      else
	databuf_data <= (others => '0');
	databuf_k <= '0';
      end if;

      databuf_ena <= even;
      
      if link_ok = '0' then
	databuf_data <= (others => '0');
	databuf_k <= '0';
	dbus_data <= (others => '0');
      end if;

      even := not even;
      event_code <= rx_data(15 downto 8);
      if link_ok = '0' or rx_charisk(1) = '1' or
	reset = '1' then
	event_code <= (others => '0');
	even := '1';
      end if;
    end if;
  end process;

  align_detect : process (event_clk, reset, rx_charisk, rx_data)
  begin
    if rising_edge(event_clk) then
      align_error <= '0';
      if rx_charisk(0) = '1' and rx_data(7) = '1' then
	align_error <= '1';
      end if;
    end if;
  end process;

  violation_detect : process (event_clk, reset, rx_clear_viol,
			      rx_disperr, rx_loss, rx_notintable, link_ok)
  begin
    if reset = '1' or rx_clear_viol = '1' then
      rx_violation <= '0';
    elsif rising_edge(event_clk) then
      if rx_disperr /= "00" or rx_loss /= "00" or rx_notintable /= "00" then
	rx_violation <= '1';
      end if;
    end if;
  end process;
	
end structure;
