---------------------------------------------------------------------------
--
--  File        : receiver_v5fxt.vhd
--
--  Title       : Event Receiver Multi-Gigabit Receiver for Lattice ECP2M
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  		
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity receiver_v5fxt is
  generic
    (
      -- Simulation attributes
      TILE_SIM_MODE                : string    := "FAST"; -- Set to Fast Functional Simulation Model
      TILE_SIM_GTXRESET_SPEEDUP    : integer   := 0; -- Set to 1 to speed up sim reset
      TILE_SIM_PLL_PERDIV2         : bit_vector:= x"0c8"; -- Set to the VCO Unit Interval time 
      
      -- Channel bonding attributes
      TILE_CHAN_BOND_MODE_0        : string    := "OFF";  -- "MASTER", "SLAVE", or "OFF"
      TILE_CHAN_BOND_LEVEL_0       : integer   := 0;     -- 0 to 7. See UG for details
      
      TILE_CHAN_BOND_MODE_1        : string    := "OFF";  -- "MASTER", "SLAVE", or "OFF"
      TILE_CHAN_BOND_LEVEL_1       : integer   := 0      -- 0 to 7. See UG for details
      );
  port (
    REFCLKP         : in std_logic;
    REFCLKN         : in std_logic;
    REFCLK_OUT      : out std_logic;
    event_clk_out   : out std_logic;
    rx_polarity     : in std_logic;
    
    -- Receiver side connections
    event_code      : out std_logic_vector(7 downto 0);
    dbus_data       : out std_logic_vector(7 downto 0);
    databuf_data    : out std_logic_vector(7 downto 0);
    databuf_k       : out std_logic;
    databuf_ena     : out std_logic;
    databuf_mode    : in std_logic;
    
    rx_link_ok      : out   std_logic;
    rx_violation    : out   std_logic;
    rx_clear_viol   : in    std_logic;

    reset           : in    std_logic;

    cdrres          : in    std_logic;
    cdrphas         : in    std_logic;
    
    -- Transmitter side connections
    tx_charisk      : in    std_logic_vector(1 downto 0);
    tx_data         : in    std_logic_vector(15 downto 0);
    tx_reset        : in    std_logic;
    tx_polarity     : in    std_logic;
    tx_event_clk    : out   std_logic;    
    
    RXN0            : in    std_logic;
    RXP0            : in    std_logic;
    RXN1            : in    std_logic;
    RXP1            : in    std_logic;

    TXN0            : out   std_logic;
    TXP0            : out   std_logic;
    TXN1            : out   std_logic;
    TXP1            : out   std_logic;
    CONTROL0        : inout std_logic_vector(35 downto 0)
    );
end receiver_v5fxt;

architecture structure of receiver_v5fxt is

  signal vcc     : std_logic;
  signal gnd     : std_logic;
  signal gnd_vec : std_logic_vector(31 downto 0);

  signal event_clk     : std_logic;
  signal CLKIN         : std_logic;
  signal refclk        : std_logic;
  signal refclk_i      : std_logic;
  signal plllkdet      : std_logic;
  signal tx_outclk     : std_logic;
  
  signal rx_mgt_pwd    : std_logic;
  signal rx_reset      : std_logic;
  signal rx_bufstatus  : std_logic_vector(1 downto 0);
  signal rx_charisk    : std_logic_vector(1 downto 0);
  signal rx_rundisp    : std_logic_vector(1 downto 0);
  signal rx_commadet   : std_logic;
  signal rx_data       : std_logic_vector(15 downto 0);
  signal rx_disperr    : std_logic_vector(1 downto 0);
  signal rx_loss       : std_logic_vector(1 downto 0);
  signal rx_notintable : std_logic_vector(1 downto 0);
  signal rx_realign    : std_logic;
  signal rxrecclk      : std_logic;
  signal rxusrclk      : std_logic;
  signal txusrclk      : std_logic;
  signal rxcdrreset    : std_logic;
  signal rxpmasetphase : std_logic;

  signal RXEQMIX       : std_logic_vector(1 downto 0);
  
  signal link_ok       : std_logic;
  signal align_error   : std_logic;
  signal rx_error      : std_logic;
  
  signal mgt_tx_charisk : std_logic_vector(1 downto 0);
  signal mgt_tx_data    : std_logic_vector(15 downto 0);
  signal mgt_tx_reset   : std_logic;

  signal rx_powerdown   : std_logic;
  signal tx_powerdown   : std_logic;

  -- RX Datapath signals
    signal rxdata0_i                        :   std_logic_vector(31 downto 0);      
    signal rxcharisk0_float_i               :   std_logic_vector(1 downto 0);
    signal rxdisperr0_float_i               :   std_logic_vector(1 downto 0);
    signal rxnotintable0_float_i            :   std_logic_vector(1 downto 0);
    signal rxrundisp0_float_i               :   std_logic_vector(1 downto 0);
    

    -- TX Datapath signals
    signal txdata0_i                        :   std_logic_vector(31 downto 0);
    signal txkerr0_float_i                  :   std_logic_vector(1 downto 0);
    signal txrundisp0_float_i               :   std_logic_vector(1 downto 0);
   

    -- RX Datapath signals
    signal rxdata1_i                        :   std_logic_vector(31 downto 0);      
    signal rxcharisk1_float_i               :   std_logic_vector(1 downto 0);
    signal rxdisperr1_float_i               :   std_logic_vector(1 downto 0);
    signal rxnotintable1_float_i            :   std_logic_vector(1 downto 0);
    signal rxrundisp1_float_i               :   std_logic_vector(1 downto 0);
    

    -- TX Datapath signals
    signal txdata1_i                        :   std_logic_vector(31 downto 0);
    signal txkerr1_float_i                  :   std_logic_vector(1 downto 0);
    signal txrundisp1_float_i               :   std_logic_vector(1 downto 0);

  signal TRIG0 : std_logic_vector(63 downto 0);
  
  component ila_rx
    PORT (
      CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
      CLK : IN STD_LOGIC;
      TRIG0 : IN STD_LOGIC_VECTOR(63 DOWNTO 0));
  end component;

begin
  ----------------------------- GTX_DUAL Instance  --------------------------   
  i_ila : ila_rx
    port map (
      CONTROL => CONTROL0,
      CLK => rxusrclk,
      TRIG0 => TRIG0);

  gtx_dual_i:GTX_DUAL
    generic map
    (

        --_______________________ Simulation-Only Attributes ___________________

        SIM_RECEIVER_DETECT_PASS_0  =>       TRUE,
        
        SIM_RECEIVER_DETECT_PASS_1  =>       TRUE,

        SIM_MODE                    =>       TILE_SIM_MODE,
        SIM_GTXRESET_SPEEDUP        =>       TILE_SIM_GTXRESET_SPEEDUP,
        SIM_PLL_PERDIV2             =>       TILE_SIM_PLL_PERDIV2,
  

        --___________________________ Shared Attributes ________________________

        -------------------------- Tile and PLL Attributes ---------------------

        CLK25_DIVIDER               =>       5, 
        CLKINDC_B                   =>       TRUE,
        CLKRCV_TRST                 =>       TRUE,
        OOB_CLK_DIVIDER             =>       4,
        OVERSAMPLE_MODE             =>       FALSE,
        PLL_COM_CFG                 =>       x"21680a",
        PLL_CP_CFG                  =>       x"00",
        PLL_DIVSEL_FB               =>       4,
        PLL_DIVSEL_REF              =>       1,
        PLL_FB_DCCEN                =>       FALSE,
        PLL_LKDET_CFG               =>       "101",
        PLL_TDCC_CFG                =>       "000",
        PMA_COM_CFG                 =>       x"000000000000000000",


        --____________________ Transmit Interface Attributes ___________________


        ------------------- TX Buffering and Phase Alignment -------------------   

        TX_BUFFER_USE_0             =>       TRUE,
        TX_XCLK_SEL_0               =>       "TXOUT",
        TXRX_INVERT_0               =>       "011",        

        TX_BUFFER_USE_1             =>       TRUE,
        TX_XCLK_SEL_1               =>       "TXOUT",
        TXRX_INVERT_1               =>       "011",        

        --------------------- TX Gearbox Settings -----------------------------

        GEARBOX_ENDEC_0             =>       "000", 
        TXGEARBOX_USE_0            =>        FALSE,

        GEARBOX_ENDEC_1            =>        "000", 
        TXGEARBOX_USE_1            =>        FALSE,

        --------------------- TX Serial Line Rate settings ---------------------   

        PLL_TXDIVSEL_OUT_0          =>       2,

        PLL_TXDIVSEL_OUT_1          =>       2,

        --------------------- TX Driver and OOB signalling --------------------  

        CM_TRIM_0                   =>       "10",
        PMA_TX_CFG_0                =>       x"80082",
        TX_DETECT_RX_CFG_0          =>       x"1832",
        TX_IDLE_DELAY_0             =>       "010",

        CM_TRIM_1                   =>       "10",
        PMA_TX_CFG_1                =>       x"80082",
        TX_DETECT_RX_CFG_1          =>       x"1832",
        TX_IDLE_DELAY_1             =>       "010",

        ------------------ TX Pipe Control for PCI Express/SATA ---------------

        COM_BURST_VAL_0             =>       "1111",

        COM_BURST_VAL_1             =>       "1111",
        --_______________________ Receive Interface Attributes ________________


        ------------ RX Driver,OOB signalling,Coupling and Eq,CDR -------------  

        AC_CAP_DIS_0                =>       TRUE,
        OOBDETECT_THRESHOLD_0       =>       "111",
        PMA_CDR_SCAN_0              =>       x"6404035",
        PMA_RX_CFG_0                =>       x"0f44088",
        RCV_TERM_GND_0              =>       FALSE,
        RCV_TERM_VTTRX_0            =>       FALSE,
        TERMINATION_IMP_0           =>       50,

        AC_CAP_DIS_1                =>       TRUE,
        OOBDETECT_THRESHOLD_1       =>       "111",
        PMA_CDR_SCAN_1              =>       x"6404035",
        PMA_RX_CFG_1                =>       x"0f44088",  
        RCV_TERM_GND_1              =>       FALSE,
        RCV_TERM_VTTRX_1            =>       FALSE,
        TERMINATION_IMP_1           =>       50,

        TERMINATION_CTRL            =>       "10100",
        TERMINATION_OVRD            =>       FALSE,

        ---------------- RX Decision Feedback Equalizer(DFE)  ----------------  

        DFE_CFG_0                   =>       "1001111011",
                 
        DFE_CFG_1                   =>       "1001111011",

        DFE_CAL_TIME                =>       "00110",

        --------------------- RX Serial Line Rate Attributes ------------------   

        PLL_RXDIVSEL_OUT_0          =>       2,
        PLL_SATA_0                  =>       FALSE,

        PLL_RXDIVSEL_OUT_1          =>       2,
        PLL_SATA_1                  =>       FALSE,

        ----------------------- PRBS Detection Attributes ---------------------  

        PRBS_ERR_THRESHOLD_0        =>       x"00000001",

        PRBS_ERR_THRESHOLD_1        =>       x"00000001",

        ---------------- Comma Detection and Alignment Attributes -------------  

        ALIGN_COMMA_WORD_0          =>       1,
        COMMA_10B_ENABLE_0          =>       "1111111111",
        COMMA_DOUBLE_0              =>       FALSE,
        DEC_MCOMMA_DETECT_0         =>       TRUE,
        DEC_PCOMMA_DETECT_0         =>       TRUE,
        DEC_VALID_COMMA_ONLY_0      =>       FALSE,
        MCOMMA_10B_VALUE_0          =>       "1010000011",
        MCOMMA_DETECT_0             =>       TRUE,
        PCOMMA_10B_VALUE_0          =>       "0101111100",
        PCOMMA_DETECT_0             =>       TRUE,
        RX_SLIDE_MODE_0             =>       "PCS",

        ALIGN_COMMA_WORD_1          =>       1,
        COMMA_10B_ENABLE_1          =>       "1111111111",
        COMMA_DOUBLE_1              =>       FALSE,
        DEC_MCOMMA_DETECT_1         =>       TRUE,
        DEC_PCOMMA_DETECT_1         =>       TRUE,
        DEC_VALID_COMMA_ONLY_1      =>       FALSE,
        MCOMMA_10B_VALUE_1          =>       "1010000011",
        MCOMMA_DETECT_1             =>       TRUE,
        PCOMMA_10B_VALUE_1          =>       "0101111100",
        PCOMMA_DETECT_1             =>       TRUE,
        RX_SLIDE_MODE_1             =>       "PCS",

        ------------------ RX Loss-of-sync State Machine Attributes -----------  

        RX_LOSS_OF_SYNC_FSM_0       =>       FALSE,
        RX_LOS_INVALID_INCR_0       =>       8,
        RX_LOS_THRESHOLD_0          =>       128,

        RX_LOSS_OF_SYNC_FSM_1       =>       FALSE,
        RX_LOS_INVALID_INCR_1       =>       8,
        RX_LOS_THRESHOLD_1          =>       128,

        --------------------- RX Gearbox Settings -----------------------------

        RXGEARBOX_USE_0             =>       FALSE,

        RXGEARBOX_USE_1             =>       FALSE,

        -------------- RX Elastic Buffer and Phase alignment Attributes -------   

        PMA_RXSYNC_CFG_0            =>       x"00",
        RX_BUFFER_USE_0             =>       FALSE,
        RX_XCLK_SEL_0               =>       "RXUSR",

        PMA_RXSYNC_CFG_1            =>       x"00",
        RX_BUFFER_USE_1             =>       FALSE,
        RX_XCLK_SEL_1               =>       "RXUSR",                   

        ------------------------ Clock Correction Attributes ------------------   

        CLK_CORRECT_USE_0           =>       FALSE,
        CLK_COR_ADJ_LEN_0           =>       2,
        CLK_COR_DET_LEN_0           =>       2,
        CLK_COR_INSERT_IDLE_FLAG_0  =>       FALSE,
        CLK_COR_KEEP_IDLE_0         =>       FALSE,
        CLK_COR_MAX_LAT_0           =>       20,
        CLK_COR_MIN_LAT_0           =>       16,
        CLK_COR_PRECEDENCE_0        =>       TRUE,
        CLK_COR_REPEAT_WAIT_0       =>       0,
        CLK_COR_SEQ_1_1_0           =>       "0000000000",
        CLK_COR_SEQ_1_2_0           =>       "0000000000",
        CLK_COR_SEQ_1_3_0           =>       "0000000000",
        CLK_COR_SEQ_1_4_0           =>       "0000000000",
        CLK_COR_SEQ_1_ENABLE_0      =>       "0000",
        CLK_COR_SEQ_2_1_0           =>       "0000000000",
        CLK_COR_SEQ_2_2_0           =>       "0000000000",
        CLK_COR_SEQ_2_3_0           =>       "0000000000",
        CLK_COR_SEQ_2_4_0           =>       "0000000000",
        CLK_COR_SEQ_2_ENABLE_0      =>       "0000",
        CLK_COR_SEQ_2_USE_0         =>       FALSE,
        RX_DECODE_SEQ_MATCH_0       =>       TRUE,

        CLK_CORRECT_USE_1           =>       FALSE,
        CLK_COR_ADJ_LEN_1           =>       2,
        CLK_COR_DET_LEN_1           =>       2,
        CLK_COR_INSERT_IDLE_FLAG_1  =>       FALSE,
        CLK_COR_KEEP_IDLE_1         =>       FALSE,
        CLK_COR_MAX_LAT_1           =>       20,
        CLK_COR_MIN_LAT_1           =>       16,
        CLK_COR_PRECEDENCE_1        =>       TRUE,
        CLK_COR_REPEAT_WAIT_1       =>       0,
        CLK_COR_SEQ_1_1_1           =>       "0000000000",
        CLK_COR_SEQ_1_2_1           =>       "0000000000",
        CLK_COR_SEQ_1_3_1           =>       "0000000000",
        CLK_COR_SEQ_1_4_1           =>       "0000000000",
        CLK_COR_SEQ_1_ENABLE_1      =>       "0000",
        CLK_COR_SEQ_2_1_1           =>       "0000000000",
        CLK_COR_SEQ_2_2_1           =>       "0000000000",
        CLK_COR_SEQ_2_3_1           =>       "0000000000",
        CLK_COR_SEQ_2_4_1           =>       "0000000000",
        CLK_COR_SEQ_2_ENABLE_1      =>       "0000",
        CLK_COR_SEQ_2_USE_1         =>       FALSE,
        RX_DECODE_SEQ_MATCH_1       =>       TRUE,

        ------------------------ Channel Bonding Attributes -------------------   

        CB2_INH_CC_PERIOD_0         =>       8,
        CHAN_BOND_1_MAX_SKEW_0      =>       1,
        CHAN_BOND_2_MAX_SKEW_0      =>       1,
        CHAN_BOND_KEEP_ALIGN_0      =>       FALSE,
        CHAN_BOND_LEVEL_0           =>       TILE_CHAN_BOND_LEVEL_0,
        CHAN_BOND_MODE_0            =>       TILE_CHAN_BOND_MODE_0,
        CHAN_BOND_SEQ_1_1_0         =>       "0000000000",
        CHAN_BOND_SEQ_1_2_0         =>       "0000000000",
        CHAN_BOND_SEQ_1_3_0         =>       "0000000000",
        CHAN_BOND_SEQ_1_4_0         =>       "0000000000",
        CHAN_BOND_SEQ_1_ENABLE_0    =>       "0000",
        CHAN_BOND_SEQ_2_1_0         =>       "0000000000",
        CHAN_BOND_SEQ_2_2_0         =>       "0000000000",
        CHAN_BOND_SEQ_2_3_0         =>       "0000000000",
        CHAN_BOND_SEQ_2_4_0         =>       "0000000000",
        CHAN_BOND_SEQ_2_ENABLE_0    =>       "0000",
        CHAN_BOND_SEQ_2_USE_0       =>       FALSE,  
        CHAN_BOND_SEQ_LEN_0         =>       1,
        PCI_EXPRESS_MODE_0          =>       FALSE,   
     
        CB2_INH_CC_PERIOD_1         =>       8,
        CHAN_BOND_1_MAX_SKEW_1      =>       1,
        CHAN_BOND_2_MAX_SKEW_1      =>       1,
        CHAN_BOND_KEEP_ALIGN_1      =>       FALSE,
        CHAN_BOND_LEVEL_1           =>       TILE_CHAN_BOND_LEVEL_1,
        CHAN_BOND_MODE_1            =>       TILE_CHAN_BOND_MODE_1,
        CHAN_BOND_SEQ_1_1_1         =>       "0000000000",
        CHAN_BOND_SEQ_1_2_1         =>       "0000000000",
        CHAN_BOND_SEQ_1_3_1         =>       "0000000000",
        CHAN_BOND_SEQ_1_4_1         =>       "0000000000",
        CHAN_BOND_SEQ_1_ENABLE_1    =>       "0000",
        CHAN_BOND_SEQ_2_1_1         =>       "0000000000",
        CHAN_BOND_SEQ_2_2_1         =>       "0000000000",
        CHAN_BOND_SEQ_2_3_1         =>       "0000000000",
        CHAN_BOND_SEQ_2_4_1         =>       "0000000000",
        CHAN_BOND_SEQ_2_ENABLE_1    =>       "0000",
        CHAN_BOND_SEQ_2_USE_1       =>       FALSE,  
        CHAN_BOND_SEQ_LEN_1         =>       1,
        PCI_EXPRESS_MODE_1          =>       FALSE,

        -------- RX Attributes to Control Reset after Electrical Idle  ------

        RX_EN_IDLE_HOLD_DFE_0       =>       TRUE,
        RX_EN_IDLE_RESET_BUF_0      =>       TRUE,
        RX_IDLE_HI_CNT_0            =>       "1000",
        RX_IDLE_LO_CNT_0            =>       "0000",

        RX_EN_IDLE_HOLD_DFE_1       =>       TRUE,
        RX_EN_IDLE_RESET_BUF_1      =>       TRUE,
        RX_IDLE_HI_CNT_1            =>       "1000",
        RX_IDLE_LO_CNT_1            =>       "0000",


        CDR_PH_ADJ_TIME             =>       "01010",
        RX_EN_IDLE_RESET_FR         =>       TRUE,
        RX_EN_IDLE_HOLD_CDR         =>       FALSE,
        RX_EN_IDLE_RESET_PH         =>       TRUE,

        ------------------ RX Attributes for PCI Express/SATA ---------------

        RX_STATUS_FMT_0             =>       "PCIE",
        SATA_BURST_VAL_0            =>       "100",
        SATA_IDLE_VAL_0             =>       "100",
        SATA_MAX_BURST_0            =>       9,
        SATA_MAX_INIT_0             =>       27,
        SATA_MAX_WAKE_0             =>       9,
        SATA_MIN_BURST_0            =>       5,
        SATA_MIN_INIT_0             =>       15,
        SATA_MIN_WAKE_0             =>       5,
        TRANS_TIME_FROM_P2_0        =>       x"003c",
        TRANS_TIME_NON_P2_0         =>       x"0019",
        TRANS_TIME_TO_P2_0          =>       x"0064",

        RX_STATUS_FMT_1             =>       "PCIE",
        SATA_BURST_VAL_1            =>       "100",
        SATA_IDLE_VAL_1             =>       "100",
        SATA_MAX_BURST_1            =>       9,
        SATA_MAX_INIT_1             =>       27,
        SATA_MAX_WAKE_1             =>       9,
        SATA_MIN_BURST_1            =>       5,
        SATA_MIN_INIT_1             =>       15,
        SATA_MIN_WAKE_1             =>       5,
        TRANS_TIME_FROM_P2_1        =>       x"003c",
        TRANS_TIME_NON_P2_1         =>       x"0019",
        TRANS_TIME_TO_P2_1          =>       x"0064"
    ) 
    port map 
    (
        ------------------------ Loopback and Powerdown Ports ----------------------
        LOOPBACK0                       =>      gnd_vec(2 downto 0),
        LOOPBACK1                       =>      gnd_vec(2 downto 0),
        RXPOWERDOWN0                    =>      gnd_vec(1 downto 0),
        RXPOWERDOWN1                    =>      gnd_vec(1 downto 0),
        TXPOWERDOWN0                    =>      gnd_vec(1 downto 0),
        TXPOWERDOWN1                    =>      gnd_vec(1 downto 0),
        -------------- Receive Ports - 64b66b and 64b67b Gearbox Ports -------------
        RXDATAVALID0                    =>      open,
        RXDATAVALID1                    =>      open,
        RXGEARBOXSLIP0                  =>      gnd,
        RXGEARBOXSLIP1                  =>      gnd,
        RXHEADER0                       =>      open,
        RXHEADER1                       =>      open,
        RXHEADERVALID0                  =>      open,
        RXHEADERVALID1                  =>      open,
        RXSTARTOFSEQ0                   =>      open,
        RXSTARTOFSEQ1                   =>      open,
        ----------------------- Receive Ports - 8b10b Decoder ----------------------
        RXCHARISCOMMA0(3 downto 2)      =>      open,
        RXCHARISCOMMA0(1 downto 0)      =>      open,
        RXCHARISCOMMA1(3 downto 2)      =>      open,
        RXCHARISCOMMA1(1 downto 0)      =>      open,
        RXCHARISK0(3 downto 2)          =>      open,
        RXCHARISK0(1 downto 0)          =>      rx_charisk,
        RXCHARISK1(3 downto 2)          =>      open,
        RXCHARISK1(1 downto 0)          =>      open,
        RXDEC8B10BUSE0                  =>      vcc,
        RXDEC8B10BUSE1                  =>      vcc,
        RXDISPERR0(3 downto 2)          =>      open,
        RXDISPERR0(1 downto 0)          =>      rx_disperr,
        RXDISPERR1(3 downto 2)          =>      open,
        RXDISPERR1(1 downto 0)          =>      open,
        RXNOTINTABLE0(3 downto 2)       =>      open,
        RXNOTINTABLE0(1 downto 0)       =>      rx_notintable,
        RXNOTINTABLE1(3 downto 2)       =>      open,
        RXNOTINTABLE1(1 downto 0)       =>      open,
        RXRUNDISP0(3 downto 2)          =>      open,
        RXRUNDISP0(1 downto 0)          =>      rx_rundisp,
        RXRUNDISP1(3 downto 2)          =>      open,
        RXRUNDISP1(1 downto 0)          =>      open,
        ------------------- Receive Ports - Channel Bonding Ports ------------------
        RXCHANBONDSEQ0                  =>      open,
        RXCHANBONDSEQ1                  =>      open,
        RXCHBONDI0                      =>      gnd_vec(3 downto 0),
        RXCHBONDI1                      =>      gnd_vec(3 downto 0),
        RXCHBONDO0                      =>      open,
        RXCHBONDO1                      =>      open,
        RXENCHANSYNC0                   =>      gnd,
        RXENCHANSYNC1                   =>      gnd,
        ------------------- Receive Ports - Clock Correction Ports -----------------
        RXCLKCORCNT0                    =>      open,
        RXCLKCORCNT1                    =>      open,
        --------------- Receive Ports - Comma Detection and Alignment --------------
        RXBYTEISALIGNED0                =>      open,
        RXBYTEISALIGNED1                =>      open,
        RXBYTEREALIGN0                  =>      open,
        RXBYTEREALIGN1                  =>      open,
        RXCOMMADET0                     =>      rx_commadet,
        RXCOMMADET1                     =>      open,
        RXCOMMADETUSE0                  =>      vcc,
        RXCOMMADETUSE1                  =>      vcc,
        RXENMCOMMAALIGN0                =>      gnd,
        RXENMCOMMAALIGN1                =>      gnd,
        RXENPCOMMAALIGN0                =>      gnd,
        RXENPCOMMAALIGN1                =>      gnd,
        RXSLIDE0                        =>      gnd,
        RXSLIDE1                        =>      gnd,
        ----------------------- Receive Ports - PRBS Detection ---------------------
        PRBSCNTRESET0                   =>      gnd,
        PRBSCNTRESET1                   =>      gnd,
        RXENPRBSTST0                    =>      gnd_vec(1 downto 0),
        RXENPRBSTST1                    =>      gnd_vec(1 downto 0),
        RXPRBSERR0                      =>      open,
        RXPRBSERR1                      =>      open,
        ------------------- Receive Ports - RX Data Path interface -----------------
	RXDATA0(31 downto 16)           =>      open,
        RXDATA0(15 downto 0)            =>      rx_data,
        RXDATA1                         =>      open,
        RXDATAWIDTH0                    =>      "01",
        RXDATAWIDTH1                    =>      "01",
        RXRECCLK0                       =>      rxrecclk,
        RXRECCLK1                       =>      open,
        RXRESET0                        =>      rx_reset,
        RXRESET1                        =>      vcc,
        RXUSRCLK0                       =>      rxusrclk,
        RXUSRCLK1                       =>      gnd,
        RXUSRCLK20                      =>      rxusrclk,
        RXUSRCLK21                      =>      gnd,
        ------------ Receive Ports - RX Decision Feedback Equalizer(DFE) -----------
        DFECLKDLYADJ0                   =>      gnd_vec(5 downto 0),
        DFECLKDLYADJ1                   =>      gnd_vec(5 downto 0),
        DFECLKDLYADJMONITOR0            =>      open,
        DFECLKDLYADJMONITOR1            =>      open,
        DFEEYEDACMONITOR0               =>      open,
        DFEEYEDACMONITOR1               =>      open,
        DFESENSCAL0                     =>      open,
        DFESENSCAL1                     =>      open,
        DFETAP10                        =>      gnd_vec(4 downto 0),
        DFETAP11                        =>      gnd_vec(4 downto 0),
        DFETAP1MONITOR0                 =>      open,
        DFETAP1MONITOR1                 =>      open,
        DFETAP20                        =>      gnd_vec(4 downto 0),
        DFETAP21                        =>      gnd_vec(4 downto 0),
        DFETAP2MONITOR0                 =>      open,
        DFETAP2MONITOR1                 =>      open,
        DFETAP30                        =>      gnd_vec(3 downto 0),
        DFETAP31                        =>      gnd_vec(3 downto 0),
        DFETAP3MONITOR0                 =>      open,
        DFETAP3MONITOR1                 =>      open,
        DFETAP40                        =>      gnd_vec(3 downto 0),
        DFETAP41                        =>      gnd_vec(3 downto 0),
        DFETAP4MONITOR0                 =>      open,
        DFETAP4MONITOR1                 =>      open,
        ------- Receive Ports - RX Driver,OOB signalling,Coupling and Eq.,CDR ------
        RXCDRRESET0                     =>      rxcdrreset,
        RXCDRRESET1                     =>      vcc,
        RXELECIDLE0                     =>      open,
        RXELECIDLE1                     =>      open,
        RXENEQB0                        =>      gnd,
        RXENEQB1                        =>      gnd,
        RXEQMIX0                        =>      RXEQMIX,
        RXEQMIX1                        =>      RXEQMIX,
        RXEQPOLE0                       =>      "0000",
        RXEQPOLE1                       =>      "0000",
        RXN0                            =>      RXN0,
        RXN1                            =>      RXN1,
        RXP0                            =>      RXP0,
        RXP1                            =>      RXP1,
        -------- Receive Ports - RX Elastic Buffer and Phase Alignment Ports -------
        RXBUFRESET0                     =>      gnd,
        RXBUFRESET1                     =>      gnd,
        RXBUFSTATUS0                    =>      open,
        RXBUFSTATUS1                    =>      open,
        RXCHANISALIGNED0                =>      open,
        RXCHANISALIGNED1                =>      open,
        RXCHANREALIGN0                  =>      open,
        RXCHANREALIGN1                  =>      open,
        RXENPMAPHASEALIGN0              =>      vcc,
        RXENPMAPHASEALIGN1              =>      vcc,
        RXPMASETPHASE0                  =>      rxpmasetphase,
        RXPMASETPHASE1                  =>      gnd,
        RXSTATUS0                       =>      open,
        RXSTATUS1                       =>      open,
        --------------- Receive Ports - RX Loss-of-sync State Machine --------------
        RXLOSSOFSYNC0                   =>      rx_loss,
        RXLOSSOFSYNC1                   =>      open,
        ---------------------- Receive Ports - RX Oversampling ---------------------
        RXENSAMPLEALIGN0                =>      gnd,
        RXENSAMPLEALIGN1                =>      gnd,
        RXOVERSAMPLEERR0                =>      open,
        RXOVERSAMPLEERR1                =>      open,
        -------------- Receive Ports - RX Pipe Control for PCI Express -------------
        PHYSTATUS0                      =>      open,
        PHYSTATUS1                      =>      open,
        RXVALID0                        =>      open,
        RXVALID1                        =>      open,
        ----------------- Receive Ports - RX Polarity Control Ports ----------------
        RXPOLARITY0                     =>      rx_polarity,
        RXPOLARITY1                     =>      gnd,
        ------------- Shared Ports - Dynamic Reconfiguration Port (DRP) ------------
        DADDR                           =>      gnd_vec(6 downto 0),
        DCLK                            =>      gnd,
        DEN                             =>      gnd,
        DI                              =>      gnd_vec(15 downto 0),
        DO                              =>      open,
        DRDY                            =>      open,
        DWE                             =>      gnd,
        --------------------- Shared Ports - Tile and PLL Ports --------------------
        CLKIN                           =>      CLKIN,
        GTXRESET                        =>      reset,
        GTXTEST                         =>      "10000000000000",
        INTDATAWIDTH                    =>      vcc,
        PLLLKDET                        =>      plllkdet,
        PLLLKDETEN                      =>      vcc,
        PLLPOWERDOWN                    =>      gnd,
        REFCLKOUT                       =>      refclk_i,
        REFCLKPWRDNB                    =>      vcc,
        RESETDONE0                      =>      open,
        RESETDONE1                      =>      open,
        -------------- Transmit Ports - 64b66b and 64b67b Gearbox Ports ------------
        TXGEARBOXREADY0                 =>      open,
        TXGEARBOXREADY1                 =>      open,
        TXHEADER0                       =>      gnd_vec(2 downto 0),
        TXHEADER1                       =>      gnd_vec(2 downto 0),
        TXSEQUENCE0                     =>      gnd_vec(6 downto 0),
        TXSEQUENCE1                     =>      gnd_vec(6 downto 0),
        TXSTARTSEQ0                     =>      gnd,
        TXSTARTSEQ1                     =>      gnd,
        ---------------- Transmit Ports - 8b10b Encoder Control Ports --------------
        TXBYPASS8B10B0                  =>      gnd_vec(3 downto 0),
        TXBYPASS8B10B1                  =>      gnd_vec(3 downto 0),
        TXCHARDISPMODE0                 =>      gnd_vec(3 downto 0),
        TXCHARDISPMODE1                 =>      gnd_vec(3 downto 0),
        TXCHARDISPVAL0                  =>      gnd_vec(3 downto 0),
        TXCHARDISPVAL1                  =>      gnd_vec(3 downto 0),
        TXCHARISK0(3 downto 2)          =>      gnd_vec(1 downto 0),
        TXCHARISK0(1 downto 0)          =>      tx_charisk,
        TXCHARISK1(3 downto 2)          =>      gnd_vec(1 downto 0),
        TXCHARISK1(1 downto 0)          =>      gnd_vec(1 downto 0),
        TXENC8B10BUSE0                  =>      vcc,
        TXENC8B10BUSE1                  =>      vcc,
        TXKERR0                         =>      open,
        TXKERR1                         =>      open,
        TXRUNDISP0                      =>      open,
        TXRUNDISP1                      =>      open,
        ------------- Transmit Ports - TX Buffering and Phase Alignment ------------
        TXBUFSTATUS0                    =>      open,
        TXBUFSTATUS1                    =>      open,
        ------------------ Transmit Ports - TX Data Path interface -----------------
	TXDATA0(31 downto 16)           =>      gnd_vec(31 downto 16),
        TXDATA0(15 downto 0)            =>      tx_data,
        TXDATA1                         =>      gnd_vec(31 downto 0),
        TXDATAWIDTH0                    =>      "01",
        TXDATAWIDTH1                    =>      "01",
        TXOUTCLK0                       =>      tx_outclk,
        TXOUTCLK1                       =>      open,
        TXRESET0                        =>      tx_reset,
        TXRESET1                        =>      vcc,
        TXUSRCLK0                       =>      txusrclk,
        TXUSRCLK1                       =>      gnd,
        TXUSRCLK20                      =>      txusrclk,
        TXUSRCLK21                      =>      gnd,
        --------------- Transmit Ports - TX Driver and OOB signalling --------------
        TXBUFDIFFCTRL0                  =>      "101",
        TXBUFDIFFCTRL1                  =>      "101",
        TXDIFFCTRL0                     =>      gnd_vec(2 downto 0),
        TXDIFFCTRL1                     =>      gnd_vec(2 downto 0),
        TXINHIBIT0                      =>      gnd,
        TXINHIBIT1                      =>      gnd,
        TXN0                            =>      TXN0,
        TXN1                            =>      TXN1,
        TXP0                            =>      TXP0,
        TXP1                            =>      TXP1,
        TXPREEMPHASIS0(3)               =>      gnd,
        TXPREEMPHASIS0(2 downto 0)      =>      gnd_vec(2 downto 0),
        TXPREEMPHASIS1(3)               =>      gnd,
        TXPREEMPHASIS1(2 downto 0)      =>      gnd_vec(2 downto 0),
        -------- Transmit Ports - TX Elastic Buffer and Phase Alignment Ports ------
        TXENPMAPHASEALIGN0              =>      gnd,
        TXENPMAPHASEALIGN1              =>      gnd,
        TXPMASETPHASE0                  =>      gnd,
        TXPMASETPHASE1                  =>      gnd,
        --------------------- Transmit Ports - TX PRBS Generator -------------------
        TXENPRBSTST0                    =>      gnd_vec(1 downto 0),
        TXENPRBSTST1                    =>      gnd_vec(1 downto 0),
        -------------------- Transmit Ports - TX Polarity Control ------------------
        TXPOLARITY0                     =>      tx_polarity,
        TXPOLARITY1                     =>      gnd,
        ----------------- Transmit Ports - TX Ports for PCI Express ----------------
        TXDETECTRX0                     =>      gnd,
        TXDETECTRX1                     =>      gnd,
        TXELECIDLE0                     =>      gnd,
        TXELECIDLE1                     =>      gnd,
        --------------------- Transmit Ports - TX Ports for SATA -------------------
        TXCOMSTART0                     =>      gnd,
        TXCOMSTART1                     =>      gnd,
        TXCOMTYPE0                      =>      gnd,
        TXCOMTYPE1                      =>      gnd

    );

  refclk_ibufds_i : IBUFDS
    port map
    (
      O	 => CLKIN,
      I	 => REFCLKP,			-- Connect to package pin Y4
      IB => REFCLKN			-- Connect to package pin Y3
      );

  i_bufg0: BUFG
    port map (
      O => rxusrclk,
      I => rxrecclk);

  i_bufg1: BUFG
    port map (
      O => txusrclk,
      I => tx_outclk);

  i_bufg2: BUFG
    port map (
      O => refclk,
      I => refclk_i);

  REFCLK_OUT <= refclk;
  tx_event_clk <= txusrclk;
  
  vcc <= '1';
  gnd <= '0';
  gnd_vec <= (others => '0');

  RXEQMIX <= "01";
  
      -- MGT0 - Event link
      -- MGT1 - unused

--  refclk_out <= refclk;
  
  event_clk_out <= rxusrclk;
  event_clk <= rxusrclk;

  rx_powerdown <= '0';
  tx_powerdown <= '0';

  error_detect : process (rxusrclk, rx_data, rx_charisk,
                         rx_disperr, rx_notintable, rx_loss)
  begin
    if rising_edge(rxusrclk) then
      rx_error <= '0';
      if (rx_charisk(0) = '1' and rx_data(7) = '1') or
        rx_disperr /= "00" or rx_notintable /= "00" or
        rx_loss(1) = '1' then
        rx_error <= '1';
      end if;
    end if;
  end process;

  process (refclk, reset, rx_loss, rx_charisk, link_ok, rx_disperr)
    variable prescaler : unsigned(10 downto 0);
    variable count : unsigned(3 downto 0);
--    variable prescaler : natural range 0 to MGT_RX_PRESCALER - 1;
--    variable count : natural range 0 to MGT_RX_TIMEOUT - 1;
--    variable k_counter : natural range 0 to MGT_RX_K_COUNTER - 1;
    variable rx_error_sync : std_logic;
    variable rx_error_sync_1 : std_logic;
    variable loss_lock : std_logic;
    variable rx_error_count : std_logic_vector(5 downto 0);
    variable reset_sync : std_logic_vector(1 downto 0);
  begin
    rx_link_ok <= link_ok;

    if rising_edge(refclk) then
      if prescaler(prescaler'high) = '1' then
	link_ok <= '0';
	if count = 0 then
	  link_ok <= '1';
	end if;
      
	rxcdrreset <= not cdrres;
	if count = 15 then
          rxcdrreset <= '1';
        end if;

        rxpmasetphase <= not cdrphas;
--        if count = 8 then
--          rxpmasetphase <= '1';
--        end if;

	if count /= 0 then
	  count := count - 1;
	end if;

      end if;

      if count = 0 then
        if loss_lock = '1' then
          count := "1111";
        end if;
      end if;

      if link_ok = '0' then
        loss_lock := rx_error;
      else
        loss_lock := rx_error_count(5);
      end if;

      if link_ok = '1' then
        if rx_error_sync = '1' then
          if rx_error_count(5) = '0' then
            rx_error_count := rx_error_count - 1;
          end if;
        else
          if prescaler(prescaler'high) = '1' and
            (rx_error_count(5) = '1' or rx_error_count(4) = '0') then
            rx_error_count := rx_error_count + 1;
          end if;
        end if;
      else
        rx_error_count := "011111";
      end if;
            
      if prescaler(prescaler'high) = '1' then
        prescaler := "01111111111";
      else
        prescaler := prescaler - 1;
      end if;

      rx_error_sync := rx_error_sync_1;
      rx_error_sync_1 := rx_error;
    end if;

    if reset_sync(0) = '1' then
      count := "1111";
    end if;

    -- Synchronize asynchronous resets
    reset_sync(0) := reset_sync(1);
    reset_sync(1) := '0';
    if reset = '1' or plllkdet = '0' then
      reset_sync(1) := '1';
    end if;
  end process;

  reg_dbus_data : process (event_clk, link_ok, rx_data)
    variable even : std_logic;
  begin
    if rising_edge(event_clk) then
      if databuf_mode = '0' or even = '0' then
	dbus_data <= rx_data(7 downto 0);
      end if;
      if databuf_mode = '1' then
	if even = '1' then
	  databuf_data <= rx_data(7 downto 0);
	  databuf_k <= rx_charisk(0);
	end if;
      else
	databuf_data <= (others => '0');
	databuf_k <= '0';
      end if;

      databuf_ena <= even;
      
      if link_ok = '0' then
	databuf_data <= (others => '0');
	databuf_k <= '0';
	dbus_data <= (others => '0');
      end if;

      even := not even;
      event_code <= rx_data(15 downto 8);
      if link_ok = '0' or rx_charisk(1) = '1' or
	reset = '1' then
	event_code <= (others => '0');
	even := '1';
      end if;
    end if;
  end process;

  align_detect : process (event_clk, reset, rx_charisk, rx_data)
  begin
    if rising_edge(event_clk) then
      align_error <= '0';
      if rx_charisk(0) = '1' and rx_data(7) = '1' then
	align_error <= '1';
      end if;
    end if;
  end process;

  violation_detect : process (event_clk, reset, rx_clear_viol,
			      rx_disperr, rx_loss, rx_notintable, link_ok)
  begin
    if reset = '1' or rx_clear_viol = '1' then
      rx_violation <= '0';
    elsif rising_edge(event_clk) then
      if rx_disperr /= "00" or rx_loss(1) = '1' or
        rx_notintable /= "00" then
	rx_violation <= '1';
      end if;
    end if;
  end process;

  TRIG0(15 downto 0) <= rx_data;
  TRIG0(17 downto 16) <= rx_charisk;
  TRIG0(19 downto 18) <= rx_disperr;
  TRIG0(21 downto 20) <= rx_loss;
  TRIG0(23 downto 22) <= rx_notintable;
  TRIG0(24) <= link_ok;
  TRIG0(47 downto 32) <= tx_data;
  TRIG0(49 downto 48) <= tx_charisk;
  
end structure;
