---------------------------------------------------------------------------
--
--  File        : sfp_serial.vhd
--
--  Title       : SFP transceiver serial EEPROM/status read
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
use work.evr_bs_pkg.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity sfp_serial is
  port (
    clk      : in std_logic;
    usec_ena : in std_logic;
    sda      : inout std_logic;
    scl      : out std_logic;
    wr_data  : out std_logic_vector(7 downto 0);
    wr_addr  : out std_logic_vector(8 downto 0);
    wr_ena   : out std_logic);
end sfp_serial;

architecture structure of sfp_serial is

begin

-- Read EEPROM
--
-- SDA 1100 aaaabbbbccccddddeeeeffffgggghhhhZZZZ
-- SCL 0111 001000100010001000100010001000100010
--     ^ start condition
--          ^ write 8 bits MSB first (device address 0xa0 for EEPROM)
--                                            ^ read ack (SDA should be low)
--
-- SDA 00000000000000000000000000000000ZZZZ
-- SCL 001000100010001000100010001000100010
--     ^ write address
--                                       ^ read ack (SDA should be low)
--
-- SDA aaaabbbbccccddddeeeeffffgggghhhhZZZZ
-- SCL 001000100010001000100010001000100010
--     ^ write 8 bits MSB first (device address 0xa1 for EEPROM)
--                               ^ read ack (SDA should be low)
--
-- SDA ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ0000
-- SCL 001000100010001000100010001000100010
--       ^   ^   ^   ^   ^   ^   ^   ^ read 8 bits MSB first
--                                       ^ ack cycle
--
-- repeat previous cycle for all bytes to be read.
--
-- SDA 0011
-- SCL 0111
--     ^ end condition

  process (clk, usec_ena)
    variable cnt        : std_logic_vector(4 downto 0) := "01001";
    variable ena_10us   : std_logic;
    variable phase      : std_logic_vector(3 downto 0) := "0000";
    variable next_phase : std_logic := '0';
    variable step_cnt   : std_logic_vector(5 downto 0) := "000000";
    variable din        : std_logic_vector(7 downto 0);
    variable address    : std_logic_vector(8 downto 0) := "000000000";
  begin
    if rising_edge(clk) then
      wr_ena <= '0';

      if ena_10us = '1' then
	next_phase := '0';
	sda <= 'Z';
	scl <= '0';
	if step_cnt(1 downto 0) = "10" then
	  scl <= '1';
	end if;

	case phase is
	  when "0000" | "1000" => -- Start condition
	    case step_cnt is
	      when "000000" =>
		sda <= '1';
	      when "000001" =>
		sda <= '1';
		scl <= '1';
	      when "000010" =>
		sda <= '0';
		scl <= '1';
	      when others =>
		sda <= '0';
		scl <= '1';
		next_phase := '1';
	    end case;
	  when "0001" =>
	    case step_cnt(5 downto 2) is
	      when "0000" => sda <= '1';
	      when "0001" => sda <= '0';
	      when "0010" => sda <= '1';
	      when "0011" => sda <= '0';
	      when "0100" => sda <= '0';
	      when "0101" => sda <= '0';
	      when "0110" => sda <= '0';
	      when "0111" => sda <= '0';
	      when others =>
		if step_cnt(1 downto 0) = "11" then
		  next_phase := '1';
		end if;
	    end case;
	  when "0010" | "1010" =>
	    case step_cnt(5 downto 2) is
	      when "0000" => sda <= '0';
	      when "0001" => sda <= '0';
	      when "0010" => sda <= '0';
	      when "0011" => sda <= '0';
	      when "0100" => sda <= '0';
	      when "0101" => sda <= '0';
	      when "0110" => sda <= '0';
	      when "0111" => sda <= '0';
	      when others =>
		if step_cnt(1 downto 0) = "11" then
		  next_phase := '1';
		end if;
	    end case;
	  when "0011" =>
	    case step_cnt(5 downto 2) is
	      when "0000" => sda <= '1';
	      when "0001" => sda <= '0';
	      when "0010" => sda <= '1';
	      when "0011" => sda <= '0';
	      when "0100" => sda <= '0';
	      when "0101" => sda <= '0';
	      when "0110" => sda <= '0';
	      when "0111" => sda <= '1';
	      when others =>
		if step_cnt(1 downto 0) = "11" then
		  next_phase := '1';
		end if;
	    end case;
	  when "0100" =>
	    if step_cnt(5) = '0' and step_cnt(1 downto 0) = "10" then
	      din := din(7 downto 1) & sda;
	    end if;
	    if step_cnt(5) = '1' and step_cnt(1 downto 0) = "00" then
	      wr_ena <= '1';
	      wr_data <= din;
	      wr_addr <= address;
	    end if;
	    if step_cnt(5) = '1' and step_cnt(1 downto 0) = "11" then
	      if address(8) = '0' then
		address := address + 1;
		step_cnt := "000000";
	      else
		next_phase := '1';
	      end if;
	    end if;
	  when "0101" | "1101" =>
	    case step_cnt is
	      when "000000" =>
		sda <= '0';
	      when "000001" =>
		sda <= '0';
		scl <= '1';
	      when "000010" =>
		sda <= '1';
		scl <= '1';
	      when others =>
		sda <= '1';
		scl <= '1';
		next_phase := '1';
	    end case;
	  when "1001" =>
	    case step_cnt(5 downto 2) is
	      when "0000" => sda <= '1';
	      when "0001" => sda <= '0';
	      when "0010" => sda <= '1';
	      when "0011" => sda <= '0';
	      when "0100" => sda <= '0';
	      when "0101" => sda <= '0';
	      when "0110" => sda <= '1';
	      when "0111" => sda <= '0';
	      when others =>
		if step_cnt(1 downto 0) = "11" then
		  next_phase := '1';
		end if;
	    end case;
	  when "1011" =>
	    case step_cnt(5 downto 2) is
	      when "0000" => sda <= '1';
	      when "0001" => sda <= '0';
	      when "0010" => sda <= '1';
	      when "0011" => sda <= '0';
	      when "0100" => sda <= '0';
	      when "0101" => sda <= '0';
	      when "0110" => sda <= '1';
	      when "0111" => sda <= '1';
	      when others =>
		if step_cnt(1 downto 0) = "11" then
		  next_phase := '1';
		end if;
	    end case;
	  when "1100" =>
	    if step_cnt(5) = '0' and step_cnt(1 downto 0) = "10" then
	      din := din(7 downto 1) & sda;
	    end if;
	    if step_cnt(5) = '1' and step_cnt(1 downto 0) = "00" then
	      wr_ena <= '1';
	    end if;
	    if step_cnt(5) = '1' and step_cnt(1 downto 0) = "11" then
	      if address(8) = '1' then
		address := address + 1;
		step_cnt := "000000";
	      else
		next_phase := '1';
	      end if;
	    end if;
	  when others =>
	end case;

	if next_phase = '1' then
	  phase := phase + 1;
	  step_cnt := (others => '0');
	end if;
		
		
      end if;

      ena_10us := '0';
      if usec_ena = '1' then
	cnt := cnt - 1;
	if cnt(cnt'high) = '1' then
	  ena_10us := '1';
	  cnt := "01001";
	end if;
      end if;
    end if;
  end process;
  
end structure;
  
    
