--------------------------------------------------------------------------
--
--  File        : vmeevr230rf.vhd
--
--  Title       : VME-EVR-230RF Top Level
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_bs_pkg.all;
use work.evr_pkg.all;
use work.opb_vme_pkg.all;
use work.opb_ubicom_pkg.all;

entity vmeevr230rf is
  port (
    CLK33        : in std_logic;  -- 33 MHz crystal oscillator input
    LA           : inout std_logic_vector(31 downto 0);
    LD           : inout std_logic_vector(31 downto 0);
    vmeAM        : inout std_logic_vector(5 downto 0);
    nvmeIACK     : in std_logic;
    nvmeASI      : in std_logic;  -- VME AS* input
    nvmeASO      : out std_logic; -- VME AS* output, OE controlled by VMEMSTR
    nvmeDSI      : in std_logic_vector(1 downto 0);
                                  -- VME DS0*, DS1* input
    nvmeDSO      : out std_logic_vector(1 downto 0);
                                  -- VME DS0*, DS1* output,
                                  -- OE controlled by VMEMSTR
    nvmeRETRYI   : in std_logic;  -- VME RETRY* input
    nvmeRETRYO   : out std_logic; -- VME RETRY* output
    RETRYEN      : out std_logic; -- VME RETRY* OE
    nvmeIACKIN   : in std_logic;
    nvmeIACKOUT  : out std_logic; -- VME IACKOUT* output,
                                  -- OE controlled by nDCHENA
    nvmeDTACKI   : in std_logic;  -- VME DTACK* input
    nvmeDTACKO   : out std_logic := '1'; -- VME DTACK* output
    DTACKEN      : out std_logic := '0'; -- VME DTACK* OE
    nvmeWRITEI   : in std_logic;  -- VME WRITE* input
    nvmeWRITEO   : out std_logic; -- VME WRITE* output,
                                  -- OE controlled by VMEMSTR
    nvmeBRI      : in std_logic_vector(3 downto 0);
                                  -- VME BR0* to BR3* inputs
    nvmeBRO      : out std_logic_vector(3 downto 0);
                                  -- VME BR0* to BR3* OC outputs
    nvmeBGIN     : in std_logic_vector(3 downto 0);
                                  -- VME BGIN0* to BGIN3* inputs
    nvmeBGOUT    : out std_logic_vector(3 downto 0);
    				  -- VME BGOUT0* to BGOUT3* outputs,
                                  -- OE controlled by nDCHENA
    VMEMSTR      : out std_logic;
    DCHENA       : out std_logic;
    nSYSRESETO   : out std_logic;
    nSYSCON      : in std_logic;  -- nSYSCON jumper input
    nvmeSYSFAILI : in std_logic;  -- VME SYSFAIL* input
    nvmeSYSFAILO : out std_logic; -- VME SYSFAIL* OC output
    nvmeACFAILI  : in std_logic;  -- VME ACFAIL* input
    nvmeBERRI    : in std_logic;  -- VME BERR* input
    nvmeBERRO    : out std_logic; -- VME BERR* OC output
    nvmeBBSYI    : in std_logic;  -- VME BBSY* input
    vmeBBSYEN    : out std_logic; -- VME BBSY* OE
    nvmeBCLRI    : in std_logic;  -- VME BCLR* input
    nvmeBCLRO    : out std_logic; -- VME BCLR* output
    vmeBCLREN    : out std_logic; -- VME BCLR* OE
    nAOE         : out std_logic; -- VME address nOE
    ACLKAB       : out std_logic; -- VME address A to B clock (local -> VME)
    ACLKBA       : out std_logic; -- VME address B to A clock (VME -> local)
    ADIR         : out std_logic; -- VME address direction (1 = local -> VME)
    ALE          : out std_logic; -- VME address LE
    AMCLKAB      : out std_logic; -- VME AM A to B clock (local -> VME)
    AMCLKBA      : out std_logic; -- VME AM B to A clock (VME -> local)
    AMDIR        : out std_logic; -- VME AM DIR (1 = local -> VME) 
    AMLE         : out std_logic; -- VME AM LE
    nAMOE        : out std_logic; -- VME AM nOE
    nvmeLWORDI   : in std_logic;  -- VME LWORD* input
    nvmeLWORDO   : out std_logic; -- VME LWORD* output,
                                  -- OE controlled by VMEMSTR
    nDOE2        : out std_logic; -- VME D16 to D31 OE
    DCLKAB2      : out std_logic; -- VME D16 to D31 A to B clock (local -> VME)
    DCLKBA2      : out std_logic; -- VME D16 to D31 B to A clock (VME -> local)
    DDIR2        : out std_logic; -- VME D16 to D31 DIR (1 = local -> VME)
    DLE2         : out std_logic; -- VME D16 to D31 LE
    nDOE         : out std_logic; -- VME D0 to D15 OE
    DCLKAB       : out std_logic; -- VME D0 to D15 A to B clock (local -> VME)
    DCLKBA       : out std_logic; -- VME D0 to D15 B to A clock (VME -> local)
    DDIR         : out std_logic; -- VME D0 to D15 DIR (1 = local -> VME)
    DLE          : out std_logic; -- VME D0 to D15 LE
    nvmeIRQI     : in std_logic_vector(7 downto 1);
    nvmeIRQO     : out std_logic_vector(7 downto 1);
    nGA          : in std_logic_vector(5 downto 0);
                                  -- VME geographical addressing pins
                                  -- GA0* to GA4*, nGA5 is GAP*
    nLII         : in std_logic;  -- LI/I* input

    HSOPEN       : in std_logic;  -- Handle switch position (1 = open)
    BLUEOFF      : out std_logic; -- (1 = turn off blue led)
    PWRDWN       : out std_logic; -- (1 = power down board)
    
    LED          : out std_logic_vector(8 downto 0);
                                  -- LED1 and LED2 controlled by IP2022,
                                  -- not available in FPGA

    ULE          : in std_logic;
    nUBERR       : out std_logic;
    nUACK        : out std_logic;
    nUIRQ        : out std_logic;
    nURD         : in std_logic;
    nUWR         : in std_logic;
    UA           : in std_logic_vector(16 downto 0);
                                  -- Ubicom address bits A16-A9 and A0,
                                  -- A8-A1 are multiplexed with D7-D0
    UD           : inout std_logic_vector(7 downto 0);

    nEINTR       : in  std_logic;
    nESRDY       : in  std_logic;
    EBCLK        : out std_logic;
    nEDATACS     : out std_logic;
    nERDYRTN     : out std_logic;
    nEBE         : out std_logic_vector(3 downto 0);
    nECYCLE      : out std_logic;
    ESWR         : out std_logic;
    nEVLBUS      : out std_logic;
    EAEN         : out std_logic;
    
    DRD          : inout std_logic_vector(31 downto 0); -- DRAM data bus
    DRA          : out std_logic_vector(12 downto 0);   -- DRAM address bus
    DQM          : out std_logic_vector(3 downto 0);    -- DRAM I/O mask
    DRCLK        : out std_logic; -- DRAM clock
    nDRWE        : out std_logic; -- DRAM write wnable WE*
    nDRCAS       : out std_logic; -- DRAM column address strobe CAS*
    nDRRAS       : out std_logic; -- DRAM row address strobe RAS*
    nDRCS        : out std_logic; -- DRAM chip select CS*
    DRCKE        : out std_logic; -- DRAM clock enable
    DRBA         : out std_logic_vector(1 downto 0);    -- DRAM bank address

    RXLOS        : in std_logic;
    MOD_DEF0     : in std_logic;    -- Grounded on SFP module
    MOD_DEF1     : out std_logic;   -- SFP module clock
    MOD_DEF2     : inout std_logic; -- SPF module data
    TXFAULT      : in std_logic;
    
    ADNLOS2      : in std_logic;
    ADNLOL2      : in std_logic;
    RXSDAB2      : inout std_logic;
    RXSCKB2      : out std_logic;

    TXSEL        : out std_logic;
    RXSEL        : out std_logic;
    
    CGDI         : out std_logic;
    CGSK         : out std_logic;
    CGCS         : out std_logic;
    CGLOCK       : in std_logic;
    OSCENA       : out std_logic;
    OSCCLK       : out std_logic;
    OSCCLKB      : out std_logic;

    BREFCLKN     : in std_logic; -- Reference clock from fractional
    BREFCLKP     : in std_logic; -- divider
    BREFCLK2N    : in std_logic; -- Resynchronised clock from
    BREFCLK2P    : in std_logic; -- MC100EP52

    BREFSEL      : out std_logic;

    RXREFCLK     : out std_logic;
    
    RECCLKN      : out std_logic;
    RECCLKP      : out std_logic;

    TTLIN        : in  std_logic_vector(1 downto 0);
    TTLOUT       : out std_logic_vector(3 downto 0);
    UNIVIN       : in  std_logic_vector(3 downto 0);
    UNIVOUT      : out std_logic_vector(3 downto 0);

    GPIO         : inout std_logic_vector(7 downto 0);
    
    IO           : out std_logic_vector(15 downto 0);
    TBIN         : in std_logic_vector(15 downto 0);
--    IOZ          : in std_logic_vector(31 downto 1);

    nIF          : in std_logic_vector(3 downto 2);
    nEVGIF       : in std_logic;  -- EVG transition module detect signal
    nEVRIF       : in std_logic;  -- EVR transition module detect signal
    PWRONIF      : out std_logic; -- Power control for hot-swappable
                                  -- transition modules

    -- MGT I/O pins
    RXN1        : in std_logic;  -- AK4 Event Link RXN
    RXP1        : in std_logic;  -- AK5 Event Link RXP
    TXN1        : out std_logic; -- AK7 Event Link TXN
    TXP1        : out std_logic; -- AK6 Event Link TXP
    RXN2        : in std_logic;  -- AK12 n.c.
    RXP2        : in std_logic;  -- AK11 n.c.
    TXN2        : out std_logic; -- AK14 CML OUT4-
    TXP2        : out std_logic; -- AK13 CML OUT4+
    RXN3        : in std_logic;  -- AK17 n.c.
    RXP3        : in std_logic;  -- AK18 n.c.
    TXN3        : out std_logic; -- AK20 CML OUT5-
    TXP3        : out std_logic; -- AK19 CML OUT5+
    RXN4        : in std_logic;  -- AK24 n.c.
    RXP4        : in std_logic;  -- AK25 n.c.
    TXN4        : out std_logic; -- AK27 CML OUT6-
    TXP4        : out std_logic  -- AK26 CML OUT6+
    );

end vmeevr230rf;   

architecture rtl of vmeevr230rf is

  signal clk             : std_logic;
  signal usec_ena        : std_logic;
  
  signal nDCHENA_int     : std_logic;
  signal nBGOUT          : std_logic_vector(3 downto 0);
  
  signal CR_CSR_address  : std_logic_vector(18 downto 2);
  signal CR_CSR_data_in  : std_logic_vector(7 downto 0);
  signal CR_CSR_data_out : std_logic_vector(7 downto 0);
  signal CR_CSR_ack      : std_logic;
  signal CR_CSR_rd       : std_logic;
  signal CR_CSR_wr       : std_logic;
  
  signal VME64x_BAR      : std_logic_vector(7 downto 0);
  signal VME64x_ADER0    : std_logic_vector(31 downto 0);
  signal VME64x_ADER1    : std_logic_vector(31 downto 0);
  signal VME64x_ADER2    : std_logic_vector(31 downto 0);
  signal module_fail     : std_logic;
  signal module_ena      : std_logic;
  signal module_reset    : std_logic;
  signal init_reset      : std_logic;
  signal SYSFAIL_ena     : std_logic;
  signal VME_irq_level   : std_logic_vector(2 downto 0);

  signal evr_irq         : std_logic;
  signal evr_irq_vector  : std_logic_vector(7 downto 0);
  
  signal local_rd_out    : std_logic;
  signal local_wr_out    : std_logic;
  signal VME_request_out : std_logic;
  signal nDTACK_out      : std_logic;
  signal module_nBERR    : std_logic;
  
  signal debug_out       : std_logic_vector(31 downto 0);

  signal dummy_io        : std_logic_vector(31 downto 0) := X"00000000";
  signal dummy_iodir     : std_logic_vector(3 downto 0);
  
  signal nTBID           : std_logic_vector(3 downto 0);
  
  signal evr_reset       : std_logic;

  signal SDRAM_DQ_I         : std_logic_vector(31 downto 0);
  signal SDRAM_DQ_O         : std_logic_vector(31 downto 0);
  signal SDRAM_DQ_T         : std_logic_vector(31 downto 0);
  signal SDRAM_init_done    : std_logic;

  signal vme_OPB_Clk : std_logic;
  signal vme_OPB_Rst : std_logic;
  signal vme_Sl_DBus : std_logic_vector(0 to 31);
  signal vme_Sl_errAck : std_logic;
  signal vme_Sl_retry : std_logic;
  signal vme_Sl_toutSup : std_logic;
  signal vme_Sl_xferAck : std_logic;
  signal vme_OPB_ABus : std_logic_vector(0 to 31);
  signal vme_OPB_BE : std_logic_vector(0 to 3);
  signal vme_OPB_DBus : std_logic_vector(0 to 31);
  signal vme_OPB_RNW : std_logic;
  signal vme_OPB_select : std_logic;
  signal vme_OPB_seqAddr : std_logic;
  signal vme_M_ABus : std_logic_vector(0 to 31);
  signal vme_M_BE : std_logic_vector(0 to 3);
  signal vme_M_busLock : std_logic;
  signal vme_M_request : std_logic;
  signal vme_M_RNW : std_logic;
  signal vme_M_select : std_logic;
  signal vme_M_seqAddr : std_logic;
  signal vme_OPB_errAck : std_logic;
  signal vme_OPB_MGrant : std_logic;
  signal vme_OPB_retry : std_logic;
  signal vme_OPB_timeout : std_logic;
  signal vme_OPB_xferAck : std_logic;
  signal vme_IP2INTC_Irpt : std_logic;

  signal VME_debug : std_logic_vector(39 downto 0);
  
  signal ubi_OPB_Clk : std_logic;
  signal ubi_OPB_Rst : std_logic;
  signal ubi_M_DBus : std_logic_vector(0 to 31);
  signal ubi_OPB_DBus : std_logic_vector(0 to 31);
  signal ubi_M_ABus : std_logic_vector(0 to 31);
  signal ubi_M_BE : std_logic_vector(0 to 3);
  signal ubi_M_busLock : std_logic;
  signal ubi_M_request : std_logic;
  signal ubi_M_RNW : std_logic;
  signal ubi_M_select : std_logic;
  signal ubi_M_seqAddr : std_logic;
  signal ubi_OPB_errAck : std_logic;
  signal ubi_OPB_MGrant : std_logic;
  signal ubi_OPB_retry : std_logic;
  signal ubi_OPB_timeout : std_logic;
  signal ubi_OPB_xferAck : std_logic;

  signal crcsr_OPB_Clk : std_logic;
  signal crcsr_OPB_Rst : std_logic;
  signal crcsr_Sl_DBus : std_logic_vector(0 to 31);
  signal crcsr_Sl_errAck : std_logic;
  signal crcsr_Sl_retry : std_logic;
  signal crcsr_Sl_toutSup : std_logic;
  signal crcsr_Sl_xferAck : std_logic;
  signal crcsr_OPB_ABus : std_logic_vector(0 to 31);
  signal crcsr_OPB_BE : std_logic_vector(0 to 3);
  signal crcsr_OPB_DBus : std_logic_vector(0 to 31);
  signal crcsr_OPB_RNW : std_logic;
  signal crcsr_OPB_select : std_logic;
  signal crcsr_OPB_seqAddr : std_logic;
  signal crcsr_debug : std_logic_vector(6 downto 0);
  
  signal evr_OPB_Clk : std_logic;
  signal evr_OPB_Rst : std_logic;
  signal evr_Sl_DBus : std_logic_vector(0 to 31);
  signal evr_Sl_errAck : std_logic;
  signal evr_Sl_retry : std_logic;
  signal evr_Sl_toutSup : std_logic;
  signal evr_Sl_xferAck : std_logic;
  signal evr_OPB_ABus : std_logic_vector(0 to 31);
  signal evr_OPB_BE : std_logic_vector(0 to 3);
  signal evr_OPB_DBus : std_logic_vector(0 to 31);
  signal evr_OPB_RNW : std_logic;
  signal evr_OPB_select : std_logic;
  signal evr_OPB_seqAddr : std_logic;

  signal CMLRXN : std_logic_vector(0 to 2);
  signal CMLRXP : std_logic_vector(0 to 2);
  signal CMLTXN : std_logic_vector(0 to 2);
  signal CMLTXP : std_logic_vector(0 to 2);

  signal TXLOOPBACK : std_logic;
  signal RXLOOPBACK : std_logic;

  component ibufgds
  port (O  : out std_ulogic;
        I  : in std_ulogic;
        IB : in std_ulogic);
  end component;

  component bufg
  port (O  : out std_ulogic;
        I  : in std_ulogic);
  end component;

  component IOBUF is
    port (
      I : in std_logic;
      IO : inout std_logic;
      O : out std_logic;
      T : in std_logic
      );
  end component;

  component local_clock is
    port (
      clkin      : in std_logic;
      clkfxout   : out std_logic
      );
  end component;

  component system is
    port (
      sys_clk : in std_logic;
      sys_rst : in std_logic;
      opb_ms_0_OPB_Clk : out std_logic;
      opb_ms_0_OPB_Rst : out std_logic;
      opb_ms_0_Sl_DBus : in std_logic_vector(0 to 31);
      opb_ms_0_Sl_errAck : in std_logic;
      opb_ms_0_Sl_retry : in std_logic;
      opb_ms_0_Sl_toutSup : in std_logic;
      opb_ms_0_Sl_xferAck : in std_logic;
      opb_ms_0_OPB_ABus : out std_logic_vector(0 to 31);
      opb_ms_0_OPB_BE : out std_logic_vector(0 to 3);
      opb_ms_0_OPB_DBus : out std_logic_vector(0 to 31);
      opb_ms_0_OPB_RNW : out std_logic;
      opb_ms_0_OPB_select : out std_logic;
      opb_ms_0_OPB_seqAddr : out std_logic;
      opb_ms_0_M_ABus : in std_logic_vector(0 to 31);
      opb_ms_0_M_BE : in std_logic_vector(0 to 3);
      opb_ms_0_M_busLock : in std_logic;
      opb_ms_0_M_request : in std_logic;
      opb_ms_0_M_RNW : in std_logic;
      opb_ms_0_M_select : in std_logic;
      opb_ms_0_M_seqAddr : in std_logic;
      opb_ms_0_OPB_errAck : out std_logic;
      opb_ms_0_OPB_MGrant : out std_logic;
      opb_ms_0_OPB_retry : out std_logic;
      opb_ms_0_OPB_timeout : out std_logic;
      opb_ms_0_OPB_xferAck : out std_logic;
      opb_ms_0_IP2INTC_Irpt : in std_logic;
      opb_m_1_OPB_Clk : out std_logic;
      opb_m_1_OPB_Rst : out std_logic;
      opb_m_1_M_DBus : in std_logic_vector(0 to 31);
      opb_m_1_OPB_DBus : out std_logic_vector(0 to 31);
      opb_m_1_M_ABus : in std_logic_vector(0 to 31);
      opb_m_1_M_BE : in std_logic_vector(0 to 3);
      opb_m_1_M_busLock : in std_logic;
      opb_m_1_M_request : in std_logic;
      opb_m_1_M_RNW : in std_logic;
      opb_m_1_M_select : in std_logic;
      opb_m_1_M_seqAddr : in std_logic;
      opb_m_1_OPB_errAck : out std_logic;
      opb_m_1_OPB_MGrant : out std_logic;
      opb_m_1_OPB_retry : out std_logic;
      opb_m_1_OPB_timeout : out std_logic;
      opb_m_1_OPB_xferAck : out std_logic;
      opb_s_2_OPB_Clk : out std_logic;
      opb_s_2_OPB_Rst : out std_logic;
      opb_s_2_Sl_DBus : in std_logic_vector(0 to 31);
      opb_s_2_Sl_errAck : in std_logic;
      opb_s_2_Sl_retry : in std_logic;
      opb_s_2_Sl_toutSup : in std_logic;
      opb_s_2_Sl_xferAck : in std_logic;
      opb_s_2_OPB_ABus : out std_logic_vector(0 to 31);
      opb_s_2_OPB_BE : out std_logic_vector(0 to 3);
      opb_s_2_OPB_DBus : out std_logic_vector(0 to 31);
      opb_s_2_OPB_RNW : out std_logic;
      opb_s_2_OPB_select : out std_logic;
      opb_s_2_OPB_seqAddr : out std_logic;
      opb_s_3_OPB_Clk : out std_logic;
      opb_s_3_OPB_Rst : out std_logic;
      opb_s_3_Sl_DBus : in std_logic_vector(0 to 31);
      opb_s_3_Sl_errAck : in std_logic;
      opb_s_3_Sl_retry : in std_logic;
      opb_s_3_Sl_toutSup : in std_logic;
      opb_s_3_Sl_xferAck : in std_logic;
      opb_s_3_OPB_ABus : out std_logic_vector(0 to 31);
      opb_s_3_OPB_BE : out std_logic_vector(0 to 3);
      opb_s_3_OPB_DBus : out std_logic_vector(0 to 31);
      opb_s_3_OPB_RNW : out std_logic;
      opb_s_3_OPB_select : out std_logic;
      opb_s_3_OPB_seqAddr : out std_logic;
      chipscope_opb_iba_0_iba_trig_in_pin : in std_logic_vector(39 downto 0)
      );
  end component;

  component opb_vme is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_ETHBASEADDR      : std_logic_vector;
      C_ETHBASEMASK      : std_logic_vector;
      C_ADER0_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
      C_ADER0_CMP_LOW    : natural := 16; -- Size 16 = 64k
      C_ADER1_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
      C_ADER1_CMP_LOW    : natural := 16; -- Size 16 = 64k
      C_ADER2_CMP_HIGH   : natural := 31; -- Address range -1 (A32) 
      C_ADER2_CMP_LOW    : natural := 16; -- Size 16 = 64k
      C_CRCSR_CMP_HIGH   : natural := 23;
      C_CRCSR_CMP_LOW    : natural := 19;
      C_FUNC0_LOCAL_BASE : std_logic_vector;
      C_FUNC1_LOCAL_BASE : std_logic_vector;
      C_FUNC2_LOCAL_BASE : std_logic_vector;
      C_CRCSR_LOCAL_BASE : std_logic_vector;
      C_OPB_AWIDTH       : integer               := 32;
      C_OPB_DWIDTH       : integer               := 32;
      C_FAMILY           : string                := "virtex2p");
    port (
      -- Shared VME/MAC signals
      LA           : inout std_logic_vector(31 downto 0);
      LD           : inout std_logic_vector(31 downto 0);
      
      -- VME signals
      vmeAM        : inout std_logic_vector(5 downto 0);
      nvmeIACK     : in std_logic;
      nvmeASI      : in std_logic;  -- VME AS* input
      nvmeASO      : out std_logic; -- VME AS* output, OE controlled by VMEMSTR
      nvmeDSI      : in std_logic_vector(1 downto 0);
      -- VME DS0*, DS1* input
      nvmeDSO      : out std_logic_vector(1 downto 0);
      -- VME DS0*, DS1* output,
      -- OE controlled by VMEMSTR
      nvmeRETRYI   : in std_logic;  -- VME RETRY* input
      nvmeRETRYO   : out std_logic; -- VME RETRY* output
      RETRYEN      : out std_logic; -- VME RETRY* OE
      nvmeIACKIN   : in std_logic;  -- VME IACKIN* input
      nvmeIACKOUT  : out std_logic; -- VME IACKOUT* output
      nvmeDTACKI   : in std_logic;  -- VME DTACK* input
      nvmeDTACKO   : out std_logic := '1'; -- VME DTACK* output
      DTACKEN      : out std_logic := '0'; -- VME DTACK* OE
      nvmeWRITEI   : in std_logic;  -- VME WRITE* input
      nvmeWRITEO   : out std_logic; -- VME WRITE* output,
      -- OE controlled by VMEMSTR
      nvmeBRI      : in std_logic_vector(3 downto 0);
      -- VME BR0* to BR3* inputs
      nvmeBRO      : out std_logic_vector(3 downto 0);
      -- VME BR0* to BR3* OC outputs
      nvmeBGIN     : in std_logic_vector(3 downto 0);
      -- VME BGIN0* to BGIN3* inputs
      nBGOUT       : out std_logic_vector(3 downto 0);
      -- VME BGOUT0* to BGOUT3* outputs,
      VMEMSTR      : out std_logic;
      nSYSCON      : in std_logic;  -- nSYSCON jumper input
      nvmeBERRI    : in std_logic;  -- VME BERR* input
      nvmeBERRO    : out std_logic; -- VME BERR* OC output
      nvmeBBSYI    : in std_logic;  -- VME BBSY* input
      vmeBBSYEN    : out std_logic; -- VME BBSY* OE
      nvmeBCLRI    : in std_logic;  -- VME BCLR* input
      nvmeBCLRO    : out std_logic; -- VME BCLR* output
      vmeBCLREN    : out std_logic; -- VME BCLR* OE
      nAOE         : out std_logic; -- VME address nOE
      ACLKAB       : out std_logic; -- VME address A to B clock (local -> VME)
      ACLKBA       : out std_logic; -- VME address B to A clock (VME -> local)
      ADIR         : out std_logic; -- VME address direction (1 = local -> VME)
      ALE          : out std_logic; -- VME address LE
      AMCLKAB      : out std_logic; -- VME AM A to B clock (local -> VME)
      AMCLKBA      : out std_logic; -- VME AM B to A clock (VME -> local)
      AMDIR        : out std_logic; -- VME AM DIR (1 = local -> VME) 
      AMLE         : out std_logic; -- VME AM LE
      nAMOE        : out std_logic; -- VME AM nOE
      nvmeLWORDI   : in std_logic;  -- VME LWORD* input
      nvmeLWORDO   : out std_logic; -- VME LWORD* output,
      -- OE controlled by VMEMSTR
      nDOE2        : out std_logic; -- VME D16 to D31 OE
      DCLKAB2      : out std_logic; -- VME D16 to D31 A to B clock (local -> VME)
      DCLKBA2      : out std_logic; -- VME D16 to D31 B to A clock (VME -> local)
      DDIR2        : out std_logic; -- VME D16 to D31 DIR (1 = local -> VME)
      DLE2         : out std_logic; -- VME D16 to D31 LE
      nDOE         : out std_logic; -- VME D0 to D15 OE
      DCLKAB       : out std_logic; -- VME D0 to D15 A to B clock (local -> VME)
      DCLKBA       : out std_logic; -- VME D0 to D15 B to A clock (VME -> local)
      DDIR         : out std_logic; -- VME D0 to D15 DIR (1 = local -> VME)
      DLE          : out std_logic; -- VME D0 to D15 LE
      nvmeIRQO     : out std_logic_vector(7 downto 1);
      
      -- MAC signals
      nEINTR       : in  std_logic;
      nESRDY       : in  std_logic;
      EBCLK        : out std_logic;
      nEDATACS     : out std_logic;
      nERDYRTN     : out std_logic;
      nEBE         : out std_logic_vector(3 downto 0);
      nECYCLE      : out std_logic;
      ESWR         : out std_logic;
      nEVLBUS      : out std_logic;
      EAEN         : out std_logic;
      
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      M_ABus          : out   std_logic_vector(0 to 31);
      M_BE            : out   std_logic_vector(0 to 3);
      M_busLock       : out   std_logic;
      M_request       : out   std_logic;
      M_RNW           : out   std_logic;
      M_select        : out   std_logic;
      M_seqAddr       : out   std_logic;
      OPB_errAck      : in    std_logic;
      OPB_MGrant      : in    std_logic;
      OPB_retry       : in    std_logic;
      OPB_timeout     : in    std_logic;
      OPB_xferAck     : in    std_logic;
      IP2INTC_Irpt    : out   std_logic;

      opb_irq         : in    std_logic_vector(0 downto 0);
      VME_irq_level   : in    std_logic_vector(2 downto 0);
      VME_irq_vector  : in    std_logic_vector(7 downto 0);
      VME64X_ADER0    : in    std_logic_vector(31 downto 0);
      VME64X_ADER1    : in    std_logic_vector(31 downto 0);
      VME64X_ADER2    : in    std_logic_vector(31 downto 0);
      VME64x_BAR      : in    std_logic_vector(7 downto 0));
  end component; 

  component opb_ubicom is
    port (
          -- Ubicom interface signals
    ULE          : in std_logic;
    nUBERR       : out std_logic;
    nUACK        : out std_logic;
    nUIRQ        : out std_logic;
    nURD         : in std_logic;
    nUWR         : in std_logic;
    UA           : in std_logic_vector(16 downto 0);
                                  -- Ubicom address bits A16-A9 and A0,
                                  -- A8-A1 are multiplexed with D7-D0
    UD           : inout std_logic_vector(7 downto 0);

    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    M_DBus          : out   std_logic_vector(0 to 31);
    OPB_DBus        : in    std_logic_vector(0 to 31);
    M_ABus          : out   std_logic_vector(0 to 31);
    M_BE            : out   std_logic_vector(0 to 3);
    M_busLock       : out   std_logic;
    M_request       : out   std_logic;
    M_RNW           : out   std_logic;
    M_select        : out   std_logic;
    M_seqAddr       : out   std_logic;
    OPB_errAck      : in    std_logic;
    OPB_MGrant      : in    std_logic;
    OPB_retry       : in    std_logic;
    OPB_timeout     : in    std_logic;
    OPB_xferAck     : in    std_logic);
  end component;

  component opb_cr_csr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR      : std_logic_vector;
      C_BASEMASK      : std_logic_vector);
    port (
    -- Bus protocol ports, do not add to or delete
    OPB_Clk         : in    std_logic;
    OPB_Rst         : in    std_logic;
    Sl_DBus         : out   std_logic_vector(0 to 31);
    Sl_errAck       : out   std_logic;
    Sl_retry        : out   std_logic;
    Sl_toutSup      : out   std_logic;
    Sl_xferAck      : out   std_logic;
    OPB_ABus        : in    std_logic_vector(0 to 31);
    OPB_BE          : in    std_logic_vector(0 to 3);
    OPB_DBus        : in    std_logic_vector(0 to 31);
    OPB_RNW         : in    std_logic;
    OPB_select      : in    std_logic;
    OPB_seqAddr     : in    std_logic;

    nGA           : in  std_logic_vector(5 downto 0);
                                  -- VME geographical addressing pins
                                  -- GA0* to GA4*, nGA5 is GAP*
    module_nBERR  : in  std_logic;
    module_fail   : in  std_logic;
    VME64x_BAR    : out std_logic_vector(7 downto 0);
    VME64x_ADER0  : out std_logic_vector(31 downto 0);
    VME64x_ADER1  : out std_logic_vector(31 downto 0);
    VME64x_ADER2  : out std_logic_vector(31 downto 0);
    SYSFAIL_ena   : out std_logic;
    module_ena    : out std_logic;
    module_reset  : out std_logic;
    VME_irq_level : out std_logic_vector(2 downto 0);
    VME_irq_vector : out std_logic_vector(7 downto 0);
    
    nSYSRESETO    : out std_logic := 'Z';
    PWRDWN        : out std_logic := '0';
    HSOPEN        : in std_logic;
    BLUEOFF       : out std_logic;
    PWRONIF       : out std_logic;
    nTBID         : in std_logic_vector(3 downto 0);
    debug_out     : out std_logic_vector(6 downto 0)
    );
  end component;

  component opb_evr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
      UNIVIN          : in std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

      GPIO            : inout std_logic_vector(C_EVR_GPIOS-1 downto 0);

      TBIN            : in std_logic_vector(0 to C_EVR_TB_INPUTS-1);
      TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic; -- Reference clock input from
      BREFCLKP        : in std_logic; -- fractional synthesizer
      BREFCLK2N       : in std_logic; -- Resynchronised clock input
      BREFCLK2P       : in std_logic; -- from FPGA external FF

      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;
      
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXN
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXN
      CMLRXN          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLRXP          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXN          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXP          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic;

      -- SFP transceiver serial interface signals
      sfp_moddef0 : in std_logic;
      sfp_sda     : inout std_logic;
      sfp_scl     : out std_logic;

      TXLOOPBACK  : out std_logic; -- when '1' loop back FPGA TX to FPGA RX
      RXLOOPBACK  : out std_logic; -- when '1' loop back optical RX signal
                                   -- to transceiver TX port
    
      debug       : out std_logic_vector(39 downto 0)
      );
  end component;

  component adi_config is
    port (
      clk      : in std_logic;
      usec_ena : in std_logic;
      sck      : out std_logic;
      sda      : inout std_logic;
      status   : out std_logic
    );
  end component;

begin

  i_lclk : local_clock
    port map (
      clkin => CLK33,
      clkfxout => clk);

  i_adi : adi_config
    port map (
      clk => clk,
      usec_ena => usec_ena,
      sck => RXSCKB2,
      sda => RXSDAB2);

  i_opb : system
    port map (
      sys_clk => clk,
      sys_rst => init_reset,
      opb_ms_0_OPB_Clk => vme_OPB_Clk,
      opb_ms_0_OPB_Rst => vme_OPB_Rst,
      opb_ms_0_Sl_DBus => vme_Sl_DBus,
      opb_ms_0_Sl_errAck => vme_Sl_errAck,
      opb_ms_0_Sl_retry => vme_Sl_retry,
      opb_ms_0_Sl_toutSup => vme_Sl_toutSup,
      opb_ms_0_Sl_xferAck => vme_Sl_xferAck,
      opb_ms_0_OPB_ABus => vme_OPB_ABus,
      opb_ms_0_OPB_BE => vme_OPB_BE,
      opb_ms_0_OPB_DBus => vme_OPB_DBus,
      opb_ms_0_OPB_RNW => vme_OPB_RNW,
      opb_ms_0_OPB_select => vme_OPB_select,
      opb_ms_0_OPB_seqAddr => vme_OPB_seqAddr,
      opb_ms_0_M_ABus => vme_M_ABus,
      opb_ms_0_M_BE => vme_M_BE,
      opb_ms_0_M_busLock => vme_M_busLock,
      opb_ms_0_M_request => vme_M_request,
      opb_ms_0_M_RNW => vme_M_RNW,
      opb_ms_0_M_select => vme_M_select,
      opb_ms_0_M_seqAddr => vme_M_seqAddr,
      opb_ms_0_OPB_errAck => vme_OPB_errAck,
      opb_ms_0_OPB_MGrant => vme_OPB_MGrant,
      opb_ms_0_OPB_retry => vme_OPB_retry,
      opb_ms_0_OPB_timeout => vme_OPB_timeout,
      opb_ms_0_OPB_xferAck => vme_OPB_xferAck,
      opb_ms_0_IP2INTC_Irpt => vme_IP2INTC_Irpt,
      opb_m_1_OPB_Clk => ubi_OPB_Clk,
      opb_m_1_OPB_Rst => ubi_OPB_Rst,
      opb_m_1_M_DBus => ubi_M_DBus,
      opb_m_1_OPB_DBus => ubi_OPB_DBus,
      opb_m_1_M_ABus => ubi_M_ABus,
      opb_m_1_M_BE => ubi_M_BE,
      opb_m_1_M_busLock => ubi_M_busLock,
      opb_m_1_M_request => ubi_M_request,
      opb_m_1_M_RNW => ubi_M_RNW,
      opb_m_1_M_select => ubi_M_select,
      opb_m_1_M_seqAddr => ubi_M_seqAddr,
      opb_m_1_OPB_errAck => ubi_OPB_errAck,
      opb_m_1_OPB_MGrant => ubi_OPB_MGrant,
      opb_m_1_OPB_retry => ubi_OPB_retry,
      opb_m_1_OPB_timeout => ubi_OPB_timeout,
      opb_m_1_OPB_xferAck => ubi_OPB_xferAck,
      opb_s_2_OPB_Clk => crcsr_OPB_Clk,
      opb_s_2_OPB_Rst => crcsr_OPB_Rst,
      opb_s_2_Sl_DBus => crcsr_Sl_DBus,
      opb_s_2_Sl_errAck => crcsr_Sl_errAck,
      opb_s_2_Sl_retry => crcsr_Sl_retry,
      opb_s_2_Sl_toutSup => crcsr_Sl_toutSup,
      opb_s_2_Sl_xferAck => crcsr_Sl_xferAck,
      opb_s_2_OPB_ABus => crcsr_OPB_ABus,
      opb_s_2_OPB_BE => crcsr_OPB_BE,
      opb_s_2_OPB_DBus => crcsr_OPB_DBus,
      opb_s_2_OPB_RNW => crcsr_OPB_RNW,
      opb_s_2_OPB_select => crcsr_OPB_select,
      opb_s_2_OPB_seqAddr => crcsr_OPB_seqAddr,
      opb_s_3_OPB_Clk => evr_OPB_Clk,
      opb_s_3_OPB_Rst => evr_OPB_Rst,
      opb_s_3_Sl_DBus => evr_Sl_DBus,
      opb_s_3_Sl_errAck => evr_Sl_errAck,
      opb_s_3_Sl_retry => evr_Sl_retry,
      opb_s_3_Sl_toutSup => evr_Sl_toutSup,
      opb_s_3_Sl_xferAck => evr_Sl_xferAck,
      opb_s_3_OPB_ABus => evr_OPB_ABus,
      opb_s_3_OPB_BE => evr_OPB_BE,
      opb_s_3_OPB_DBus => evr_OPB_DBus,
      opb_s_3_OPB_RNW => evr_OPB_RNW,
      opb_s_3_OPB_select => evr_OPB_select,
      opb_s_3_OPB_seqAddr => evr_OPB_seqAddr,
      chipscope_opb_iba_0_iba_trig_in_pin => VME_debug
      );

  i_opb_vme : opb_vme
    generic map (
      C_ETHBASEADDR => ETH_LOCAL_BASE,
      C_ETHBASEMASK => LOCAL_BASE_MASK,
      C_ADER0_CMP_HIGH => 31, -- Address range -1 (A16) 
      C_ADER0_CMP_LOW =>  11, -- Size = 2k
      C_FUNC0_LOCAL_BASE => EVR_LOCAL_BASE,
      C_ADER1_CMP_HIGH => 31, -- Address range -1 (A24/A32) 
      C_ADER1_CMP_LOW =>  16, -- Size = 64k
      C_FUNC1_LOCAL_BASE => EVR_LOCAL_BASE,
      C_FUNC2_LOCAL_BASE => EVR_LOCAL_BASE,
      C_CRCSR_LOCAL_BASE => CRCSR_LOCAL_BASE
      )
    port map (
      LA => LA,
      LD => LD,
      vmeAM => vmeAM,
      nvmeIACK => nvmeIACK,
      nvmeASI => nvmeASI,
      nvmeASO => nvmeASO,
      nvmeDSI => nvmeDSI,
      nvmeDSO => nvmeDSO,
      nvmeRETRYI => nvmeRETRYI,
      nvmeRETRYO => nvmeRETRYO,
      RETRYEN => RETRYEN,
      nvmeIACKIN => nvmeIACKIN,
      nvmeIACKOUT => nvmeIACKOUT,
      nvmeDTACKI => nvmeDTACKI,
      nvmeDTACKO => nvmeDTACKO,
      DTACKEN => DTACKEN,
      nvmeWRITEI => nvmeWRITEI,
      nvmeWRITEO => nvmeWRITEO,
      nvmeBRI => nvmeBRI,
      nvmeBRO => nvmeBRO,
      nvmeBGIN => nvmeBGIN,
      nBGOUT => nvmeBGOUT,
      VMEMSTR => VMEMSTR,
      nSYSCON => nSYSCON,
      nvmeBERRI => nvmeBERRI,
      nvmeBERRO => module_nBERR,
      nvmeBBSYI => nvmeBBSYI,
      vmeBBSYEN => vmeBBSYEN,
      nvmeBCLRI => nvmeBCLRI,
      nvmeBCLRO => nvmeBCLRO,
      vmeBCLREN => vmeBCLREN,
      nAOE => nAOE,
      ACLKAB => ACLKAB,
      ACLKBA => ACLKBA,
      ADIR => ADIR,
      ALE => ALE,
      AMCLKAB => AMCLKAB,
      AMCLKBA => AMCLKBA,
      AMDIR => AMDIR,
      AMLE => AMLE,
      nAMOE => nAMOE,
      nvmeLWORDI => nvmeLWORDI,
      nvmeLWORDO => nvmeLWORDO,
      nDOE2 => nDOE2,
      DCLKAB2 => DCLKAB2,
      DCLKBA2 => DCLKBA2,
      DDIR2 => DDIR2,
      DLE2 => DLE2,
      nDOE => nDOE,
      DCLKAB => DCLKAB,
      DCLKBA => DCLKBA,
      DDIR => DDIR,
      DLE => DLE,
      nvmeIRQO => nvmeIRQO,
      nEINTR => nEINTR,
      nESRDY => nESRDY,
      EBCLK => EBCLK,
      nEDATACS => nEDATACS,
      nERDYRTN => nERDYRTN,
      nEBE => nEBE,
      nECYCLE => nECYCLE,
      ESWR => ESWR,
      nEVLBUS => nEVLBUS,
      EAEN => EAEN,
      OPB_Clk => clk,
      OPB_Rst => vme_OPB_Rst,
      Sl_DBus => vme_Sl_DBus,
      Sl_errAck => vme_Sl_errAck,
      Sl_retry => vme_Sl_retry,
      Sl_toutSup => vme_Sl_toutSup,
      Sl_xferAck => vme_Sl_xferAck,
      OPB_ABus => vme_OPB_ABus,
      OPB_BE => vme_OPB_BE,
      OPB_DBus => vme_OPB_DBus,
      OPB_RNW => vme_OPB_RNW,
      OPB_select => vme_OPB_select,
      OPB_seqAddr => vme_OPB_seqAddr,
      M_ABus => vme_M_ABus,
      M_BE => vme_M_BE,
      M_busLock => vme_M_busLock,
      M_request => vme_M_request,
      M_RNW => vme_M_RNW,
      M_select => vme_M_select,
      M_seqAddr => vme_M_seqAddr,
      OPB_errAck => vme_OPB_errAck,
      OPB_MGrant => vme_OPB_MGrant,
      OPB_retry => vme_OPB_retry,
      OPB_timeout => vme_OPB_timeout,
      OPB_xferAck => vme_OPB_xferAck,
      IP2INTC_Irpt => vme_IP2INTC_Irpt,
      opb_irq(0) => evr_irq,
      VME_irq_level => VME_irq_level,
      VME_irq_vector => evr_irq_vector,
      VME64X_ADER0 => VME64X_ADER0,
      VME64X_ADER1 => VME64X_ADER1,
      VME64X_ADER2 => VME64X_ADER2,
      VME64x_BAR => VME64x_BAR);

  i_opb_ubicom : opb_ubicom
    port map (
      ULE => ULE,
      nUBERR => nUBERR,
      nUACK => nUACK,
      nUIRQ => nUIRQ,
      nURD => nURD,
      nUWR => nUWR,
      UA => UA,
      UD => UD,
      OPB_Clk => clk,
      OPB_Rst => ubi_OPB_Rst,
      M_DBus => ubi_M_DBus,
      OPB_DBus => ubi_OPB_DBus,
      M_ABus => ubi_M_ABus,
      M_BE => ubi_M_BE,
      M_busLock => ubi_M_busLock,
      M_request => ubi_M_request,
      M_RNW => ubi_M_RNW,
      M_select => ubi_M_select,
      M_seqAddr => ubi_M_seqAddr,
      OPB_errAck => ubi_OPB_errAck,
      OPB_MGrant => ubi_OPB_MGrant,
      OPB_retry => ubi_OPB_retry,
      OPB_timeout => ubi_OPB_timeout,
      OPB_xferAck => ubi_OPB_xferAck);

  i_opb_cr_csr : opb_cr_csr
    generic map (
      C_BASEADDR => CRCSR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => clk,
      OPB_Rst => crcsr_OPB_Rst,
      Sl_DBus => crcsr_Sl_DBus,
      Sl_errAck => crcsr_Sl_errAck,
      Sl_retry => crcsr_Sl_retry,
      Sl_toutSup => crcsr_Sl_toutSup,
      Sl_xferAck => crcsr_Sl_xferAck,
      OPB_ABus => crcsr_OPB_ABus,
      OPB_BE => crcsr_OPB_BE,
      OPB_DBus => crcsr_OPB_DBus,
      OPB_RNW => crcsr_OPB_RNW,
      OPB_select => crcsr_OPB_select,
      OPB_seqAddr => crcsr_OPB_seqAddr,
      nGA => nGA,
      module_nBERR => module_nBERR,
      module_fail => module_fail,
      VME64x_BAR => VME64x_BAR,
      VME64x_ADER0 => VME64x_ADER0,
      VME64x_ADER1 => VME64x_ADER1,
      VME64x_ADER2 => VME64x_ADER2,
      SYSFAIL_ena => SYSFAIL_ena,
      module_ena => module_ena,
      module_reset => module_reset,
      VME_irq_level => VME_irq_level, 
      VME_irq_vector => evr_irq_vector,
      nSYSRESETO => nSYSRESETO,
      PWRDWN => PWRDWN,
      HSOPEN => HSOPEN,
      BLUEOFF => BLUEOFF,
      PWRONIF => PWRONIF,
      nTBID => nTBID);

  i_opb_evr : opb_evr
    generic map (
      C_BASEADDR => EVR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => clk,
      OPB_Rst => evr_OPB_Rst,
      Sl_DBus => evr_Sl_DBus,
      Sl_errAck => evr_Sl_errAck,
      Sl_retry => evr_Sl_retry,
      Sl_toutSup => evr_Sl_toutSup,
      Sl_xferAck => evr_Sl_xferAck,
      OPB_ABus => evr_OPB_ABus,
      OPB_BE => evr_OPB_BE,
      OPB_DBus => evr_OPB_DBus,
      OPB_RNW => evr_OPB_RNW,
      OPB_select => evr_OPB_select,
      OPB_seqAddr => evr_OPB_seqAddr,

      ip_irq_out => evr_irq,
    
      TTLIN => TTLIN,
      TTLOUT => TTLOUT,
      UNIVIN => UNIVIN,
      UNIVOUT => UNIVOUT,

      GPIO => GPIO,
      
      TBIN => TBIN,
      TBOUT => IO,

      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      BREFCLK2N => BREFCLK2N,
      BREFCLK2P => BREFCLK2P,

      RECCLKN => RECCLKN,
      RECCLKP => RECCLKP,
      
      LED_fail => LED(0),
      LED_rx_link => LED(4),
      LED_ena => LED(3),
      LED_event_in => LED(6),
      LED_event_out => LED(5),
      LED_rx_fail => LED(7),
      LED_error => LED(8),
    
      EVRXN => RXN1,
      EVRXP => RXP1,
      EVTXN => TXN1,
      EVTXP => TXP1,
      CMLRXN => CMLRXN,
      CMLRXP => CMLRXP,
      CMLTXN => CMLTXN,
      CMLTXP => CMLTXP,

      CGSK => CGSK,
      CGDI => CGDI,
      CGCS => CGCS,
      CGLOCK => CGLOCK,

      sfp_moddef0 => MOD_DEF0,
      sfp_sda => MOD_DEF2,
      sfp_scl => MOD_DEF1,
      
      TXLOOPBACK => TXLOOPBACK,
      RXLOOPBACK => RXLOOPBACK);

  CMLRXN <= RXN2 & RXN3 & RXN4;
  CMLRXP <= RXP2 & RXP3 & RXP4;
  TXN2 <= CMLTXN(0);
  TXN3 <= CMLTXN(1);
  TXN4 <= CMLTXN(2);
  TXP2 <= CMLTXP(0);
  TXP3 <= CMLTXP(1);
  TXP4 <= CMLTXP(2);
  
  DCHENA <= '1';
  module_fail <= '0';
  
  nvmeSYSFAILO <= '1';

  nvmeBERRO <= module_nBERR;

  nTBID(0) <= nEVGIF;
  nTBID(1) <= nEVRIF;
  nTBID(2) <= nIF(2);
  nTBID(3) <= nIF(3);
  
  DRD <= (others => '0');
  DRA(12 downto 0) <= (others => '0');
  DQM <= (others => '0');
  DRCLK <= '0';
  nDRWE <= '1';
  nDRCAS <= '1';
  nDRRAS <= '1';
  nDRCS <= '1';
  DRCKE <= '0';
  DRBA <= "00";

--  RXSDAB2 <= '0';
--  RXSCKB2 <= '0';

--  TXSEL <= '0';
--  RXSEL <= '0';

  BREFSEL <= '0';

  RXREFCLK <= '1';

  OSCENA <= '1';
  
  Synchronous_reset: process (clk)
    variable reset_delay : std_logic_vector(4 downto 0) := "01111";
  begin
    if rising_edge(clk) then
      init_reset <= '0';
      if reset_delay(reset_delay'high) = '0' then
	init_reset <= '1';
	reset_delay := reset_delay - 1;
      end if;
    end if;
  end process;

  crosspoint_switch: process (RXLOOPBACK, TXLOOPBACK)
  begin
    if C_EVR_ENABLE_BACKWARD_CHANNEL = TRUE then
      -- Get gigabit signal from transmitter (if not in loopback mode)
      TXSEL <= not RXLOOPBACK;
    else
      -- Get gigabit signal from transceiver RX input
      TXSEL <= '0';
    end if;
    RXSEL <= TXLOOPBACK;
  end process;

  usec_enable_generation : process (clk)
    variable counter : std_logic_vector(7 downto 0) := (others => '0');
  begin
    if rising_edge(clk) then
      usec_ena <= '0';
      if counter(counter'high) = '1' then
        counter := X"20";
        usec_ena <= '1';
      end if;
      counter := counter - 1;
    end if;
  end process;

  VME_debug(39 downto 34) <= vmeAM;
  VME_debug(33 downto 32) <= nvmeDSI;
  VME_debug(31) <= nvmeASI;
  VME_debug(30) <= nvmeDTACKI;
  VME_debug(29) <= nvmeWRITEI;
  VME_debug(28) <= nvmeLWORDI;
  VME_debug(27 downto 16) <= LD(11 downto 0);
  VME_debug(15 downto 0) <= LA(15 downto 0);

end rtl;
